/*
            Copyright(C) 1997-2022.  Sinowealth Tech. Co., Ltd.
********************************************************************************
* @file    CFG33F2054.c 
* @author  
* @version V0.0.0
* @date    26-May-2024
* @brief   Keil configuration Wizard file.
********************************************************************************
*/

#include "CFG33F2054.h"
#include "sh33f2054.h"

#ifdef _MODULE_DBG_PRINTF

void flush_printfbuffer(void);
#if defined ( __CC_ARM )
struct __FILE { int handle; /* Add whatever you need here */ };
FILE __stdout;
FILE __stdin;
#elif defined(__ICCARM__)
struct __FILE { int handle; /* Add whatever you need here */ };
typedef struct __FILE FILE;
FILE __stdout;
FILE __stdin;
#endif
  
#include <stdio.h>



/**
  * @brief    retarget initial function, parameters defined in sh32f2xx_config.h 
  * \n  1. initial the retarget device
  * \n  2. can be selected as ITM or UART
  * @retval  None
  */
#define PTN_DBG_UART_BAUDRATE 115200
  
void RetargetInit(void)
{
#ifdef _DBG_DEVICE_UART
    
    uint32_t module_clk = (SYS_CLK/APB_F);
   	uint32_t sbrt;
    uint32_t fine;

    /* Enable Clock Gates */    
    RCC->RCCLOCK = 0x33CC;
    RCC->APB0ENR.BIT.UART0EN = 1;
    RCC->AHBENR.BIT.GPIOEN = 1;

    /* GPIO Config */
    GPIOB->AFRH.BIT.AFR8 = 7; //RX
    GPIOB->AFRH.BIT.AFR9 = 8; //TX

    PTN_DBG_UART->CR.BIT.SBRTEN = 1;
	  PTN_DBG_UART->CR.BIT.SM = 1;
    sbrt = module_clk /16/ PTN_DBG_UART_BAUDRATE-1;
    fine = module_clk / PTN_DBG_UART_BAUDRATE - (16*(sbrt+1));    
    
    PTN_DBG_UART->BRT.BIT.SBRT = sbrt;
    PTN_DBG_UART->BRT.BIT.BFINE = fine;//10;//0;
    
    PTN_DBG_UART->CR.BIT.RIE = 0;   
    PTN_DBG_UART->CR.BIT.TIE = 0;   
    PTN_DBG_UART->CR.BIT.REN = 1;   
    PTN_DBG_UART->CR.BIT.TEN = 1; 
#endif    
}


#ifdef _DBG_DEVICE_UART
/**
  * @brief send data to UART
  * @param ch: data to transmit
  * @retval  uint8_t data to transmit
  */
static uint8_t dbg_uart_send(uint8_t ch)
{
   while(PTN_DBG_UART->FR.BIT.TI == 0)
   {
        CLR_WDT();
   }
	 PTN_DBG_UART->TDR.BIT.TDR = ch;
   return (ch);
}   

/**
  * @brief read data from to UART
  * @retval  uint8_t data readed
  */
static uint8_t dbg_uart_read()
{
	 while(PTN_DBG_UART->FR.BIT.RI == 0)
   {
       CLR_WDT();
   }        
   return (uint8_t)PTN_DBG_UART->RDR.BIT.RDR;
}
#endif 
    

/**
  * @brief    output character function
  * @param  ch  output character
  * @param  f   file handle
  * @retval  int   outputted character
  */
#if defined(__clang__)
int fputc(int ch, FILE *f __attribute__((unused))) 
#else
int fputc(int ch, FILE *f) 
#endif
{  
#if !defined(__GNUC__)   
    f = f;
#elif !defined(__clang__)    
    f = 0;    
#endif  

#ifdef _DBG_DEVICE_UART
   if(ch == '\n'){
       dbg_uart_send('\r');
   }
   return (int)dbg_uart_send((uint8_t)ch);
#endif  
}

/**
  * @brief    input retarget
  * @param  f  file handle
  * @retval  int character which read
  */
#if defined(__clang__)
int fgetc(FILE *f __attribute__((unused))) 
#else
int fgetc(FILE *f) 
#endif
{
  char ch;
#if !defined(__GNUC__)   
    f = f;
#elif !defined(__clang__) 
    f = 0;
#endif  

#ifdef _DBG_DEVICE_UART
  ch = dbg_uart_read();
  dbg_uart_send(ch);
  if(ch == '\r')
     dbg_uart_send('\n');
#endif 
   return ch;
}

#if defined ( __clang__ )
#ifndef __MICROLIB
/*fix problem: printf enter __bkpt(0) assert when no microlib selected */
#define MEM_32(addr) (*(volatile uint32_t*)(addr))
_ARMABI FILE *freopen(const char * __restrict filename __attribute__((unused)),
                      const char * __restrict mode __attribute__((unused)),
                      FILE * __restrict stream)
{
    uint32_t addr = (uint32_t)stream;
    uint32_t off = MEM_32(addr + 0x50);
    MEM_32(addr+0x4) = 0;
    MEM_32(addr+0x10) = 0;
    MEM_32(addr+0xC) = 2;
    MEM_32(addr+0x14) = 1;                        
    MEM_32(addr+0x50) = off | 1;
    return stream;
}
#endif
#endif

#if defined(__ICCARM__)

#ifdef _DBG_DEVICE_UART
/*! for replace the function in getchar.o */
int _UngetChar(void)
{
    return 0;
}
/*! for replace the function in getchar.o */
int __ungetchar(int a)
{
    return 0;
}
/**
  * @brief  replace the function getchar for IAR scanf
  * @retval  int data readed from input device
  */
int getchar(void)
{
   char ch = dbg_uart_read();
   dbg_uart_send(ch);
   if(ch == '\r')
      dbg_uart_send('\n');
   return (int)ch;
}

/**
  * @brief  replace the function putchar for IAR printf
  * @param ch data to output device
  * @retval  int data to output device
  */
int putchar(int ch) 
{  
   if(ch == '\n')
   {
       dbg_uart_send('\r');
   }
   return (int)dbg_uart_send(ch);
}
#endif /*_DBG_DEVICE_UART*/

#endif /*__ICCARM__*/



#if defined(__GNUC__)

#if !defined(__CC_ARM)
/**
  * @brief  provide the function for GCC printf
  * @param  fd: file pointer no used
  * @param  pBuffer data buffer
  * @param  size data buffer size
  * @retval int outputted data size
  */
extern int _write (int fd, char *pBuffer, int size);
#if defined(__clang__)
int _write (int fd __attribute__((unused)), char *pBuffer, int size)  
#else
int _write (int fd, char *pBuffer, int size)  
#endif
{  
   int i;
#if !defined(__clang__)    
   fd = 0;
#endif
    
#ifdef _DBG_DEVICE_UART   
   for (i = 0; i < size; i++)  
   {  
      if(pBuffer[i] == '\n')
      {
        dbg_uart_send('\r');
      }
      dbg_uart_send(pBuffer[i]);
   }     
#endif   
   return size;  
}  

/**
  * @brief  provide the function for GCC scanf
  * @param  fd: file pointer no used
  * @param  pBuffer data buffer
  * @param  size data buffer size
  * @retval int readed data size
  */
extern int _read (int fd, char *pBuffer, int size);  
#if defined(__clang__)
int _read (int fd __attribute__((unused)), char *pBuffer, int size)      
#else
int _read (int fd, char *pBuffer, int size)  
#endif
{  
    int i; 
    
#if !defined(__clang__)    
   fd = 0;
#endif
    
#ifdef _DBG_DEVICE_UART
    for(i = 0; i < size; i++)
    {
        pBuffer[i]=dbg_uart_read();
        dbg_uart_send(pBuffer[i]);
        if(pBuffer[i] == '\r')
        {
            dbg_uart_send('\n');
            i += 1;
            pBuffer[i]='\n';
            i++;
            break;
        }
    }    
#endif    
   return i;  
}      

/*! flush printf buffer to output device */
void flush_printfbuffer(void)
{
    fflush(0);
}
#endif

#endif  /* __GNUC__ */
  
#endif /* _MODULE_DBG_PRINTF */

void Peripherals_InitCfg(void)
{
#if CFG_SYSCFG_EN	
	SYSCFG_InitCfg();
#endif		
	
#if CFG_IWDT_EN
	IWDT_InitCfg();
	CLR_WDT();	
#endif 
	
#if CFG_WWDT_EN
	WWDT_InitCfg();
#endif 
	
#if CFG_GPIO_EN
	GPIO_InitCfg();
#endif	

#if CFG_AMOC_EN	
	AMOC_InitCfg();
#endif	
	
#if CFG_ADC_EN	
	ADC_InitCfg();
#endif
	
	NVIC_InitCfg();

#if CFG_SYSTICK_EN	
	SysTick_InitCfg();
#endif	

#if CFG_EXTI_EN	
	EXTI_InitCfg();
#endif

#if CFG_MCM_EN	
	MCM0_InitCfg();
	MCM1_InitCfg();
#endif	

#if CFG_PCA_EN	
	PCA_InitCfg();
#endif	

#if CFG_TIM_EN
	TIM_InitCfg();
#endif 

#if CFG_MACP_EN
	MACP_InitCfg();
#endif 
	
#if CFG_QEI_EN
	QEI_InitCfg();
#endif 
	
#if CFG_UART_EN	
	UART_InitCfg();
#endif

#if CFG_SPI_EN	
	SPI_InitCfg();
#endif	
	
#if CFG_TWI_EN	
	TWI_InitCfg();
#endif
}

#if CFG_SYSCFG_EN
void SYSCFG_InitCfg(void)
{
	//Enable SYSCFG Clock
	RCC_WriteEnable();
    
    RCC->AHBENR.BIT.SYSCFGEN = 1;
    RCC_WriteDisable();	
	
	//Set the registers
	SYSCFG->PWRCR.V32 = CFG_SYSCFG_PWRCR_INIT;
    SYSCFG->SAFR.V32  = (0x5AA50000UL) | CFG_SYSCFG_SAFR_INIT;
}
#endif

void IWDT_InitCfg(void)
{
	//Set the registers
	IWDT->CR.V32 = (0x5AA5<<16) | CFG_IWDTRLR_INIT | CFG_IWDTPR_INIT;	
}

void WWDT_InitCfg(void)
{
	//Enable WWDT Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.WWDTEN = 1;
  RCC_WriteDisable();	
	
	//Set the registers
	WWDT->CR.V32 = (0x5AA5<<16) | CFG_WWDT_CR_INIT;
	WWDT->WTR.V32 = (0x5AA5<<16) | CFG_WWDT_WTR_INIT;
}

void GPIO_InitCfg(void)
{
  //Enable GPIO Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.GPIOEN = 1;
  RCC_WriteDisable();
	
#if CFG_GPIOA_EN
	//Set the registers
	GPIOA->ODR = CFG_GPIO_ODR_A_INIT;
	GPIOA->MODER.V32 = CFG_GPIO_MODER_A_INIT;
	GPIOA->OTYPER.V32 = CFG_GPIO_OTYPER_A_INIT;
	GPIOA->PUPDR.V32 = CFG_GPIO_PUPDR_A_INIT;
	GPIOA->AFRL.V32 = CFG_GPIO_AFRL_A_INIT;
	GPIOA->AFRH.V32 = CFG_GPIO_AFRH_A_INIT;
	GPIOA->IE.V32 = CFG_GPIO_IE_A_INIT;
	GPIOA->TTLEN.V32 = CFG_GPIO_TTLEN_A_INIT;
	GPIOA->LCKR.V32 = CFG_GPIO_LCKR_A_INIT;
#endif

#if CFG_GPIOB_EN
	//Set the registers
	GPIOB->ODR = CFG_GPIO_ODR_B_INIT;
	GPIOB->MODER.V32 = CFG_GPIO_MODER_B_INIT;
	GPIOB->OTYPER.V32 = CFG_GPIO_OTYPER_B_INIT;
	GPIOB->PUPDR.V32 = CFG_GPIO_PUPDR_B_INIT;
	GPIOB->AFRL.V32 = CFG_GPIO_AFRL_B_INIT;
	GPIOB->AFRH.V32 = CFG_GPIO_AFRH_B_INIT;
	GPIOB->IE.V32 = CFG_GPIO_IE_B_INIT;
	GPIOB->TTLEN.V32 = CFG_GPIO_TTLEN_B_INIT;
	GPIOB->LCKR.V32 = CFG_GPIO_LCKR_B_INIT;
#endif

#if CFG_GPIOC_EN
	//Set the registers
	GPIOC->ODR = CFG_GPIO_ODR_C_INIT;
	GPIOC->MODER.V32 = CFG_GPIO_MODER_C_INIT;
	GPIOC->OTYPER.V32 = CFG_GPIO_OTYPER_C_INIT;
	GPIOC->PUPDR.V32 = CFG_GPIO_PUPDR_C_INIT;
	GPIOC->AFRL.V32 = CFG_GPIO_AFRL_C_INIT;
	GPIOC->AFRH.V32 = CFG_GPIO_AFRH_C_INIT;
	GPIOC->IE.V32 = CFG_GPIO_IE_C_INIT;
	GPIOC->TTLEN.V32 = CFG_GPIO_TTLEN_C_INIT;
	GPIOC->LCKR.V32 = CFG_GPIO_LCKR_C_INIT;
#endif

#if CFG_GPIOD_EN
	//Set the registers
	GPIOD->ODR = CFG_GPIO_ODR_D_INIT;
	GPIOD->MODER.V32 = CFG_GPIO_MODER_D_INIT;
	GPIOD->OTYPER.V32 = CFG_GPIO_OTYPER_D_INIT;
	GPIOD->PUPDR.V32 = CFG_GPIO_PUPDR_D_INIT;
	GPIOD->AFRL.V32 = CFG_GPIO_AFRL_D_INIT;
	GPIOD->AFRH.V32 = CFG_GPIO_AFRH_D_INIT;
	GPIOD->IE.V32 = CFG_GPIO_IE_D_INIT;
	GPIOD->TTLEN.V32 = CFG_GPIO_TTLEN_D_INIT;
	GPIOD->LCKR.V32 = CFG_GPIO_LCKR_D_INIT;
#endif

#if CFG_GPIOE_EN
	//Set the registers
	GPIOE->ODR = CFG_GPIO_ODR_E_INIT;
	GPIOE->MODER.V32 = CFG_GPIO_MODER_E_INIT;
	GPIOE->PUPDR.V32 = CFG_GPIO_PUPDR_E_INIT;
	GPIOE->AFRL.V32 = CFG_GPIO_AFRL_E_INIT;
	GPIOE->IE.V32 = CFG_GPIO_IE_E_INIT;
	GPIOE->LCKR.V32 = CFG_GPIO_LCKR_E_INIT;
#endif
}


void EXTI_InitCfg(void)
{
    RCC_WriteEnable();
    RCC->APB0ENR.BIT.EXTIEN = 1;
    RCC_WriteDisable();
    
	//Set the registers
	EXTI->IMR = CFG_EXTI_IMR_INIT;
	EXTI->EMR = CFG_EXTI_EMR_INIT;
	EXTI->TMSR = CFG_EXTI_TMSR_INIT;
	EXTI->RTSR = CFG_EXTI_RTSR_INIT;
	EXTI->FTSR = CFG_EXTI_FTSR_INIT;
	EXTI->CFGL.V32 = CFG_EXTI_CFGL_INIT;
	EXTI->CFGH.V32 = CFG_EXTI_CFGH_INIT;
	EXTI->SAMPL.V32 = CFG_EXTI_SAMPL_INIT;
	EXTI->SAMPH.V32 = CFG_EXTI_SAMPH_INIT;
}


void DMA_InitCfg(void)
{
	//Enable DMA Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.DMAEN = 1;
  RCC_WriteDisable();
	
#if CFG_DMA_CH0_EN
	//Set the registers
	DMA->NPKT0 = CFG_DMA_NPKT0_INIT;
	DMA->SAR0 =	CFG_DMA_SAR0_INIT;
	DMA->DAR0 =	CFG_DMA_DAR0_INIT;
	DMA->CCR0.V32 = CFG_DMA_CCR0_INIT | CFG_DMA_CH0_EN;
#endif

#if CFG_DMA_CH1_EN
	//Set the registers
	DMA->NPKT1 = CFG_DMA_NPKT1_INIT;
	DMA->SAR1 =	CFG_DMA_SAR1_INIT;
	DMA->DAR1 =	CFG_DMA_DAR1_INIT;
	DMA->CCR1.V32 = CFG_DMA_CCR1_INIT | CFG_DMA_CH1_EN;
#endif
	
#if CFG_DMA_CH2_EN
	//Set the registers
	DMA->CCR2.V32 = CFG_DMA_CCR2_INIT;
	DMA->NPKT2 = CFG_DMA_NPKT2_INIT;
	DMA->SAR2 =	CFG_DMA_SAR2_INIT;
	DMA->DAR2 =	CFG_DMA_DAR2_INIT;
#endif
	
#if CFG_DMA_CH3_EN
	//Set the registers
	DMA->CCR3.V32 = CFG_DMA_CCR3_INIT;
	DMA->NPKT3 = CFG_DMA_NPKT3_INIT;
	DMA->SAR3 =	CFG_DMA_SAR3_INIT;
	DMA->DAR3 =	CFG_DMA_DAR3_INIT;
#endif
	
#if CFG_DMA_CH4_EN
	//Set the registers
	DMA->CCR4.V32 = CFG_DMA_CCR4_INIT;
	DMA->NPKT4 = CFG_DMA_NPKT4_INIT;
	DMA->SAR4 =	CFG_DMA_SAR4_INIT;
	DMA->DAR4 =	CFG_DMA_DAR4_INIT;
#endif
	
#if CFG_DMA_CH5_EN
	//Set the registers
	DMA->CCR5.V32 = CFG_DMA_CCR5_INIT;
	DMA->NPKT5 = CFG_DMA_NPKT5_INIT;
	DMA->SAR5 =	CFG_DMA_SAR5_INIT;
	DMA->DAR5 =	CFG_DMA_DAR5_INIT;
#endif

#if CFG_DMA_CH6_EN
	//Set the registers
	DMA->CCR6.V32 = CFG_DMA_CCR4_INIT;
	DMA->NPKT6 = CFG_DMA_NPKT4_INIT;
	DMA->SAR6 =	CFG_DMA_SAR4_INIT;
	DMA->DAR6 =	CFG_DMA_DAR4_INIT;
#endif
	
#if CFG_DMA_CH7_EN
	//Set the registers
	DMA->CCR7.V32 = CFG_DMA_CCR5_INIT;
	DMA->NPKT7 = CFG_DMA_NPKT5_INIT;
	DMA->SAR7 =	CFG_DMA_SAR5_INIT;
	DMA->DAR7 =	CFG_DMA_DAR5_INIT;
#endif
	
  DMA->CSR.V32 = CFG_DMA_CSR_INIT; 
}


void MCM0_InitCfg(void)
{
	//Enable MCM0 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.MCM0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	MCM0->PWMRLDEN0 = 0x55;
	
	MCM0->PWMREMAP = CFG_MCM0_PWMREMAP_INIT;
	
	MCM0->PWMCON1.V32 = CFG_MCM0_PWMCON1_INIT;
	MCM0->PWMCON2.V32 = CFG_MCM0_PWMCON2_INIT;	
	
	MCM0->PWMP = CFG_MCM0_PWMP_INIT;
	MCM0->PWM0D = CFG_MCM0_PWM0D_INIT;
	MCM0->PWM1D = CFG_MCM0_PWM1D_INIT;
	MCM0->PWM2D = CFG_MCM0_PWM2D_INIT;
	MCM0->PWM01D = CFG_MCM0_PWM01D_INIT; 
	MCM0->PWM11D = CFG_MCM0_PWM11D_INIT; 
	MCM0->PWM21D = CFG_MCM0_PWM21D_INIT; 

	MCM0->PWMDT21	= CFG_MCM0_PWMDT21_INIT;
	MCM0->PWMDT20	= CFG_MCM0_PWMDT20_INIT;
	MCM0->PWMDT11	= CFG_MCM0_PWMDT11_INIT;
	MCM0->PWMDT10	= CFG_MCM0_PWMDT10_INIT;
	MCM0->PWMDT01	= CFG_MCM0_PWMDT01_INIT;
	MCM0->PWMDT00	= CFG_MCM0_PWMDT00_INIT;
	
	MCM0->PWMPSQ = CFG_MCM0_PWMPSQ_INIT;
	
	MCM0->PMANUALCON1.V32 = CFG_MCM0_PMANUALCON1_INIT;
	MCM0->PMANUALCON2.V32 = CFG_MCM0_PMANUALCON2_INIT;	
		
	MCM0->PWMCMP1 = CFG_MCM0_PWMCMP1_INIT;
	MCM0->PWMCMP2 = CFG_MCM0_PWMCMP2_INIT;
	MCM0->PWMCMP3 = CFG_MCM0_PWMCMP3_INIT;
	MCM0->PWMCMP4 = CFG_MCM0_PWMCMP4_INIT;
	
	MCM0->PWMINTEN.V32 = CFG_MCM0_PWMINTEN_INIT;
	MCM0->PSCON.V32 = CFG_MCM0_PSCON_INIT;	
	MCM0->PWMDMAX = CFG_MCM0_PWMDMAX_INIT;	
	MCM0->PWMDMIN = CFG_MCM0_PWMDMIN_INIT;	
	MCM0->PWMDCMP1 = CFG_MCM0_PWMDCMP1_INIT;
	MCM0->PWMDCMP2 = CFG_MCM0_PWMDCMP2_INIT;
	
	MCM0->SC0CON.V32 = CFG_MCM0_SC0CON_INIT;
	MCM0->SC1CON.V32 = CFG_MCM0_SC1CON_INIT;
	MCM0->SC2CON.V32 = CFG_MCM0_SC2CON_INIT;

	MCM0->FLTWEN = 0x33CC;
	MCM0->FLTCON.V32 = CFG_MCM0_FLTCON_INIT;
	MCM0->POSCR.V32 = CFG_MCM0_POSCR_INIT;	
	MCM0->POSTDCR.V32 = CFG_MCM0_POSTDCR_INIT;
	MCM0->FLTWEN = 0x0000;
	MCM0->PWMOE = CFG_MCM0_PWMOE;
	MCM0->PWMRLDEN1 = 0xAA;
}

void MCM1_InitCfg(void)
{
	//Enable MCM1 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.MCM1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	MCM1->PWMRLDEN0 = 0x55;
	
	MCM1->PWMREMAP = CFG_MCM1_PWMREMAP_INIT;
	
	MCM1->PWMCON1.V32 = CFG_MCM1_PWMCON1_INIT;
	MCM1->PWMCON2.V32 = CFG_MCM1_PWMCON2_INIT;	
	
	MCM1->PWMP = CFG_MCM1_PWMP_INIT;
	MCM1->PWM0D = CFG_MCM1_PWM0D_INIT;
	MCM1->PWM1D = CFG_MCM1_PWM1D_INIT;
	MCM1->PWM2D = CFG_MCM1_PWM2D_INIT;
	MCM1->PWM01D = CFG_MCM1_PWM01D_INIT; 
	MCM1->PWM11D = CFG_MCM1_PWM11D_INIT; 
	MCM1->PWM21D = CFG_MCM1_PWM21D_INIT; 

	MCM1->PWMDT21	= CFG_MCM1_PWMDT21_INIT;
	MCM1->PWMDT20	= CFG_MCM1_PWMDT20_INIT;
	MCM1->PWMDT11	= CFG_MCM1_PWMDT11_INIT;
	MCM1->PWMDT10	= CFG_MCM1_PWMDT10_INIT;
	MCM1->PWMDT01	= CFG_MCM1_PWMDT01_INIT;
	MCM1->PWMDT00	= CFG_MCM1_PWMDT00_INIT;
	
	MCM1->PWMPSQ = CFG_MCM1_PWMPSQ_INIT;
	
	MCM1->PMANUALCON1.V32 = CFG_MCM1_PMANUALCON1_INIT;
	MCM1->PMANUALCON2.V32 = CFG_MCM1_PMANUALCON2_INIT;	
		
	MCM1->PWMCMP1 = CFG_MCM1_PWMCMP1_INIT;
	MCM1->PWMCMP2 = CFG_MCM1_PWMCMP2_INIT;
	MCM1->PWMCMP3 = CFG_MCM1_PWMCMP3_INIT;
	MCM1->PWMCMP4 = CFG_MCM1_PWMCMP4_INIT;
	
	MCM1->PWMINTEN.V32 = CFG_MCM1_PWMINTEN_INIT;
	MCM1->PSCON.V32 = CFG_MCM1_PSCON_INIT;	
	MCM1->PWMDMAX = CFG_MCM1_PWMDMAX_INIT;	
	MCM1->PWMDMIN = CFG_MCM1_PWMDMIN_INIT;	
	MCM1->PWMDCMP1 = CFG_MCM1_PWMDCMP1_INIT;
	MCM1->PWMDCMP2 = CFG_MCM1_PWMDCMP2_INIT;
	
	MCM1->SC0CON.V32 = CFG_MCM1_SC0CON_INIT;
	MCM1->SC1CON.V32 = CFG_MCM1_SC1CON_INIT;
	MCM1->SC2CON.V32 = CFG_MCM1_SC2CON_INIT;

	MCM1->FLTWEN = 0x33CC;
	MCM1->FLTCON.V32 = CFG_MCM1_FLTCON_INIT;
	MCM1->POSCR.V32 = CFG_MCM1_POSCR_INIT;	
	MCM1->POSTDCR.V32 = CFG_MCM1_POSTDCR_INIT;
	MCM1->FLTWEN = 0x0000;
	MCM1->PWMOE = CFG_MCM1_PWMOE;
	MCM1->PWMRLDEN1 = 0xAA;
}


void MACP_InitCfg(void)
{
	//Enable MACP Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.MACPEN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	MACP->CORDCSR0.V32 = CFG_MACP_CORDCSR0_INIT;	
	MACP->CORDCSR1.V32 = CFG_MACP_CORDCSR1_INIT;
	MACP->SVUALPHA = CFG_MACP_SVUALPHA_INIT;
	MACP->SVUBETA = CFG_MACP_SVUBETA_INIT;
	MACP->SVIQN.V32 = CFG_MACP_SVIQN_INIT;
}

void TIM_InitCfg(void)
{

#if CFG_TIM0_EN
	//Enable TIM0 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.TIM0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	TIM0->TCNT = CFG_TIM0_TCNT_INIT;
	TIM0->TPR  = CFG_TIM0_TPR_INIT;
	TIM0->PSQ  = CFG_TIM0_PSQ_INIT;	
	TIM0->CR.V32  = (CFG_TIM0_CR_START) |CFG_TIM0_CR_INIT;
#endif

#if CFG_TIM1_EN
	//Enable TIM1 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.TIM1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	TIM1->TCNT = CFG_TIM1_TCNT_INIT;
	TIM1->TPR = CFG_TIM1_TPR_INIT;
	TIM1->PSQ = CFG_TIM1_PSQ_INIT;	
	TIM1->CR.V32 = (CFG_TIM1_CR_START) |CFG_TIM1_CR_INIT;
#endif
}


void ADC_InitCfg(void)
{
#if CFG_ADC0_EN
  //Enable ADC0 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.ADC0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	ADC0->ADCON2.V32 = CFG_ADC0_ADCON2_INIT;
	ADC0->ADCMPCON.V32 = CFG_ADC0_ADCMPCON_INIT;
	ADC0->ADDLT = CFG_ADC0_ADDLT_INIT;
	ADC0->ADDGT = CFG_ADC0_ADDGT_INIT;	
	ADC0->SEQCHSEL0.V32 = CFG_ADC0_SEQCHSEL0_INIT;	
	ADC0->ADGAPON.V32 = CFG_ADC0_ADGAPON_INIT;
	ADC0->ADCON1.V32 = CFG_ADC0_ADCON1_INIT;
#endif

#if CFG_ADC1_EN
  //Enable ADC1 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.ADC1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	ADC1->ADCON2.V32 = CFG_ADC1_ADCON2_INIT;
	ADC1->ADCMPCON.V32 = CFG_ADC1_ADCMPCON_INIT;
	ADC1->ADDLT = CFG_ADC1_ADDLT_INIT;
	ADC1->ADDGT = CFG_ADC1_ADDGT_INIT;	
	ADC1->SEQCHSEL0.V32 = CFG_ADC1_SEQCHSEL0_INIT;	
	ADC1->ADGAPON.V32 = CFG_ADC1_ADGAPON_INIT;
	ADC1->ADCON1.V32 = CFG_ADC1_ADCON1_INIT;
#endif

#if CFG_ADC2_EN
  //Enable ADC2 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.ADC2EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	ADC2->ADCON2.V32 = CFG_ADC2_ADCON2_INIT;
	ADC2->ADCMPCON.V32 = CFG_ADC2_ADCMPCON_INIT;
	ADC2->ADDLT = CFG_ADC2_ADDLT_INIT;
	ADC2->ADDGT = CFG_ADC2_ADDGT_INIT;	
	ADC2->SEQCHSEL0.V32 = CFG_ADC2_SEQCHSEL0_INIT;
	ADC2->ADGAPON.V32 = CFG_ADC2_ADGAPON_INIT;
	ADC2->ADCON1.V32 = CFG_ADC2_ADCON1_INIT;
#endif
}

void QEI_InitCfg(void)
{
  //Enable QEI Clock
#if CFG_QEI0_EN	
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.QEI0EN = 1;
  RCC_WriteDisable();
	//Set the registers
	QEI0->QEICON.V32 = CFG_QEI0_QEICON_INIT;
	QEI0->QFLTCON.V32 = CFG_QEI0_QFLTCON_INIT;
	QEI0->QPOSMAX = CFG_QEI0_QPOSMAX_INIT;
	QEI0->QCNTMIN = CFG_QEI0_QCNTMIN_INIT;
	QEI0->QTPR = CFG_QEI0_QTPR_INIT;
	QEI0->QTPSQ = CFG_QEI0_QTPSQ_INIT;
	QEI0->QEIINT.V32 = (uint32_t)CFG_QEI0_QEIINT_INIT;
#endif	

#if CFG_QEI1_EN	
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.QEI1EN = 1;
  RCC_WriteDisable();
	//Set the registers
	QEI1->QEICON.V32 = CFG_QEI1_QEICON_INIT;
	QEI1->QFLTCON.V32 = CFG_QEI1_QFLTCON_INIT;
	QEI1->QPOSMAX = CFG_QEI1_QPOSMAX_INIT;
	QEI1->QCNTMIN = CFG_QEI1_QCNTMIN_INIT;
	QEI1->QTPR = CFG_QEI1_QTPR_INIT;
	QEI1->QTPSQ = CFG_QEI1_QTPSQ_INIT;
	QEI1->QEIINT.V32 = (uint32_t)CFG_QEI1_QEIINT_INIT;
#endif	
}

void AMOC_InitCfg(void)
{
  //Enable AMOC Clock
	RCC_WriteEnable();	
  RCC->APB0ENR.BIT.AMOCEN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	AMOC->CMP0CON.V32 = CFG_AMOC_CMP0CON_INIT;
	AMOC->CMP1CON.V32 = CFG_AMOC_CMP1CON_INIT;
	AMOC->CMP2CON.V32 = CFG_AMOC_CMP2CON_INIT;
	AMOC->OPCON.V32 = CFG_AMOC_OPCON_INIT;
}

void UART_InitCfg(void)
{
#if CFG_UART0_EN
  //Enable UART1 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.UART0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	UART0->ADDR.V32 = CFG_UART0_ADDR_INIT;
	UART0->BRT.V32 = CFG_UART0_BRT_INIT;
	UART0->CR.V32 = CFG_UART0_CR_INIT;
#endif
	
#if CFG_UART1_EN
  //Enable UART1 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.UART1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	UART1->ADDR.V32 = CFG_UART1_ADDR_INIT;
	UART1->BRT.V32 = CFG_UART1_BRT_INIT;
	UART1->CR.V32 = CFG_UART1_CR_INIT;	
#endif

#if CFG_UART2_EN
  //Enable UART2 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.UART2EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	UART2->ADDR.V32 = CFG_UART2_ADDR_INIT;
	UART2->BRT.V32 = CFG_UART2_BRT_INIT;
	UART2->CR.V32 = CFG_UART2_CR_INIT;
#endif	

#if CFG_UART3_EN
  //Enable UART3 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.UART3EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	UART3->ADDR.V32 = CFG_UART3_ADDR_INIT;
	UART3->BRT.V32 = CFG_UART3_BRT_INIT;
	UART3->CR.V32 = CFG_UART3_CR_INIT;
#endif	
}

void SPI_InitCfg(void)
{
#if CFG_SPI0_EN
  //Enable SPI0 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.SPI0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	SPI0->CR.V32 = CFG_SPI0_CR_INIT;	
#endif

#if CFG_SPI1_EN
  //Enable SPI1 Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.SPI1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	SPI1->CR.V32 = CFG_SPI1_CR_INIT;	
#endif
}

void TWI_InitCfg(void)
{
  //Enable TWI Clock
	RCC_WriteEnable();
  RCC->APB0ENR.BIT.TWI0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	TWI0->HOC = CFG_TWI_HOC_INIT;
	TWI0->BRT = CFG_TWI_BRT_INIT;
	TWI0->ADDR.V32 = CFG_TWI_ADDR_INIT;
	TWI0->CR.V32 = CFG_TWI_CR_INIT;
}
void SysTick_InitCfg(void)
{
	//set reload register 
	SysTick->LOAD  = CFG_SYSTICK_LOAD_INIT; 

	//Load the SysTick Counter Value 	
  SysTick->VAL   = CFG_SYSTICK_VAL_INIT;                          
  SysTick->CTRL  = (CFG_SYSTICK_CR_CLK << 2) |	
                   (CFG_SYSTICK_INT_EN << 1) |
                   CFG_SYSTICK_START_EN;    
}

void NVIC_InitCfg(void)
{
	//Setting Interrupt Set Enable Register 
	NVIC->ISER[0] = CFG_NVIC_ISER0_INIT;
	NVIC->ISER[1] = CFG_NVIC_ISER1_INIT;

#if CFG_PRI_EN
	//Setting PRI 
  SCB->AIRCR = (0x05FA << 16) | CFG_SCB_PRIGROUP_INIT;
	
	//Setting Exception Priority Register
	SCB->SHPR[0] = CFG_SCB_SHP4_INIT;
	SCB->SHPR[1] = CFG_SCB_SHP5_INIT;
	SCB->SHPR[2] = CFG_SCB_SHP6_INIT;	
	
	SCB->SHPR[7] = CFG_SCB_SHP11_INIT;
	SCB->SHPR[8] = CFG_SCB_SHP12_INIT;
	SCB->SHPR[10] = CFG_SCB_SHP14_INIT;
	SCB->SHPR[11] = CFG_SCB_SHP15_INIT;
	
	//Setting Interrupt Priority Register
	NVIC->IPR[0] = CFG_NVIC_IP0_INIT;
	NVIC->IPR[1] = CFG_NVIC_IP1_INIT;
	NVIC->IPR[2] = CFG_NVIC_IP2_INIT;
	NVIC->IPR[3] = CFG_NVIC_IP3_INIT;
	NVIC->IPR[4] = CFG_NVIC_IP4_INIT;
	NVIC->IPR[5] = CFG_NVIC_IP5_INIT;
	NVIC->IPR[6] = CFG_NVIC_IP6_INIT;
	NVIC->IPR[7] = CFG_NVIC_IP7_INIT;
	NVIC->IPR[8] = CFG_NVIC_IP8_INIT;
	NVIC->IPR[9] = CFG_NVIC_IP9_INIT;
	NVIC->IPR[10] = CFG_NVIC_IP10_INIT;
	NVIC->IPR[11] = CFG_NVIC_IP11_INIT;
	NVIC->IPR[12] = CFG_NVIC_IP12_INIT;
	NVIC->IPR[13] = CFG_NVIC_IP13_INIT;
	NVIC->IPR[14] = CFG_NVIC_IP14_INIT;
	NVIC->IPR[15] = CFG_NVIC_IP15_INIT;
	NVIC->IPR[16] = CFG_NVIC_IP16_INIT;
	NVIC->IPR[17] = CFG_NVIC_IP17_INIT;
	NVIC->IPR[18] = CFG_NVIC_IP18_INIT;
	NVIC->IPR[19] = CFG_NVIC_IP19_INIT;
	NVIC->IPR[20] = CFG_NVIC_IP20_INIT;
	NVIC->IPR[21] = CFG_NVIC_IP21_INIT;	
	NVIC->IPR[22] = CFG_NVIC_IP22_INIT;
	NVIC->IPR[23] = CFG_NVIC_IP23_INIT;
	NVIC->IPR[24] = CFG_NVIC_IP24_INIT;
	NVIC->IPR[25] = CFG_NVIC_IP25_INIT;
	NVIC->IPR[26] = CFG_NVIC_IP26_INIT;
	NVIC->IPR[27] = CFG_NVIC_IP27_INIT;
	NVIC->IPR[28] = CFG_NVIC_IP28_INIT;	
	NVIC->IPR[29] = CFG_NVIC_IP29_INIT;	
	NVIC->IPR[30] = CFG_NVIC_IP30_INIT;
	NVIC->IPR[31] = CFG_NVIC_IP31_INIT;
	NVIC->IPR[32] = CFG_NVIC_IP32_INIT;
	NVIC->IPR[33] = CFG_NVIC_IP33_INIT;
	NVIC->IPR[34] = CFG_NVIC_IP34_INIT;
	NVIC->IPR[35] = CFG_NVIC_IP35_INIT;
	NVIC->IPR[36] = CFG_NVIC_IP36_INIT;
	NVIC->IPR[37] = CFG_NVIC_IP37_INIT;
	NVIC->IPR[38] = CFG_NVIC_IP38_INIT;
	NVIC->IPR[39] = CFG_NVIC_IP39_INIT;
	NVIC->IPR[40] = CFG_NVIC_IP40_INIT;
	NVIC->IPR[41] = CFG_NVIC_IP41_INIT;
	NVIC->IPR[42] = CFG_NVIC_IP42_INIT;
	NVIC->IPR[43] = CFG_NVIC_IP43_INIT;
	NVIC->IPR[44] = CFG_NVIC_IP44_INIT;
	NVIC->IPR[45] = CFG_NVIC_IP45_INIT;
	NVIC->IPR[46] = CFG_NVIC_IP46_INIT;
	NVIC->IPR[47] = CFG_NVIC_IP47_INIT;	
	NVIC->IPR[48] = CFG_NVIC_IP48_INIT;
	NVIC->IPR[49] = CFG_NVIC_IP49_INIT;
#endif
}

void PCA_InitCfg(void)
{
#if CFG_PCA0_EN
  //Enable PCA0 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.PCA0EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	PCA0->LCKR			= 0x33CC; 
	PCA0->CFGR.V32 	= CFG_PCA0_CFGR_INIT; 
	PCA0->CNT.V32  			= CFG_PCA0_CNT_INIT;  
	PCA0->PSC  			= CFG_PCA0_PSC_INIT;  
	PCA0->PR.V32   	= CFG_PCA0_PR_INIT;   
	PCA0->ADTR 			= CFG_PCA0_ADTR_INIT; 
	PCA0->CCR0.V32 			= CFG_PCA0_CCR0_INIT; 
	PCA0->CCR1.V32 			= CFG_PCA0_CCR1_INIT; 
	PCA0->CCR2.V32 			= CFG_PCA0_CCR2_INIT; 
	PCA0->CCMR0.V32 = CFG_PCA0_CCMR0_INIT;
	PCA0->CCMR1.V32 = CFG_PCA0_CCMR1_INIT;
	PCA0->CCMR2.V32 = CFG_PCA0_CCMR2_INIT;
	PCA0->CR = CFG_PCA0_CR_INIT;
	PCA0->LCKR			= 0x0; 	
	
	PCA0->FLTWPR 		= 0x33CC;
	PCA0->FLTCR.V32 = CFG_PCA0_FLTCR_INIT;
	PCA0->FLTWPR 		= 0;	
#endif

#if CFG_PCA1_EN
  //Enable PCA1 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.PCA1EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	PCA1->LCKR			= 0x33CC;             
	PCA1->CFGR.V32 	= CFG_PCA1_CFGR_INIT; 
	PCA1->CNT.V32  			= CFG_PCA1_CNT_INIT;  
	PCA1->PSC  			= CFG_PCA1_PSC_INIT;  
	PCA1->PR.V32   			= CFG_PCA1_PR_INIT;   
	PCA1->CCMR0.V32 = CFG_PCA1_CCMR0_INIT;
	PCA1->CCMR1.V32 = CFG_PCA1_CCMR1_INIT;
	PCA1->CCMR2.V32 = CFG_PCA1_CCMR2_INIT;
	PCA1->CCR0.V32 			= CFG_PCA1_CCR0_INIT; 
	PCA1->CCR1.V32 			= CFG_PCA1_CCR1_INIT; 
	PCA1->CCR2.V32 			= CFG_PCA1_CCR2_INIT; 
	PCA1->ADTR 			= CFG_PCA1_ADTR_INIT; 
	PCA1->CR = CFG_PCA1_CR_INIT;
	PCA1->LCKR			= 0x0;
	                                      
	PCA1->FLTWPR 		= 0x33CC;             
	PCA1->FLTCR.V32 = CFG_PCA1_FLTCR_INIT;
	PCA1->FLTWPR 		= 0;	    
#endif
	
#if CFG_PCA2_EN
  //Enable PCA2 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.PCA2EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	PCA2->LCKR			= 0x33CC; 
	PCA2->CFGR.V32 	= CFG_PCA2_CFGR_INIT; 
	PCA2->CNT.V32  			= CFG_PCA2_CNT_INIT;  
	PCA2->PSC  			= CFG_PCA2_PSC_INIT;  
	PCA2->PR.V32   	= CFG_PCA2_PR_INIT;   
	PCA2->ADTR 			= CFG_PCA2_ADTR_INIT; 
	PCA2->CCR0.V32 			= CFG_PCA2_CCR0_INIT; 
	PCA2->CCR1.V32 			= CFG_PCA2_CCR1_INIT; 
	PCA2->CCR2.V32 			= CFG_PCA2_CCR2_INIT; 
	PCA2->CCMR0.V32 = CFG_PCA2_CCMR0_INIT;
	PCA2->CCMR1.V32 = CFG_PCA2_CCMR1_INIT;
	PCA2->CCMR2.V32 = CFG_PCA2_CCMR2_INIT;
	PCA2->CR = CFG_PCA2_CR_INIT;
	PCA2->LCKR			= 0x0; 	
	
	PCA2->FLTWPR 		= 0x33CC;
	PCA2->FLTCR.V32 = CFG_PCA2_FLTCR_INIT;
	PCA2->FLTWPR 		= 0;	
#endif

#if CFG_PCA3_EN
  //Enable PCA3 Clock
	RCC_WriteEnable();
  RCC->AHBENR.BIT.PCA3EN = 1;
  RCC_WriteDisable();
	
	//Set the registers
	PCA3->LCKR			= 0x33CC;             
	PCA3->CFGR.V32 	= CFG_PCA3_CFGR_INIT; 
	PCA3->CNT.V32  			= CFG_PCA3_CNT_INIT;  
	PCA3->PSC  			= CFG_PCA3_PSC_INIT;  
	PCA3->PR.V32   			= CFG_PCA3_PR_INIT;   
	PCA3->CCMR0.V32 = CFG_PCA3_CCMR0_INIT;
	PCA3->CCMR1.V32 = CFG_PCA3_CCMR1_INIT;
	PCA3->CCMR2.V32 = CFG_PCA3_CCMR2_INIT;
	PCA3->CCR0.V32 			= CFG_PCA3_CCR0_INIT; 
	PCA3->CCR1.V32 			= CFG_PCA3_CCR1_INIT; 
	PCA3->CCR2.V32 			= CFG_PCA3_CCR2_INIT; 
	PCA3->ADTR 			= CFG_PCA3_ADTR_INIT; 
	PCA3->CR = CFG_PCA3_CR_INIT;
	PCA3->LCKR			= 0x0;
	                                      
	PCA3->FLTWPR 		= 0x33CC;             
	PCA3->FLTCR.V32 = CFG_PCA3_FLTCR_INIT;
	PCA3->FLTWPR 		= 0;	    
#endif
}

