/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_rcc.h
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file provides reset and clock module's APIs
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * COPYRIGHT 2022 Sinowealth
  *
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __SH3xF2xx_SC0_RCC_H
#define __SH3xF2xx_SC0_RCC_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/  
#include "sh3xf2xx_sc0_lib.h"
/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @addtogroup RCC_MODULE RCC
  * @{
  */ 

/** @defgroup RCC_Group_Constant  Public Constants
  * @{
  */ 
/*!< CLOCK SOURCE CONSTANT */
typedef enum {
    RCC_SYS_SRC_HSI = 0,
    RCC_SYS_SRC_HSE = 1,
    RCC_SYS_SRC_LSI = 2,
    RCC_SYS_SRC_PLL = 3,
}RCC_SysSource_Type;

/*!  check clock source */
#define IS_SYS_SOURCE(src) ((((src) == RCC_SYS_SRC_HSI) || ((src) == RCC_SYS_SRC_HSE) || ((src) == RCC_SYS_SRC_PLL))|| ((src) == RCC_SYS_SRC_LSI))

/*!  PLL XTPRE CLOCK CONSTANT */
typedef enum {
    PLL_XTPRE_DIV1  = 1,
    PLL_XTPRE_DIV2  = 2,
    PLL_XTPRE_DIV3  = 3,
    PLL_XTPRE_DIV4  = 4,
    PLL_XTPRE_DIV5  = 5,
    PLL_XTPRE_DIV6  = 6,
    PLL_XTPRE_DIV7  = 7,
}RCC_PLL_XTPRE_Type;
/*!  check pll 's pre-divider */
#define IS_PLL_XTPRE(div) (((div) >= PLL_XTPRE_DIV1) && ((div) <= PLL_XTPRE_DIV7))

/*!  PLL SOURCE CONSTANT */
typedef enum {
   PLL_SRC_HSI  = 0,
   PLL_SRC_HSE  = 1,
}RCC_PLL_SRC_Type;
/*!  check pll's input source */
#define IS_PLL_SRC(src) (((src) == PLL_SRC_HSI) || ((src) == PLL_SRC_HSE))


/*! RCC AHB CLOCK CONSTANT */
typedef enum {
    RCC_HCLK_DIV1  =  0,
    RCC_HCLK_DIV2  =  1,
    RCC_HCLK_DIV4  =  2,
    RCC_HCLK_DIV8  =  3,
    RCC_HCLK_DIV16 =  4,
    RCC_HCLK_DIV32 =  5,
}RCC_HCLK_DIV_Type;
/*!  check AHB frequence's divider value */
#define IS_HCLK_DIV(div)   (((div) == RCC_HCLK_DIV1)  \
                                      || ((div) == RCC_HCLK_DIV2)  \
                                      || ((div) == RCC_HCLK_DIV4)  \
                                      || ((div) == RCC_HCLK_DIV8)  \
                                      || ((div) == RCC_HCLK_DIV16)  \
                                      || ((div) == RCC_HCLK_DIV32))
 

/*! RCC APB0 CLOCK CONSTANT */
typedef enum {
    RCC_PCLK_DIV1   = 0,
    RCC_PCLK_DIV2   = 1,
    RCC_PCLK_DIV4   = 2,
    RCC_PCLK_DIV8   = 3,
    RCC_PCLK_DIV16  = 4,
}RCC_PCLK_DIV_Type;
/*!  check APB1 frequence's divider value */
#define IS_PCLK_DIV(div)   (((div) == RCC_PCLK_DIV1)  \
                                       || ((div) == RCC_PCLK_DIV2)  \
                                       || ((div) == RCC_PCLK_DIV4)  \
                                       || ((div) == RCC_PCLK_DIV8)  \
                                       || ((div) == RCC_PCLK_DIV16))


/*!  RCC HSI TRIM CONSTANT onestep about 0.1% */
typedef enum {
    RCC_HSITRIM_INC0  =   0, /*!< + 0 step */
    RCC_HSITRIM_INC1  =   1, /*!< + 1 step */
    RCC_HSITRIM_INC2  =   2, /*!< + 2 step */
    RCC_HSITRIM_INC3  =   3, /*!< + 3 step */
    RCC_HSITRIM_DEC4  =   4, /*!< - 4 step */
    RCC_HSITRIM_DEC3  =   5, /*!< - 3 step */
    RCC_HSITRIM_DEC2  =   6, /*!< - 2 step */
    RCC_HSITRIM_DEC1  =   7, /*!< - 1 step */
}RCC_HSITRIM_Type;
/*!  check HSITRIM's value */
#define IS_HSITRIM(trim)    (((trim) == RCC_HSITRIM_INC0)   \
                                       || ((trim) == RCC_HSITRIM_INC1)   \
                                       || ((trim) == RCC_HSITRIM_INC2)   \
                                       || ((trim) == RCC_HSITRIM_INC3)   \
                                       || ((trim) == RCC_HSITRIM_DEC4)   \
                                       || ((trim) == RCC_HSITRIM_DEC3)   \
                                       || ((trim) == RCC_HSITRIM_DEC2)   \
                                       || ((trim) == RCC_HSITRIM_DEC1))

/*!  RCC Interrupt constant */
typedef enum
{
    RCC_INT_HSERDY = RCC_CISTR_HSERDYIF_Msk, /*!< Interrupt for HSE ready */
    RCC_INT_PLLRDY = RCC_CISTR_PLLRDYIF_Msk, /*!< Interrupt for PLL ready */
    RCC_INT_CSMHSE = RCC_CISTR_HSECSMF_Msk,  /*!< Interrupt for CMS HSE error */
    RCC_INT_CSMPLL = RCC_CISTR_PLLCSMF_Msk,  /*!< Interrupt for CMS PLL error */ 
}RCC_INT_Type;
/*! check interrupt enable items */
#define IS_RCC_INTSRC(src) (((src)==RCC_INT_HSERDY) || ((src)==RCC_INT_PLLRDY))

/*! check interrupt enable items */
#define IS_RCC_INTSRCS(srcs) (((srcs)==RCC_INT_HSERDY)  \
                             || ((srcs)==RCC_INT_PLLRDY) \
                             || ((srcs) == (RCC_INT_HSERDY|RCC_INT_PLLRDY)))

/*! check interrupt flag items */
#define IS_RCC_INTFLAG(flag) (((flag)==RCC_INT_HSERDY) \
                             || ((flag)==RCC_INT_PLLRDY) \
                             || ((flag)==RCC_INT_CSMHSE) \
                             || ((flag)==RCC_INT_CSMPLL))
/*! All INT Flags */
#define RCC_INT_ALL    (RCC_INT_HSERDY|RCC_INT_PLLRDY|RCC_INT_CSMHSE|RCC_INT_CSMPLL)
/*! check interrupt flag items */
#define IS_RCC_INTFLAGS(flags)  ((((flags ) & (uint32_t)RCC_INT_ALL) != 0) && (((uint32_t)(flags) & (~(uint32_t)RCC_INT_ALL)) == 0))


/*!  RCC AHB Modules */
typedef enum{
    RCC_AHB_GPIO   = 1,
    RCC_AHB_MCM0   = 2,
    RCC_AHB_MCM1   = 3,
    RCC_AHB_MACP   = 4,
    RCC_AHB_CRC    = 5,
    RCC_AHB_PCA0   = 6,
    RCC_AHB_PCA1   = 7,
    RCC_AHB_PCA2   = 8,
    RCC_AHB_PCA3   = 9,
    RCC_AHB_DMA    = 10,
    RCC_AHB_SYSCFG = 11,
    RCC_AHB_SDFM0  = 12,
    RCC_AHB_ADC0   = 13,
    RCC_AHB_ADC1   = 14,
    RCC_AHB_ADC2   = 15,
}RCC_AHB_Type;


/*! All AHB Modules */
#define RCC_AHB_ALL    (0xFFFFU)

/*! check AHB Module source */
#define IS_AHB_MODULES(m)   ((((m ) & RCC_AHB_ALL) != 0) && (((m) & (~RCC_AHB_ALL)) == 0))

/*!  RCC APB0 Modules */
typedef enum{
    RCC_APB0_TIM0   = 1,
    RCC_APB0_TIM1   = 2,
    RCC_APB0_QEI0   = 3,
    RCC_APB0_QEI1   = 4,
    RCC_APB0_UART0  = 5,
    RCC_APB0_UART1  = 6,
    RCC_APB0_UART2  = 7,
    RCC_APB0_UART3  = 8,
    RCC_APB0_SPI0   = 9,
    RCC_APB0_SPI1   = 10,
    RCC_APB0_TWI0   = 11,
    RCC_APB0_WWDT   = 12,
    RCC_APB0_EXTI   = 13,
    RCC_APB0_AMOC   = 14,
    RCC_APB0_CAN0   = 15,
}RCC_APB0_Type;

/*! All APB0  Modules */
#define RCC_APB0_ALL    (0x7FFFFU)

/*! check APB0 clock source */
#define IS_APB0_MODULES(m) ((((m ) & RCC_APB0_ALL) != 0) && (((m) & (~RCC_APB0_ALL)) == 0))


/*!  RCC Reset Flags */
typedef enum{
    RCC_RST_PIN      =  RCC_RSTSTR_PINRSTF_Msk,
    RCC_RST_LVR      =  RCC_RSTSTR_LVRSTF_Msk,
    RCC_RST_POWERON  =  RCC_RSTSTR_PORSTF_Msk,
    RCC_RST_SOFTWARE =  RCC_RSTSTR_SWRSTF_Msk,
    RCC_RST_IWDT     =  RCC_RSTSTR_IWDTRSTF_Msk,
    RCC_RST_WWDT     =  RCC_RSTSTR_WWDTRSTF_Msk,
}RCC_RESET_Type;

/*! Check RESET Type */
#define IS_RCC_RESET_Type(rst)    (  ((rst ) == RCC_RST_PIN ) \
                                  || ((rst ) == RCC_RST_LVR ) \
                                  || ((rst ) == RCC_RST_POWERON ) \
                                  || ((rst ) == RCC_RST_SOFTWARE ) \
                                  || ((rst ) == RCC_RST_IWDT ) \
                                  || ((rst ) == RCC_RST_WWDT ) \
                                  )
/*! All Reset Sources */                                  
#define RCC_RESET_ALL     (RCC_RST_PIN | RCC_RST_LVR | RCC_RST_POWERON | RCC_RST_SOFTWARE | RCC_RST_IWDT | RCC_RST_WWDT)
/*! Check RESET Types */                                  
#define IS_RCC_RESET_Types(rst)  ((((rst ) & RCC_RESET_ALL) != 0) && (((rst) & (~RCC_RESET_ALL)) == 0))


/*!< CLOCK SMOOTH SWITCH DELAY*/
typedef enum {
    RCC_CLKSS_DELAY_3  = 0,
    RCC_CLKSS_DELAY_2  = 1,
    RCC_CLKSS_DELAY_1  = 2,
    RCC_CLKSS_DELAY_0  = 3,
}RCC_CLKSS_Type;


/*!< CLOCK GATE ON DELAY */
typedef enum {
    RCC_CLKGON_DELAY_3 = 0,
    RCC_CLKGON_DELAY_2 = 1,
    RCC_CLKGON_DELAY_1 = 2,
    RCC_CLKGON_DELAY_0 = 3,
}RCC_CLKGON_Type;


/**
  * @}
  */ 

/** @defgroup RCC_Group_Types  Public Types
  * @{
  */     

/* Exported types ------------------------------------------------------------*/
    
/*! @struct  RCC_Clocks_TypeDef
  *  RCC structure of clock frequency datas, used to get current system's clock frequency
  */    
typedef struct
{
  uint32_t sysFreq;     /*!<  System clock frequency, unit is Hz */
  uint32_t hclkFreq;    /*!< AHB bus clock frequency, unit is Hz */
  uint32_t pclkFreq;   /*!< APB0 bus clock frequency, unit is Hz */
}RCC_Clocks_TypeDef;
    

/*! @struct  RCC_PLL_InitTypeDef
  *   PLL Initial structure
  */    
typedef struct
{
  uint8_t   xpreDiv;   /*!<  Pre-divider for input clock */
  uint8_t   pllSRC;    /*!< PLL clock source */
  uint8_t   pllMul;    /*!< PLL's F Value: 15~78*/
  uint8_t   pllDiv;    /*!< PLL's K Value: 1~16*/
}RCC_PLL_InitTypeDef;

/**
  * @}
  */ 

/* Exported macro ------------------------------------------------------------*/
/** @defgroup RCC_Group_Macro  Public Macros
  * @{
  */ 

/** 
  *@brief  Unlock the RCC module's regsiter. RCC registers can be modified.
  */
#define RCC_REGS_UNLOCK() RCC->RCCLOCK=0x33CC

/** 
  *@brief  Lock the RCC module's regsiter. RCC registers cannot be modified.
  */
#define RCC_REGS_LOCK()   RCC->RCCLOCK=0


/** 
  *@brief Calculate PLL's F value. Input value is 15~78.
  */
#define PLL_PLLF(f)      (f)

      
/**
  * @}
  */ 

/* Exported functions --------------------------------------------------------*/ 
/** @addtogroup RCC_Group_Pub_Funcs Public Functions
  * @{
  */     

/* get current clock requency*/
void RCC_GetClocksFreq(RCC_Clocks_TypeDef* Clocks);

/*  restore default clocks*/
void RCC_Reset(void);

/*  configure HSE*/
void RCC_HSEOnOff(SH_CmdState OnOffState);

/*  wati HSE ready */
SH_ErrorStatus RCC_WaitForHSEReady(uint32_t TimeOut);

/*  adjust HSI frequency */
uint32_t RCC_AdjustHSITrimValue(RCC_HSITRIM_Type HSITrimValue);
/* Fill the RCC_PLL_InitStruct member with its default value */
void RCC_PLLStructInit(RCC_PLL_InitTypeDef* PLLInit);
/* configure PLL's parameters */
void RCC_PLLConfig(RCC_PLL_InitTypeDef* PLLInit);

/* enable or disable PLL */
void RCC_PLLOnOff(SH_CmdState OnOffState);

/* Wait PLL ready*/
SH_ErrorStatus RCC_WaitForPLLReady(uint32_t TimeOut);

/* configure system's input clock source*/
void RCC_SysClkConfig(RCC_SysSource_Type SysClkSrc);

/* get current system clock's source */
RCC_SysSource_Type RCC_GetSysClkSource(void);

/*  configure HCLK*/
void RCC_HCLKConfig(RCC_HCLK_DIV_Type HCLK_DIV);

/* configure PCLK1 */
void RCC_PCLKConfig(RCC_PCLK_DIV_Type PCLK_DIV);


/* configure  RCC interrupt */ 
void RCC_INTConfig(uint32_t RCC_IT, SH_FunctionalState NewState);

/* get RCC interrupt flag */
SH_FlagStatus  RCC_GetINTStatus(RCC_INT_Type INTFlag);

/* clear RCC interrupt flag */
void  RCC_ClearINTStatus(uint32_t INTCFlag);


/* configure AHB modules' clock gate */ 
void  RCC_AHBPeriphClockOnOff(uint32_t AHBModules,SH_CmdState OnOffState);

/* configure APB1 modules' clock gate */ 
void  RCC_APB0PeriphClockOnOff(uint32_t APB0Modules,SH_CmdState OnOffState);


/* reset AHB modules */ 
void RCC_AHBPeriphReset(uint32_t AHBModules);

/* reset APB1 modules*/ 
void RCC_APB0PeriphReset(uint32_t APB0Modules);


/*  get RCC reset flags */
SH_FlagStatus RCC_GetResetFlag(RCC_RESET_Type ResetFlags);

/*  clear RCC reset flags */
void RCC_ClearResetFlag(uint8_t ResetFlags);


/* CSM enable or disable*/
void RCC_CSMOnOff(SH_CmdState OnOffState);

/* CSM enable or disable*/
SH_FlagStatus RCC_CSMGetFlagStatus(uint32_t CSM_Flag);

/* CSM clear flag*/
void RCC_CSMClearFlag(void);

/* Clock smooth switch*/
void RCC_ClockSmoothSwitch(RCC_CLKSS_Type dlymode);

/* Clock Gate On Delay*/
void RCC_ClockOnDelay(RCC_CLKGON_Type dlymode);

/* HSE tune on for different crystal type*/
SH_Bool RCC_HSE_TuneOn(void);


/**
  * @}
  */ 


#ifdef __cplusplus
}
#endif

#endif /*__SH3xF2xx_SC0_RCC_H */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
