/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_sdfm.h
  * @author  sinowealth
  * @version V1.0.0
  * @date    01-March-2024 
  * @brief   This file contains all the functions prototypes for the qei firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * COPYRIGHT 2024 Sinowealth
  *
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __SH3xF2xx_SC0_SDFM_H
#define __SH3xF2xx_SC0_SDFM_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_lib.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @addtogroup SDFM_MODULE SDFM
  * @{
  */ 

/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/

/** @defgroup SDFM_Exported_Constants Public Constants
  * @{
  */ 
typedef struct
{
    union {
        __IO  uint32_t  V32;                                /*!< 0040H */
        struct {
            __IO  uint32_t  MOD       : 3;  /*!< [b2~b0]*/
            __IO  uint32_t  SDCLKSEL  : 1;  /*!< [b3~b3]*/
            __IO  uint32_t  SDCLKSYNC : 1;  /*!< [b4~b4]*/
            __IO  uint32_t  rev0      : 1;  /*!< [b5~b5]*/
            __IO  uint32_t  SDDATASYNC: 1;  /*!< [b6~b6]*/
            __IO  uint32_t  rev1      :25;  /*!< [b31~b7]*/
        }BIT;
    }CTLPARM;                               /*!< 0040H */
    union {
        __IO  uint32_t  V32;                                /*!< 0044H */
        struct {
            __IO  uint32_t  DOSR      : 8;  /*!< [b7~b0]*/
            __IO  uint32_t  FEN       : 1;  /*!< [b8~b8]*/
            __IO  uint32_t  AE        : 1;  /*!< [b9~b9]*/
            __IO  uint32_t  SST       : 2;  /*!< [b11~b10]*/
            __IO  uint32_t  SDSYNCEN  : 1;  /*!< [b12~b12]*/
            __IO  uint32_t  rev0      :19;  /*!< [b31~b13]*/
        }BIT;
    }DFPARM;                               /*!< 0044H */
    union {
        __IO  uint32_t  V32;                                /*!< 0048H */
        struct {
            __IO  uint32_t  rev0      :10;  /*!< [b9~b0]*/
            __IO  uint32_t  DR        : 1;  /*!< [b10~b10]*/
            __IO  uint32_t  SH        : 5;  /*!< [b15~b11]*/
            __IO  uint32_t  rev1      :16;  /*!< [b31~b16]*/
        }BIT;
    }DPARM;                               /*!< 0048H */
    __IO uint32_t  CMPH1;                               /*!< 004CH */
    __IO uint32_t  CMPL1;                               /*!< 0050H */
    union {
        __IO  uint32_t  V32;                                /*!< 0054H */
        struct {
            __IO  uint32_t  COSR      : 5;  /*!< [b4~b0]*/
            __IO  uint32_t  CEVT1EN   : 1;  /*!< [b5~b5]*/
            __IO  uint32_t  CEVT2EN   : 1;  /*!< [b6~b6]*/
            __IO  uint32_t  CS1CS0    : 2;  /*!< [b8~b7]*/
            __IO  uint32_t  MFIE      : 1;  /*!< [b9~b9]*/
            __IO  uint32_t  HZEN      : 1;  /*!< [b10~b10]*/
            __IO  uint32_t  CEVT1SEL  : 2;  /*!< [b12~b11]*/
            __IO  uint32_t  CEN       : 1;  /*!< [b13~b13]*/
            __IO  uint32_t  CEVT2SEL  : 2;  /*!< [b15~b14]*/
            __IO  uint32_t  rev0      :16;  /*!< [b31~b16]*/
        }BIT;
    }CPARM;                               /*!< 0054H */
    union {
        __IO  uint32_t  V32;                                /*!< 0058H */
        struct {
    __IO uint32_t  DATA16:16;                               /*!< 0058H */
    __IO uint32_t  DATA32HI:16;                             /*!< 005AH */
        }BIT;
    }RDATA;                              /*!< 0058H */
    union {
        __IO  uint32_t  V32;                                /*!< 005CH */
        struct {
    __IO uint32_t  DATA16:16;                               /*!< 005CH */
    __IO uint32_t  DATA32HI:16;                             /*!< 005EH */
        }BIT;
    }DATFIFO;                              /*!< 005CH */
    __IO uint32_t  CDATA;                               /*!< 0060H */
    __IO uint32_t  CMPH2;                               /*!< 0064H */
    __IO uint32_t  CMPHZ;                               /*!< 0068H */
    union {
        __IO  uint32_t  V32;                                /*!< 006CH */
        struct {
            __IO  uint32_t  SDFFIL    : 5;  /*!< [b4~b0]*/
            __IO  uint32_t  rev0      : 1;  /*!< [b5~b5]*/
            __IO  uint32_t  SDFFST    : 5;  /*!< [b10~b6]*/
            __IO  uint32_t  rev1      : 1;  /*!< [b11~b11]*/
            __IO  uint32_t  FFIEN     : 1;  /*!< [b12~b12]*/
            __IO  uint32_t  FFEN      : 1;  /*!< [b13~b13]*/
            __IO  uint32_t  DRINTSEL  : 1;  /*!< [b14~b14]*/
            __IO  uint32_t  OVFIEN    : 1;  /*!< [b15~b15]*/
            __IO  uint32_t  rev2      :16;  /*!< [b31~b16]*/
        }BIT;
    }FIFOCTL;                               /*!< 006CH */
    union {
        __IO  uint32_t  V32;                                /*!< 0070H */
        struct {
            __IO  uint32_t  SYNCSEL   : 6;  /*!< [b5~b0]*/
            __IO  uint32_t  WTSYNCEN  : 1;  /*!< [b6~b6]*/
            __IO  uint32_t  WTSYNFLG  : 1;  /*!< [b7~b7]*/
            __IO  uint32_t  WTSYNCLR  : 1;  /*!< [b8~b8]*/
            __IO  uint32_t  FFSYNCCLREN: 1;  /*!< [b9~b9]*/
            __IO  uint32_t  WTSCLREN  : 1;  /*!< [b10~b10]*/
            __IO  uint32_t  rev0      :21;  /*!< [b31~b11]*/
        }BIT;
    }SYNC;                               /*!< 0070H */
    __IO uint32_t  CMPL2;                               /*!< 0074H */
    union {
        __IO  uint32_t  V32;                                /*!< 0078H */
        struct {
            __IO  uint32_t  rev0      : 2;  /*!< [b1~b0]*/
            __IO  uint32_t  CEVT1DFS  : 2;  /*!< [b3~b2]*/
            __IO  uint32_t  rev1      : 6;  /*!< [b9~b4]*/
            __IO  uint32_t  CEVT2DFS  : 2;  /*!< [b11~b10]*/
            __IO  uint32_t  rev2      :20;  /*!< [b31~b12]*/
        }BIT;
    }COMPCTL;                               /*!< 0078H */
    union {
        __IO  uint32_t  V32;                                /*!< 007CH */
        struct {
            __IO  uint32_t  rev0      : 4;  /*!< [b3~b0]*/
            __IO  uint32_t  SAMPWIN   : 5;  /*!< [b8~b4]*/
            __IO  uint32_t  THRESH    : 5;  /*!< [b13~b9]*/
            __IO  uint32_t  rev1      : 1;  /*!< [b14~b14]*/
            __IO  uint32_t  FILINIT   : 1;  /*!< [b15~b15]*/
            __IO  uint32_t  rev2      :16;  /*!< [b31~b16]*/
        }BIT;
    }COMPEVT2CTL;                               /*!< 007CH */
    __IO uint32_t  COMPEVT2CLKCTL;                      /*!< 0080H */
    union {
        __IO  uint32_t  V32;                                /*!< 0084H */
        struct {
            __IO  uint32_t  rev0      : 4;  /*!< [b3~b0]*/
            __IO  uint32_t  SAMPWIN   : 5;  /*!< [b8~b4]*/
            __IO  uint32_t  THRESH    : 5;  /*!< [b13~b9]*/
            __IO  uint32_t  rev1      : 1;  /*!< [b14~b14]*/
            __IO  uint32_t  FILINIT   : 1;  /*!< [b15~b15]*/
            __IO  uint32_t  rev2      :16;  /*!< [b31~b16]*/
        }BIT;
    }COMPEVT1CTL;                               /*!< 0084H */
    __IO uint32_t  COMPEVT1CLKCTL;                      /*!< 0088H */
    union {
        __IO  uint32_t  V32;                                /*!< 008CH */
        struct {
            __IO  uint32_t  LCKCOMPCTL: 1;  /*!< [b0~b0]*/
            __IO  uint32_t  rev0      : 2;  /*!< [b2~b1]*/
            __IO  uint32_t  LCKCOMPEVT: 1;  /*!< [b3~b3]*/
            __IO  uint32_t  rev1      :28;  /*!< [b31~b4]*/
        }BIT;
    }COMPLOCK;                               /*!< 008CH */
                              /*!< 017CH */
}SDFM_FILTER_TypeDef;


/*! structure SDFM_InitTypeDef definition */
typedef struct
{
    uint16_t CFltHL1;             /*!< Compare filter high limtitation Value 1*/
    uint16_t CFltLL1;             /*!< Compare filter low limtitation Value 1*/
    
    uint16_t CFltHL2;             /*!< Compare filter high limtitation Value 2*/
    uint16_t CFltLL2;             /*!< Compare filter low limtitation Value 2*/
    
    uint16_t CFltHZ;              /*!< Compare filter zero Value 2*/
    uint16_t CEvt1SmpClk;         /*!< CEVT1 sampling clock, n+1 module clocks*/
    
    uint16_t CEvt2SmpClk;         /*!< CEVT2 sampling clock, n+1 module clocks*/    
    uint8_t DataSyncEn;           /*!< Enable or disable data signal sync.*/
    uint8_t ClkSyncEn;            /*!< Enable or disable clock signal sync*/
    
    uint8_t ClkSel;               /*!< Select filter clock source, 0 for own clock pin, 1 for channel1 clock pin*/
    uint8_t DemodulMode;          /*!< Specifie demodulation mode */
    uint8_t SyncEn;               /*!< Enable or disable Filter Sync feature */
    uint8_t DataFltMethod;        /*!< Specifie data filter's algorithm method */
    
    uint8_t DataRdyFlagEn;        /*!< Enable or disable data ready flag*/
    uint8_t DataFltEn;            /*!< Enable or disable data filter */
    uint8_t DataOSR;              /*!< Data OSR select 0xFF is 256 */
    uint8_t DataShiftBits;        /*!< Data shift bits*/
    
    uint8_t DataFmt;              /*!< Data format, 0 for 16bits and 1 for 32 bits*/                      
    uint8_t CEvt1Sel;             /*!< Compare event2 select*/                     
    uint8_t CEvt2Sel;             /*!< Compare event1 select*/                     
    uint8_t CmpFltEn;             /*!< Compare filter select*/
    
    uint8_t CmpHZEn;              /*!< Compare filter zero compare enable or disable*/
    uint8_t DemodulErrEn;         /*!< Demodulate error enable or disable*/
    uint8_t CmpFltMethod;         /*!< Specifie compare filter's algorithm method*/
    uint8_t CEvt1En;              /*!< Enable or disable compare event1*/
    
    uint8_t CEvt2En;              /*!< Enable or disable compare event2*/
    uint8_t CmpOSR;               /*!< Compare OSR select 0x1F is 32 */    
    uint8_t FifoItOvfEn;          /*!< FIFO overflow interrupt enable or disable */
    uint8_t DRdyItSel;            /*!< Select Data ready interrupt source select */
    
    uint8_t FifoEn;               /*!< Data FIFO enable or disable */
    uint8_t FifoItEn;             /*!< Data FIFO interrupt enable or disable */    
    uint8_t FifoDepth;            /*!< Depth of data FIFO buffer */        
    uint8_t SyncFlagAutoClrEn;    /*!< Enable or disable auto clear sync flag by fifo interrupt */
    
    uint8_t FifoAutoClrEn;        /*!< Enable or disable auto clear fifo by sync event */    
    uint8_t Data2FifoMode;        /*!< Select data write to fifo method*/
    uint8_t SyncSrcSel;           /*!< Select sync source */      
    uint8_t CEvt1OutSrcSel;       /*!< Select CEVT1 signal output source, through digital sampling or direct */  
    
    uint8_t CEvt2OutSrcSel;       /*!< Select CEVT2 signal output source, through digital sampling or direct */   
    uint8_t CEvt1SmpWnd;          /*!< Select CEVT1 sampling window */   
    uint8_t CEvt1Thresh;          /*!< Select CEVT1 opposite state threshold*/   
    uint8_t CEvt1RstEn;           /*!< Reset CEVT1's all sampling counter */   
    
    uint8_t CEvt2SmpWnd;          /*!< Select CEVT2 sampling window */       
    uint8_t CEvt2Thresh;          /*!< Select CEVT2 opposite state threshold */   
    uint8_t CEvt2RstEn;           /*!< Reset CEVT2's all sampling counter */   
    uint8_t reverse;
}SDFM_FilterInit_TypeDef;

/** @defgroup SDFM_Filter_Clock_Source
  * @{
  */ 
#define SDFM_FLT_CLK_OWN          (uint8_t)0x00 /*!< Own Clock pin*/
#define SDFM_FLT_CLK_SD1          (uint8_t)0x01 /*!< Channel 1 Clock pin */

/**
  * @}
  */

/** @defgroup SDFM_Filter_Index_Source
  * @{
  */ 
#define SDFM_FILTER_1          0x00 /*!< Filter 1*/
#define SDFM_FILTER_2          0x01 /*!< Filter 2 */
#define SDFM_FILTER_3          0x02 /*!< Filter 3 */
#define SDFM_FILTER_4          0x03 /*!< Filter 4 */

/**
  * @}
  */


/** @defgroup SDFM_Demodulte_Mode
  * @{
  */ 
#define SDFM_FLT_DEMOD_RISE_EDGE       (uint8_t)0x00 /*!< sampling at rising edge*/
#define SDFM_FLT_DEMOD_DESC_EDGE       (uint8_t)0x01 /*!< sampling at descending edge */
#define SDFM_FLT_DEMOD_BOTH_EDGE       (uint8_t)0x02 /*!< sampling at both edges */
#define SDFM_FLT_DEMOD_MANCHESTER_R1   (uint8_t)0x03 /*!< rising edge as 1 for machester decoding */
#define SDFM_FLT_DEMOD_MANCHESTER_R0   (uint8_t)0x04 /*!< rising edge as 0 for machester decoding  */
#define SDFM_FLT_DEMOD_SECOND_CLK      (uint8_t)0x05 /*!< sampling at the second clock */
/**
  * @}
  */

/** @defgroup SDFM_Algorithm
  * @{
  */ 
#define SDFM_ALGTHM_SINCFAST          (uint8_t)0x00 /*!< SincFast Method*/
#define SDFM_ALGTHM_SINC1             (uint8_t)0x01 /*!< Sinc1 Method*/
#define SDFM_ALGTHM_SINC2             (uint8_t)0x02 /*!< Sinc2 Method*/
#define SDFM_ALGTHM_SINC3             (uint8_t)0x03 /*!< Sinc3 Method*/
/**
  * @}
  */
 

/** @defgroup SDFM_Data_Format
  * @{
  */ 
#define SDFM_DFMT_BITS16          (uint8_t)0x00/*!< 16-bits format */
#define SDFM_DFMT_BITS32          (uint8_t)0x01/*!< 32-bits format */
/**
  * @}
  */
  
/** @defgroup SDFM_Data_Ready_Interrupt_Source Data Ready Interrupt Source
  * @{
  */ 
#define SDFM_DRDY_IT_AF          (uint8_t)0x00/*!< Index Filter Enable */
#define SDFM_DRDY_IT_FIFO        (uint8_t)0x01/*!< Index Filter Enable */
/**
  * @}
  */  

/** @defgroup SDFM_Data_2_FIFO_Method  Data to FIFO Method
  * @{
  */ 
#define SDFM_DATA2FIFO_NOWAIT             (uint8_t)0x00 /*!< every data write to fifo */
#define SDFM_DATA2FIFO_WAIT_SYNC          (uint8_t)0x01 /*!< only data after sync event can write to fifo */
/**
  * @}
  */


/** @defgroup SDFM_SYNC_Source   Sync Event Source
  * @{
  */ 
#define SDFM_SYNC_SRC_MCM0_CMP1             (uint8_t)0x00 /*!< Sync event source MCM0 CMP1 */
#define SDFM_SYNC_SRC_MCM1_CMP1             (uint8_t)0x01 /*!< Sync event source MCM1 CMP1 */
#define SDFM_SYNC_SRC_MCM0_CMP2             (uint8_t)0x04 /*!< Sync event source MCM0 CMP2 */
#define SDFM_SYNC_SRC_MCM1_CMP2             (uint8_t)0x05 /*!< Sync event source MCM1 CMP2 */
#define SDFM_SYNC_SRC_MCM0_CMP3             (uint8_t)0x08 /*!< Sync event source MCM0 CMP3 */
#define SDFM_SYNC_SRC_MCM1_CMP3             (uint8_t)0x09 /*!< Sync event source MCM1 CMP3 */
#define SDFM_SYNC_SRC_MCM0_CMP4             (uint8_t)0x0C /*!< Sync event source MCM0 CMP4 */
#define SDFM_SYNC_SRC_MCM1_CMP4             (uint8_t)0x0D /*!< Sync event source MCM1 CMP4 */
#define SDFM_SYNC_SRC_PCA0                  (uint8_t)0x10 /*!< Sync event source PCA0 */
#define SDFM_SYNC_SRC_TIM1                  (uint8_t)0x11 /*!< Sync event source TIM1 */
#define SDFM_SYNC_SRC_PCA1                  (uint8_t)0x14 /*!< Sync event source PCA1 */
#define SDFM_SYNC_SRC_EXTI1                 (uint8_t)0x15 /*!< Sync event source EXTI1 */
#define SDFM_SYNC_SRC_PCA2                  (uint8_t)0x18 /*!< Sync event source PCA2 */
#define SDFM_SYNC_SRC_PCA3                  (uint8_t)0x1C /*!< Sync event source PCA3 */
/**
  * @}
  */

/** @defgroup SDFM_Data_2_FIFO_Method  Data to FIFO Method
  * @{
  */ 
#define SDFM_CEVT_OUT_DIRECT               (uint8_t)0x00 /*!< cevt signal output directly */
#define SDFM_CEVT_OUT_NONE                 (uint8_t)0x01 /*!< no cevt signal output */
#define SDFM_CEVT_OUT_SAMPLING             (uint8_t)0x02 /*!< cevt signal output after sampling */
/**
  * @}  
  */

/** @defgroup SDFM_COMPEVT_Register_Lock  Lock compare event register
  * @{
  */ 
#define SDFM_COMPEVT_LOCK               (1UL<<3) /*!< cevt signal output directly */
#define SDFM_COMPCTL_LOCK               (1UL<<0) /*!< no cevt signal output */
/**
  * @}  
  */


/** @defgroup SDFM_Interrupt_Flag  SDFM interrupt flag
  * @{
  */ 
#define SDFM_SDIFLG_MIF                   ((uint32_t)1 << 31) /*!< Error interrupt flag */
#define SDFM_SDIFLG_SDFFINT4              ((uint32_t)1 << 23) /*!< FIFO4 ready interrupt flag */
#define SDFM_SDIFLG_SDFFINT3              ((uint32_t)1 << 22) /*!< FIFO3 ready interrupt flag */
#define SDFM_SDIFLG_SDFFINT2              ((uint32_t)1 << 21) /*!< FIFO2 ready interrupt flag */
#define SDFM_SDIFLG_SDFFINT1              ((uint32_t)1 << 20) /*!< FIFO1 ready interrupt flag */
#define SDFM_SDIFLG_SDFFOVF4              ((uint32_t)1 << 19) /*!< FIFO4 overflow interrupt flag */
#define SDFM_SDIFLG_SDFFOVF3              ((uint32_t)1 << 18) /*!< FIFO3 overflow interrupt flag */
#define SDFM_SDIFLG_SDFFOVF2              ((uint32_t)1 << 17) /*!< FIFO2 overflow interrupt flag */
#define SDFM_SDIFLG_SDFFOVF1              ((uint32_t)1 << 16) /*!< FIFO1 overflow interrupt flag */
#define SDFM_SDIFLG_AF4                   ((uint32_t)1 << 15) /*!< Filter4 aata ready interrupt flag */
#define SDFM_SDIFLG_AF3                   ((uint32_t)1 << 14) /*!< Filter3 aata ready interrupt flag */
#define SDFM_SDIFLG_AF2                   ((uint32_t)1 << 13) /*!< Filter2 aata ready interrupt flag */
#define SDFM_SDIFLG_AF1                   ((uint32_t)1 << 12) /*!< Filter1 aata ready interrupt flag */
#define SDFM_SDIFLG_MF4                   ((uint32_t)1 << 11) /*!< Filter4 demodule error interrupt flag */
#define SDFM_SDIFLG_MF3                   ((uint32_t)1 << 10) /*!< Filter3 demodule error interrupt flag */
#define SDFM_SDIFLG_MF2                   ((uint32_t)1 << 9) /*!< Filter2 demodule error interrupt flag */
#define SDFM_SDIFLG_MF1                   ((uint32_t)1 << 8) /*!< Filter1 demodule error interrupt flag */
#define SDFM_SDIFLG_FLT4CEVT2             ((uint32_t)1 << 7) /*!< Filter4 event2 interrupt flag */
#define SDFM_SDIFLG_FLT4CEVT1             ((uint32_t)1 << 6) /*!< Filter4 event1 interrupt flag */
#define SDFM_SDIFLG_FLT3CEVT2             ((uint32_t)1 << 5) /*!< Filter3 event2 interrupt flag */
#define SDFM_SDIFLG_FLT3CEVT1             ((uint32_t)1 << 4) /*!< Filter3 event1 interrupt flag */
#define SDFM_SDIFLG_FLT2CEVT2             ((uint32_t)1 << 3) /*!< Filter2 event2 interrupt flag */
#define SDFM_SDIFLG_FLT2CEVT1             ((uint32_t)1 << 2) /*!< Filter2 event1 interrupt flag */
#define SDFM_SDIFLG_FLT1CEVT2             ((uint32_t)1 << 1) /*!< Filter1 event2 interrupt flag */
#define SDFM_SDIFLG_FLT1CEVT1             ((uint32_t)1 << 0) /*!< Filter1 event1 interrupt flag */
/**
  * @}
  */
  
/**
  * @}
  */



/**
  * @}
  */ 


/* Exported macro ------------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/ 

/** @addtogroup SDFM_Exported_Functions Public Functions
  * @{
  */
/* SDFM module reset */
extern void SDFM_Reset(SDFM_TypeDef* SDFMx);
/* Initialize SDFM structure */
extern void SDFM_FilterStructInit(SDFM_FilterInit_TypeDef* SDFM_FltInitStruct);
/* Get filter handler */
extern SDFM_FILTER_TypeDef* SDFM_GetFilter(SDFM_TypeDef* SDFMx,int nFilter);    
/* Initialize SDFM module */
extern void SDFM_FilterInit( SDFM_FILTER_TypeDef* pFilterX, SDFM_FilterInit_TypeDef* SDFM_FltInitStruct);
/* Turn on or turn off SDFM Data Filter  */
extern void SDFM_DataFltOnOff( SDFM_FILTER_TypeDef* pFilterX, SH_CmdState OnOffState);
/* Turn on or turn off SDFM Compare Filter  */
extern void SDFM_CmpFltOnOff( SDFM_FILTER_TypeDef* pFilterX, SH_CmdState OnOffState);
/* Set Filter's manchester clock options */
extern void SDFM_ManchesterClk( SDFM_FILTER_TypeDef* pFilterX, int nSmpPos, int nClkCycle);
/* Get SDFM interrupt Flags */
extern SH_FlagStatus SDFM_GetINTFlag(SDFM_TypeDef* SDFMx,uint32_t SDFM_Flag);   
/* Clear SDFM interrupt flags */
extern void SDFM_ClearINTFlag(SDFM_TypeDef* SDFMx, uint32_t SDFM_Flag);
/* Config SDFM interrupt  */
extern void SDFM_INTConfig(SDFM_TypeDef* SDFMx, uint32_t SDFM_Flag, SH_FunctionalState NewState);
/* Enable or disable SDFM's error interrupt*/
extern void SDFM_ErrINTConfig(SDFM_TypeDef* SDFMx, SH_FunctionalState NewState);
/* Get filter's Cross-Zero flag */
extern SH_FlagStatus SDFM_GetHZFlag(SDFM_FILTER_TypeDef* pFilterX);
/* Clear filter's Cross-Zero flag */
extern void SDFM_ClearHZFlag(SDFM_FILTER_TypeDef* pFilterX);
/* Get filter's sync flag */
extern SH_FlagStatus SDFM_GetSyncFlag(SDFM_FILTER_TypeDef* pFilterX);
/* Clear filter's sync flag */
extern void SDFM_ClearSyncFlag(SDFM_FILTER_TypeDef* pFilterX);

/* Get data from main filter */
extern uint32_t SDFM_GetMainData(SDFM_FILTER_TypeDef* pFilterX);
/* Get data from compare filter */
extern uint32_t SDFM_GetCmpData(SDFM_FILTER_TypeDef* pFilterX);
/* Get data from fifo */
extern uint32_t SDFM_GetFifoData(SDFM_FILTER_TypeDef* pFilterX);
/* Get fifo depth */
extern uint32_t SDFM_GetFifoDepth(SDFM_FILTER_TypeDef* pFilterX);
/* Get fifo data count */
extern uint32_t SDFM_GetFifoCnt(SDFM_FILTER_TypeDef* pFilterX);
/* Set fifo depth */
extern void SDFM_SetFifoDepth(SDFM_FILTER_TypeDef* pFilterX, uint32_t nDepth);
/* Lock COMP Register */
extern void SDFM_LockCompReg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_LockBits);
/* Set Event1 signal output mode */
extern void SDFM_CEvt1OutCfg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_CEvtOutMode);
/* Set Event2 signal output mode */
extern void SDFM_CEvt2OutCfg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_CEvtOutMode);




/**
  * @}
  */


#ifdef __cplusplus
}
#endif

#endif /*__SH3xF2xx_SC0_SDFM_H */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
