/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_TIM.h
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file contains all the functions prototypes for the PPP firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * COPYRIGHT 2022 Sinowealth
  *
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __SH3xF2xx_SC0_TIM_H
#define __SH3xF2xx_SC0_TIM_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_lib.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @addtogroup TIM_MODULE TIM
  * @{
  */ 

/* Exported types ------------------------------------------------------------*/

/** @defgroup TIM_Group_Types  Public Types
  * @{
  */     

/*! @struct  TIM_InitTypeDef
  *   structure for TIM 
  */   
typedef struct
{
    uint32_t TIM_Prescaler;   /*!< Specifie the TIM prescaler value.
                                   This parameter can be 0 to 0xFFFF */
                                   
    uint32_t TIM_Period;      /*!< Specifie the TIM period value.
                                   This parameter can be 0 to 0xFFFF */ 
                                   
    uint32_t TIM_Count;      /*!< Specifie the TIM count value.
                                   This parameter can be 0 to 0xFFFF */ 	
                                   
    uint8_t TIM_CLKSource;   /*!< Specifie the TIM's clock sources */
                                   
    uint8_t TIM_OPMode;      /*!< Specifie the TIM OP mode */
	
    uint16_t TIM_FilterTime;  /*!< Specifie the TIM Filter Time.*/

}TIM_InitTypeDef;
/**
  * @}
  */

  
/* Exported constants --------------------------------------------------------*/


/** @defgroup TIM_Group_Macro  Public Macros
  * @{
  */  
  
#define IS_TIM_ALL_PERIPH(PERIPH)  ((PERIPH == TIM0) || (PERIPH == TIM1))/*!< check the parameters */
     
/*!< TIM_Clock_Sources */   
#define TIM_CLKSource_PCLK          (uint16_t)0x0000/*!< Internal APB clock (PCLK) */
#define TIM_CLKSource_TX            (uint16_t)0x0010/*!< Tx (x=7, 8) port input, automatic pull-up */
#define TIM_CLKSource_LSI           (uint16_t)0x0020/*!< Internal low frequency 128KHz RC oscillator (LSICLK)  */
#define IS_TIM_CLK_SOURCE(SOURCE)   ((SOURCE == TIM_CLKSource_PCLK) \
                                    || (SOURCE == TIM_CLKSource_LSI) \
                                    || (SOURCE == TIM_CLKSource_TX))/*!< check the parameters */

/*!< TIM_OP_Mode */  
#define TIM_OPMode_Continue    (uint16_t)0x0000 /*!< When timer x overflows, the timer doesn't stop */
#define TIM_OPMode_OnePulse    (uint16_t)0x0008 /*!< When timer x overflows, the timer stops */
#define IS_TIM_OP_MODE(MODE)   ((MODE == TIM_OPMode_Continue) \
                               || (MODE == TIM_OPMode_OnePulse))/*!< check the parameters */


/*!< TIM_FilterTime */
#define TIM_FilterTime_NO    		(uint16_t)0x0000 /*!< No filter */
#define TIM_FilterTime_8PCLK    	(uint16_t)0x1000 /*!< Filter time constant is 8*PCLK */
#define TIM_FilterTime_16PCLK    	(uint16_t)0x2000 /*!< Filter time constant is 16*PCLK */
#define TIM_FilterTime_32PCLK    	(uint16_t)0x3000 /*!< Filter time constant is 32*PCLK */
#define IS_TIM_FilterTime(ECF)   ((ECF == TIM_FilterTime_NO) \
                               || (ECF == TIM_FilterTime_8PCLK)\
															 || (ECF == TIM_FilterTime_16PCLK)\
															 || (ECF == TIM_FilterTime_32PCLK))  /*!< check the parameters */

  
/**
  * @}
  */
	
/** @addtogroup TIM_Public_Functions Public Functions
  * @{
  */ 

/* Exported macro ------------------------------------------------------------*/
/* Exported functions --------------------------------------------------------*/ 
/* Reset the TIM peripheral */
void TIM_Reset(TIMER32_TypeDef* TIMx);
/* Fills each TIM_InitStruct member with its default value.  */
void TIM_StructInit(TIM_InitTypeDef* TIM_InitStruct);
/* Initializes the TIMx peripheral according to the specified */
void TIM_Init(TIMER32_TypeDef* TIMx, TIM_InitTypeDef* TIM_InitStruct);
/* Enable or disable the specified TIM interrupt */
void TIM_INTConfig(TIMER32_TypeDef* TIMx,    SH_FunctionalState NewState);
/* Open or close the TIM peripheral. */
void TIM_OnOff(TIMER32_TypeDef* TIMx,      SH_CmdState OnOffState);
/* Enable or disable the TIMx's output pin. */
void TIM_OutPutConfig(TIMER32_TypeDef* TIMx, SH_FunctionalState NewState);
/* Enable or disable the TIMx's trigger core function. */
void TIM_EventTrigger(TIMER32_TypeDef* TIMx, SH_FunctionalState NewState);
/* Enable or disable the TIMx's trigger function. */
void TIM_ExtPeripheralsTrigger(TIMER32_TypeDef* TIMx, SH_FunctionalState NewState);
/* Check whether the specified TIM flag is set or not */
SH_FlagStatus TIM_GetFlagStatus(TIMER32_TypeDef* TIMx);
/* Clear the TIMx's pending flag */
void TIM_ClearFlag(TIMER32_TypeDef* TIMx);
/* Get the TIMx Period Register value */
uint32_t TIM_GetCounter(TIMER32_TypeDef* TIMx);
/* Get the TIMx Period Register value */
uint32_t TIM_GetPrescaler(TIMER32_TypeDef* TIMx);
/* Get the TIMx Period Register value */
uint32_t TIM_GetPeriod(TIMER32_TypeDef* TIMx);
/* Set the TIMx Counter Register value */
void TIM_SetCounter(TIMER32_TypeDef* TIMx, uint32_t Counter);
/* Set the TIMx Prescaler Register value */
void TIM_SetPrescaler(TIMER32_TypeDef* TIMx, uint32_t Prescaler);
/* Set the TIMx Period Register value */
void TIM_SetPeriod(TIMER32_TypeDef* TIMx, uint32_t Period);
/* Set the TIMx Filter Time */
void TIM_SetFilterTime(TIMER32_TypeDef* TIMx, uint32_t FilterTime);
/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /*__SH3xF2xx_SC0_TIM_H */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/
