/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_qei.c
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_qei.h"
#include "sh3xf2xx_sc0_rcc.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @defgroup QEI_MODULE QEI 
  * @brief QEI driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup QEI_Exported_Functions Public Functions
  * @{
  */ 

/** @defgroup QEI_Group1 Initialization and Configuration
 *  @brief    Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      Initialization and Configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */

/**
  * @brief  Reset the QEI peripheral registers to their default reset values.
  * @retval None
  */
void QEI_Reset(QEI_TypeDef* QEIx)
{
    if(QEIx == QEI0)
        RCC_APB0PeriphReset(RCC_APB0_QEI0);
    else    
        RCC_APB0PeriphReset(RCC_APB0_QEI1);
}

/**
  * @brief  Fills each QEI_InitStruct member with its default value.
  * @param  QEI_InitStruct : pointer to a @ref QEI_InitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void QEI_StructInit(QEI_InitTypeDef* QEI_InitStruct)
{
    QEI_InitStruct->QEI_PosCntMode = QEI_PosCntMode_2Edge_IndexRST;
    QEI_InitStruct->QEI_PosCntMaxValue = 0xFFF0;
    QEI_InitStruct->QEI_IndexRSTPosConterCmd = QEI_IndexRSTPosConter_Enable;
    QEI_InitStruct->QEI_ABSwapCmd = QEI_ABSwap_Disable;
    QEI_InitStruct->QEI_IndexFilterCmd = QEI_IndexFilter_Enable;
    QEI_InitStruct->QEI_IndexFilterPrescaler = QEI_IndexFilterPrescaler_4;
    QEI_InitStruct->QEI_QEABFilterCmd = QEI_QEABFilter_Enable;
    QEI_InitStruct->QEI_QEABFilterPrescaler = QEI_QEABFilterPrescaler_4;
    QEI_InitStruct->QEI_PosCntPrescaler = QEI_PosCntPrescaler_1;
    QEI_InitStruct->QEI_QTimerPeriod = 0xFFFFFFFF;
    QEI_InitStruct->QEI_PosCounterMinValue = 0x0000;
    QEI_InitStruct->QEI_TimerPrescaler = 0x0000;
}

/**
  * @brief  Initialize the QEI peripheral according to the specified parameters in the QEI_InitStruct.  
  * @param  QEI_InitStruct: pointer to a QEI_InitTypeDef structure that contains
  *         the configuration information for the QEI peripheral.
  * @retval None
  */
void QEI_Init(QEI_TypeDef* QEIx, QEI_InitTypeDef* QEI_InitStruct)
{
    uint32_t tempReg1 = 0;
    uint32_t tempReg2 = 0;

    /* Check the parameters */
    assert_param(IS_QEI_POS_CNT_MODE(QEI_InitStruct->QEI_PosCntMode));
    assert_param(IS_QEI_INDEX_RST_POS_COUNTER(QEI_InitStruct->QEI_IndexRSTPosConterCmd));
    assert_param(IS_QEI_AB_SWAP(QEI_InitStruct->QEI_ABSwapCmd));
    assert_param(IS_QEI_INDEX_FILTER(QEI_InitStruct->QEI_IndexFilterCmd));
    assert_param(IS_QEI_INDEX_FILTER_PRESCALER(QEI_InitStruct->QEI_IndexFilterPrescaler));
    assert_param(IS_QEI_AB_FILTER(QEI_InitStruct->QEI_QEABFilterCmd));
    assert_param(IS_QEI_QEAB_FILTER_PRESCALER(QEI_InitStruct->QEI_QEABFilterPrescaler));
    assert_param(IS_QEI_POSCNT_PRESCALER(QEI_InitStruct->QEI_PosCntPrescaler));
    
    tempReg1 = QEIx->QEICON.V32;
    tempReg2 = QEIx->QFLTCON.V32;

    tempReg1 &= ~(QEI_QEICON_QPMOD_Msk | QEI_QEICON_QIDXEN_Msk | QEI_QEICON_QSWAP_Msk \
                 | QEI_QEICON_QTEPS_Msk);
    tempReg1 |= QEI_InitStruct->QEI_PosCntMode | QEI_InitStruct->QEI_IndexRSTPosConterCmd \
                | QEI_InitStruct->QEI_ABSwapCmd | QEI_InitStruct->QEI_PosCntPrescaler;
    QEIx->QEICON.V32 = tempReg1;

    tempReg2 &= ~(QEI_QFLTCON_QABCPS_Msk | QEI_QFLTCON_QABFEN_Msk | QEI_QFLTCON_QIDXCPS_Msk \
                | QEI_QFLTCON_QIDXFEN_Msk);
    tempReg2 |= QEI_InitStruct->QEI_IndexFilterCmd | QEI_InitStruct->QEI_IndexFilterPrescaler \
                | QEI_InitStruct->QEI_QEABFilterCmd | QEI_InitStruct->QEI_QEABFilterPrescaler;
    QEIx->QFLTCON.V32 = tempReg2;
    
    QEIx->QTPSQ    = QEI_InitStruct->QEI_TimerPrescaler;
    QEIx->QPOSMAX  = QEI_InitStruct->QEI_PosCntMaxValue;
    QEIx->QTPR     = QEI_InitStruct->QEI_QTimerPeriod;
    QEIx->QCNTMIN  = QEI_InitStruct->QEI_PosCounterMinValue;              
}


/**
  * @}
  */
  

/** @defgroup QEI_Group2 QEI Config
 *  @brief    QEI Config
 *  @verbatim   
 ===============================================================================
                               QEI Config
 ===============================================================================

  * @endverbatim
  * @{
  */
  

/**
  * @brief  Open or close the QEI peripheral.  
  * @param  OnOffState: state of the QEI peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void QEI_OnOff(QEI_TypeDef* QEIx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        /* Enable the QEI peripheral */
        QEIx->QEICON.V32 |= QEI_QEICON_QEIEN_Msk; 
    }
    else
    {
        /* Disable the QEI peripheral */
        QEIx->QEICON.V32 &= ~QEI_QEICON_QEIEN_Msk;
    }
}

/**
  * @brief  Set the QEI's position counter register new value.  
  * @param  QEI_PosCounter: new value of the Position Register.
  *          This parameter can be 0 to 0xFFFF.
  * @retval None
  */
void QEI_SetPositionCounter(QEI_TypeDef* QEIx, uint16_t QEI_PosCounter)
{
    QEIx->QPOSCNT = QEI_PosCounter;
}

/**
  * @brief  Get the QEI's position counter register value.  
  * @retval The Position Register value.
  */
uint16_t QEI_GetPositonCounter(QEI_TypeDef* QEIx)
{
    return (uint16_t)QEIx->QPOSCNT;
}

/**
  * @brief  Get latch register value of the QTimer's Counter.  
  * @retval Value of QTimer's Counter.
  */
uint32_t QEI_GetCaptureLatchValue(QEI_TypeDef* QEIx)
{
    return QEIx->QTMLAT;
}

/**
  * @brief Get position counter direction   
  * @retval QEI counter direction @ref E_QEI_COUNTER_DIR
  *     @arg @b eQEICounterDirDec  Counter decrease - QEA behind QEB
  *     @arg @b eQEICounterDirInc  Counter increase - QEA ahead QEB
*/
E_QEI_COUNTER_DIR QEI_GetPosCounterDir(QEI_TypeDef* QEIx)
{
    E_QEI_COUNTER_DIR eDir;

    if ((QEIx->QEICON.V32 & QEI_QEICON_QPDIR_Msk))
        eDir = eQEICounterDirInc;
    else
        eDir = eQEICounterDirDec;

    return eDir;
}

/**
  * @brief Get INDEX input level   
  * @retval INDEX input level  @ref E_QEI_INDEX_LEVEL
  *     @arg @b  eQEIIndexLow   INDEX input is level low
  *     @arg @b  eQEIIndexHigh  INDEX input is level high
  */
E_QEI_INDEX_LEVEL QEI_GetIndexLevel(QEI_TypeDef* QEIx)
{
    E_QEI_INDEX_LEVEL eLevel;
    if(QEIx->QEICON.BIT.QIDXS)
        eLevel = eQEIIndexHigh;
    else
        eLevel = eQEIIndexLow;
    return eLevel;
}

/**
  * @}
  */
  

/** @defgroup QEI_Group3 Qtimer Config
 *  @brief    Qtimer Config
 *  @verbatim   
 ===============================================================================
                               Qtimer Config
 ===============================================================================

  * @endverbatim
  * @{
  */
  
/**
  * @brief  Set the QEI's QTimer counter register new value.  
  * @param  QEI_QTimerCounter: new value of the QTimer Counter Register.
  *          This parameter can be 0 to 0xFFFFFFFF.
  * @retval None
  */
void QEI_SetQTimerCounter(QEI_TypeDef* QEIx, uint32_t QEI_QTimerCounter)
{
    QEIx->QCNT = QEI_QTimerCounter;
}

/**
  * @brief  Get the QEI's QTimer counter register value.  
  * @retval The QTimer Counter Register value.
  */
uint32_t QEI_GetQTimerCounter(QEI_TypeDef* QEIx)
{
    return QEIx->QCNT;
}

/**
  * @brief  Set the QEI's QTimer period register new value.  
  * @param  QEI_QTimerPeriod: new value of the QTimer Counter Register.
  *          This parameter can be 0 to 0xFFFFFFFF.
  * @retval None
  */
void QEI_SetQTimerPeriod(QEI_TypeDef* QEIx, uint32_t QEI_QTimerPeriod)
{
    QEIx->QTPR = QEI_QTimerPeriod;
}

/**
  * @brief  Get the QEI's QTimer Period register value.  
  * @retval The QTimer Counter Register value.
  */
uint32_t QEI_GetQTimerPeriod(QEI_TypeDef* QEIx)
{
    return QEIx->QTPR;
}

/**
  * @brief  Sets the QEI Prescaler Register value
  * @param  Prescaler: specifies the Prescaler register new value.
  * @retval None
  */
void QEI_SetPrescaler(QEI_TypeDef* QEIx, uint16_t Prescaler)
{
    QEIx->QTPSQ = Prescaler;    
}

/**
  * @brief  Get the QEI Prescaler Register value
  * @retval The Prescaler Register value.
  */
uint16_t QEI_GetPrescaler(QEI_TypeDef* QEIx)
{
    return (uint16_t)QEIx->QTPSQ;    
}


/**
  * @brief  Open or close the QTimer function.  
  * @param  OnOffState: state of the QTimer function.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void QEI_QTimerOnOff(QEI_TypeDef* QEIx, SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));

    if (OnOffState != SH_OFF)
    {
        /* Enable the QTimer */
        QEIx->QEICON.V32 |= QEI_QEICON_QTSR_Msk; 
    }
    else
    {
        /* Disable the QTimer */
        QEIx->QEICON.V32 &= ~QEI_QEICON_QTSR_Msk;
    }
}


/**
  * @}
  */
  
/** @defgroup QEI_Group4 QEI Interrupt
 *  @brief    QEI Interrupt
 *
@verbatim   
 ===============================================================================
                               QEI Interrupt
 ===============================================================================

@endverbatim
  * @{
  */


/**
  * @brief  Enable or disable the QEI's interrupts.
  * @param  QEI_INT: specifie the QEI interrupts to be enabled or disabled.
  *          This parameter can be any combination of the following values:
  *            @arg QEI_INT_PE: QEI phase error interrupt source
  *            @arg QEI_INT_CE: 
  *            @arg QEI_INT_Update:
  *            @arg QEI_INT_CAP: 
  *            @arg QEI_INT_OV:   
  * @param  NewState: new state of the interrupts.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval None
  */
void QEI_INTConfig(QEI_TypeDef* QEIx, uint16_t QEI_INT, SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_QEI_INT(QEI_INT));
    assert_param(IS_FUNCTION_STATE(NewState));

    if (NewState != SH_DISABLE)
    {
        QEIx->QEIINT.V32 |= QEI_INT;
    }
    else
    {
        QEIx->QEIINT.V32 &= ~QEI_INT;
    }
}


#define IS_QEI_FLAG(FLAG)         ((FLAG & (uint16_t)0xFFE0) == (uint16_t)0x00 \
                                   && (FLAG != (uint16_t)0x00))/*!< Check QEI flag parameter */

/**
  * @brief  Check whether the specified QEI flag is set or not.
  * @param  QEI_Flag: specifie the QEI flag to check.
  *          This parameter can be one of the following values:
  *            @arg QEI_FLAG_QPEIF: 
  *            @arg QEI_FLAG_QCEIF: 
  *            @arg QEI_FLAG_QEIIF:
  *            @arg QEI_FLAG_QTCAPIF: 
  *            @arg QEI_FLAG_QTIF:   
  * @retval None
  */
SH_FlagStatus QEI_GetFlagStatus(QEI_TypeDef* QEIx, uint16_t QEI_Flag)
{
    SH_FlagStatus bitStatus;
    
    /* Check the parameters */
    assert_param(IS_QEI_GET_ONE_FLAG(QEI_Flag));   

    if ((QEIx->QTINTF.V32 & QEI_Flag) != (uint32_t)SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    return bitStatus;
}

/**
  * @brief  Clear the QEI's pending flags.
  * @param  QEI_Flag: specifie the flags to be clear.
  *          This parameter can be any combination of the following values:
  *            @arg QEI_FLAG_QPEIF: 
  *            @arg QEI_FLAG_QCEIF: 
  *            @arg QEI_FLAG_QEIIF:
  *            @arg QEI_FLAG_QTCAPIF: 
  *            @arg QEI_FLAG_QTIF:   
  * @retval None
  */
void QEI_ClearFlag(QEI_TypeDef* QEIx, uint16_t QEI_Flag)
{
    /* Check the parameters */
    assert_param(IS_QEI_FLAG(QEI_Flag));

    QEIx->QTINTF.V32 = (uint32_t)QEI_Flag << 16;
}
 
 
/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/

