/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_qei.c
  * @author  sinowealth
  * @version V1.0.0
  * @date    01-March-2024 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2024 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_sdfm.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @defgroup SDFM_MODULE SDFM 
  * @brief SDFM driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup SDFM_Exported_Functions Public Functions
  * @{
  */ 

/** @defgroup SDFM_Group1 Initialization and Configuration
 *  @brief    Initialization and Configuration
 *
@verbatim    
 ===============================================================================
                      Initialization and Configuration
 ===============================================================================  
  
@endverbatim
  * @{
  */
  
static SDFM_FILTER_TypeDef* const SH_SDFM0_Filters[4]={
    ((SDFM_FILTER_TypeDef *)(SDFM0_BASE+0x40)),
    ((SDFM_FILTER_TypeDef *)(SDFM0_BASE+0x90)),
    ((SDFM_FILTER_TypeDef *)(SDFM0_BASE+0xE0)),
    ((SDFM_FILTER_TypeDef *)(SDFM0_BASE+0x130))
};

/**
  * @brief  Get filter handler.
  * @param  nFilter: contains
  *            @arg SDFM_FILTER_1
  *            @arg SDFM_FILTER_2
  *            @arg SDFM_FILTER_3
  *            @arg SDFM_FILTER_4
  * @retval Filter handler
  */
SDFM_FILTER_TypeDef* SDFM_GetFilter(SDFM_TypeDef* SDFMx,int nFilter)
{
    if(SDFMx == SDFM0)
    {
        return SH_SDFM0_Filters[nFilter];
    }
    return NULL;    
}
    
/**
  * @brief  Reset the SDFM peripheral registers to their default reset values.
  * @retval None
  */
void SDFM_Reset(SDFM_TypeDef* SDFMx)
{
    if(SDFMx == SDFM0)
        RCC_AHBPeriphReset(RCC_AHB_SDFM0); 
}

/**
  * @brief  Fills each SDFM_InitStruct member with its default value.
  * @param  SDFM_InitStruct : pointer to a @ref SDFM_InitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void SDFM_FilterStructInit(SDFM_FilterInit_TypeDef* SDFM_FltInitStruct)
{
    int nsize = sizeof(SDFM_FilterInit_TypeDef);
    int i;
    volatile unsigned long* pd = (volatile unsigned long*)(void*)SDFM_FltInitStruct;
    for(i =  0; i < nsize /4; i++)
    {
        *pd = 0;
        pd++;
    }
}

/**
  * @brief  Initialize the SDFM peripheral according to the specified parameters in the SDFM_FltInitStruct.  
  * @param  SDFM_InitStruct: pointer to a SDFM_FilterInit_TypeDef structure that contains
  *         the configuration information for the SDFM peripheral.
  * @retval None
  */
void SDFM_FilterInit( SDFM_FILTER_TypeDef* pFilterX, SDFM_FilterInit_TypeDef* SDFM_FltInitStruct)
{
    pFilterX->CTLPARM.V32 = (((uint32_t)SDFM_FltInitStruct->DataSyncEn)   << SDFM_SD1_CTLPARM_SDDATASYNC_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->ClkSyncEn)   << SDFM_SD1_CTLPARM_SDCLKSYNC_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->ClkSel)      << SDFM_SD1_CTLPARM_SDCLKSEL_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DemodulMode) << SDFM_SD1_CTLPARM_MOD_Pos);
    
    pFilterX->DFPARM.V32 = (((uint32_t)SDFM_FltInitStruct->SyncEn)          << SDFM_SD1_DFPARM_SDSYNCEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DataFltMethod) << SDFM_SD1_DFPARM_SST_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DataRdyFlagEn) << SDFM_SD1_DFPARM_AE_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DataFltEn)     << SDFM_SD1_DFPARM_FEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DataOSR)       << SDFM_SD1_DFPARM_DOSR_Pos);

    pFilterX->DPARM.V32 = (((uint32_t)SDFM_FltInitStruct->DataShiftBits) << SDFM_SD1_DPARM_SH_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DataFmt)    << SDFM_SD1_DPARM_DR_Pos);
    
    pFilterX->CMPH1 = SDFM_FltInitStruct->CFltHL1;
    pFilterX->CMPL1 = SDFM_FltInitStruct->CFltLL1;
    

    pFilterX->CMPH2 = SDFM_FltInitStruct->CFltHL2;
    pFilterX->CMPL2 = SDFM_FltInitStruct->CFltLL2;
    pFilterX->CMPHZ = SDFM_FltInitStruct->CFltHZ;
    

    pFilterX->CPARM.V32 = (((uint32_t)SDFM_FltInitStruct->CEvt2Sel)        << SDFM_SD1_CPARM_CEVT2SEL_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CmpFltEn)     << SDFM_SD1_CPARM_CEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CEvt1Sel)     << SDFM_SD1_CPARM_CEVT1SEL_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CmpHZEn)      << SDFM_SD1_CPARM_HZEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DemodulErrEn) << SDFM_SD1_CPARM_MFIE_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CmpFltMethod) << SDFM_SD1_CPARM_CS1CS0_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CEvt2En)      << SDFM_SD1_CPARM_CEVT2EN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CEvt1En)      << SDFM_SD1_CPARM_CEVT1EN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CmpOSR)       << SDFM_SD1_CPARM_COSR_Pos);

    pFilterX->FIFOCTL.V32 = (((uint32_t)SDFM_FltInitStruct->FifoItOvfEn)   << SDFM_SD1_FIFOCTL_OVFIEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->DRdyItSel)    << SDFM_SD1_FIFOCTL_DRINTSEL_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->FifoEn)       << SDFM_SD1_FIFOCTL_FFEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->FifoItEn)     << SDFM_SD1_FIFOCTL_FFIEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->FifoDepth)    << SDFM_SD1_FIFOCTL_SDFFIL_Pos);

    pFilterX->SYNC.V32 = (((uint32_t)SDFM_FltInitStruct->SyncFlagAutoClrEn) << SDFM_SD1_SYNC_WTSCLREN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->FifoAutoClrEn) << SDFM_SD1_SYNC_FFSYNCCLREN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->Data2FifoMode) << SDFM_SD1_SYNC_WTSYNCEN_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->SyncSrcSel)    << SDFM_SD1_SYNC_SYNCSEL_Pos);

    pFilterX->COMPCTL.V32 = (((uint32_t)SDFM_FltInitStruct->CEvt2OutSrcSel) << SDFM_SD1_COMPCTL_CEVT2DFS_Pos)
                            |(((uint32_t)SDFM_FltInitStruct->CEvt1OutSrcSel)<< SDFM_SD1_COMPCTL_CEVT1DFS_Pos);

    pFilterX->COMPEVT2CTL.V32 = (((uint32_t)SDFM_FltInitStruct->CEvt2RstEn)  << SDFM_SD1_COMPEVT2CTL_FILINIT_Pos)
                                |(((uint32_t)SDFM_FltInitStruct->CEvt2Thresh)<< SDFM_SD1_COMPEVT2CTL_THRESH_Pos)
                                |(((uint32_t)SDFM_FltInitStruct->CEvt2SmpWnd)<< SDFM_SD1_COMPEVT2CTL_SAMPWIN_Pos);

    pFilterX->COMPEVT1CTL.V32 = (((uint32_t)SDFM_FltInitStruct->CEvt1RstEn)  << SDFM_SD1_COMPEVT1CTL_FILINIT_Pos)
                                |(((uint32_t)SDFM_FltInitStruct->CEvt1Thresh)<< SDFM_SD1_COMPEVT1CTL_THRESH_Pos)
                                |(((uint32_t)SDFM_FltInitStruct->CEvt1SmpWnd)<< SDFM_SD1_COMPEVT1CTL_SAMPWIN_Pos);
                                
    pFilterX->COMPEVT1CLKCTL = SDFM_FltInitStruct->CEvt1SmpClk;
    pFilterX->COMPEVT2CLKCTL = SDFM_FltInitStruct->CEvt2SmpClk;
                                     
}


/**
  * @}
  */
  

/** @defgroup SDFM_Group2 SDFM Config
 *  @brief    SDFM Config
 *  @verbatim   
 ===============================================================================
                               SDFM Config
 ===============================================================================

  * @endverbatim
  * @{
  */
  

/**
  * @brief  Turn on or turn off SDFM Data Filter.  
  * @param  OnOffState: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void SDFM_DataFltOnOff(SDFM_FILTER_TypeDef* pFilterX, SH_CmdState OnOffState)
{
    if(OnOffState == SH_ON)
    {
        SDFM0->SDMFILEN.V32 = (1UL << SDFM_SDMFILEN_MFE_Pos);
        pFilterX->DFPARM.BIT.FEN = 1;
    }
    else
    {
        pFilterX->DFPARM.BIT.FEN = 0;
    }
}

/**
  * @brief  Turn on or turn off SDFM Compare Filter.  
  * @param  OnOffState: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ON
  *            @arg SH_OFF
  * @retval None
  */
void SDFM_CmpFltOnOff(SDFM_FILTER_TypeDef* pFilterX, SH_CmdState OnOffState)
{
    if(OnOffState == SH_ON)
    {
        pFilterX->CPARM.BIT.CEN = 1;
    }
    else
    {
        pFilterX->CPARM.BIT.CEN = 0;
    }
}



/**
  * @brief   Set Filter's manchester clock options.  
  * @param  nSmpPos: Sampling position.
  * @param  nClkCycle: Sampling clock.
  * @retval None
  */
void SDFM_ManchesterClk(SDFM_FILTER_TypeDef* pFilterX, int nSmpPos, int nClkCycle)
{
        
   if(pFilterX == SH_SDFM0_Filters[0])
   {
       SDFM0->MCSTCLKCFG1.V32 = (((uint32_t)nSmpPos)<<SDFM_MCSTCLKCFG1_SMPPOS_Pos) 
                                |(((uint32_t)nClkCycle)<<SDFM_MCSTCLKCFG1_SCLKCYCLE_Pos);
   }
   else if(pFilterX == SH_SDFM0_Filters[1])
   {
       SDFM0->MCSTCLKCFG2.V32 = (((uint32_t)nSmpPos)<<SDFM_MCSTCLKCFG2_SMPPOS_Pos) 
                                |(((uint32_t)nClkCycle)<<SDFM_MCSTCLKCFG2_SCLKCYCLE_Pos);
   }
   else if(pFilterX == SH_SDFM0_Filters[2])
   {
       SDFM0->MCSTCLKCFG3.V32 = (((uint32_t)nSmpPos)<<SDFM_MCSTCLKCFG3_SMPPOS_Pos) 
                                |(((uint32_t)nClkCycle)<<SDFM_MCSTCLKCFG3_SCLKCYCLE_Pos);
   }
   else if(pFilterX == SH_SDFM0_Filters[3])
   {
       SDFM0->MCSTCLKCFG4.V32 = (((uint32_t)nSmpPos)<<SDFM_MCSTCLKCFG4_SMPPOS_Pos) 
                                |(((uint32_t)nClkCycle)<<SDFM_MCSTCLKCFG4_SCLKCYCLE_Pos);
   }
}


/**
  * @}
  */
  

/** @defgroup SDFM_Group2 SDFM Flags
 *  @brief    SDFM Config
 *  @verbatim   
 ===============================================================================
                               SDFM Flags
 ===============================================================================

  * @endverbatim
  * @{
  */
  

/**
  * @brief  Get SDFM interrupt Flags.  
  * @param  SDFM_Flag: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SDFM_SDIFLG_MIF
  *            @arg SDFM_SDIFLG_SDFFINT4
  *            @arg SDFM_SDIFLG_SDFFINT3
  *            @arg SDFM_SDIFLG_SDFFINT2
  *            @arg SDFM_SDIFLG_SDFFINT1
  *            @arg SDFM_SDIFLG_SDFFOVF4
  *            @arg SDFM_SDIFLG_SDFFOVF3
  *            @arg SDFM_SDIFLG_SDFFOVF2
  *            @arg SDFM_SDIFLG_SDFFOVF1
  *            @arg SDFM_SDIFLG_AF4
  *            @arg SDFM_SDIFLG_AF3
  *            @arg SDFM_SDIFLG_AF2
  *            @arg SDFM_SDIFLG_AF1
  *            @arg SDFM_SDIFLG_MF4
  *            @arg SDFM_SDIFLG_MF3
  *            @arg SDFM_SDIFLG_MF2
  *            @arg SDFM_SDIFLG_MF1
  *            @arg SDFM_SDIFLG_FLT4CEVT2
  *            @arg SDFM_SDIFLG_FLT4CEVT1
  *            @arg SDFM_SDIFLG_FLT3CEVT2
  *            @arg SDFM_SDIFLG_FLT3CEVT1
  *            @arg SDFM_SDIFLG_FLT2CEVT2
  *            @arg SDFM_SDIFLG_FLT2CEVT1
  *            @arg SDFM_SDIFLG_FLT1CEVT2
  *            @arg SDFM_SDIFLG_FLT1CEVT1
  * @retval SH_SET,SH_RESET
  */
SH_FlagStatus SDFM_GetINTFlag(SDFM_TypeDef* SDFMx,uint32_t SDFM_Flag)
{
    if(SDFMx->SDIFLG.V32 & SDFM_Flag)
        return SH_SET;
    else
        return SH_RESET;
}

/**
  * @brief  Clear SDFM interrupt Flags.  
  * @param  SDFM_Flag: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SDFM_SDIFLG_MIF
  *            @arg SDFM_SDIFLG_SDFFINT4
  *            @arg SDFM_SDIFLG_SDFFINT3
  *            @arg SDFM_SDIFLG_SDFFINT2
  *            @arg SDFM_SDIFLG_SDFFINT1
  *            @arg SDFM_SDIFLG_SDFFOVF4
  *            @arg SDFM_SDIFLG_SDFFOVF3
  *            @arg SDFM_SDIFLG_SDFFOVF2
  *            @arg SDFM_SDIFLG_SDFFOVF1
  *            @arg SDFM_SDIFLG_AF4
  *            @arg SDFM_SDIFLG_AF3
  *            @arg SDFM_SDIFLG_AF2
  *            @arg SDFM_SDIFLG_AF1
  *            @arg SDFM_SDIFLG_MF4
  *            @arg SDFM_SDIFLG_MF3
  *            @arg SDFM_SDIFLG_MF2
  *            @arg SDFM_SDIFLG_MF1
  *            @arg SDFM_SDIFLG_FLT4CEVT2
  *            @arg SDFM_SDIFLG_FLT4CEVT1
  *            @arg SDFM_SDIFLG_FLT3CEVT2
  *            @arg SDFM_SDIFLG_FLT3CEVT1
  *            @arg SDFM_SDIFLG_FLT2CEVT2
  *            @arg SDFM_SDIFLG_FLT2CEVT1
  *            @arg SDFM_SDIFLG_FLT1CEVT2
  *            @arg SDFM_SDIFLG_FLT1CEVT1
  * @retval none
  */
void SDFM_ClearINTFlag(SDFM_TypeDef* SDFMx, uint32_t SDFM_Flag)
{
    SDFMx->SDIFLGCLR.V32 = SDFM_Flag;
    
}

/**
  * @brief  Enable or disable SDFM's error interrupt.  
  * @param  SDFM_Flag: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SDFM_SDIFLG_MIF
  *            @arg SDFM_SDIFLG_SDFFINT4
  *            @arg SDFM_SDIFLG_SDFFINT3
  *            @arg SDFM_SDIFLG_SDFFINT2
  *            @arg SDFM_SDIFLG_SDFFINT1
  *            @arg SDFM_SDIFLG_SDFFOVF4
  *            @arg SDFM_SDIFLG_SDFFOVF3
  *            @arg SDFM_SDIFLG_SDFFOVF2
  *            @arg SDFM_SDIFLG_SDFFOVF1
  *            @arg SDFM_SDIFLG_AF4
  *            @arg SDFM_SDIFLG_AF3
  *            @arg SDFM_SDIFLG_AF2
  *            @arg SDFM_SDIFLG_AF1
  *            @arg SDFM_SDIFLG_MF4
  *            @arg SDFM_SDIFLG_MF3
  *            @arg SDFM_SDIFLG_MF2
  *            @arg SDFM_SDIFLG_MF1
  *            @arg SDFM_SDIFLG_FLT4CEVT2
  *            @arg SDFM_SDIFLG_FLT4CEVT1
  *            @arg SDFM_SDIFLG_FLT3CEVT2
  *            @arg SDFM_SDIFLG_FLT3CEVT1
  *            @arg SDFM_SDIFLG_FLT2CEVT2
  *            @arg SDFM_SDIFLG_FLT2CEVT1
  *            @arg SDFM_SDIFLG_FLT1CEVT2
  *            @arg SDFM_SDIFLG_FLT1CEVT1
  * @param  NewState: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval none
  */
void SDFM_INTConfig(SDFM_TypeDef* SDFMx, uint32_t SDFM_Flag, SH_FunctionalState NewState)
{
    if(NewState == SH_ENABLE)
    {
        if(SDFM_Flag & SDFM_SDIFLG_MIF)
            SDFMx->SDCTL.V32 = (1UL<<SDFM_SDCTL_MIE_Pos);
        
         
            
    }
    else
    {
        if(SDFM_Flag & SDFM_SDIFLG_MIF)
            SDFMx->SDCTL.V32 = (0UL<<SDFM_SDCTL_MIE_Pos);        
    }
    
     //filter 1
    if(SDFM_Flag & SDFM_SDIFLG_SDFFINT1)
        SDFMx->SD1_FIFOCTL.BIT.FFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_SDFFOVF1)
        SDFMx->SD1_FIFOCTL.BIT.OVFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_AF1)
        SDFMx->SD1_DFPARM.BIT.AE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_MF1)
        SDFMx->SD1_CPARM.BIT.MFIE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT1CEVT1)
        SDFMx->SD1_CPARM.BIT.CEVT1EN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT1CEVT2)
        SDFMx->SD1_CPARM.BIT.CEVT2EN = NewState;
    
    //filter2
    if(SDFM_Flag & SDFM_SDIFLG_SDFFINT2)
        SDFMx->SD2_FIFOCTL.BIT.FFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_SDFFOVF2)
        SDFMx->SD2_FIFOCTL.BIT.OVFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_AF2)
        SDFMx->SD2_DFPARM.BIT.AE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_MF2)
        SDFMx->SD2_CPARM.BIT.MFIE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT2CEVT1)
        SDFMx->SD2_CPARM.BIT.CEVT1EN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT2CEVT2)
        SDFMx->SD2_CPARM.BIT.CEVT2EN = NewState;        
    
    //filter3
    if(SDFM_Flag & SDFM_SDIFLG_SDFFINT3)
        SDFMx->SD3_FIFOCTL.BIT.FFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_SDFFOVF3)
        SDFMx->SD3_FIFOCTL.BIT.OVFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_AF3)
        SDFMx->SD3_DFPARM.BIT.AE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_MF3)
        SDFMx->SD3_CPARM.BIT.MFIE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT3CEVT1)
        SDFMx->SD3_CPARM.BIT.CEVT1EN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT3CEVT2)
        SDFMx->SD3_CPARM.BIT.CEVT2EN = NewState;  

    //filter4
    if(SDFM_Flag & SDFM_SDIFLG_SDFFINT4)
        SDFMx->SD4_FIFOCTL.BIT.FFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_SDFFOVF4)
        SDFMx->SD4_FIFOCTL.BIT.OVFIEN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_AF4)
        SDFMx->SD4_DFPARM.BIT.AE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_MF4)
        SDFMx->SD4_CPARM.BIT.MFIE = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT4CEVT1)
        SDFMx->SD4_CPARM.BIT.CEVT1EN = NewState;
    if(SDFM_Flag & SDFM_SDIFLG_FLT4CEVT2)
        SDFMx->SD4_CPARM.BIT.CEVT2EN = NewState; 
    
}


/**
  * @brief  Enable or disable SDFM's error interrupt.  
  * @param  NewState: state of the SDFM peripheral.
  *          This parameter can be one of the following values:
  *            @arg SH_ENABLE
  *            @arg SH_DISABLE
  * @retval none
  */
void SDFM_ErrINTConfig(SDFM_TypeDef* SDFMx, SH_FunctionalState NewState)
{
    if(NewState == SH_ENABLE)
        SDFMx->SDCTL.V32 = (1UL<<SDFM_SDCTL_MIE_Pos);
    else
        SDFMx->SDCTL.V32 = (0UL<<SDFM_SDCTL_MIE_Pos);
}

/**
  * @brief  Get filter's Cross-Zero flag.  
  * @param  pFilterX: pointer to SDFM_FILTER_TypeDef.
  * @retval SH_SET,SH_RESET
  */
SH_FlagStatus SDFM_GetHZFlag(SDFM_FILTER_TypeDef* pFilterX)
{
    uint32_t flag = SDFM0->SDSTATUS.V32;
    if(pFilterX == SH_SDFM0_Filters[0])
        flag &= (1UL<<0);
    else if(pFilterX == SH_SDFM0_Filters[1])
        flag &= (1UL<<1);
    else if(pFilterX == SH_SDFM0_Filters[2])
        flag &= (1UL<<2);
    else if(pFilterX == SH_SDFM0_Filters[3])
        flag &= (1UL<<3);
    
    if(flag)
        return SH_SET;
    else
        return SH_RESET;    
}

/**
  * @brief  Clear filter's Cross-Zero flag.  
  * @param  pFilterX: pointer to SDFM_FILTER_TypeDef.
  * @retval none
  */
void SDFM_ClearHZFlag(SDFM_FILTER_TypeDef* pFilterX)
{
    if(pFilterX == SH_SDFM0_Filters[0])
        SDFM0->SDCTL.V32 |= (1UL<<0);
    else if(pFilterX == SH_SDFM0_Filters[1])
        SDFM0->SDCTL.V32 |= (1UL<<1);
    else if(pFilterX == SH_SDFM0_Filters[2])
        SDFM0->SDCTL.V32 |= (1UL<<2);
    else if(pFilterX == SH_SDFM0_Filters[3])
        SDFM0->SDCTL.V32 |= (1UL<<3);
}

/**
  * @brief  Get filter's sync flag.  
  * @param  pFilterX: pointer to SDFM_FILTER_TypeDef.
  * @retval none
  */
SH_FlagStatus SDFM_GetSyncFlag(SDFM_FILTER_TypeDef* pFilterX)
{
    return (SH_FlagStatus)pFilterX->SYNC.BIT.WTSYNFLG;
}
/**
  * @brief  Clear filter's sync flag  
  * @param  pFilterX: pointer to SDFM_FILTER_TypeDef.
  * @retval none
  */
void SDFM_ClearSyncFlag(SDFM_FILTER_TypeDef* pFilterX)
{
    pFilterX->SYNC.BIT.WTSYNCLR = 1;    
}


/**
  * @}
  */


/** @defgroup SDFM_Group2 SDFM Flags
 *  @brief    SDFM Config
 *  @verbatim   
 ===============================================================================
                               SDFM Flags
 ===============================================================================

  * @endverbatim
  * @{
  */

/**
  * @brief  Get data from main filter.  
  * @retval data in compare filter
  */
uint32_t SDFM_GetMainData(SDFM_FILTER_TypeDef* pFilterX)
{
    return pFilterX->RDATA.V32;        
}

/**
  * @brief  Get data from compare filter.  
  * @retval data in compare filter
  */
uint32_t SDFM_GetCmpData(SDFM_FILTER_TypeDef* pFilterX)
{
        return pFilterX->CDATA;
}

/**
  * @brief  Get data from fifo.  
  * @retval first data in fifo
  */
uint32_t SDFM_GetFifoData(SDFM_FILTER_TypeDef* pFilterX)
{
    return pFilterX->DATFIFO.V32;
}

/**
  * @brief  Get fifo depth
  * @retval fifo depth
  */
uint32_t SDFM_GetFifoDepth(SDFM_FILTER_TypeDef* pFilterX)
{
    return pFilterX->FIFOCTL.BIT.SDFFIL;
}
/**
  * @brief  Get fifo data count.  
  * @retval data count in fifo
  */
uint32_t SDFM_GetFifoCnt(SDFM_FILTER_TypeDef* pFilterX)
{
    return pFilterX->FIFOCTL.BIT.SDFFST;
}
/**
  * @brief  Set fifo depth .  
  * @retval none
  */
void SDFM_SetFifoDepth(SDFM_FILTER_TypeDef* pFilterX, uint32_t nDepth)
{
    pFilterX->FIFOCTL.BIT.SDFFIL = nDepth;
}
/**
  * @}
  */


/** @defgroup SDFM_Group2 SDFM Flags
 *  @brief    SDFM Config
 *  @verbatim   
 ===============================================================================
                               SDFM Flags
 ===============================================================================

  * @endverbatim
  * @{
  */


/**
  * @brief  Lock COMP Register.  
  * @param  SDFM_LockBits: 
  *          This parameter can be one of the following values:
  *            @arg SDFM_COMPEVT_LOCK
  *            @arg SDFM_COMPCTL_LOCK
  * @retval none
  */
void SDFM_LockCompReg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_LockBits)
{
        pFilterX->COMPLOCK.V32 = SDFM_LockBits;
}


/* Set Event1 signal output mode */
/**
  * @brief  Set Event1 signal output mode.  
  * @param  SDFM_CEvtOutMode: 
  *          This parameter can be one of the following values:
  *            @arg SDFM_CEVT_OUT_DIRECT
  *            @arg SDFM_CEVT_OUT_NONE
  *            @arg SDFM_CEVT_OUT_SAMPLING
  * @retval none
  */
void SDFM_CEvt1OutCfg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_CEvtOutMode)
{
    pFilterX->COMPCTL.BIT.CEVT1DFS = SDFM_CEvtOutMode;
}

/* Set Event2 signal output mode */
/**
  * @brief  Set Event2 signal output mode.  
  * @param  SDFM_CEvtOutMode: 
  *          This parameter can be one of the following values:
  *            @arg SDFM_CEVT_OUT_DIRECT
  *            @arg SDFM_CEVT_OUT_NONE
  *            @arg SDFM_CEVT_OUT_SAMPLING
  * @retval none
  */
void SDFM_CEvt2OutCfg(SDFM_FILTER_TypeDef* pFilterX,uint32_t SDFM_CEvtOutMode)
{
    pFilterX->COMPCTL.BIT.CEVT2DFS = SDFM_CEvtOutMode;
}

/**
  * @}
  */


/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/

