/*
            Copyright(C) 1997-2021.  Sinowealth Tech. Co., Ltd.
********************************************************************************
* @file    CFG33F2053.h  
* @author  
* @version V0.0.0
* @date    26-Oct-2023
* @brief   Keil configuration Wizard file.
********************************************************************************
*/
#ifndef __CFG33F2053_H__
#define __CFG33F2053_H__

#if defined(__clang__)

#if (__ARMCC_VERSION >= 6190000)
#pragma clang diagnostic ignored "-Winvalid-utf8"
#pragma clang diagnostic ignored "-Wstrict-prototypes"
#endif

#if (__ARMCC_VERSION >= 6200000)
#pragma clang diagnostic ignored "-Wunsafe-buffer-usage"
#endif

#endif

/*****************************************************************************
//--------- <<< Use Configuration Wizard in Context Menu >>> -----------------
*****************************************************************************/
//<h>Debug Options
/**********************
//<o> Default Assert Function
//  <0=>DISABLE    
//  <1=>ENABLE
**********************/     
#define     ASSERT_ENABLE       0
#if ASSERT_ENABLE==1
#define DEFAULT_ASSERT_ENABLE
#endif    


#ifdef __cplusplus
extern "C" {
extern void RetargetInit(void);
}
#else
extern void RetargetInit(void);
#endif
   

/**********************
// debug printf
**********************/    


/**********************
//Debug Printf Options
//====== Debug Printf ========    
//<o>Debug Printf Device
//   <0=>No Printf support
//   <1=>UART0
//   <2=>UART1
//   <3=>UART2
//   <4=>UART3
//   </h> 
**********************/     
#define     USE_MODULE_DBG_PRINTF 1
#if USE_MODULE_DBG_PRINTF
#define _MODULE_DBG_PRINTF
#endif

   
#if USE_MODULE_DBG_PRINTF == 1
#define _DBG_DEVICE_UART
#define PTN_DBG_UART UART0
#elif USE_MODULE_DBG_PRINTF == 2
#define _DBG_DEVICE_UART
#define PTN_DBG_UART UART1
#elif USE_MODULE_DBG_PRINTF == 3
#define _DBG_DEVICE_UART
#define PTN_DBG_UART UART2
#elif USE_MODULE_DBG_PRINTF == 4
#define _DBG_DEVICE_UART
#define PTN_DBG_UART UART3
#endif

/***********************
//<e>Use Default SystemInit
************************/
#define     USE_DEFAULT_SYSTEMINIT 1
#if USE_DEFAULT_SYSTEMINIT
#define DEFAULT_SYSTEMINIT
#endif


/***********************
//<o> Clock Source  
//    <0=>HSI (12MHz)
//    <1=>HSE       
//    <2=>RC128K
//    <3=>PLL
**********************/     
#define     CLK_SRC             3
/*********************
//<o> HSI CLK 
**********************/
#define     HSI_CLK             12000000
/*********************
//<o> HSE CLK 
**********************/
#define     HSE_CLK             8000000
/*********************/


/*********************
//<e> Use PLL
//<i>PLLCLK = PLLCLKIN/N*PLL_MUL/PLL_DIV/2
**********************/
#define     PLL_EN              1

/***********************
//<o> PLL Clock Source  
//    <0=>HSI (24MHz)
//    <1=>HSE       
//<i>PLLCLK = PLLCLKIN/N*PLL_MUL/PLL_DIV/2
**********************/
#define     PLL_CLK_SRC         0

/*********************
//<o> N:CLK/n TO PLL 
//    <1=>FCLK/1
//    <2=>FCLK/2 (default)
//    <3=>FCLK/3
//    <4=>FCLK/4
//    <5=>FCLK/5
//    <6=>FCLK/6
//    <7=>FCLK/7
//<i>PLLCLK = PLLCLKIN/N*PLL_MUL/PLL_DIV/2
**********************/
#define   PLL_XTPRE           2

#define   PLL_IN        PLL_CLK_SRC
/*********************
//<o> PLL_MUL
//<i>PLLCLK = PLLCLKIN/N*PLL_MUL/PLL_DIV/2
**********************/
#define     PLL_MUL             80

/*********************
//<o> PLL_DIV
//    <0=>K=1
//    <1=>K=2 (default)
//    <2=>K=4 
//    <3=>K=8
//    <4=>K=16
//    <5=>K=32
//<i>PLLCLK = PLLCLKIN/N*PLL_MUL/PLL_DIV/2
**********************/
#define     PLL_DIV            1

/*********************
//</e>
**********************/
#if PLL_MUL < 30
#error "PLL MUL Need 31~255"
#endif
#if PLL_MUL > 255
#error "PLL MUL Need 31~255"
#endif


/*********************
//<o>AHB CLOCK (HCLK)
//    <0=>FCLK/1
//    <1=>FCLK/2
//    <2=>FCLK/4
//    <3=>FCLK/8
**********************/
#define     CLK_AHB             0


/*********************
//<o>APB0 CLOCK
//    <0=>HCLK/1
//    <1=>HCLK/2
//    <2=>HCLK/4
//    <3=>HCLK/8
**********************/
#define     CLK_APB0            1


/*********************
//<o>FLASH Latency
//    <0=>0
//    <1=>1
//    <2=>2
//    <3=>3
//    <4=>4
//    <5=>5
//    <6=>6
//    <7=>7
**********************/
#define     FLASH_LATENCY       5
/*********************
//<q> FLASH Instruction Buffer
**********************/
#define     FLASH_IBUFFER_EN         1
/*********************
//<q> FLASH Data Buffer
**********************/
#define     FLASH_DBUFFER_EN        1
/*********************
//<q> FLASH Cache
**********************/
#define     FLASH_CACHE_EN           1

/***********************
//</e>
************************/


#if  (CLK_SRC == 0)
#define CLK_HZ   HSI_CLK
#elif (CLK_SRC == 1)
#define CLK_HZ   HSE_CLK
#else
#if (PLL_EN == 0)
#error ("System Clock Source is PLL, so PLL must enable")
#endif
#if (PLL_CLK_SRC == 0)
#define CLK_HZ   HSI_CLK
#else
#define CLK_HZ   HSE_CLK
#endif
#endif

#define PLL_CLK  ((PLL_CLK_SRC?HSE_CLK:24000000)/PLL_XTPRE*PLL_MUL/(1<<PLL_DIV)/2)


#if (CLK_SRC == 3)
#define SYS_CLK  (PLL_CLK)/(1<<CLK_AHB)
#else
#define SYS_CLK  CLK_HZ/(1<<CLK_AHB)
#endif

#if SYS_CLK > 50000000  && FLASH_LATENCY < 1
#error ("Flash Latency need >= 1")
#endif
#if SYS_CLK > 100000000  && FLASH_LATENCY < 2
#error ("Flash Latency need >= 2")
#endif
#if SYS_CLK > 150000000  && FLASH_LATENCY < 3
#error ("Flash Latency need >= 3")
#endif
#if SYS_CLK > 200000000  && FLASH_LATENCY < 4
#error ("Flash Latency need >= 4")
#endif
#if SYS_CLK > 250000000  && FLASH_LATENCY < 5
#error ("Flash Latency need >= 5")
#endif


#if (CLK_AHB == 0)
#define AHB_F 1
#elif (CLK_AHB == 1)
#define AHB_F 2
#elif (CLK_AHB == 2)
#define AHB_F 4
#elif (CLK_AHB == 3)
#define AHB_F 8
#elif (CLK_AHB == 4)
#define AHB_F 16
#elif (CLK_AHB == 5)
#define AHB_F 32
#endif


#if (CLK_APB0 == 1)
#define APB_F 2
#elif (CLK_APB0 == 0)
#define APB_F 1
#elif (CLK_APB0 == 2)
#define APB_F 4
#elif (CLK_APB0 == 3)
#define APB_F 8
#elif (CLK_APB0 == 4)
#define APB_F 16
#endif


//<h>NVIC&SysTick
//<e>SysTick
#define     CFG_SYSTICK_EN      0
// <o1>SysTickʼֵ(us) <0-100000000>
//<i>Systick counter is a 24 bit counter, and the cycle value shall be reasonably set according to the clock selection
// <o2>SysTickֵ(us) <0-100000000>
//<i>Systick counter is a 24 bit counter, and the cycle value shall be reasonably set according to the clock selection
// <o3>SysTickʱѡ
//    <0=>ⲿοʱHCLK/8
//    <1=>ںʱHCLK
// <q0>SysTickжʹλ
// <q4>SysTickϵʼʱ


#define     CFG_SYSTICK_INT_EN  0
#define     CFG_SYSTICK_PERIOD_VALUE 1000
#define     CFG_SYSTICK_RL_VALUE 1000
#define     CFG_SYSTICK_CR_CLK  0
#define     CFG_SYSTICK_START_EN 1

#if CFG_SYSTICK_CR_CLK
#define CFG_SYSTICK_CLK SYS_CLK
#else
#define CFG_SYSTICK_CLK SYS_CLK/8
#endif

#define CFG_SYSTICK_VAL_INIT (unsigned long)(CFG_SYSTICK_PERIOD_VALUE*(CFG_SYSTICK_CLK/1000000.0))
#define CFG_SYSTICK_LOAD_INIT (unsigned long)(CFG_SYSTICK_RL_VALUE*(CFG_SYSTICK_CLK/1000000.0))
//</e>


//<o0.8..10>жȼλ
//<i>PRIGROUP ȼвռʽȼ
//<0=>3λռʽȼ, 0λȼ
//<5=>2λռʽȼ, 1λȼ
//<6=>1λռʽȼ, 2λȼ
//<7=>0λռʽȼ, 3λȼ
#define     CFG_SCB_PRIGROUP_INIT 0x0


//<e>жȼ
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_PRI_EN          0


//<o0.5..7>PRI_4 洢 fault ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP4_INIT 0x0


//<o0.5..7>PRI_5  fault ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP5_INIT 0x0


//<o0.5..7>PRI_6 ÷ fault ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP6_INIT 0x0


//<o0.5..7>PRI_11 SVC ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP11_INIT 0x0


//<o0.5..7>PRI_12 Լȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP12_INIT 0x0


//<o0.5..7>PRI_14 PendSV ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP14_INIT 0x0


//<o0.5..7>PRI_15 SysTick ȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_SCB_SHP15_INIT 0x0
                                           

//<o0.5..7>HWINT_0 WWDTжϵȼ <0-7>  
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP0_INIT 0x0


//<o0.5..7>HWINT_1 BODжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP1_INIT 0x0


//<o0.5..7>HWINT_2 RCCжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP2_INIT 0x0


//<o0.5..7>HWINT_3 EXTI0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP3_INIT 0x0


//<o0.5..7>HWINT_4 EXTI1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP4_INIT 0x0


//<o0.5..7>HWINT_5 EXTI2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP5_INIT 0x0


//<o0.5..7>HWINT_6 EXTI3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP6_INIT 0x0


//<o0.5..7>HWINT_7 EXIT4_7жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP7_INIT 0x0


//<o0.5..7>HWINT_8 DMA_CH0жϵȼ <0-7>
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP8_INIT 0x0


//<o0.5..7>HWINT_9 DMA_CH1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP9_INIT 0x0


//<o0.5..7>HWINT_10 DMA_CH2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP10_INIT 0x0


//<o0.5..7>HWINT_11 DMA_CH3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP11_INIT 0x0


//<o0.5..7>HWINT_12 DMA_CH4жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP12_INIT 0x0


//<o0.5..7>HWINT_13 DMA_CH5-7жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP13_INIT 0x0


//<o0.5..7>HWINT_14 MCM0_FLTжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP14_INIT 0x0


//<o0.5..7>HWINT_15 MCM1_FLTжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP15_INIT 0x0


//<o0.5..7>HWINT_16 MCM0_PWMжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP16_INIT 0x0


//<o0.5..7>HWINT_17 MCM1_PWMжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP17_INIT 0x0


//<o0.5..7>HWINT_18 ADC0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP18_INIT 0x0


//<o0.5..7>HWINT_19 ADC1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP19_INIT 0x0


//<o0.5..7>HWINT_20 ADC2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP20_INIT 0x0


//<o0.5..7>HWINT_21 PCA_FLT0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP21_INIT 0x0


//<o0.5..7>HWINT_22 PCA_FLT1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP22_INIT 0x0


//<o0.5..7>HWINT_23 PCA_FLT2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP23_INIT 0x0


//<o0.5..7>HWINT_24 PCA_FLT3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP24_INIT 0x0


//<o0.5..7>HWINT_25 PCA0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP25_INIT 0x0


//<o0.5..7>HWINT_26 PCA1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP26_INIT 0x0


//<o0.5..7>HWINT_27 PCA2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP27_INIT 0x0


//<o0.5..7>HWINT_28 PCA3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP28_INIT 0x0

//<o0.5..7>HWINT_29 CMP0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP29_INIT 0x0

//<o0.5..7>HWINT_30 CMP1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP30_INIT 0x0

//<o0.5..7>HWINT_31 CMP2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP31_INIT 0x0

//<o0.5..7>HWINT_32 QEI0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP32_INIT 0x0

//<o0.5..7>HWINT_33 QEI1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP33_INIT 0x0

//<o0.5..7>HWINT_34 CAN0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP34_INIT 0x0

//<o0.5..7>HWINT_35 TWI0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP35_INIT 0x0

//<o0.5..7>HWINT_36 SPI0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP36_INIT 0x0

//<o0.5..7>HWINT_37 SPI1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP37_INIT 0x0

//<o0.5..7>HWINT_38 UART0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP38_INIT 0x0

//<o0.5..7>HWINT_39 UART1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP39_INIT 0x0

//<o0.5..7>HWINT_40 UART2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP40_INIT 0x0

//<o0.5..7>HWINT_41 UART3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP41_INIT 0x0

//<o0.5..7>HWINT_42 TIM0жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP42_INIT 0x0

//<o0.5..7>HWINT_43 TIM1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP43_INIT 0x0

//<o0.5..7>HWINT_44 EXTI8_15жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP44_INIT 0x0

//<o0.5..7>HWINT_45 SDFM_ERRжϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP45_INIT 0x0

//<o0.5..7>HWINT_46 SDFM_DR1жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP46_INIT 0x0

//<o0.5..7>HWINT_47 SDFM_DR2жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP47_INIT 0x0


//<o0.5..7>HWINT_48 SDFM_DR3жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP48_INIT 0x0


//<o0.5..7>HWINT_49 SDFM_DR4жϵȼ <0-7> 
//<i>3BitȼΧ0-7
//<i>ѡ3λռʽȼ, 0λȼ3Bitݱʾ8ռȼ
//<i>2λռʽȼ, 1λȼ2Bitݱʾ4ռȼ1bitʾ2ȼ
//<i>1λռʽȼ, 2λȼ1Bitݱʾ2ռȼ2bitʾ4ȼ
//<i>0λռʽȼ, 3λȼ3bitʾ8ȼ
#define     CFG_NVIC_IP49_INIT 0x0
//</e>
//</h>

//<e>SYS_CFG
#define     CFG_SYSCFG_EN       0
//<h>PWRCR
//<q0.11>LVR(Low Voltage Reset)ʹ
//<O0.9..10>LVR Level
//   <0=>4.1V
//   <1=>3.7V
//   <2=>3.1V
//<q0.8>BOD(Brownout Detection)ʹ
//<q0.7>BODжʹ
//<o0.5..6>BODģʽѡ
//    <0=>VDDѹСBOD޵BODʱ
//    <1=>VDDѹӴBOD޵СBODʱ
//    <2=>VDDѹӴ޵СޣС޵
//<o0.0..3>BOD޵ѹλ
//    <0=>2.8V
//    <1=>2.9V
//    <2=>3.0V
//    <3=>3.1V
//    <4=>3.2V
//    <5=>3.3V
//    <6=>3.4V
//    <7=>3.5V
//    <8=>3.6V
//    <9=>3.7V
//    <10=>3.8V
//    <11=>3.9V
//    <12=>4.0V
//    <13=>4.1V
//    <14=>4.2V
//    <15=>4.3V
//</h>
#define     CFG_SYSCFG_PWRCR_INIT 0x0

//<h>SAFR
//<o0.0..1>OSCCFG:XTAL1/XTAL2 Port Function
//   <0=>XTAL1/XTAL2ȫ
//   <1=>ⲿӿڣ
//   <2=>XTAL1ʱӣXTAL2ΪGPIOʹ 
//<o0.2>PA10 Emulation Port Fuction
//     <0=>PA10 (SWDIO)
//     <1=>PA10 ͨIO
//<o0.3>PA11 Emulation Port Fuction
//     <0=>PA11 (SWCLK)
//     <1=> PA11 ͨIO
//<o0.4>PA12 Emulation Port Fuction
//     <0=>PA12 (SWCLK)
//     <1=>PA12 ͨIO
//<q0.7>EXTI7NMIжʹλ
//<q0.8>BODNMIжʹλ
//<q0.9>CSMNMIжʹλ
//</h>
#define     CFG_SYSCFG_SAFR_INIT 0x0

//<h>CRAMLOCK
//<q0.0>CRAMLCK0 CRAM Sector0
//<q0.1>CRAMLCK1 CRAM Sector1
//<q0.2>CRAMLCK2 CRAM Sector2
//<q0.3>CRAMLCK3 CRAM Sector3
//<q0.4>CRAMLCK4 CRAM Sector4 
//<q0.5>CRAMLCK5 CRAM Sector5 
//<q0.6>CRAMLCK6 CRAM Sector6 
//<q0.7>CRAMLCK7 CRAM Sector7 
//<q0.8>CRAMLCK8 CRAM Sector8
//<q0.9>CRAMLCK9 CRAM Sector9 
//<q0.10>CRAMLCK10 CRAM Sector10 
//<q0.11>CRAMLCK11 CRAM Sector11 
//<q0.12>CRAMLCK12 CRAM Sector12
//<q0.13>CRAMLCK13 CRAM Sector13
//<q0.14>CRAMLCK14 CRAM Sector14 
//<q0.15>CRAMLCK15 CRAM Sector15   
//<i>CRAM sectorʼַΪ0x1000 0000ÿһsectorΪ1Kֽڣܹ16Kֽڡ   
//</h>
#define     CFG_SYSCFG_CRAMLOCK_INIT 0x0

//***********************************************************/
//</e>

//<e>IWDT
#define     CFG_IWDT_EN         0
// <q0.15> IWDTON_Źʹ
//    <i>0:
//    <i>1:IWDT
//<i>CR_ŹƼĴ
//<i>0x000 b31~16 b15    b14~12 b11~0   
//<i>CR    LOCK   IWDTON IWDTPR IWDTRLR 
//<i>˿λֻOP_IWDT=0101bʱЧ
//<i>˿λĿͨOptionرտŹͨʽ򿪡
//<i>λá1Ӳڸλ塯0
// <o0.0..11> IWDTRLR_ŹؼĴ<0-4096>
//    ڶ忴Źװֵ 
#define     CFG_IWDTRLR_INIT    4095

// <o0.12..14> IWDTPR_Źʱ
//<0=>ֵ128ms
//<1=>ֵ256ms
//<2=>ֵ512ms
//<3=>ֵ1.024s (Ĭ)
//<4=>ֵ2.048s
//<5=>ֵ4.096s 
//<6=>ֵ8.192s
//<7=>ֵ16.384s
//<i>IWDTPR[2:0]ϵĬֵ011b
#define     CFG_IWDTPR_INIT     12288
//***********************************************************/
//</e>


//<e>WWDT
#define     CFG_WWDT_EN         0
//<i>CR_ڿŹƼĴ
//<i>0x000 b31~16 b15    b14    b13~11 b10~8  b7~0    
//<i>CR    LOCK   WWDTON WWDTIE -      WWDTPR WWDTRLR 
//<i>WTR_ڿŹڼĴ
//<i>0x00C b31~16 b15~8 b7~0    
//<i>WTR   LOCK   -     WWDTWTR 
// <q0.15> WWDTON_ڿŹʹ
//    <i>0: ֹWWDT
//    <i>1: WWDT
//<i>λá1Ӳڸλ塯0
// <o0.0..7> WWDTRLR_ڿŹؼĴ<0-255>
//    ڶ忴Źװֵ
// <o0.8..10> WWDTPR_ڿŹʱԤƵ
//    <0=>ԤƵ=1x256
//    <1=>ԤƵ=2x256
//    <2=>ԤƵ=4x256
//    <3=>ԤƵ=8x256
//    <4=>ԤƵ=16x256
//    <5=>ԤƵ=32x256
//    <6=>ԤƵ=64x256
//    <7=>ԤƵ=128x256
// <q0.14> WWDTIE_ڿŹιжʹ
//    <i>0: ֹWWDTж
//    <i>1: WWDTж
#define     CFG_WWDT_CR_INIT    255


#if ((CFG_WWDT_CR_INIT & (1<<14)) != 0)
	#define CFG_WWDT_INT_EN 1
#else
	#define CFG_WWDT_INT_EN 0
#endif


// <o0.0..7> WWDTWTR_ڿŹֵĴ<0-255>
//    ڶ忴ŹĴֵ
//    üĴдֻLOCKλΪ0x5AA5ʱд룬ơ
#define     CFG_WWDT_WTR_INIT   255
//***********************************************************/
//</e>




//<e>GPIO
#define     CFG_GPIO_EN         0


//<e>GPIOA  
#define     CFG_GPIOA_EN        0

// <h> Pin0  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA0_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA0_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA0_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA0_OT          0


// <q> IE_˿ʹλ
#define     CFG_PA0_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA0_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA0_LOCK        0
// </h>


// <h> Pin1  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA1_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA1_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA1_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA1_OT          0


// <q> IE_˿ʹλ
#define     CFG_PA1_IE          1

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA1_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA1_LOCK        0
// </h>


// <h> Pin2  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA2_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA2_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA2_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PA2_IE          1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA2_TTLEN       0

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA2_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA2_LOCK        0
// </h>


// <h> Pin3  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA3_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA3_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA3_ODR         0




// <q> IE_˿ʹλ
#define     CFG_PA3_IE          1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA3_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA3_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA3_LOCK        0
// </h>


// <h> Pin4  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA4_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA4_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA4_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PA4_IE          1

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA4_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA4_LOCK        0
// </h>


// <h> Pin5  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA5_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA5_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA5_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA5_OT          0


// <q> IE_˿ʹλ
#define     CFG_PA5_IE          1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA5_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA5_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA5_LOCK        0
// </h>


// <h> Pin6  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog_Analog
//    <15=>AF15_Analog
#define     CFG_PA6_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA6_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA6_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA6_OT          0


// <q> IE_˿ʹλ
#define     CFG_PA6_IE          1


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA6_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA6_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA6_LOCK        0
// </h>


// <h> Pin7 
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA7_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA7_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA7_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PA7_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA7_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA7_LOCK        0
// </h>


// <h> Pin8  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA8_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA8_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA8_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PA8_IE          1




// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA8_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA8_LOCK        0
// </h>


// <h> Pin9  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA9_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA9_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA9_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA9_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA9_IE          1


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA9_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA9_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA9_LOCK        0
// </h>


// <h> Pin10  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA10_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA10_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA10_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA10_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA10_IE         1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA10_TTLEN      0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA10_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA10_LOCK       0
// </h>


// <h> Pin11  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA11_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA11_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA11_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA11_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA11_IE         1


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA11_TTLEN      0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA11_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA11_LOCK       0
// </h>


// <h> Pin12  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA12_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA12_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA12_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA12_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA12_IE         1


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA12_TTLEN      0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA12_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA12_LOCK       0
// </h>


// <h> Pin13  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA13_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA13_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA13_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PA13_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA13_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA13_LOCK       0
// </h>


// <h> Pin14  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA14_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA14_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA14_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA14_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA14_IE         1


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA14_TTLEN      0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA14_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA14_LOCK       0
// </h>


// <h> Pin15  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PA15_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PA15_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PA15_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PA15_OT         0


// <q> IE_˿ʹλ
#define     CFG_PA15_IE         1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PA15_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PA15_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PA15_LOCK       0
// </h>
//</e>


#define CFG_GPIO_OTYPER_A_INIT     (((uint32_t)CFG_PA15_OT<<15)\
                                      |(CFG_PA14_OT<<14)\
                                      |(CFG_PA12_OT<<12)\
																		  |(CFG_PA11_OT<<11)\
																		  |(CFG_PA10_OT<<10)\
                                      |(CFG_PA9_OT<<9)\
																		  |(CFG_PA6_OT<<6)\
                                      |(CFG_PA5_OT<<5)\
																		  |(CFG_PA1_OT<<1)\
																		  | CFG_PA0_OT)

#define CFG_GPIO_TTLEN_A_INIT     	(((uint32_t)CFG_PA15_TTLEN<<15)\
																		|(CFG_PA14_TTLEN<<14)\
																		|(CFG_PA12_TTLEN<<12)\
																		|(CFG_PA11_TTLEN<<11)\
																		|(CFG_PA10_TTLEN<<10)\
																		|(CFG_PA9_TTLEN<<9)\
																		|(CFG_PA6_TTLEN<<6)\
																		|(CFG_PA5_TTLEN<<5)\
																		|(CFG_PA3_TTLEN<<3)\
																		|(CFG_PA2_TTLEN<<2))

#define CFG_GPIO_IE_A_INIT     	(((uint32_t)CFG_PA15_IE<<15)\
																		|(CFG_PA14_IE<<14)\
																		|(CFG_PA13_IE<<13)\
																		|(CFG_PA12_IE<<12)\
																		|(CFG_PA11_IE<<11)\
																		|(CFG_PA10_IE<<10)\
																		|(CFG_PA9_IE<<9)\
																		|(CFG_PA8_IE<<8)\
																		|(CFG_PA7_IE<<7)\
																		|(CFG_PA6_IE<<6)\
																		|(CFG_PA5_IE<<5)\
																		|(CFG_PA4_IE<<4)\
																		|(CFG_PA3_IE<<3)\
																		|(CFG_PA2_IE<<2)\
																		|(CFG_PA1_IE<<1)\
																		|CFG_PA0_IE)
																		

#define CFG_GPIO_PUPDR_A_INIT     	(((uint32_t)CFG_PA15_PUPDR<<30)\
																		|(CFG_PA14_PUPDR<<28)\
																		|(CFG_PA13_PUPDR<<26)\
																		|(CFG_PA12_PUPDR<<24)\
																		|(CFG_PA11_PUPDR<<22)\
																		|(CFG_PA10_PUPDR<<20)\
																		|(CFG_PA9_PUPDR<<18)\
																		|(CFG_PA8_PUPDR<<16)\
																		|(CFG_PA7_PUPDR<<14)\
																		|(CFG_PA6_PUPDR<<12)\
																		|(CFG_PA5_PUPDR<<10)\
																		|(CFG_PA4_PUPDR<<8)\
																		|(CFG_PA3_PUPDR<<6)\
																		|(CFG_PA2_PUPDR<<4)\
																		|(CFG_PA1_PUPDR<<2)\
																		|CFG_PA0_PUPDR)
																		

#define CFG_GPIO_AFRH_A_INIT     		(((uint32_t)CFG_PA15_AFR<<28)\
																		|(CFG_PA14_AFR<<24)\
																		|(CFG_PA13_AFR<<20)\
																		|(CFG_PA12_AFR<<16)\
																		|(CFG_PA11_AFR<<12)\
																		|(CFG_PA10_AFR<<8)\
																		|(CFG_PA9_AFR<<4)\
																		|CFG_PA8_AFR)
															

#define CFG_GPIO_AFRL_A_INIT				(((uint32_t)CFG_PA7_AFR<<28)\
																		|(CFG_PA6_AFR<<24)\
																		|(CFG_PA5_AFR<<20)\
																		|(CFG_PA4_AFR<<16)\
																		|(CFG_PA3_AFR<<12)\
																		|(CFG_PA2_AFR<<8)\
																		|(CFG_PA1_AFR<<4)\
																		|CFG_PA0_AFR)																		
																		

#define CFG_GPIO_ODR_A_INIT         (((uint32_t)CFG_PA15_ODR<<15)\
																		|(CFG_PA14_ODR<<14)\
																		|(CFG_PA13_ODR<<13)\
																		|(CFG_PA12_ODR<<12)\
																		|(CFG_PA11_ODR<<11)\
																		|(CFG_PA10_ODR<<10)\
																		|(CFG_PA9_ODR<<9)\
																		|(CFG_PA8_ODR<<8)\
																		|(CFG_PA7_ODR<<7)\
																		|(CFG_PA6_ODR<<6)\
																		|(CFG_PA5_ODR<<5)\
																		|(CFG_PA4_ODR<<4)\
																		|(CFG_PA3_ODR<<3)\
																		|(CFG_PA2_ODR<<2)\
																		|(CFG_PA1_ODR<<1)\
																		|CFG_PA0_ODR)																		
																		

#define CFG_GPIO_MODER_A_INIT     	(((uint32_t)CFG_PA15_MODER<<15)\
																		|(CFG_PA14_MODER<<14)\
																		|(CFG_PA13_MODER<<13)\
																		|(CFG_PA12_MODER<<12)\
																		|(CFG_PA11_MODER<<11)\
																		|(CFG_PA10_MODER<<10)\
																		|(CFG_PA9_MODER<<9)\
																		|(CFG_PA8_MODER<<8)\
																		|(CFG_PA7_MODER<<7)\
																		|(CFG_PA6_MODER<<6)\
																		|(CFG_PA5_MODER<<5)\
																		|(CFG_PA4_MODER<<4)\
																		|(CFG_PA3_MODER<<3)\
																		|(CFG_PA2_MODER<<2)\
																		|(CFG_PA1_MODER<<1)\
																		|CFG_PA0_MODER)	
																		


#define CFG_GPIO_LCKR_A_INIT     	  (((uint32_t)CFG_PA15_LOCK<<15)\
																		|(CFG_PA14_LOCK<<14)\
																		|(CFG_PA13_LOCK<<13)\
																		|(CFG_PA12_LOCK<<12)\
																		|(CFG_PA11_LOCK<<11)\
																		|(CFG_PA10_LOCK<<10)\
																		|(CFG_PA9_LOCK<<9)\
																		|(CFG_PA8_LOCK<<8)\
																		|(CFG_PA7_LOCK<<7)\
																		|(CFG_PA6_LOCK<<6)\
																		|(CFG_PA5_LOCK<<5)\
																		|(CFG_PA4_LOCK<<4)\
																		|(CFG_PA3_LOCK<<3)\
																		|(CFG_PA2_LOCK<<2)\
																		|(CFG_PA1_LOCK<<1)\
																		|CFG_PA0_LOCK)	
																		

//<e>GPIOB  
#define     CFG_GPIOB_EN        0


// <h> Pin0  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB0_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB0_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB0_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB0_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB0_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB0_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB0_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB0_LOCK        0
// </h>


// <h> Pin1  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB1_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB1_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB1_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB1_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB1_TTLEN       0

// <q> IE_˿ʹλ
#define     CFG_PB1_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB1_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB1_LOCK        0
// </h>


// <h> Pin2  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB2_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB2_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB2_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB2_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB2_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB2_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB2_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB2_LOCK        0
// </h>


// <h> Pin3  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB3_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB3_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB3_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB3_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB3_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB3_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB3_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB3_LOCK        0
// </h>


// <h> Pin4  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB4_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB4_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB4_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB4_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB4_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB4_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB4_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB4_LOCK        0
// </h>


// <h> Pin5  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB5_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB5_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB5_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB5_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB5_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB5_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB5_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB5_LOCK        0
// </h>


// <h> Pin6  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB6_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB6_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB6_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PB6_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB6_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB6_LOCK        0
// </h>


// <h> Pin7 
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB7_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB7_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB7_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB7_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB7_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB7_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB7_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB7_LOCK        0
// </h>


// <h> Pin8  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB8_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB8_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB8_ODR         0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB8_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB8_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB8_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB8_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB8_LOCK        0
// </h>


// <h> Pin9  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB9_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB9_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB9_ODR         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB9_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB9_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB9_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB9_LOCK        0
// </h>


// <h> Pin10  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB10_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB10_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB10_ODR        0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB10_OT         0

// <q> IE_˿ʹλ
#define     CFG_PB10_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB10_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB10_LOCK       0
// </h>


// <h> Pin11  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB11_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB11_MODER      0

// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB11_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PB11_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB11_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB11_LOCK       0
// </h>


// <h> Pin12  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB12_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB12_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB12_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB12_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB12_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB12_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB12_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB12_LOCK       0
// </h>


// <h> Pin13  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB13_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB13_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB13_ODR        0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB13_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB13_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB13_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB13_LOCK       0
// </h>


// <h> Pin14  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB14_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB14_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB14_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB14_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB14_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB14_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB14_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB14_LOCK       0
// </h>


// <h> Pin15  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PB15_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PB15_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PB15_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PB15_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PB15_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PB15_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PB15_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PB15_LOCK       0
// </h>
//</e>


#define CFG_GPIO_OTYPER_B_INIT     (((uint32_t)CFG_PB15_OT<<15)\
                                                |(CFG_PB14_OT<<14)\
                                                |(CFG_PB10_OT<<10)\
                                                |(CFG_PB8_OT<<8)\
                                                |(CFG_PB7_OT<<7)\
                                                |(CFG_PB5_OT<<5)\
                                                |(CFG_PB4_OT<<4)\
                                                |(CFG_PB3_OT<<3)\
                                                |(CFG_PB2_OT<<2)\
                                                |(CFG_PB1_OT<<1)\
                                                | CFG_PB0_OT)

#define CFG_GPIO_TTLEN_B_INIT     	(((uint32_t)CFG_PB15_TTLEN<<15)\
                                                |(CFG_PB14_TTLEN<<14)\
                                                |(CFG_PB13_TTLEN<<13)\
                                                |(CFG_PB12_TTLEN<<12)\
                                                |(CFG_PB9_TTLEN<<9)\
                                                |(CFG_PB8_TTLEN<<8)\
                                                |(CFG_PB7_TTLEN<<7)\
                                                |(CFG_PB5_TTLEN<<5)\
                                                |(CFG_PB4_TTLEN<<4)\
                                                |(CFG_PB3_TTLEN<<3)\
                                                |(CFG_PB2_TTLEN<<2)\
                                                |(CFG_PB1_TTLEN<<1)\
                                                |CFG_PB0_TTLEN)																	

#define CFG_GPIO_IE_B_INIT     	(((uint32_t)CFG_PB15_IE<<15)\
                                            |(CFG_PB14_IE<<14)\
                                            |(CFG_PB13_IE<<13)\
                                            |(CFG_PB12_IE<<12)\
                                            |(CFG_PB11_IE<<11)\
                                            |(CFG_PB10_IE<<10)\
                                            |(CFG_PB9_IE<<9)\
                                            |(CFG_PB8_IE<<8)\
                                            |(CFG_PB7_IE<<7)\
                                            |(CFG_PB6_IE<<6)\
                                            |(CFG_PB5_IE<<5)\
                                            |(CFG_PB4_IE<<4)\
                                            |(CFG_PB3_IE<<3)\
                                            |(CFG_PB2_IE<<2)\
                                            |(CFG_PB1_IE<<1)\
                                            |CFG_PB0_IE)
																		

#define CFG_GPIO_PUPDR_B_INIT     	(((uint32_t)CFG_PB15_PUPDR<<30)\
                                                |(CFG_PB14_PUPDR<<28)\
                                                |(CFG_PB13_PUPDR<<26)\
                                                |(CFG_PB12_PUPDR<<24)\
                                                |(CFG_PB11_PUPDR<<22)\
                                                |(CFG_PB10_PUPDR<<20)\
                                                |(CFG_PB9_PUPDR<<18)\
                                                |(CFG_PB8_PUPDR<<16)\
                                                |(CFG_PB7_PUPDR<<14)\
                                                |(CFG_PB6_PUPDR<<12)\
                                                |(CFG_PB5_PUPDR<<10)\
                                                |(CFG_PB4_PUPDR<<8)\
                                                |(CFG_PB3_PUPDR<<6)\
                                                |(CFG_PB2_PUPDR<<4)\
                                                |(CFG_PB1_PUPDR<<2)\
                                                |CFG_PB0_PUPDR)
                                                                                

#define CFG_GPIO_AFRH_B_INIT     		(((uint32_t)CFG_PB15_AFR<<28)\
                                                    |(CFG_PB14_AFR<<24)\
                                                    |(CFG_PB13_AFR<<20)\
                                                    |(CFG_PB12_AFR<<16)\
                                                    |(CFG_PB11_AFR<<12)\
                                                    |(CFG_PB10_AFR<<8)\
                                                    |(CFG_PB9_AFR<<4)\
                                                    |CFG_PB8_AFR)
                                        

#define CFG_GPIO_AFRL_B_INIT				(((uint32_t)CFG_PB7_AFR<<28)\
                                                        |(CFG_PB6_AFR<<24)\
                                                        |(CFG_PB5_AFR<<20)\
                                                        |(CFG_PB4_AFR<<16)\
                                                        |(CFG_PB3_AFR<<12)\
                                                        |(CFG_PB2_AFR<<8)\
                                                        |(CFG_PB1_AFR<<4)\
                                                        |CFG_PB0_AFR)																		
                                                        

#define CFG_GPIO_ODR_B_INIT         (((uint32_t)CFG_PB15_ODR<<15)\
																		|(CFG_PB14_ODR<<14)\
																		|(CFG_PB13_ODR<<13)\
																		|(CFG_PB12_ODR<<12)\
																		|(CFG_PB11_ODR<<11)\
																		|(CFG_PB10_ODR<<10)\
																		|(CFG_PB9_ODR<<9)\
																		|(CFG_PB8_ODR<<8)\
																		|(CFG_PB7_ODR<<7)\
																		|(CFG_PB6_ODR<<6)\
																		|(CFG_PB5_ODR<<5)\
																		|(CFG_PB4_ODR<<4)\
																		|(CFG_PB3_ODR<<3)\
																		|(CFG_PB2_ODR<<2)\
																		|(CFG_PB1_ODR<<1)\
																		|CFG_PB0_ODR)																		
																		

#define CFG_GPIO_MODER_B_INIT     	(((uint32_t)CFG_PB15_MODER<<15)\
																		|(CFG_PB14_MODER<<14)\
																		|(CFG_PB13_MODER<<13)\
																		|(CFG_PB12_MODER<<12)\
																		|(CFG_PB11_MODER<<11)\
																		|(CFG_PB10_MODER<<10)\
																		|(CFG_PB9_MODER<<9)\
																		|(CFG_PB8_MODER<<8)\
																		|(CFG_PB7_MODER<<7)\
																		|(CFG_PB6_MODER<<6)\
																		|(CFG_PB5_MODER<<5)\
																		|(CFG_PB4_MODER<<4)\
																		|(CFG_PB3_MODER<<3)\
																		|(CFG_PB2_MODER<<2)\
																		|(CFG_PB1_MODER<<1)\
																		|CFG_PB0_MODER)	


#define CFG_GPIO_LCKR_B_INIT     	  (((uint32_t)CFG_PB15_LOCK<<15)\
																		|(CFG_PB14_LOCK<<14)\
																		|(CFG_PB13_LOCK<<13)\
																		|(CFG_PB12_LOCK<<12)\
																		|(CFG_PB11_LOCK<<11)\
																		|(CFG_PB10_LOCK<<10)\
																		|(CFG_PB9_LOCK<<9)\
																		|(CFG_PB8_LOCK<<8)\
																		|(CFG_PB7_LOCK<<7)\
																		|(CFG_PB6_LOCK<<6)\
																		|(CFG_PB5_LOCK<<5)\
																		|(CFG_PB4_LOCK<<4)\
																		|(CFG_PB3_LOCK<<3)\
																		|(CFG_PB2_LOCK<<2)\
																		|(CFG_PB1_LOCK<<1)\
																		|CFG_PB0_LOCK)	
																		

//<e>GPIOC  
#define     CFG_GPIOC_EN        0


// <h> Pin0  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC0_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC0_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC0_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC0_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC0_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC0_LOCK        0
// </h>


// <h> Pin1  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC1_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC1_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC1_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PC1_OT          0


// <q> IE_˿ʹλ
#define     CFG_PC1_IE          1

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC1_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC1_LOCK        0
// </h>


// <h> Pin2  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC2_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC2_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC2_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC2_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC2_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC2_LOCK        0
// </h>


// <h> Pin3  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC3_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC3_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC3_ODR         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PC3_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PC3_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC3_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC3_LOCK        0
// </h>


// <h> Pin4  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC4_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC4_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC4_ODR         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PC4_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PC4_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC4_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC4_LOCK        0
// </h>


// <h> Pin5  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC5_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC5_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC5_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC5_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC5_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC5_LOCK        0
// </h>


// <h> Pin6  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC6_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC6_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC6_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC6_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC6_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC6_LOCK        0
// </h>


// <h> Pin7 
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC7_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC7_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC7_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC7_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC7_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC7_LOCK        0
// </h>


// <h> Pin8  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC8_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC8_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC8_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PC8_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC8_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC8_LOCK        0
// </h>


// <h> Pin9  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC9_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC9_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC9_ODR         0


// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PC9_OT          0


// <q> IE_˿ʹλ
#define     CFG_PC9_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC9_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC9_LOCK        0
// </h>


// <h> Pin10  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC10_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC10_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC10_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PC10_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC10_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC10_LOCK       0
// </h>


// <h> Pin11  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC11_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC11_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC11_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PC11_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC11_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC11_LOCK       0
// </h>


// <h> Pin12  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC12_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC12_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC12_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PC12_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC12_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC12_LOCK       0
// </h>



// <h> Pin13  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC13_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC13_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC13_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PC13_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC13_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC13_LOCK       0
// </h>


// <h> Pin14  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC14_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC14_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC14_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PC14_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PC14_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PC14_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC14_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC14_LOCK       0
// </h>


// <h> Pin15  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PC15_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PC15_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PC15_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PC15_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PC15_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PC15_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PC15_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PC15_LOCK       0
// </h>

//</e>


#define CFG_GPIO_OTYPER_C_INIT     (((uint32_t)CFG_PC15_OT<<15)\
																		|(CFG_PC14_OT<<14))
																	

#define CFG_GPIO_IE_C_INIT     	(((uint32_t)CFG_PC15_IE<<15)\
																		|(CFG_PC14_IE<<14)\
																		|(CFG_PC13_IE<<13)\
																		|(CFG_PC12_IE<<12)\
																		|(CFG_PC11_IE<<11)\
																		|(CFG_PC10_IE<<10)\
																		|(CFG_PC9_IE<<9)\
																		|(CFG_PC8_IE<<8)\
																		|(CFG_PC7_IE<<7)\
																		|(CFG_PC6_IE<<6)\
																		|(CFG_PC5_IE<<5)\
																		|(CFG_PC4_IE<<4)\
																		|(CFG_PC3_IE<<3)\
																		|(CFG_PC2_IE<<2)\
																		|(CFG_PC1_IE<<1)\
																		|CFG_PC0_IE)
																		

#define CFG_GPIO_PUPDR_C_INIT     	(((uint32_t)CFG_PC15_PUPDR<<30)\
																		|(CFG_PC14_PUPDR<<28)\
																		|(CFG_PC13_PUPDR<<26)\
																		|(CFG_PC12_PUPDR<<24)\
																		|(CFG_PC11_PUPDR<<22)\
																		|(CFG_PC10_PUPDR<<20)\
																		|(CFG_PC9_PUPDR<<18)\
																		|(CFG_PC8_PUPDR<<16)\
																		|(CFG_PC7_PUPDR<<14)\
																		|(CFG_PC6_PUPDR<<12)\
																		|(CFG_PC5_PUPDR<<10)\
																		|(CFG_PC4_PUPDR<<8)\
																		|(CFG_PC3_PUPDR<<6)\
																		|(CFG_PC2_PUPDR<<4)\
																		|(CFG_PC1_PUPDR<<2)\
																		|CFG_PC0_PUPDR)
																		

#define CFG_GPIO_AFRH_C_INIT     		(((uint32_t)CFG_PC15_AFR<<28)\
																		|(CFG_PC14_AFR<<24)\
																		|(CFG_PC13_AFR<<20)\
																		|(CFG_PC12_AFR<<16)\
																		|(CFG_PC11_AFR<<12)\
																		|(CFG_PC10_AFR<<8)\
																		|(CFG_PC9_AFR<<4)\
																		|CFG_PC8_AFR)
															

#define CFG_GPIO_AFRL_C_INIT				(((uint32_t)CFG_PC7_AFR<<28)\
																		|(CFG_PC6_AFR<<24)\
																		|(CFG_PC5_AFR<<20)\
																		|(CFG_PC4_AFR<<16)\
																		|(CFG_PC3_AFR<<12)\
																		|(CFG_PC2_AFR<<8)\
																		|(CFG_PC1_AFR<<4)\
																		|CFG_PC0_AFR)																		
																		

#define CFG_GPIO_ODR_C_INIT         (((uint32_t)CFG_PC15_ODR<<15)\
																		|(CFG_PC14_ODR<<14)\
																		|(CFG_PC13_ODR<<13)\
																		|(CFG_PC12_ODR<<12)\
																		|(CFG_PC11_ODR<<11)\
																		|(CFG_PC10_ODR<<10)\
																		|(CFG_PC9_ODR<<9)\
																		|(CFG_PC8_ODR<<8)\
																		|(CFG_PC7_ODR<<7)\
																		|(CFG_PC6_ODR<<6)\
																		|(CFG_PC5_ODR<<5)\
																		|(CFG_PC4_ODR<<4)\
																		|(CFG_PC3_ODR<<3)\
																		|(CFG_PC2_ODR<<2)\
																		|(CFG_PC1_ODR<<1)\
																		|CFG_PC0_ODR)																		
																		

#define CFG_GPIO_MODER_C_INIT     	(((uint32_t)CFG_PC15_MODER<<15)\
																		|(CFG_PC14_MODER<<14)\
																		|(CFG_PC13_MODER<<13)\
																		|(CFG_PC12_MODER<<12)\
																		|(CFG_PC11_MODER<<11)\
																		|(CFG_PC10_MODER<<10)\
																		|(CFG_PC9_MODER<<9)\
																		|(CFG_PC8_MODER<<8)\
																		|(CFG_PC7_MODER<<7)\
																		|(CFG_PC6_MODER<<6)\
																		|(CFG_PC5_MODER<<5)\
																		|(CFG_PC4_MODER<<4)\
																		|(CFG_PC3_MODER<<3)\
																		|(CFG_PC2_MODER<<2)\
																		|(CFG_PC1_MODER<<1)\
																		|CFG_PC0_MODER)	


#define CFG_GPIO_LCKR_C_INIT     	  (((uint32_t)CFG_PC15_LOCK<<15)\
																		|(CFG_PC14_LOCK<<14)\
																		|(CFG_PC13_LOCK<<13)\
																		|(CFG_PC12_LOCK<<12)\
																		|(CFG_PC11_LOCK<<11)\
																		|(CFG_PC10_LOCK<<10)\
																		|(CFG_PC9_LOCK<<9)\
																		|(CFG_PC8_LOCK<<8)\
																		|(CFG_PC7_LOCK<<7)\
																		|(CFG_PC6_LOCK<<6)\
																		|(CFG_PC5_LOCK<<5)\
																		|(CFG_PC4_LOCK<<4)\
																		|(CFG_PC3_LOCK<<3)\
																		|(CFG_PC2_LOCK<<2)\
																		|(CFG_PC1_LOCK<<1)\
																		|CFG_PC0_LOCK)	


#define CFG_GPIO_TTLEN_C_INIT     	(((uint32_t)CFG_PC15_TTLEN<<15)\
																		|(CFG_PC14_TTLEN<<14)\
																		|(CFG_PC4_TTLEN<<4)\
																		|(CFG_PC3_TTLEN<<3))	
																		

//<e>GPIOD  
#define     CFG_GPIOD_EN        0


// <h> Pin0  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD0_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD0_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD0_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD0_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD0_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD0_LOCK        0
// </h>


// <h> Pin1  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD1_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD1_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD1_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD1_IE          0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD1_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD1_LOCK        0
// </h>


// <h> Pin2  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD2_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD2_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD2_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD2_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD2_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD2_LOCK        0
// </h>


// <h> Pin3  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD3_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD3_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD3_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD3_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD3_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD3_LOCK        0
// </h>


// <h> Pin4  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD4_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD4_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD4_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD4_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD4_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD4_LOCK        0
// </h>


// <h> Pin5  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD5_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD5_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD5_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD5_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD5_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD5_LOCK        0
// </h>


// <h> Pin6  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD6_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD6_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD6_ODR         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD6_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PD6_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD6_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD6_LOCK        0
// </h>


// <h> Pin7 
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD7_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD7_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD7_ODR         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD7_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PD7_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD7_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD7_LOCK        0
// </h>


// <h> Pin8  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD8_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD8_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD8_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD8_IE          1

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD8_TTLEN       0


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD8_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD8_LOCK        0
// </h>


// <h> Pin9  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD9_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD9_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD9_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PD9_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD9_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD9_LOCK        0
// </h>


// <h> Pin10  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD10_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD10_MODER      0

// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD10_ODR        0

// <q> IE_˿ʹλ
#define     CFG_PD10_IE         1

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD10_PUPDR      0

// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD10_LOCK       0
// </h>


// <h> Pin11  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD11_AFR        0

// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD11_MODER      0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PD11_OT         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD11_TTLEN       0

// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD11_ODR        0


// <q> IE_˿ʹλ
#define     CFG_PD11_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD11_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD11_LOCK       0
// </h>


// <h> Pin12  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD12_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD12_MODER      0

// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD12_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PD12_OT         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD12_TTLEN       0

// <q> IE_˿ʹλ
#define     CFG_PD12_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD12_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD12_LOCK       0
// </h>


// <h> Pin13  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD13_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD13_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD13_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PD13_OT         0


// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD13_TTLEN       0


// <q> IE_˿ʹλ
#define     CFG_PD13_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD13_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD13_LOCK       0
// </h>


// <h> Pin14  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD14_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD14_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD14_ODR        0

// <o> OT_˿ʽ
//    <0=>λ״̬
//    <1=>©
#define     CFG_PD14_OT         0

// <o> TTLEN_˿TTLƽѡĴ
//    <0=>ѡCMOS루λ״̬
//    <1=>ѡTTL
#define     CFG_PD14_TTLEN       0

// <q> IE_˿ʹλ
#define     CFG_PD14_IE         1

// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD14_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD14_LOCK       0
// </h>


// <h> Pin15  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PD15_AFR        0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PD15_MODER      0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PD15_ODR        0

// <q> IE_˿ʹλ
#define     CFG_PD15_IE         1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PD15_PUPDR      0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PD15_LOCK       0
// </h>
//</e>


#define CFG_GPIO_OTYPER_D_INIT     (((uint32_t)CFG_PD14_OT<<14)\
                                      |(CFG_PD13_OT<<13)\
                                      |(CFG_PD12_OT<<12)\
																			|(CFG_PD11_OT<<11))

#define CFG_GPIO_TTLEN_D_INIT     	(((uint32_t)CFG_PD14_TTLEN<<14)\
																		|(CFG_PD13_TTLEN<<13)\
																		|(CFG_PD12_TTLEN<<12)\
																		|(CFG_PD11_TTLEN<<11)\
																		|(CFG_PD8_TTLEN<<8)\
																		|(CFG_PD7_TTLEN<<7)\
																		|(CFG_PD6_TTLEN<<6))																	

#define CFG_GPIO_IE_D_INIT     	(((uint32_t)CFG_PD15_IE<<15)\
																		|(CFG_PD14_IE<<14)\
																		|(CFG_PD13_IE<<13)\
																		|(CFG_PD12_IE<<12)\
																		|(CFG_PD11_IE<<11)\
																		|(CFG_PD10_IE<<10)\
																		|(CFG_PD9_IE<<9)\
																		|(CFG_PD8_IE<<8)\
																		|(CFG_PD7_IE<<7)\
																		|(CFG_PD6_IE<<6)\
																		|(CFG_PD5_IE<<5)\
																		|(CFG_PD4_IE<<4)\
																		|(CFG_PD3_IE<<3)\
																		|(CFG_PD2_IE<<2)\
																		|(CFG_PD1_IE<<1)\
																		|CFG_PD0_IE)
																		

#define CFG_GPIO_PUPDR_D_INIT     	(((uint32_t)CFG_PD15_PUPDR<<30)\
																		|(CFG_PD14_PUPDR<<28)\
																		|(CFG_PD13_PUPDR<<26)\
																		|(CFG_PD12_PUPDR<<24)\
																		|(CFG_PD11_PUPDR<<22)\
																		|(CFG_PD10_PUPDR<<20)\
																		|(CFG_PD9_PUPDR<<18)\
																		|(CFG_PD8_PUPDR<<16)\
																		|(CFG_PD7_PUPDR<<14)\
																		|(CFG_PD6_PUPDR<<12)\
																		|(CFG_PD5_PUPDR<<10)\
																		|(CFG_PD4_PUPDR<<8)\
																		|(CFG_PD3_PUPDR<<6)\
																		|(CFG_PD2_PUPDR<<4)\
																		|(CFG_PD1_PUPDR<<2)\
																		|CFG_PD0_PUPDR)
																		

#define CFG_GPIO_AFRH_D_INIT     		(((uint32_t)CFG_PD15_AFR<<28)\
																		|(CFG_PD14_AFR<<24)\
																		|(CFG_PD13_AFR<<20)\
																		|(CFG_PD12_AFR<<16)\
																		|(CFG_PD11_AFR<<12)\
																		|(CFG_PD10_AFR<<8)\
																		|(CFG_PD9_AFR<<4)\
																		|CFG_PD8_AFR)
															

#define CFG_GPIO_AFRL_D_INIT				(((uint32_t)CFG_PD7_AFR<<28)\
																		|(CFG_PD6_AFR<<24)\
																		|(CFG_PD5_AFR<<20)\
																		|(CFG_PD4_AFR<<16)\
																		|(CFG_PD3_AFR<<12)\
																		|(CFG_PD2_AFR<<8)\
																		|(CFG_PD1_AFR<<4)\
																		|CFG_PD0_AFR)																		
																		

#define CFG_GPIO_ODR_D_INIT         (((uint32_t)CFG_PD15_ODR<<15)\
																		|(CFG_PD14_ODR<<14)\
																		|(CFG_PD13_ODR<<13)\
																		|(CFG_PD12_ODR<<12)\
																		|(CFG_PD11_ODR<<11)\
																		|(CFG_PD10_ODR<<10)\
																		|(CFG_PD9_ODR<<9)\
																		|(CFG_PD8_ODR<<8)\
																		|(CFG_PD7_ODR<<7)\
																		|(CFG_PD6_ODR<<6)\
																		|(CFG_PD5_ODR<<5)\
																		|(CFG_PD4_ODR<<4)\
																		|(CFG_PD3_ODR<<3)\
																		|(CFG_PD2_ODR<<2)\
																		|(CFG_PD1_ODR<<1)\
																		|CFG_PD0_ODR)																		
																		

#define CFG_GPIO_MODER_D_INIT     	(((uint32_t)CFG_PD15_MODER<<15)\
																		|(CFG_PD14_MODER<<14)\
																		|(CFG_PD13_MODER<<13)\
																		|(CFG_PD12_MODER<<12)\
																		|(CFG_PD11_MODER<<11)\
																		|(CFG_PD10_MODER<<10)\
																		|(CFG_PD9_MODER<<9)\
																		|(CFG_PD8_MODER<<8)\
																		|(CFG_PD7_MODER<<7)\
																		|(CFG_PD6_MODER<<6)\
																		|(CFG_PD5_MODER<<5)\
																		|(CFG_PD4_MODER<<4)\
																		|(CFG_PD3_MODER<<3)\
																		|(CFG_PD2_MODER<<2)\
																		|(CFG_PD1_MODER<<1)\
																		|CFG_PD0_MODER)	


#define CFG_GPIO_LCKR_D_INIT     	  (((uint32_t)CFG_PD15_LOCK<<15)\
																		|(CFG_PD14_LOCK<<14)\
																		|(CFG_PD13_LOCK<<13)\
																		|(CFG_PD12_LOCK<<12)\
																		|(CFG_PD11_LOCK<<11)\
																		|(CFG_PD10_LOCK<<10)\
																		|(CFG_PD9_LOCK<<9)\
																		|(CFG_PD8_LOCK<<8)\
																		|(CFG_PD7_LOCK<<7)\
																		|(CFG_PD6_LOCK<<6)\
																		|(CFG_PD5_LOCK<<5)\
																		|(CFG_PD4_LOCK<<4)\
																		|(CFG_PD3_LOCK<<3)\
																		|(CFG_PD2_LOCK<<2)\
																		|(CFG_PD1_LOCK<<1)\
																		|CFG_PD0_LOCK)	
																																				


//<e>GPIOE  
#define     CFG_GPIOE_EN        0


// <h> Pin0  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PE0_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PE0_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PE0_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PE0_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PE0_PUPDR       


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PE0_LOCK        0
// </h>


// <h> Pin1  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PE1_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PE1_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PE1_ODR         0


// <q> IE_˿ʹλ
#define     CFG_PE1_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PE1_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PE1_LOCK        0
// </h>


// <h> Pin2  
// <o> AFR_˿ڸùѡλ
//    AFRѡ:
//    <0=>AF0_Closed
//    <1=>AF1_GPIO
//    <2=>AF2_MCM1
//    <3=>AF3_MCM0
//    <4=>AF4_PCA
//    <5=>AF5_PCA
//    <6=>AF6_TIM
//    <7=>AF7_UART
//    <8=>AF8_UART
//    <9=>AF9_QEI
//    <10=>AF10_TWI
//    <11=>AF11_TWI_CPOUT
//    <12=>AF12_SPI0
//    <13=>AF13_SPI1
//    <14=>AF14_CAN_Analog
//    <15=>AF15_Analog
#define     CFG_PE2_AFR         0


// <o> MODER_˿ģʽ
//    <0=>
//    <1=> 
#define     CFG_PE2_MODER       0


// <o> ODR_˿ƽ
//    <0=>͵ƽ
//    <1=>ߵƽ
#define     CFG_PE2_ODR         0

// <q> IE_˿ʹλ
#define     CFG_PE2_IE          1


// <o> PUPDR_˿
//    <0=>λ״̬
//    <1=>
#define     CFG_PE2_PUPDR       0


// <q> LCKy_˿λ
//    <0=>˿λδ
//    <1=>˿λ
#define     CFG_PE2_LOCK        0
// </h>
//</e>

															

#define CFG_GPIO_IE_E_INIT     	(((uint32_t)CFG_PE2_IE<<2)\
																		|(CFG_PE1_IE<<1)\
																		|CFG_PE0_IE)
																		

#define CFG_GPIO_PUPDR_E_INIT     	(((uint32_t)CFG_PE2_PUPDR<<4)\
																		|(CFG_PE1_PUPDR<<2)\
																		|CFG_PE0_PUPDR)
																		
													

#define CFG_GPIO_AFRL_E_INIT				(((uint32_t)CFG_PE2_AFR<<8)\
																		|(CFG_PE1_AFR<<4)\
																		|CFG_PE0_AFR)																		
																		

#define CFG_GPIO_ODR_E_INIT         (((uint32_t)CFG_PE2_ODR<<2)\
																		|(CFG_PE1_ODR<<1)\
																		|CFG_PE0_ODR)																		
																		

#define CFG_GPIO_MODER_E_INIT     	(((uint32_t)CFG_PE2_MODER<<2)\
																		|(CFG_PE1_MODER<<1)\
																		|CFG_PE0_MODER)	


#define CFG_GPIO_LCKR_E_INIT     	  (((uint32_t)CFG_PE2_LOCK<<2)\
																		|(CFG_PE1_LOCK<<1)\
																		|CFG_PE0_LOCK)	
																		

//</e>	

																		
//<e>EXTI
#define CFG_EXTI_EN     			0
//<e9>EXTI0
// <q0> IMR0_EXTI0ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR0_EXTI0ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR0_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR0_EXTI0ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR0_EXTI0ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI0_EXTI0 
//   These bits are written by software to select the source input for the EXTI0 external interrupt. 
//    <0=>PA[0] pin 
//    <1=>PB[0] pin
//    <2=>PC[0] pin
//    <3=>PD[0] pin
//    <4=>PE[0] pin
// <o6> PS0_EXTI0ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN0_EXTI0ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR0_EXTI0ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI0_IMR_BIT		0x0
#define CFG_EXTI0_EMR_BIT		0x0
#define CFG_EXTI0_TMR_BIT		0x0
#define CFG_EXTI0_RTR_BIT		0x0
#define CFG_EXTI0_FTR_BIT		0x0
#define CFG_EXTI0_PIN_CFG		0x0
#define CFG_EXTI0_PS_BIT		0x2
#define CFG_EXTI0_SN_BIT		0x2
#define CFG_EXTI0_DMR_BIT		0x0
#define CFG_EXTI0_EN      		0

//<e9>EXTI1
// <q0> IMR1_EXTI1ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR1_EXTI1ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR1_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR1_EXTI1ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR1_EXTI1ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI1_EXTI1 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[1] pin 
//    <1=>PB[1] pin
//    <2=>PC[1] pin
//    <3=>PD[1] pin
//    <4=>PE[1] pin
// <o6> PS1_EXTI1ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN1_EXTI1ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR1_EXTI1ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI1_IMR_BIT		0x0
#define CFG_EXTI1_EMR_BIT		0x0
#define CFG_EXTI1_TMR_BIT		0x0
#define CFG_EXTI1_RTR_BIT		0x0
#define CFG_EXTI1_FTR_BIT		0x0
#define CFG_EXTI1_PIN_CFG		0x0
#define CFG_EXTI1_PS_BIT		0x2
#define CFG_EXTI1_SN_BIT		0x2
#define CFG_EXTI1_DMR_BIT		0x0
#define CFG_EXTI1_EN      		0


//<e9>EXTI2
// <q0> IMR2_EXTI2ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR2_EXTI2ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR2_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//<i>شΪone-shotʽͨclear pending flag־λ
//<i>ƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR2_EXTI2ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR2_EXTI2ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI2_EXTI2 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[2] pin 
//    <1=>PB[2] pin
//    <2=>PC[2] pin
//    <3=>PD[2] pin
// <o6> PS2_EXTI2ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN2_EXTI2ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR2_EXTI2ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI2_IMR_BIT		0x0
#define CFG_EXTI2_EMR_BIT		0x0
#define CFG_EXTI2_TMR_BIT		0x0
#define CFG_EXTI2_RTR_BIT		0x0
#define CFG_EXTI2_FTR_BIT		0x0
#define CFG_EXTI2_PIN_CFG		0x0
#define CFG_EXTI2_PS_BIT		0x2
#define CFG_EXTI2_SN_BIT		0x2
#define CFG_EXTI2_DMR_BIT		0x0
#define CFG_EXTI2_EN      		0


//<e9>EXTI3
// <q0> IMR3_EXTI3ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR3_EXTI3ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR3_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//<i>شΪone-shotʽͨclear pending flag־λ
//<i>ƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR3_EXTI3ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR3_EXTI3ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI3_EXTI3 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[3] pin 
//    <1=>PB[3] pin
//    <2=>PC[3] pin
// <o6> PS3_EXTI3ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN3_EXTI3ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR3_EXTI3ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>


#define CFG_EXTI3_IMR_BIT		0x0
#define CFG_EXTI3_EMR_BIT		0x0
#define CFG_EXTI3_TMR_BIT		0x0
#define CFG_EXTI3_RTR_BIT		0x0
#define CFG_EXTI3_FTR_BIT		0x0
#define CFG_EXTI3_PIN_CFG		0x0
#define CFG_EXTI3_PS_BIT		0x2
#define CFG_EXTI3_SN_BIT		0x2
#define CFG_EXTI3_DMR_BIT		0x0
#define CFG_EXTI3_EN      		0

//<e9>EXTI4
// <q0> IMR4_EXTI4ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR4_EXTI4ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR4_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λ
//<i>ƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR4_EXTI4ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR4_EXTI4ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI4_EXTI4 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[4] pin 
//    <1=>PB[4] pin
//    <2=>PC[4] pin
// <o6> PS4_EXTI4ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN4_EXTI4ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR4_EXTI4ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI4_IMR_BIT		0x0
#define CFG_EXTI4_EMR_BIT		0x0
#define CFG_EXTI4_TMR_BIT		0x0
#define CFG_EXTI4_RTR_BIT		0x0
#define CFG_EXTI4_FTR_BIT		0x0
#define CFG_EXTI4_PIN_CFG		0x0
#define CFG_EXTI4_PS_BIT		0x2
#define CFG_EXTI4_SN_BIT		0x2
#define CFG_EXTI4_DMR_BIT		0x0
#define CFG_EXTI4_EN      		0

//<e9>EXTI5
// <q0> IMR5_EXTI5ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR5_EXTI5ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR5_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λ
//<i>ƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR5_EXTI5ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR5_EXTI5ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI5_EXTI5 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>
//    <1=>PB[5] pin
// <o6> PS5_EXTI5ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN5_EXTI5ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR5_EXTI5ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI5_IMR_BIT		0x0
#define CFG_EXTI5_EMR_BIT		0x0
#define CFG_EXTI5_TMR_BIT		0x0
#define CFG_EXTI5_RTR_BIT		0x0
#define CFG_EXTI5_FTR_BIT		0x0
#define CFG_EXTI5_PIN_CFG		0x0
#define CFG_EXTI5_PS_BIT		0x2
#define CFG_EXTI5_SN_BIT		0x2
#define CFG_EXTI5_DMR_BIT		0x0
#define CFG_EXTI5_EN      		0

//<e9>EXTI6
// <q0> IMR6_EXTI6ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR6_EXTI6ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR6_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR6_EXTI6ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR6_EXTI6ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI6_EXTI6 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>
//    <1=>PB[6] pin
//    <2=>PC[6] pin
// <o6> PS6_EXTI6ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN6_EXTI6ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR6_EXTI6ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI6_IMR_BIT		0x0
#define CFG_EXTI6_EMR_BIT		0x0
#define CFG_EXTI6_TMR_BIT		0x0
#define CFG_EXTI6_RTR_BIT		0x0
#define CFG_EXTI6_FTR_BIT		0x0
#define CFG_EXTI6_PIN_CFG		0x0
#define CFG_EXTI6_PS_BIT		0x2
#define CFG_EXTI6_SN_BIT		0x2
#define CFG_EXTI6_DMR_BIT		0x0
#define CFG_EXTI6_EN      		0

//<e9>EXTI7
// <q0> IMR7_EXTI7ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR7_EXTI7ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR7_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR7_EXTI7ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR7_EXTI7ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI7_EXTI7 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[7] pin 
//    <1=>PB[7] pin
//    <2=>PC[7] pin
// <o6> PS7_EXTI7ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN7_EXTI7ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR7_EXTI7ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI7_IMR_BIT		0x0
#define CFG_EXTI7_EMR_BIT		0x0
#define CFG_EXTI7_TMR_BIT		0x0
#define CFG_EXTI7_RTR_BIT		0x0
#define CFG_EXTI7_FTR_BIT		0x0
#define CFG_EXTI7_PIN_CFG		0x0
#define CFG_EXTI7_PS_BIT		0x2
#define CFG_EXTI7_SN_BIT		0x2
#define CFG_EXTI7_DMR_BIT		0x0
#define CFG_EXTI7_EN      		0

//<e9>EXTI8
// <q0> IMR8_EXTI8ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR8_EXTI8ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR8_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR8_EXTI8ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR8_EXTI8ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI8_EXTI8 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[8] pin 
//    <1=>PB[8] pin
//    <2=>PC[8] pin
// <o6> PS8_EXTI8ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN8_EXTI8ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR8_EXTI8ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI8_IMR_BIT		0x0
#define CFG_EXTI8_EMR_BIT		0x0
#define CFG_EXTI8_TMR_BIT		0x0
#define CFG_EXTI8_RTR_BIT		0x0
#define CFG_EXTI8_FTR_BIT		0x0
#define CFG_EXTI8_PIN_CFG		0x0
#define CFG_EXTI8_PS_BIT		0x2
#define CFG_EXTI8_SN_BIT		0x2
#define CFG_EXTI8_DMR_BIT		0x0
#define CFG_EXTI8_EN      		0

//<e9>EXTI9
// <q0> IMR9_EXTI9ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR9_EXTI9ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR9_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR9_EXTI9ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR9_EXTI9ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI9_EXTI9 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[9] pin 
// <o6> PS9_EXTI9ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN9_EXTI9ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR9_EXTI9ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI9_IMR_BIT		0x0
#define CFG_EXTI9_EMR_BIT		0x0
#define CFG_EXTI9_TMR_BIT		0x0
#define CFG_EXTI9_RTR_BIT		0x0
#define CFG_EXTI9_FTR_BIT		0x0
#define CFG_EXTI9_PIN_CFG		0x0
#define CFG_EXTI9_PS_BIT		0x2
#define CFG_EXTI9_SN_BIT		0x2
#define CFG_EXTI9_DMR_BIT		0x0
#define CFG_EXTI9_EN      		0

//<e9>EXTI10
// <q0> IMR10_EXTI10ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR10_EXTI10ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR10_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR10_EXTI10ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR10_EXTI10ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI10_EXTI10 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[10] pin 
//    <1=>PB[10] pin
//    <2=>PC[10] pin
// <o6> PS10_EXTI10ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN10_EXTI10ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR10_EXTI10ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI10_IMR_BIT		0x0
#define CFG_EXTI10_EMR_BIT		0x0
#define CFG_EXTI10_TMR_BIT		0x0
#define CFG_EXTI10_RTR_BIT		0x0
#define CFG_EXTI10_FTR_BIT		0x0
#define CFG_EXTI10_PIN_CFG		0x0
#define CFG_EXTI10_PS_BIT			0x2
#define CFG_EXTI10_SN_BIT			0x2
#define CFG_EXTI10_DMR_BIT		0x0
#define CFG_EXTI10_EN      		0

//<e9>EXTI11
// <q0> IMR11_EXTI11ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR11_EXTI11ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR11_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR11_EXTI11ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR11_EXTI11ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI11_EXTI11 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[11] pin 
//    <1=>PB[11] pin
//    <2=>PC[11] pin
//    <3=>PD[11] pin
// <o6> PS11_EXTI11ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN11_EXTI11ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR11_EXTI11ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI11_IMR_BIT		0x0
#define CFG_EXTI11_EMR_BIT		0x0
#define CFG_EXTI11_TMR_BIT		0x0
#define CFG_EXTI11_RTR_BIT		0x0
#define CFG_EXTI11_FTR_BIT		0x0
#define CFG_EXTI11_PIN_CFG		0x0
#define CFG_EXTI11_PS_BIT			0x2
#define CFG_EXTI11_SN_BIT			0x2
#define CFG_EXTI11_DMR_BIT		0x0
#define CFG_EXTI11_EN      		0

//<e9>EXTI12
// <q0> IMR12_EXTI12ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR12_EXTI12ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR12_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR12_EXTI12ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR12_EXTI12ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI12_EXTI12 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[12] pin 
//    <1=>PB[12] pin
//    <2=>PC[12] pin
//    <3=>PD[12] pin
// <o6> PS12_EXTI12ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN12_EXTI12ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR12_EXTI12ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI12_IMR_BIT		0x0
#define CFG_EXTI12_EMR_BIT		0x0
#define CFG_EXTI12_TMR_BIT		0x0
#define CFG_EXTI12_RTR_BIT		0x0
#define CFG_EXTI12_FTR_BIT		0x0
#define CFG_EXTI12_PIN_CFG		0x0
#define CFG_EXTI12_PS_BIT			0x2
#define CFG_EXTI12_SN_BIT			0x2
#define CFG_EXTI12_DMR_BIT		0x0
#define CFG_EXTI12_EN      		0


//<e9>EXTI13
// <q0> IMR13_EXTI13ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR13_EXTI13ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR13_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR13_EXTI13ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR13_EXTI13ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI13_EXTI13 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[13] pin 
//    <1=>PB[13] pin
//    <2=>PC[13] pin
//    <3=>PD[13] pin
// <o6> PS13_EXTI13ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN13_EXTI13ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR13_EXTI13ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI13_IMR_BIT		0x0
#define CFG_EXTI13_EMR_BIT		0x0
#define CFG_EXTI13_TMR_BIT		0x0
#define CFG_EXTI13_RTR_BIT		0x0
#define CFG_EXTI13_FTR_BIT		0x0
#define CFG_EXTI13_PIN_CFG		0x0
#define CFG_EXTI13_PS_BIT			0x2
#define CFG_EXTI13_SN_BIT			0x2
#define CFG_EXTI13_DMR_BIT		0x0
#define CFG_EXTI13_EN      		0

//<e9>EXTI14
// <q0> IMR14_EXTI14ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR14_EXTI14ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR14_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//    <i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
// <q3> RTR14_EXTI14ϵ/ߵƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR14_EXTI14ϵ½/͵ƽжϺ¼
//<i>0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI14_EXTI14 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[14] pin 
//    <1=>PB[14] pin
//    <2=>PC[14] pin
//    <3=>PD[14] pin
// <o6> PS14_EXTI14ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN14_EXTI14ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR14_EXTI14ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI14_IMR_BIT		0x0
#define CFG_EXTI14_EMR_BIT		0x0
#define CFG_EXTI14_TMR_BIT		0x0
#define CFG_EXTI14_RTR_BIT		0x0
#define CFG_EXTI14_FTR_BIT		0x0
#define CFG_EXTI14_PIN_CFG		0x0
#define CFG_EXTI14_PS_BIT			0x2
#define CFG_EXTI14_SN_BIT			0x2
#define CFG_EXTI14_DMR_BIT		0x0
#define CFG_EXTI14_EN      		0

//<e9>EXTI15
// <q0> IMR15_EXTI15ϵж
//<i>0: yϵж
//<i>1: yϵж
// <q1> EMR15_EXTI15ϵ¼
//<i>0: yϵ¼
//<i>1: yϵ¼
// <o2> TMR15_EXTIϵĴģʽѡλ
//     <0=>ش
//     <1=>ƽ
//<i>شΪone-shotʽͨclear pending flag־λƽΪlevelʽֻͨⲿƽ仯־λ޷ͨ
//<q3> RTR15_EXTI15ϵ/ߵƽжϺ¼
//<i>      0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <q4> FTR15_EXTI15ϵ½/͵ƽжϺ¼
//<i>      0: ֹyϵ½/͵ƽ(жϺ¼)
//<i>1: yϵ½/͵ƽ(жϺ¼)
// <o5> EXTI15_EXTI15 
//   These bits are written by software to select the source input for the EXTI1 external interrupt. 
//    <0=>PA[15] pin 
//    <1=>PB[15] pin
//    <2=>PC[15] pin
//    <3=>PD[15] pin
// <o6> PS15_EXTI15ʱԤƵѡλ
//    <0=>1/1
//    <1=>1/4
//    <2=>1/16
//    <3=>1/128
// <o7> SN15_EXTI15ѡλ
//    <0=>1
//    <1=>2
//    <2=>3
//    <3=>4 
// <q8> DMR15_EXTI15ϵDMA
//<i>0: yϵDMA
//<i>1: yϵDMA
//</e>
#define CFG_EXTI15_IMR_BIT		0x0
#define CFG_EXTI15_EMR_BIT		0x0
#define CFG_EXTI15_TMR_BIT		0x0
#define CFG_EXTI15_RTR_BIT		0x0
#define CFG_EXTI15_FTR_BIT		0x0
#define CFG_EXTI15_PIN_CFG		0x0
#define CFG_EXTI15_PS_BIT			0x2
#define CFG_EXTI15_SN_BIT			0x2
#define CFG_EXTI15_DMR_BIT		0x0
#define CFG_EXTI15_EN      		0


//</e>
#define CFG_EXTI_IMR_INIT     (((uint32_t)CFG_EXTI15_IMR_BIT<<15)\
															|(CFG_EXTI14_IMR_BIT<<14)\
															|(CFG_EXTI13_IMR_BIT<<13)\
															|(CFG_EXTI12_IMR_BIT<<12)\
															|(CFG_EXTI11_IMR_BIT<<11)\
															|(CFG_EXTI10_IMR_BIT<<10)\
															|(CFG_EXTI9_IMR_BIT<<9)\
															|(CFG_EXTI8_IMR_BIT<<8)\
															|(CFG_EXTI7_IMR_BIT<<7)\
															|(CFG_EXTI6_IMR_BIT<<6)\
															|(CFG_EXTI5_IMR_BIT<<5)\
															|(CFG_EXTI4_IMR_BIT<<4)\
															|(CFG_EXTI3_IMR_BIT<<3)\
															|(CFG_EXTI2_IMR_BIT<<2)\
															|(CFG_EXTI1_IMR_BIT<<1)\
															|CFG_EXTI0_IMR_BIT)

#define CFG_EXTI_EMR_INIT     (((uint32_t)CFG_EXTI15_EMR_BIT<<15)\
															|(CFG_EXTI14_EMR_BIT<<14)\
															|(CFG_EXTI13_EMR_BIT<<13)\
															|(CFG_EXTI12_EMR_BIT<<12)\
															|(CFG_EXTI11_EMR_BIT<<11)\
															|(CFG_EXTI10_EMR_BIT<<10)\
															|(CFG_EXTI9_EMR_BIT<<9)\
															|(CFG_EXTI8_EMR_BIT<<8)\
															|(CFG_EXTI7_EMR_BIT<<7)\
															|(CFG_EXTI6_EMR_BIT<<6)\
															|(CFG_EXTI5_EMR_BIT<<5)\
															|(CFG_EXTI4_EMR_BIT<<4)\
															|(CFG_EXTI3_EMR_BIT<<3)\
															|(CFG_EXTI2_EMR_BIT<<2)\
															|(CFG_EXTI1_EMR_BIT<<1)\
															|CFG_EXTI0_EMR_BIT)
															
#define CFG_EXTI_TMSR_INIT     (((uint32_t)CFG_EXTI15_TMR_BIT<<15)\
															|(CFG_EXTI14_TMR_BIT<<14)\
															|(CFG_EXTI13_TMR_BIT<<13)\
															|(CFG_EXTI12_TMR_BIT<<12)\
															|(CFG_EXTI11_TMR_BIT<<11)\
															|(CFG_EXTI10_TMR_BIT<<10)\
															|(CFG_EXTI9_TMR_BIT<<9)\
															|(CFG_EXTI8_TMR_BIT<<8)\
															|(CFG_EXTI7_TMR_BIT<<7)\
															|(CFG_EXTI6_TMR_BIT<<6)\
															|(CFG_EXTI5_TMR_BIT<<5)\
															|(CFG_EXTI4_TMR_BIT<<4)\
															|(CFG_EXTI3_TMR_BIT<<3)\
															|(CFG_EXTI2_TMR_BIT<<2)\
															|(CFG_EXTI1_TMR_BIT<<1)\
															|CFG_EXTI0_TMR_BIT)
															
#define CFG_EXTI_RTSR_INIT    (((uint32_t)CFG_EXTI15_RTR_BIT<<15)\
															|(CFG_EXTI14_RTR_BIT<<14)\
															|(CFG_EXTI13_RTR_BIT<<13)\
															|(CFG_EXTI12_RTR_BIT<<12)\
															|(CFG_EXTI11_RTR_BIT<<11)\
															|(CFG_EXTI10_RTR_BIT<<10)\
															|(CFG_EXTI9_RTR_BIT<<9)\
															|(CFG_EXTI8_RTR_BIT<<8)\
															|(CFG_EXTI7_RTR_BIT<<7)\
															|(CFG_EXTI6_RTR_BIT<<6)\
															|(CFG_EXTI5_RTR_BIT<<5)\
															|(CFG_EXTI4_RTR_BIT<<4)\
															|(CFG_EXTI3_RTR_BIT<<3)\
															|(CFG_EXTI2_RTR_BIT<<2)\
															|(CFG_EXTI1_RTR_BIT<<1)\
															|CFG_EXTI0_RTR_BIT)															

#define CFG_EXTI_FTSR_INIT    (((uint32_t)CFG_EXTI15_FTR_BIT<<15)\
															|(CFG_EXTI14_FTR_BIT<<14)\
															|(CFG_EXTI13_FTR_BIT<<13)\
															|(CFG_EXTI12_FTR_BIT<<12)\
															|(CFG_EXTI11_FTR_BIT<<11)\
															|(CFG_EXTI10_FTR_BIT<<10)\
															|(CFG_EXTI9_FTR_BIT<<9)\
															|(CFG_EXTI8_FTR_BIT<<8)\
															|(CFG_EXTI7_FTR_BIT<<7)\
															|(CFG_EXTI6_FTR_BIT<<6)\
															|(CFG_EXTI5_FTR_BIT<<5)\
															|(CFG_EXTI4_FTR_BIT<<4)\
															|(CFG_EXTI3_FTR_BIT<<3)\
															|(CFG_EXTI2_FTR_BIT<<2)\
															|(CFG_EXTI1_FTR_BIT<<1)\
															|CFG_EXTI0_FTR_BIT)		
															
#define CFG_EXTI_CFGL_INIT		(((uint32_t)CFG_EXTI7_PIN_CFG<<28)\
															|(CFG_EXTI6_PIN_CFG<<24)\
															|(CFG_EXTI5_PIN_CFG<<20)\
															|(CFG_EXTI4_PIN_CFG<<16)\
															|(CFG_EXTI3_PIN_CFG<<12)\
															|(CFG_EXTI2_PIN_CFG<<8)\
															|(CFG_EXTI1_PIN_CFG<<4)\
															|CFG_EXTI0_PIN_CFG)	
															
#define CFG_EXTI_CFGH_INIT		(((uint32_t)CFG_EXTI15_PIN_CFG<<28)\
															|(CFG_EXTI14_PIN_CFG<<24)\
															|(CFG_EXTI13_PIN_CFG<<20)\
															|(CFG_EXTI12_PIN_CFG<<16)\
															|(CFG_EXTI11_PIN_CFG<<12)\
															|(CFG_EXTI10_PIN_CFG<<8)\
															|(CFG_EXTI9_PIN_CFG<<4)\
															|CFG_EXTI8_PIN_CFG)												
															
															
#define CFG_EXTI_SAMPL_INIT   (((uint32_t)CFG_EXTI7_PS_BIT<<30)\
															|(CFG_EXTI7_SN_BIT<<28)\
															|(CFG_EXTI6_PS_BIT<<26)\
															|(CFG_EXTI6_SN_BIT<<24)\
															|(CFG_EXTI5_PS_BIT<<22)\
															|(CFG_EXTI5_SN_BIT<<20)\
															|(CFG_EXTI4_PS_BIT<<18)\
															|(CFG_EXTI4_SN_BIT<<16)\
															|(CFG_EXTI3_PS_BIT<<14)\
															|(CFG_EXTI3_SN_BIT<<12)\
															|(CFG_EXTI2_PS_BIT<<10)\
															|(CFG_EXTI2_SN_BIT<<8)\
															|(CFG_EXTI1_PS_BIT<<6)\
															|(CFG_EXTI1_SN_BIT<<4)\
															|(CFG_EXTI0_PS_BIT<<2)\
															|CFG_EXTI0_SN_BIT)	
															
#define CFG_EXTI_SAMPH_INIT   (((uint32_t)CFG_EXTI15_PS_BIT<<30)\
															|(CFG_EXTI15_SN_BIT<<28)\
															|(CFG_EXTI14_PS_BIT<<26)\
															|(CFG_EXTI14_SN_BIT<<24)\
															|(CFG_EXTI13_PS_BIT<<22)\
															|(CFG_EXTI13_SN_BIT<<20)\
															|(CFG_EXTI12_PS_BIT<<18)\
															|(CFG_EXTI12_SN_BIT<<16)\
															|(CFG_EXTI11_PS_BIT<<14)\
															|(CFG_EXTI11_SN_BIT<<12)\
															|(CFG_EXTI10_PS_BIT<<10)\
															|(CFG_EXTI10_SN_BIT<<8)\
															|(CFG_EXTI9_PS_BIT<<6)\
															|(CFG_EXTI9_SN_BIT<<4)\
															|(CFG_EXTI8_PS_BIT<<2)\
															|CFG_EXTI8_SN_BIT)															
															
															

#define CFG_EXTI_DMR_INIT     (((uint32_t)CFG_EXTI15_DMR_BIT<<15)\
															|(CFG_EXTI14_DMR_BIT<<14)\
															|(CFG_EXTI13_DMR_BIT<<13)\
															|(CFG_EXTI12_DMR_BIT<<12)\
															|(CFG_EXTI11_DMR_BIT<<11)\
															|(CFG_EXTI10_DMR_BIT<<10)\
															|(CFG_EXTI9_DMR_BIT<<9)\
															|(CFG_EXTI8_DMR_BIT<<8)\
															|(CFG_EXTI7_DMR_BIT<<7)\
															|(CFG_EXTI6_DMR_BIT<<6)\
															|(CFG_EXTI5_DMR_BIT<<5)\
															|(CFG_EXTI4_DMR_BIT<<4)\
															|(CFG_EXTI3_DMR_BIT<<3)\
															|(CFG_EXTI2_DMR_BIT<<2)\
															|(CFG_EXTI1_DMR_BIT<<1)\
															|CFG_EXTI0_DMR_BIT)

#define CFG_EXTI0_INT_EN CFG_EXTI0_EN
#define CFG_EXTI1_INT_EN CFG_EXTI1_EN
#define CFG_EXTI2_INT_EN CFG_EXTI2_EN
#define CFG_EXTI3_INT_EN CFG_EXTI3_EN
#define CFG_EXTI7_4_INT_EN (CFG_EXTI4_EN | CFG_EXTI5_EN | CFG_EXTI6_EN | CFG_EXTI7_EN )
#define CFG_EXTI15_8_INT_EN (CFG_EXTI15_EN | CFG_EXTI14_EN | CFG_EXTI13_EN | CFG_EXTI12_EN | CFG_EXTI11_EN| CFG_EXTI10_EN | CFG_EXTI9_EN | CFG_EXTI8_EN)



//<e>DMA
#define CFG_DMA_EN 0
//<h>CSR_DMA״̬Ĵ
//<i>0x008 b31     b30     b29     b28     b27     b26     b25     b24     b19~16    b15~8         b7~0          
//<i>CSR   RELOAD7 RELOAD6 RELOAD5 RELOAD4 RELOAD3 RELOAD2 RELOAD1 RELOAD0 BURSTIDLE DBUSYy(y=7~0) SWTRGy(y=7~0) 
// <o0.16..19> BURSTIDLE_DMAͻͷλ
//λú
//    <0=>ͷ1
//    <1=>ͷ2
//    <2=>ͷ3
//    <3=>ͷ4
//    <4=>ͷ5
//    <5=>ͷ6
//    <6=>ͷ7
//    <7=>ͷ8
//    <8=>ͷ9
//    <9=>ͷ10
//    <10=>ͷ12
//    <11=>ͷ16
//    <12=>ͷ20
//    <13=>ͷ24
//    <14=>ͷ28
//    <15=>ͷ32

// <q0.0> SWTRG0_DMAͨ0ʹ
// <q0.1> SWTRG1_DMAͨ1ʹ
// <q0.2> SWTRG2_DMAͨ2ʹ
// <q0.3> SWTRG3_DMAͨ3ʹ
// <q0.4> SWTRG4_DMAͨ4ʹ
// <q0.5> SWTRG5_DMAͨ5ʹ
// <q0.6> SWTRG6_DMAͨ6ʹ
// <q0.7> SWTRG7_DMAͨ7ʹ

//<i>DMAͨyλãӲ
//<i>   0ɶ0д0
//<i>   1λDMAӦԶ0

// <q0.24> RELOAD0_DMAͨ0ؿλ
//    0: رԶ
//    1: Զ
// <q0.25> RELOAD1_DMAͨ1ؿλ
//    0: رԶ
//    1: Զ
// <q0.26> RELOAD2_DMAͨ2ؿλ
//    0: رԶ
//    1: Զ
// <q0.27> RELOAD3_DMAͨ3ؿλ
//    0: رԶ
//    1: Զ
// <q0.28> RELOAD4_DMAͨ4ؿλ
//    0: رԶ
//    1: Զ
// <q0.29> RELOAD5_DMAͨ5ؿλ
//    0: رԶ
//    1: Զ
// <q0.30> RELOAD6_DMAͨ6ؿλ
//    0: رԶ
//    1: Զ
// <q0.31> RELOAD7_DMAͨ7ؿλ
//    0: رԶ
//    1: Զ
//</h>
#define CFG_DMA_CSR_INIT 0x0

//<e>DMA_CH0
#define CFG_DMA_CH0_EN 0
	
//<h>CCR0_DMAͨ0üĴ
//<i>0x010 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR0  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR0_INIT	0x000000

#if ((CFG_DMA_CCR0_INIT & 0xF) != 0)
	#define CFG_DMA_CH0_INT_EN 1
#else
	#define CFG_DMA_CH0_INT_EN 0
#endif

//<h>NPKT0_DMAͨ0Ĵ
//<i>0x014 b31~13 b12~0 
//<i>NPKT0 -      NPKT  
// <o0.0..10> NPKT_ͨ0ݴburst<0-8191>
//    ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT0_INIT	0x0

//<h>SAR0_DMAͨ0ԴַĴ
//<i>0x01C b31~0 
//<i>SAR0  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//<i>  ԴݼĴĻַ
//<i>  SSIZE=01(16λ)ʹSARλԶֵַ롣
//<i>  SSIZE=10(32λ)ʹSARλԶֵַ롣
//<i>  ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR0_INIT	0x00

//<h>DAR0_DMAͨ0ĿַĴ
//<i>0x020 b31~0 
//<i>DAR0  DAR   
// <o0.0..31> DAR_DMAĿַĴ
// <i>ĿݼĴĻַ
// <i>DSIZE=01(16λ)ʹDARλԶֵַ롣
// <i>DSIZE=10(32λ)ʹDARλԶֵַ롣
// <i>ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR0_INIT	0x0


//</e>
//<e>DMA_CH1
#define CFG_DMA_CH1_EN 0

//<h>CCR1_DMAͨ1üĴ
//<i>0x030 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR1  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR1_INIT		0x000000
#if ((CFG_DMA_CCR1_INIT & 0xF) != 0)
	#define CFG_DMA_CH1_INT_EN 1
#else
	#define CFG_DMA_CH1_INT_EN 0
#endif

//<h>NPKT1_DMAͨ1Ĵ
//<i>0x034 b31~13 b12~0 
//<i>NPKT1 -      NPKT 
// <o0.0..10> NPKT_ͨ1ݴburst<0-8191>
//    ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT1_INIT	0x000

//<h>SAR1_DMAͨ1ԴַĴ
//<i>0x03C b31~0 
//<i>SAR1  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//     ԴݼĴĻַ
//     SSIZE=01(16λ)ʹSARλԶֵַ롣
//     SSIZE=10(32λ)ʹSARλԶֵַ롣
//     ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR1_INIT	0x00000000

//<h>DAR1_DMAͨ1ĿַĴ
//<i>0x040 b31~0 
//<i>DAR1  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
//    ĿݼĴĻַ
//    DSIZE=01(16λ)ʹDARλԶֵַ롣
//    DSIZE=10(32λ)ʹDARλԶֵַ롣
//    ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR1_INIT	0x00000000


//</e>
//<e>DMA_CH2
#define CFG_DMA_CH2_EN 0
//<h>CCR2_DMAͨ2üĴ
//<i>0x050 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR2  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR2_INIT		0x0

#if ((CFG_DMA_CCR2_INIT & 0xF) != 0)
	#define CFG_DMA_CH2_INT_EN 1
#else
	#define CFG_DMA_CH2_INT_EN 0
#endif

//<h>NPKT2_DMAͨ2Ĵ
//<i>0x054 b31~13 b12~0 
//<i>NPKT2 -      NPKT  
// <o0.0..10> NPKT_ͨ2ݴburst<0-8191>
//    ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT2_INIT	0x0

//<h>SAR2_DMAͨ2ԴַĴ
//<i>0x05C b31~0 
//<i>SAR2  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//     ԴݼĴĻַ
//     SSIZE=01(16λ)ʹSARλԶֵַ롣
//     SSIZE=10(32λ)ʹSARλԶֵַ롣
//     ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR2_INIT	0x0

//<h>DAR2_DMAͨ2ĿַĴ
//<i>0x060 b31~0 
//<i>DAR2  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
//    ĿݼĴĻַ
//    DSIZE=01(16λ)ʹDARλԶֵַ롣
//    DSIZE=10(32λ)ʹDARλԶֵַ롣
//    ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR2_INIT	0x0

//</e>
//<e>DMA_CH3
#define CFG_DMA_CH3_EN 0

//<h>CCR3_DMAͨ3üĴ
//<i>0x070 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR3  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR3_INIT		0x0

#if ((CFG_DMA_CCR3_INIT & 0xF) != 0)
	#define CFG_DMA_CH3_INT_EN 1
#else
	#define CFG_DMA_CH3_INT_EN 0
#endif

//<h>NPKT3_DMAͨ3Ĵ
//<i>0x074 b31~13 b12~0 
//<i>NPKT3 -      NPKT   
// <o0.0..10> NPKT_ͨ3ݴburst<0-8191>
//    ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT3_INIT	0x0

//<h>SAR3_DMAͨ3ԴַĴ
//<i>0x07C b31~0 
//<i>SAR3  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//     ԴݼĴĻַ
//     SSIZE=01(16λ)ʹSARλԶֵַ롣
//     SSIZE=10(32λ)ʹSARλԶֵַ롣
//     ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR3_INIT	0x0

//<h>DAR3_DMAͨ3ĿַĴ
//<i>0x080 b31~0 
//<i>DAR3  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
//    ĿݼĴĻַ
//    DSIZE=01(16λ)ʹDARλԶֵַ롣
//    DSIZE=10(32λ)ʹDARλԶֵַ롣
//    ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR3_INIT	0x0
//</e>


//<e>DMA_CH4
#define CFG_DMA_CH4_EN 0

//<h>CCR4_DMAͨ4üĴ
//<i>0x090 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR4  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR4_INIT		0x0

#if ((CFG_DMA_CCR4_INIT & 0xF) != 0)
	#define CFG_DMA_CH4_INT_EN 1
#else
	#define CFG_DMA_CH4_INT_EN 0
#endif

//<h>NPKT4_DMAͨ4Ĵ
//<i>0x094 b31~13 b12~0 
//<i>NPKT4 -      NPKT 
// <o0.0..10> NPKT_ͨ4ݴburst<0-8191>
//<i>ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ 
//</h>
#define CFG_DMA_NPKT4_INIT	0x0

//<h>SAR4_DMAͨ4ԴַĴ
//<i>0x09C b31~0 
//<i>SAR4  SAR   
// <o0.0..31> SAR_DMAԴַĴ
// <i> ԴݼĴĻַ
// <i> SSIZE=01(16λ)ʹSARλԶֵַ롣
// <i> SSIZE=10(32λ)ʹSARλԶֵַ롣
// <i> ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR4_INIT	0x0

//<h>DAR4_DMAͨ4ĿַĴ
//<i>0x0A0 b31~0 
//<i>DAR4  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
// <i>   ĿݼĴĻַ
// <i>   DSIZE=01(16λ)ʹDARλԶֵַ롣
// <i>   DSIZE=10(32λ)ʹDARλԶֵַ롣
// <i>   ͨ(DMA_CCRxEN=1)ʱдüĴ 
//</h>
#define CFG_DMA_DAR4_INIT	0x0
//</e>


//<e>DMA_CH5
#define CFG_DMA_CH5_EN 0

//<h>CCR5_DMAͨ5üĴ
//<i>0x0B0 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR5  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR5_INIT	0x0


//<h>NPKT5_DMAͨ5Ĵ
//<i>0x0B4 b31~13 b12~0 
//<i>NPKT5 -      NPKT 
// <o0.0..10> NPKT_ͨ5ݴburst<0-8191>
//<i>ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ 
//</h>
#define CFG_DMA_NPKT5_INIT	0x0

//<h>SAR5_DMAͨ5ԴַĴ
//<i>0x0BC b31~0 
//<i>SAR5  SAR   
// <o0.0..31> SAR_DMAԴַĴ
// <i>ԴݼĴĻַ
// <i>SSIZE=01(16λ)ʹSARλԶֵַ롣
// <i>SSIZE=10(32λ)ʹSARλԶֵַ롣
// <i>ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR5_INIT	0x0

//<h>DAR5_DMAͨ5ĿַĴ
//<i>0x0C0 b31~0 
//<i>DAR5  DAR   
// <o0.0..31> DAR_DMAĿַĴ
//  <i>ĿݼĴĻַ
//  <i>DSIZE=01(16λ)ʹDARλԶֵַ롣
//  <i>DSIZE=10(32λ)ʹDARλԶֵַ롣
//  <i>ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR5_INIT	0x0
//</e>

//<e>DMA_CH6
#define CFG_DMA_CH6_EN 0
//<h>CCR6_DMAͨ6üĴ
//<i>0x0D0 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR6  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR6_INIT		0x0


//<h>NPKT6_DMAͨ6Ĵ
//<i>0x0D4 b31~13 b12~0 
//<i>NPKT6 -      NPKT 
// <o0.0..10> NPKT_ͨ6ݴburst<0-8191>
//    ݴΪNPKT÷Χ08191ʾburst18192Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT6_INIT	0x0

//<h>SAR6_DMAͨ6ԴַĴ
//<i>0x0DC b31~0 
//<i>SAR6  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//     ԴݼĴĻַ
//     SSIZE=01(16λ)ʹSARλԶֵַ롣
//     SSIZE=10(32λ)ʹSARλԶֵַ롣
//     ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR6_INIT	0x0

//<h>DAR6_DMAͨ6ĿַĴ
//<i>0x0E0 b31~0 
//<i>DAR6  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
//    ĿݼĴĻַ
//    DSIZE=01(16λ)ʹDARλԶֵַ롣
//    DSIZE=10(32λ)ʹDARλԶֵַ롣
//    ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR6_INIT	0x0

//</e>
//<e>DMA_CH7
#define CFG_DMA_CH7_EN 0
//<h>CCR7_DMAͨ7üĴ
//<i>0x0F0 b31~24 b23     b22~20  b19~16   b15~14 b13~12 b11~10 b9~8  b7~6  b5 b4   b3   b2   b1   b0 
//<i>CCR7  -      TRGMODE STRMSEL BURSTLEN PL     SSIZE  DSIZE  SPTYP DPTYP -  TEIE BEIE HTIE TCIE EN 
// <q0.1> TCIE_ж
//    0: ֹTCж
//    1: TCж
// <q0.2> HTIE_봫ж 
//    0: ֹHTж
//    1: HTж
// <q0.3> BEIE_burstж
//    0: ֹBEж 
//    1: BEж
//    <i>burstΪ1ʱsingleģʽͬBEжϡ
// <q0.4> TEIE_ж
//    0: ֹTEж
//    1: TEж
// <o0.6..7> DPTYP_Ŀָ޸ķʽ
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.8..9> SPTYP_Դָ޸ķʽ
//λú
//    <0=>
//    <1=>ݼ
//    <2=>̶
//    <3=>ѭѭ
// <o0.10..11> DSIZE_Ŀݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ 
//    <3=>
// <o0.12..13> SSIZE_Դݿ
//Щλú
//    <0=>8λ
//    <1=>16λ 
//    <2=>32λ  
//    <3=>
// <o0.14..15> PL_ͨȼ
//Щλú
//    <0=>
//    <1=>
//    <2=> 
//    <3=>
// <o0.16..19> BURSTLEN_DMAͻλ
//λú
//    <0=>ÿburst1ݣsingle䣩
//    <1=>ÿburst2
//    <2=>ÿburst3
//    <3=>ÿburst4
//    <4=>ÿburst5
//    <5=>ÿburst6
//    <6=>ÿburst7
//    <7=>ÿburst8
//    <8=>ÿburst9
//    <9=>ÿburst10
//    <10=>ÿburst11
//    <11=>ÿburst12
//    <12=>ÿburst13
//    <13=>ÿburst14
//    <14=>ÿburst15
//    <15=>ÿburst16
//    <i>SIZEõĲͬ1ݴܰ1ֽ/2ֽ/4ֽڡ
// <o0.20..22> STRMSEL_DMAͨԴѡλ
//<i>Channel		CH0	CH1	CH2	CH3	CH4	CH5	CH6	CH7
//<i>stream0_UART	UART0_TX	UART0_RX	UART1_TX	UART1_RX	UART2_TX	UART2_RX	UART3_TX	UART3_RX
//<i>stream1_SPI	SDFM_DR1 SDFM_DR2	SDFM_DR3	SDFM_DR4	SPI0_TX	SPI0_RX	SPI1_TX	SPI1_RX
//<i>stream2_EXTI	EXTI0 EXTI1	EXTI2		EXTI3		EXTI4		EXTI5		EXTI6		EXTI17
//<i>stream3_EXTI	EXTI8 EXTI9	EXTI10	EXTI11	EXTI12	EXTI13	EXTI14	EXTI15
//<i>stream4_CMP	CMP0  CMP1	CMP2		CMP0		CMP1		CMP2		TIM0		TIM1
//<i>stream5_ADC	ADC0	ADC1	ADC2		TIM0  	TIM1  	ADC0		ADC1		ADC2
//<i>stream6_PCA	PCA0	PCA1  PCA2		PCA3  	PCA0		PCA1  	PCA2		PCA3
//<i>stream7_MCM	MCM0	MCM1	MCM0		MCM1		MCM0		MCM1		QEI0		QEI1
//    <0=>ѡstream0
//    <1=>ѡstream1
//    <2=>ѡstream2
//    <3=>ѡstream3
//    <4=>ѡstream4
//    <5=>ѡstream5
//    <6=>ѡstream6
//    <7=>ѡstream7
// <o0.23> TRGMODE_DMAģʽѡλ
//λú
//    <0=>one-shotʽ
//    <1=>to-endʽ
//</h>
#define CFG_DMA_CCR7_INIT		0x0


//<h>NPKT7_DMAͨ7Ĵ
//<i>0x0F4 b31~13 b12~0 
//<i>NPKT7 -      NPKT   
// <o0.0..10> NPKT_ͨ7ݴburst<0-511>
//    ݴΪNPKT÷Χ0511ʾburst1512Ĵֻͨ(DMA_CCRxEN=0)ʱд롣ͨüĴΪֻ
//</h>
#define CFG_DMA_NPKT7_INIT	0x0

//<h>SAR7_DMAͨ7ԴַĴ
//<i>0x0FC b31~0 
//<i>SAR7  SAR   
// <o0.0..31> SAR_DMAԴַĴ
//     ԴݼĴĻַ
//     SSIZE=01(16λ)ʹSARλԶֵַ롣
//     SSIZE=10(32λ)ʹSARλԶֵַ롣
//     ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_SAR7_INIT	0x0

//<h>DAR7_DMAͨ7ĿַĴ
//<i>0x100 b31~0 
//<i>DAR7  DAR   
// <o0.0..31> DAR_DMAĿַĴ 
//    ĿݼĴĻַ
//    DSIZE=01(16λ)ʹDARλԶֵַ롣
//    DSIZE=10(32λ)ʹDARλԶֵַ롣
//    ͨ(DMA_CCRxEN=1)ʱдüĴ
//</h>
#define CFG_DMA_DAR7_INIT	0x0
//</e>

#if (((CFG_DMA_CCR5_INIT & 0xF) != 0) || ((CFG_DMA_CCR6_INIT & 0xF) != 0) || ((CFG_DMA_CCR7_INIT & 0xF) != 0)) 
	#define CFG_DMA_CH5_7_INT_EN 1
#else
	#define CFG_DMA_CH5_7_INT_EN 0
#endif


//***********************************************************/
//</e>


//<e>MCM
#define CFG_MCM_EN	0
//***********************************************************/
//<e>MCM0
#define     CFG_MCM0_PWMOE       0
#define  CFG_MCM0_EN CFG_MCM0_PWMOE
//<i>0x00C    b31~3 b2~0  
//<i>PWMREMAP -     PWMIO 
// <o0.0..2> PWMIO_PWMIOڵӳ
//<0=>PWM
//<1=>PWM21 PWM11 PWM01 PWM2 PWM1 PWM0
//<2=>PWM0 PWM1 PWM2 PWM01 PWM11 PWM21 
//<3=>PWM21 PWM2 PWM11 PWM1 PWM01 PWM0
//<4=>PWM0 PWM01 PWM1 PWM11 PWM2 PWM21
#define CFG_MCM0_PWMREMAP_INIT	0x0

// <o> PWMPSQ_PWMƵϵ<0-65535>
//<i>PWMʱΪ(HCLK/(PSQ[15:0]+1)) 
#define     CFG_MCM0_PWMPSQ_INIT 0x0


// <o> PWMP_PWMƵ (0-100000Hz) <0-100000> 
#define     CFG_MCM0_PWMP_F      12000




//<o0>PWMD0:  PWM0ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o1>PWMD1:  PWM1ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o2>PWMD2:  PWM2ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o3>PWMD01: PWM01ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o4>PWMD11: PWM11ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o5>PWMD21: PWM21ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
#define     CFG_MCM0_PWM0_DUTY   500
#define     CFG_MCM0_PWM1_DUTY   500
#define     CFG_MCM0_PWM2_DUTY   500
#define     CFG_MCM0_PWM01_DUTY  500
#define     CFG_MCM0_PWM11_DUTY  500
#define     CFG_MCM0_PWM21_DUTY  500




// <o0.4..6> SECTOR_SVPWM趨λ
//<i>MAXMUMMIN
//    <0=>PWM2D > PWM1D > PWM0D
//    <1=>PWM2D > PWM0D > PWM1D
//    <2=>PWM0D > PWM2D > PWM1D
//    <3=>PWM0D > PWM1D > PWM2D
//    <4=>PWM1D > PWM0D > PWM2D
//    <5=>PWM1D > PWM2D > PWM0D
//    <i>趨110111ʱЧ
//    <i>SVPWM趨λͨλȷPWM0DPWM1DPWM2DĴС˳ֵʱĴС˳
#define     CFG_MCM0_PSCON_L_INIT 0x0


// <q0.7> ʹֹͣPWM
//    <i>0: ֹ
//    <i>1: ʹ
#define     CFG_MCM0_POSTDCR_INIT 0x0


//<h>PWM
// <o0.15> POUTMOD_PWM0/1/2ģʽ/λ
//    <0=>3ͨ
//    <1=>6·
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
// <o0.12..14> POSTPS_Ƶϵ
//    <0=>޺Ƶ
//    <1=>жϡź¼źŽ2Ƶ
//    <2=>жϡź¼źŽ3Ƶ
//    <3=>жϡź¼źŽ4Ƶ
//    <4=>жϡź¼źŽ5Ƶ
//    <5=>жϡź¼źŽ6Ƶ
//    <6=>жϡź¼źŽ7Ƶ
//    <7=>жϡź¼źŽ8Ƶ
// <o0.10..11> PTMOD_PWM0/1/2ģʽ
//    <0=>ضģʽ   
//    <1=>Ķģʽ
//    <2=>μģʽ
//<i>ضģʽģʽ£ʱֻᷢƥ
//<i>Ķģʽģʽ£ʱᷢƥƥ
//<i>μģʽģʽ£ʱֻᷢƥ
// <o0.9> PWMSYM_PWMζԳԿ
//    <0=>ԳPWM
//    <1=>ǶԳPWM
//<i>ĶģʽĻʱЧ
// <o0.6> PDCON0_PWM0/PWM01ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.7> PDCON1_PWM1/PWM11ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.8> PDCON2_PWM2/PWM21ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.0> PWM0S_PWM0Чƽѡλ
//    <0=>PWM0Dutyߵƽʱ͵ƽ
//    <1=>PWM0Duty͵ƽʱߵƽ
// <o0.1> PWM1S_PWM1Чƽѡλ
//    <0=>PWM1Dutyߵƽʱ͵ƽ
//    <1=>PWM1Duty͵ƽʱߵƽ
// <o0.2> PWM2S_PWM2Чƽѡλ
//    <0=>PWM2Dutyߵƽʱ͵ƽ
//    <1=>PWM2Duty͵ƽʱߵƽ
// <o0.3> PWM01S_PWM01Чƽѡλ
//    <0=>PWM01Duty͵ƽʱߵƽ
//    <1=>PWM01Dutyߵƽʱ͵ƽ
// <o0.4> PWM11S_PWM11Чƽѡλ
//    <0=>PWM11Duty͵ƽʱߵƽ
//    <1=>PWM11Dutyߵƽʱ͵ƽ
// <o0.5> PWM21S_PWM21Чƽѡλ
//    <0=>PWM21Duty͵ƽʱߵƽ
//    <1=>PWM21Dutyߵƽʱ͵ƽ
#define     CFG_MCM0_PWMCON1_INIT 0x0
//</h>


//<h>PWM
// <o> PWMDT00_PWMͨ0(0.1us) <0-100>
#define     CFG_MCM0_PWMDT00_DT  10


// <o> PWMDT01_PWMͨ0½(0.1us) <0-100>
#define     CFG_MCM0_PWMDT01_DT  10


// <o> PWMDT10_PWMͨ1(0.1us) <0-100>
#define     CFG_MCM0_PWMDT10_DT  10


// <o> PWMDT11_PWMͨ1½(0.1us) <0-100>
#define     CFG_MCM0_PWMDT11_DT  10


// <o> PWMDT20_PWMͨ2(0.1us) <0-100>
#define     CFG_MCM0_PWMDT20_DT  10


// <o> PWMDT21_PWMͨ2½(0.1us) <0-100>
#define     CFG_MCM0_PWMDT21_DT  10
//</h>


//<h>ȽϼĴAD
// <o0.0..1> CMP1_ȽϼĴ1ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ1ʱᴥAD
//    <2=>ֻPWMȽϼĴ1ʱᴥAD
//PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ1ʱᴥAD
//PTMOD = 01ĶģʽЧ    
//<i>ADCADONPWMTRGENλ1
// <o1> PWMCMP1_¼Ƚֵ1ٷֱ(0.1%) <0-1000>
// <o0.2..3> CMP2_ȽϼĴ2ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ2ʱᴥAD
//    <2=>ֻPWMȽϼĴ2ʱᴥAD
// PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ2ʱᴥAD
//PTMOD = 01    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o2> PWMCMP2_¼Ƚֵ2ٷֱ(0.1%) <0-1000>
// <o0.4..5> CMP3_ȽϼĴ3ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ3ʱᴥAD
//    <2=>ֻPWMȽϼĴ3ʱᴥAD
//PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ3ʱᴥAD
//PTMOD = 01ĶģʽЧ    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o3> PWMCMP3_¼Ƚֵ3ٷֱ(0.1%) <0-1000>
// <o0.6..7> CMP4_ȽϼĴ4ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ4ʱᴥAD
//    <2=>ֻPWMȽϼĴ4ʱᴥAD
// PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ4ʱᴥAD
//PTMOD = 01    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o4> PWMCMP4_¼Ƚֵ4ٷֱ(0.1%) <0-1000>
//</h>
//<h>ͬ
// <o0.8> OSYNC_ֶ޸PWMx/PWMx1ͬλ
//    <0=>ֶ޸ĺЧ
//    <1=>ֶ޸PWMͬЧ
//<i>ֶ޸ͨPMANUALCON1/2Ĵʵ,һĴ޸,ʱЧɴ˼Ĵ
// <o0.9> DILDEN_PWMռձȼĴЧʹλ
//    <0=>PWMͬЧ
//    <1=>Ч
// <o0.10> CILDEN_PWM¼ȽϼĴЧʹλ
//    <0=>PWMͬЧ
//    <1=>Ч
// <q0.11> ZDLDEN_PWMƥʱռձȼĴĴ
//    <i> ֵܴǷ趨0ƥʱԶڼĴƵԤƵĴκμĴ
// <q0.12> PDLDEN_PWMƥʱռձȼĴĴ
//    <i>ۼĴãƥʱPWMڼĴƵԤƵĴ,ֻĶģʽЧ
// <q0.13> ZCMLDEN_PWMƥʱ¼Ĵ
//<i>PWMCMPxx=1,2,3,4,ƥʱ¼ȽϼĴADʽĴ(CMP1~4[1-0])ʹλ      
// <q0.14> PCMLDEN_PWMƥʱ¼Ĵ
//<i>ƥʱ¼ȽϼĴADʽĴ(CMP1~4[1-0])ʹλֻĶģʽЧ
// <q0.15> MCMSYNEN_MCM0MCM1ͬʹλ
//    <i>0: رMCM0MCM1ͬ
//    <i>1: MCM0MCM1ͬ
//    <i>Ϊ˷ֹֻMCM0.PWMOE[MCMSYNEN]MCM1.PWMOE[MCMSYNEN]ͬʱ1ܿͬ,ֻĶģʽЧ
//</h>
#define     CFG_MCM0_PWMCON2_INIT 0x0
#define     CFG_MCM0_PWMCMP1_VALUE 100
#define     CFG_MCM0_PWMCMP2_VALUE 200
#define     CFG_MCM0_PWMCMP3_VALUE 999
#define     CFG_MCM0_PWMCMP4_VALUE 0
	

//<h>PWM0/1/2ֶ
//<i>PMANUALCON1_PWM0/1/2ֶüĴ1
//<i>0x058       b31~6 b5        b4        b3        b2       b1       b0       
//<i>PMANUALCON1 -     PMANUAL21 PMANUAL11 PMANUAL01 PMANUAL2 PMANUAL1 PMANUAL0 
//<i>PMANUALCON2_PWM0/1/2ֶüĴ2
//<i>0x05C       b31~6 b5     b4     b3     b2    b1    b0    
//<i>PMANUALCON2 -     POUT21 POUT11 POUT01 POUT2 POUT1 POUT0 
// <o0.0> PMANUAL0_PWM0λ
//    <0=>PWM0PWM
//    <1=>PWM0PMANUALCON2ĴеPOUT0λ
// <o0.1> PMANUAL1_PWM1λ
//    <0=>PWM1PWM
//    <1=>PWM1PMANUALCON2ĴеPOUT1λ
// <o0.2> PMANUAL2_PWM2λ
//    <0=>PWM2PWM
//    <1=>PWM2PMANUALCON2ĴеPOUT2λ
// <o0.3> PMANUAL01_PWM01λ
//    <0=>PWM01PWM
//    <1=>PWM01PMANUALCON2ĴеPOUT01λ
// <o0.4> PMANUAL11_PWM11λ
//    <0=>PWM11PWM
//    <1=>PWM11PMANUALCON2ĴеPOUT11λ
// <o0.5> PMANUAL21_PWM21λ
//    <0=>PWM21PWM
//    <1=>PWM21PMANUALCON2ĴеPOUT21λ
#define     CFG_MCM0_PMANUALCON1_INIT 0x0


//<h>PMANUALCON2_PWM0/1/2ֶüĴ2
//<i>0x05C       b31~6 b5     b4     b3     b2    b1    b0    
//<i>PMANUALCON2 -     POUT21 POUT11 POUT01 POUT2 POUT1 POUT0 
// <o0.0> POUT0_PMANUAL0 = 1ʱPWM0ڵƽ
//    <0=>0
//    <1=>1
// <o0.1> POUT1_PMANUAL1 = 1ʱPWM1ڵƽ
//    <0=>0
//    <1=>1
// <o0.2> POUT2_PMANUAL2 = 1ʱPWM2ڵƽ
//    <0=>0
//    <1=>1
// <o0.3> POUT01_PMANUAL01 = 1ʱPWM01ڵƽ
//    <0=>0
//    <1=>1
// <o0.4> POUT11_PMANUAL11 = 1ʱPWM11ڵƽ
//    <0=>0
//    <1=>1
// <o0.5> POUT21_PMANUAL21 = 1ʱPWM21ڵƽ
//    <0=>0
//    <1=>1
#define     CFG_MCM0_PMANUALCON2_INIT 0x0
//</h>
//</h>


//<h>ϼ
//<i>0x064  b31 b30 	b29 			b28 		 b27 		  b26~25 b24~23 b22       b21       b20       b19    b18     b17     b16     b15~14 b13~12 b11~10 b9    b8       b7~5    b4     b3    b2    b1       b0       
//<i>FLTCON - SDFMFLT4 SDFMFLT3 SDFMFLT2 SDFMFLT1 FOUT1  FOUT0  FLT0STATC FLT1STATC FLT2STATC FLT1EN ADC2FLT ADC1FLT ADC0FLT CMP2S  CMP1S  CMP0S  FLT1M FLT1STAT FLT2DEB FLT2EN FLT2S FLT2M FLT2STAT FLT0STAT 
// <q0.30> SDFMFLT4 SDFMͨ4ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.29> SDFMFLT3 SDFMͨ3ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.28> SDFMFLT2 SDFMͨ2ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.27> SDFMFLT1 SDFMͨ1ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <o0.23..24> FOUT0_ϷPWMxţţ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ
// <o0.25..26> FOUT1_ϷPWMx1ţţ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ
// <e0.19> FLT1EN_FLT1MCMλ
//    <i>0:ֹܽ
//    <i>1:ܴ򿪣ԴѡȽ0/1/2ADC0/1/2
// <o0.9> FLT1M_⹦ģʽѡ
//<i>ģʽ⵽ЧFLTSTAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLTSTAT޷
//<i>ģʽPWMֱɹϼƣЧFLTSTATӲ1رPWMΪЧFLTSTATԶ0PWM0/1/2νPWMʱ0ʱԶָ
//    <0=>ģʽ
//    <1=>ģʽ
// <o0.10..11> CMP0S_Ƚ0ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> CMP1S_Ƚ1ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> CMP2S_Ƚ2ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <q0.16> ADC0FLT_ADC0ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.17> ADC1FLT_ADC1ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.18> ADC2FLT_ADC2ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.30> SDFMFLT4 SDFMͨ4ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.29> SDFMFLT3 SDFMͨ3ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.28> SDFMFLT2 SDFMͨ2ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.27> SDFMFLT1 SDFMͨ1ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
//</e>

// <e0.4> FLT2EN_ϼ2ܿλ
//    <i>0:ϼ2ֹܽ
//    <i>1:ϼ2ܴ򿪣ԴΪFLT
// <o0.2> FLT2M_⹦ģʽѡ
//<i>ģʽ⵽ЧFLTSTAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLTSTAT޷
//<i>ģʽPWMֱɹϼƣЧFLTSTATӲ1رPWMΪЧFLTSTATԶ0PWM0/1/2νPWMʱ0ʱԶָ
//    <0=>ģʽ
//    <1=>ģʽ
// <o0.3> FLT2S_ϼ2Чƽѡλ
//    <0=>ϼ2 FLTߵƽЧ
//    <1=>ϼ2 FLT͵ƽЧ

// <o0.5..7> FLT2DEB_ϼ1˲ѡ
//  <0=>˲
//  <1=>0.5us
//  <2=>1us
//  <3=>2us
//  <4=>3us
//  <5=>4us
//  <6=>5us
//  <7=>6us
//  <i>˲ʱ䲻Ǿȷֵ
//  <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define CFG_MCM0_FLTCON_INIT	0x0
//</h>


//<e0.7>PWMʹ
//<i>0x064 b31~8 b7    b6 b5     b4     b3     b2    b1    b0    
//<i>POSCR -     OLSEN -  OLSG21 OLSG11 OLSG01 OLSG2 OLSG1 OLSG0 
// <o0.0> OLSG0_·PWM0Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.1> OLSG1_·PWM1Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.2> OLSG2_·PWM2Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.3> OLSG01_·PWM01Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.4> OLSG11_·PWM11Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.5> OLSG21_·PWM21Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
//</e>
#define     CFG_MCM0_POSCR_INIT 0x0

//<h>PWMDMA
//<i>0x074    b31~8 b7     b6     b5      b4      b3      b2      b1      b0      
//<i>PWMDMAEN -     PWMPDE PWMZDE PTDD2DE PTUD2DE PTDD1DE PTUD1DE PTDD0DE PTUD0DE 
// <q0.0> PTUD0DE_PWMʱʱռձȼĴPWM0DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM0DƥʱDMA
//    <i>1: PWMʱPWM0DƥʱDMA
// <q0.1> PTDD0DE_PWMʱʱռձȼĴPWM0D/PWM01DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM0D/PWM01DƥʱDMA
//    <i>1: PWMʱPWM0D/PWM01DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM0DĶǶԳģʽ£ΪPWM01D
// <q0.2> PTUD1DE_PWMʱʱռձȼĴPWM1DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM1DƥʱDMA
//    <i>1: PWMʱPWM1DƥʱDMA
// <q0.3> PTDD1DE_PWMʱʱռձȼĴPWM1D/PWM11DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM1D/PWM11DƥʱDMA
//    <i>1: PWMʱPWM1D/PWM11DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM1DĶǶԳģʽ£ΪPWM11D
// <q0.4> PTUD2DE_PWMʱʱռձȼĴPWM2DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM2DƥʱDMA
//    <i>1: PWMʱPWM2DƥʱDMA
// <q0.5> PTDD2DE_PWMʱʱռձȼĴPWM2D/PWM21DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM2D/PWM21DƥʱDMA
//    <i>1: PWMʱPWM2D/PWM21DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM2DĶǶԳģʽ£ΪPWM21D
// <q0.6> PWMZDE_PWMʱDMAʹλ
//    <i>0: ֹPWMʱDMA
//    <i>1: PWMʱDMA
// <q0.7> PWMPDE_PWMʱƥDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱƥDMA
//    <i>1: PWMʱƥDMA
#define CFG_MCM1_PWMDMAEN_INIT	0x0
//</h>


//<h>PWMж
//<i>0x074    b31~12 b11    b10 b9     b8     b7     b6     b5      b4      b3      b2      b1      b0      
//<i>PWMINTEN -      OSTDIE OIE FLT2IE FLT1IE PWMPIE PWMZIE PTDD2IE PTUD2IE PTDD1IE PTUD1IE PTDD0IE PTUD0IE
// <q0.0> PTUD0IE_PWMʱPWM0Dƥʱж
//    <i>0: ֹPWMʱPWM0Dƥʱж
//    <i>1: PWMʱPWM0Dƥʱж
// <q0.1> PTDD0IE_PWMʱPWM0Dƥʱж
//<i>ֻĶģʽЧ


// <q0.2> PTUD1IE_PWMʱPWM1Dƥʱж


// <q0.3> PTDD1IE_PWMʱPWM1Dƥʱж
//<i>ֻĶģʽЧ


// <q0.4> PTUD2IE_PWMʱPWM2Dƥʱж


// <q0.5> PTDD2IE_PWMʱPWM2Dƥʱж
//<i>ֻĶģʽЧ


// <q0.6> PWMZIE_PWMʱ㴥ж


// <q0.7> PWMPIE_PWMʱƥ䴥ж
//<i>ֻĶģʽЧ
// <q0.8> FLT1IE_1жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <q0.9> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <q0.10> OIE_·жλ
//    <i>0: ֹж
//    <i>1: ж
// <q0.11> OSTDIE_ֹͣPWMжʹλ
//    <i>0: ֹж
//    <i>1: ж
//</h>
#define     CFG_MCM0_PWMINTEN_INIT 0x0


#if ((CFG_MCM0_PWMINTEN_INIT & 0xFF) != 0)
	#define CFG_MCM0_PWM_INT_EN 1
#else
	#define CFG_MCM0_PWM_INT_EN 0
#endif


#if ((CFG_MCM0_PWMINTEN_INIT & 0x700) != 0)
	#define CFG_MCM0_FLT_INT_EN 1
#else
	#define CFG_MCM0_FLT_INT_EN 0
#endif


//PWMRLDEN
//0x078    b31~8 b7~0     
//PWMRLDEN -     PWMRLDEN 
//PWMRLDEN_Ĵ޸ĺؿƼĴ
//    0x55:ģĴ޸
//    0xAA:ģļĴ
//    PWMģļĴжϱ־ĴPWMINTFֶƼĴPMANUALCON2FLTCONPOSCRPOSTDCR⣬Ĵ޸ֻеPWMRLDEN = 0x55ʱ޸ģ޸Ч
//    PWMģһЩĴлĴռձȼĴPWMxD&PWMx1DڼĴPWMPPWMʱԤƵĴPWMPSQƵĴPOSTPS[2:0]@ PWMCON1DutyѡĴPDCONx@PWMCON1ԳԿƼĴPWMSYM@PWMCON1ĴPWMDTxxȽϼĴADλCMPx@PWMCON2PWM¼ĴPWMCMPx
#define CFG_MCM0_PWMRLDEN_INIT	0x0


//<h>PWM
//<i>PSCON
//<i>0x080 b31~12 b11~10    b9~8      b7 b6~4   b3 b2   b1      b0       
//<i>PSCON -      MINSELECT MAXSELECT -  SECTOR -  OVER STATRUN SHIFTRUN 


// <o0.8..9> MAXSELECT_ʱޱȽֵȽϺPWMռձֵѡλ
//    <0=>κʱPWMxռձȣPWMxDֵ
//    <1=>PWMxDڵPWMDMAXʱPWMxD޸ΪPWMDMAX򲻱
//    <2=>PWMxDڵPWMDMAXʱPWMxD޸ΪֵPWMP򲻱
// <o0.10..11> MINSELECT_ʱޱȽֵȽϺPWMռձֵѡλ
//    <0=>κʱPWMxռձȣPWMxDֵ
//    <1=>PWMxDСڵPWMDMINʱPWMxD޸ΪPWMDMIN򲻱
//    <2=>PWMxDСڵPWMDMINʱPWMxD޸Ϊ0򲻱
#define     CFG_MCM0_PSCON_H_INIT 0x0
#define CFG_MCM0_PSCON_INIT	(CFG_MCM0_PSCON_L_INIT|CFG_MCM0_PSCON_H_INIT)

// PWMDMAX_PWMռձޱȽϼĴ
// <o> PWMDMAX_PWMռձޱȽֵ(0.1%) <0-1000> 
//PWMDMAX
//0x080   b31~16 b15~0   
//PWMDMAX -      PWMDMAX 
#define     CFG_MCM0_PWMDMAX_VALUE 980

// PWMDMIN_PWMռձޱȽϼĴ
// <o> PWMDMIN_PWMռձޱȽֵ(0.1%) <0-1000> 
//PWMDMIN
//0x084   b31~16 b15~0   
//PWMDMIN -      PWMDMIN 
#define     CFG_MCM0_PWMDMIN_VALUE 20

// <o> PWMDCMP1_PWMռձСĴ1 <0-65535>
//<i>PWMDCMP1
//<i>0x088    b31~16 b15~0    
//<i>PWMDCMP1 -      PWMDCMP1 
#define     CFG_MCM0_PWMDCMP1_INIT 330


// <o0> PWMDCMP2_PWMռձСĴ2 <0-65535>
//<i>PWMDCMP2
//<i>0x08C    b31~16 b15~0    
//<i>PWMDCMP2 -      PWMDCMP2 
#define     CFG_MCM0_PWMDCMP2_INIT 330
//</h>


//<h>
//<e0.12>Ƚź0ʹ
//<i>0x094  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC0CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME  
//SCEN_SC0ʹ
// <o0.12..14> SCEN_0ܿλ
//    <0=>0ֹܽ
//    <1=>SC0Ϊ0
//    <2=>CMP0Ϊ0룬ߵƽЧ
//    <3=>CMP1Ϊ0룬ߵƽЧ
//    <4=>SDFMCOMPH1Ϊ0룬ߵƽЧ
//    <5=>SDFMCOMPH2Ϊ0룬ߵƽЧ
//    <6=>SDFMCOMPH3Ϊ0룬ߵƽЧ
//    <7=>SDFMCOMPH4Ϊ0룬ߵƽЧ
// <o0.11> SCS_SC0Чƽѡλ
//    <0=>SC0ߵƽЧ
//    <1=>SC0͵ƽЧ
// <q0.8> SCPWM0EN_SC0PWM0ʹ
// <q0.9> SCPWM1EN_SC0PWM1ʹ
// <q0.10> SCPWM2EN_SC0PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC0˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//   <i>˲ʱ䲻Ǿȷֵ
//   <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM0_SC0CON_INIT 0x0

//<e0.12>Ƚź1ʹ
//<i>0x098  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC1CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME 
// <o0.12..14> SCEN_1ܿλ
//    <0=>1ֹܽ
//    <1=>SC1Ϊ1
//    <2=>CMP0Ϊ1룬ߵƽЧ
//    <3=>CMP1Ϊ1룬ߵƽЧ
//    <4=>SDFMCOMPL1Ϊ1룬ߵƽЧ
//    <5=>SDFMCOMPL2Ϊ1룬ߵƽЧ
//    <6=>SDFMCOMPL3Ϊ1룬ߵƽЧ
//    <7=>SDFMCOMPL4Ϊ1룬ߵƽЧ
// <o0.11> SCS_SC1Чƽѡλ
//    <0=>SC1ߵƽЧ
//    <1=>SC1͵ƽЧ
// <q0.8> SCPWM0EN_SC1PWM0ʹ
// <q0.9> SCPWM1EN_SC1PWM1ʹ
// <q0.10> SCPWM2EN_SC1PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC1˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//   <i>˲ʱ䲻Ǿȷֵ
//   <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM0_SC1CON_INIT 0x0


//<e0.12>Ƚź2ʹ
//<i>0x09C  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC2CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME 
// <o0.12..14> SCEN_2ܿλ
//    <0=>2ֹܽ
//    <1=>SC2Ϊ2
//    <2=>CMP0Ϊ2룬ߵƽЧ
//    <3=>CMP1Ϊ2룬ߵƽЧ
//    <4=>SDFMCOMPZ1Ϊ2룬ߵƽЧ
//    <5=>SDFMCOMPZ2Ϊ2룬ߵƽЧ
//    <6=>SDFMCOMPZ3Ϊ2룬ߵƽЧ
//    <7=>SDFMCOMPZ4Ϊ2룬ߵƽЧ
// <o0.11> SCS_SC2Чƽѡλ
//    <0=>SC2ߵƽЧ
//    <1=>SC2͵ƽЧ
// <q0.8> SCPWM0EN_SC2PWM0ʹ
// <q0.9> SCPWM1EN_SC2PWM1ʹ
// <q0.10> SCPWM2EN_SC2PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC2˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM0_SC2CON_INIT 0x0
//</h>
//</e>

#define CFG_MCM0_CLK		(unsigned long)(SYS_CLK/(CFG_MCM0_PWMPSQ_INIT + 1))

#if (CFG_MCM0_PWMCON1_INIT & (1<<10)) 
	#define CFG_MCM0_PWMP_MODE 2
#else
	#define CFG_MCM0_PWMP_MODE 1
#endif


#define CFG_MCM0_PWMP_INIT (uint16_t)(CFG_MCM0_CLK/CFG_MCM0_PWMP_F/CFG_MCM0_PWMP_MODE)

#define CFG_MCM0_PWM0D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM0_DUTY/1000)
#define CFG_MCM0_PWM1D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM1_DUTY/1000)
#define CFG_MCM0_PWM2D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM2_DUTY/1000)
#define CFG_MCM0_PWM01D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM01_DUTY/1000)
#define CFG_MCM0_PWM11D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM11_DUTY/1000)
#define CFG_MCM0_PWM21D_INIT (uint16_t)(CFG_MCM0_PWMP_INIT*CFG_MCM0_PWM21_DUTY/1000)


#define CFG_MCM0_PWMDT21_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT21_DT*CFG_MCM0_CLK/10000000))
#define CFG_MCM0_PWMDT20_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT20_DT*CFG_MCM0_CLK/10000000))
#define CFG_MCM0_PWMDT11_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT11_DT*CFG_MCM0_CLK/10000000))
#define CFG_MCM0_PWMDT10_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT10_DT*CFG_MCM0_CLK/10000000))
#define CFG_MCM0_PWMDT01_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT01_DT*CFG_MCM0_CLK/10000000))
#define CFG_MCM0_PWMDT00_INIT ((uint16_t)((uint64_t)CFG_MCM0_PWMDT00_DT*CFG_MCM0_CLK/10000000))

#define CFG_MCM0_PWMDMIN_INIT	(uint16_t)(CFG_MCM0_PWMDMIN_VALUE*CFG_MCM0_PWMP_INIT/1000)
#define CFG_MCM0_PWMDMAX_INIT	(uint16_t)(CFG_MCM0_PWMDMAX_VALUE*CFG_MCM0_PWMP_INIT/1000)

#define CFG_MCM0_PWMCMP1_INIT	(uint16_t)(CFG_MCM0_PWMCMP1_VALUE*CFG_MCM0_PWMP_INIT/1000)
#define CFG_MCM0_PWMCMP2_INIT	(uint16_t)(CFG_MCM0_PWMCMP2_VALUE*CFG_MCM0_PWMP_INIT/1000)
#define CFG_MCM0_PWMCMP3_INIT	(uint16_t)(CFG_MCM0_PWMCMP3_VALUE*CFG_MCM0_PWMP_INIT/1000)
#define CFG_MCM0_PWMCMP4_INIT	(uint16_t)(CFG_MCM0_PWMCMP4_VALUE*CFG_MCM0_PWMP_INIT/1000)


//***********************************************************/
//<e>MCM1
#define     CFG_MCM1_PWMOE       0
#define  CFG_MCM1_EN CFG_MCM1_PWMOE
//<i>0x00C    b31~3 b2~0  
//<i>PWMREMAP -     PWMIO 
// <o0.0..2> PWMIO_PWMIOڵӳ
//<0=>PWM
//<1=>PWM21 PWM11 PWM01 PWM2 PWM1 PWM0
//<2=>PWM0 PWM1 PWM2 PWM01 PWM11 PWM21 
//<3=>PWM21 PWM2 PWM11 PWM1 PWM01 PWM0
//<4=>PWM0 PWM01 PWM1 PWM11 PWM2 PWM21
#define CFG_MCM1_PWMREMAP_INIT	0x0

// <o> PWMPSQ_PWMƵϵ<0-65535>
//<i>PWMʱΪ(HCLK/(PSQ[15:0]+1)) 
#define     CFG_MCM1_PWMPSQ_INIT 0x0


// <o> PWMP_PWMƵ (0-100000Hz) <0-100000> 
#define     CFG_MCM1_PWMP_F      12000


//<o0>PWMD0:  PWM0ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o1>PWMD1:  PWM1ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o2>PWMD2:  PWM2ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o3>PWMD01: PWM01ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o4>PWMD11: PWM11ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
//<o5>PWMD21: PWM21ռձ (0.1%) <0-1000>
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ
#define     CFG_MCM1_PWM0_DUTY   500
#define     CFG_MCM1_PWM1_DUTY   500
#define     CFG_MCM1_PWM2_DUTY   500
#define     CFG_MCM1_PWM01_DUTY  500
#define     CFG_MCM1_PWM11_DUTY  500
#define     CFG_MCM1_PWM21_DUTY  500


// <o0.4..6> SECTOR_SVPWM趨λ
//<i>MAXMUMMIN
//    <0=>PWM2D > PWM1D > PWM0D
//    <1=>PWM2D > PWM0D > PWM1D
//    <2=>PWM0D > PWM2D > PWM1D
//    <3=>PWM0D > PWM1D > PWM2D
//    <4=>PWM1D > PWM0D > PWM2D
//    <5=>PWM1D > PWM2D > PWM0D
//    <i>趨110111ʱЧ
//    <i>SVPWM趨λͨλȷPWM0DPWM1DPWM2DĴС˳ֵʱĴС˳
#define     CFG_MCM1_PSCON_L_INIT 0x0


// <q0.7> ʹֹͣPWM
//    <i>0: ֹ
//    <i>1: ʹ
#define     CFG_MCM1_POSTDCR_INIT 0x0


//<h>PWM
// <o0.15> POUTMOD_PWM0/1/2ģʽ/λ
//    <0=>3ͨ
//    <1=>6·
//<i>16λռձȼĴ
//<i>ģʽ
//<i>PWMxD (x = 0-2)PWMx(x = 0-2)ԭʼ
//<i>PWMx1D (x = 0-2)PWMx1(x = 0-2)ԭʼ
//<i>ģʽԳƲ
//<i>PWM0DͬʱPWM0/01ԭʼ
//<i>PWM1DͬʱPWM1/11ԭʼ
//<i>PWM2DͬʱPWM2/21ԭʼ
//<i>ģʽ:ǶԳƲΣ(Ķģʽ´)
//<i>PWM0DͬʱPWM0/01
//<i>PWM01DͬʱPWM0/01̵ԭʼ
//<i>PWM1DͬʱPWM1/11̵ԭʼ
//<i>PWM11DͬʱPWM1/11̵ԭʼ
//<i>PWM2DͬʱPWM2/21̵ԭʼ
//<i>PWM21DͬʱPWM2/21̵ԭʼ

// <o0.12..14> POSTPS_Ƶϵ
//    <0=>޺Ƶ
//    <1=>жϡź¼źŽ2Ƶ
//    <2=>жϡź¼źŽ3Ƶ
//    <3=>жϡź¼źŽ4Ƶ
//    <4=>жϡź¼źŽ5Ƶ
//    <5=>жϡź¼źŽ6Ƶ
//    <6=>жϡź¼źŽ7Ƶ
//    <7=>жϡź¼źŽ8Ƶ
// <o0.10..11> PTMOD_PWM0/1/2ģʽ
//    <0=>ضģʽ   
//    <1=>Ķģʽ
//    <2=>μģʽ
//<i>ضģʽģʽ£ʱֻᷢƥ
//<i>Ķģʽģʽ£ʱᷢƥƥ
//<i>μģʽģʽ£ʱֻᷢƥ
// <o0.9> PWMSYM_PWMζԳԿ
//    <0=>ԳPWM
//    <1=>ǶԳPWM
//<i>ĶģʽĻʱЧ
// <o0.6> PDCON0_PWM0/PWM01ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.7> PDCON1_PWM1/PWM11ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.8> PDCON2_PWM2/PWM21ԭʼDUTYλ
//    <0=>ֵСռձȼĴʱΪDUTY
//    <1=>ֵڵռձȼĴʱΪDUTY
// <o0.0> PWM0S_PWM0Чƽѡλ
//    <0=>PWM0Dutyߵƽʱ͵ƽ
//    <1=>PWM0Duty͵ƽʱߵƽ
// <o0.1> PWM1S_PWM1Чƽѡλ
//    <0=>PWM1Dutyߵƽʱ͵ƽ
//    <1=>PWM1Duty͵ƽʱߵƽ
// <o0.2> PWM2S_PWM2Чƽѡλ
//    <0=>PWM2Dutyߵƽʱ͵ƽ
//    <1=>PWM2Duty͵ƽʱߵƽ
// <o0.3> PWM01S_PWM01Чƽѡλ
//    <0=>PWM01Duty͵ƽʱߵƽ
//    <1=>PWM01Dutyߵƽʱ͵ƽ
// <o0.4> PWM11S_PWM11Чƽѡλ
//    <0=>PWM11Duty͵ƽʱߵƽ
//    <1=>PWM11Dutyߵƽʱ͵ƽ
// <o0.5> PWM21S_PWM21Чƽѡλ
//    <0=>PWM21Duty͵ƽʱߵƽ
//    <1=>PWM21Dutyߵƽʱ͵ƽ
#define     CFG_MCM1_PWMCON1_INIT 0x0
//</h>


//<h>PWM
// <o> PWMDT00_PWMͨ0(0.1us) <0-100>
#define     CFG_MCM1_PWMDT00_DT  10

// <o> PWMDT01_PWMͨ0½(0.1us) <0-100>
#define     CFG_MCM1_PWMDT01_DT  10

// <o> PWMDT10_PWMͨ1(0.1us) <0-100>
#define     CFG_MCM1_PWMDT10_DT  10

// <o> PWMDT11_PWMͨ1½(0.1us) <0-100>
#define     CFG_MCM1_PWMDT11_DT  10

// <o> PWMDT20_PWMͨ2(0.1us) <0-100>
#define     CFG_MCM1_PWMDT20_DT  10

// <o> PWMDT21_PWMͨ2½(0.1us) <0-100>
#define     CFG_MCM1_PWMDT21_DT  10
//</h>


//<h>ȽϼĴAD
// <o0.0..1> CMP1_ȽϼĴ1ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ1ʱᴥAD
//    <2=>ֻPWMȽϼĴ1ʱᴥAD
//PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ1ʱᴥAD
//PTMOD = 01ĶģʽЧ    
//<i>ADCADONPWMTRGENλ1
// <o1> PWMCMP1_¼Ƚֵ1ٷֱ(0.1%) <0-1000>
// <o0.2..3> CMP2_ȽϼĴ2ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ2ʱᴥAD
//    <2=>ֻPWMȽϼĴ2ʱᴥAD
// PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ2ʱᴥAD
//PTMOD = 01    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o2> PWMCMP2_¼Ƚֵ2ٷֱ(0.1%) <0-1000>
// <o0.4..5> CMP3_ȽϼĴ3ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ3ʱᴥAD
//    <2=>ֻPWMȽϼĴ3ʱᴥAD
//PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ3ʱᴥAD
//PTMOD = 01ĶģʽЧ    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o3> PWMCMP3_¼Ƚֵ3ٷֱ(0.1%) <0-1000>
// <o0.6..7> CMP4_ȽϼĴ4ADλ
//    <0=>AD
//    <1=>ֻҪPWMȽϼĴ4ʱᴥAD
//    <2=>ֻPWMȽϼĴ4ʱᴥAD
// PTMOD = 01ĶģʽЧ
//    <3=>ֻҪPWMڼȽϼĴ4ʱᴥAD
//PTMOD = 01    ADCADONPWMTRGENλ1£
//<i>ADCADONPWMTRGENλ1
// <o4> PWMCMP4_¼Ƚֵ4ٷֱ(0.1%) <0-1000>
//</h>
//<h>ͬ
// <o0.8> OSYNC_ֶ޸PWMx/PWMx1ͬλ
//    <0=>ֶ޸ĺЧ
//    <1=>ֶ޸PWMͬЧ
//<i>ֶ޸ͨPMANUALCON1/2Ĵʵ,һĴ޸,ʱЧɴ˼Ĵ
// <o0.9> DILDEN_PWMռձȼĴЧʹλ
//    <0=>PWMͬЧ
//    <1=>Ч
// <o0.10> CILDEN_PWM¼ȽϼĴЧʹλ
//    <0=>PWMͬЧ
//    <1=>Ч
// <q0.11> ZDLDEN_PWMƥʱռձȼĴĴ
//    <i> ֵܴǷ趨0ƥʱԶڼĴƵԤƵĴκμĴ
// <q0.12> PDLDEN_PWMƥʱռձȼĴĴ
//    <i>ۼĴãƥʱPWMڼĴƵԤƵĴ,ֻĶģʽЧ
// <q0.13> ZCMLDEN_PWMƥʱ¼Ĵ
//<i>PWMCMPxx=1,2,3,4,ƥʱ¼ȽϼĴADʽĴ(CMP1~4[1-0])ʹλ      
// <q0.14> PCMLDEN_PWMƥʱ¼Ĵ
//<i>ƥʱ¼ȽϼĴADʽĴ(CMP1~4[1-0])ʹλֻĶģʽЧ
// <q0.15> MCMSYNEN_MCM0MCM1ͬʹλ
//    <i>0: رMCM0MCM1ͬ
//    <i>1: MCM0MCM1ͬ
//    <i>Ϊ˷ֹֻMCM0.PWMOE[MCMSYNEN]MCM1.PWMOE[MCMSYNEN]ͬʱ1ܿͬ,ֻĶģʽЧ
//</h>

#define     CFG_MCM1_PWMCON2_INIT 0x0
#define     CFG_MCM1_PWMCMP1_VALUE 100
#define     CFG_MCM1_PWMCMP2_VALUE 200
#define     CFG_MCM1_PWMCMP3_VALUE 999
#define     CFG_MCM1_PWMCMP4_VALUE 0
	

//<h>PWM0/1/2ֶ
//<i>PMANUALCON1_PWM0/1/2ֶüĴ1
//<i>0x058       b31~6 b5        b4        b3        b2       b1       b0       
//<i>PMANUALCON1 -     PMANUAL21 PMANUAL11 PMANUAL01 PMANUAL2 PMANUAL1 PMANUAL0 
//<i>PMANUALCON2_PWM0/1/2ֶüĴ2
//<i>0x05C       b31~6 b5     b4     b3     b2    b1    b0    
//<i>PMANUALCON2 -     POUT21 POUT11 POUT01 POUT2 POUT1 POUT0 
// <o0.0> PMANUAL0_PWM0λ
//    <0=>PWM0PWM
//    <1=>PWM0PMANUALCON2ĴеPOUT0λ
// <o0.1> PMANUAL1_PWM1λ
//    <0=>PWM1PWM
//    <1=>PWM1PMANUALCON2ĴеPOUT1λ
// <o0.2> PMANUAL2_PWM2λ
//    <0=>PWM2PWM
//    <1=>PWM2PMANUALCON2ĴеPOUT2λ
// <o0.3> PMANUAL01_PWM01λ
//    <0=>PWM01PWM
//    <1=>PWM01PMANUALCON2ĴеPOUT01λ
// <o0.4> PMANUAL11_PWM11λ
//    <0=>PWM11PWM
//    <1=>PWM11PMANUALCON2ĴеPOUT11λ
// <o0.5> PMANUAL21_PWM21λ
//    <0=>PWM21PWM
//    <1=>PWM21PMANUALCON2ĴеPOUT21λ
#define     CFG_MCM1_PMANUALCON1_INIT 0x0


//<h>PMANUALCON2_PWM0/1/2ֶüĴ2
//<i>0x05C       b31~6 b5     b4     b3     b2    b1    b0    
//<i>PMANUALCON2 -     POUT21 POUT11 POUT01 POUT2 POUT1 POUT0 
// <o0.0> POUT0_PMANUAL0 = 1ʱPWM0ڵƽ
//    <0=>0
//    <1=>1
// <o0.1> POUT1_PMANUAL1 = 1ʱPWM1ڵƽ
//    <0=>0
//    <1=>1
// <o0.2> POUT2_PMANUAL2 = 1ʱPWM2ڵƽ
//    <0=>0
//    <1=>1
// <o0.3> POUT01_PMANUAL01 = 1ʱPWM01ڵƽ
//    <0=>0
//    <1=>1
// <o0.4> POUT11_PMANUAL11 = 1ʱPWM11ڵƽ
//    <0=>0
//    <1=>1
// <o0.5> POUT21_PMANUAL21 = 1ʱPWM21ڵƽ
//    <0=>0
//    <1=>1
#define     CFG_MCM1_PMANUALCON2_INIT 0x0
//</h>
//</h>


//<h>ϼ
//<i>0x064  b31 b30 	b29 			b28 		 b27 		  b26~25 b24~23 b22       b21       b20       b19    b18     b17     b16     b15~14 b13~12 b11~10 b9    b8       b7~5    b4     b3    b2    b1       b0       
//<i>FLTCON - SDFMFLT4 SDFMFLT3 SDFMFLT2 SDFMFLT1 FOUT1  FOUT0  FLT0STATC FLT1STATC FLT2STATC FLT1EN ADC2FLT ADC1FLT ADC0FLT CMP2S  CMP1S  CMP0S  FLT1M FLT1STAT FLT2DEB FLT2EN FLT2S FLT2M FLT2STAT FLT0STAT 
// <o0.23..24> FOUT0_ϷPWMxţţ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ
// <o0.25..26> FOUT1_ϷPWMx1ţţ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ

// <e0.19> FLT1EN_FLT1MCMλ
//    <i>0:ֹܽ
//    <i>1:ܴ򿪣ԴѡȽ0/1/2ADC0/1/2
// <o0.9> FLT1M_⹦ģʽѡ
//<i>ģʽ⵽ЧFLTSTAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLTSTAT޷
//<i>ģʽPWMֱɹϼƣЧFLTSTATӲ1رPWMΪЧFLTSTATԶ0PWM0/1/2νPWMʱ0ʱԶָ
//    <0=>ģʽ
//    <1=>ģʽ
// <o0.10..11> CMP0S_Ƚ0ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> CMP1S_Ƚ1ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> CMP2S_Ƚ2ѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <q0.16> ADC0FLT_ADC0ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.17> ADC1FLT_ADC1ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.18> ADC2FLT_ADC2ѡλ
//    <i>0: ֹܽ
//    <i>1: ܴ
// <q0.30> SDFMFLT4 SDFMͨ4ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.29> SDFMFLT3 SDFMͨ3ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.28> SDFMFLT2 SDFMͨ2ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
// <q0.27> SDFMFLT1 SDFMͨ1ʹλ
//    <i>0:ֹܽ
//    <i>1:ܴ
//</e>

// <e0.4> FLT2EN_ϼ2ܿλ
//    <i>0:ϼ2ֹܽ
//    <i>1:ϼ2ܴ򿪣ԴΪFLT
// <o0.2> FLT2M_⹦ģʽѡ
//<i>ģʽ⵽ЧFLTSTAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLTSTAT޷
//<i>ģʽPWMֱɹϼƣЧFLTSTATӲ1رPWMΪЧFLTSTATԶ0PWM0/1/2νPWMʱ0ʱԶָ
//    <0=>ģʽ
//    <1=>ģʽ
// <o0.3> FLT2S_ϼ2Чƽѡλ
//    <0=>ϼ2 FLTߵƽЧ
//    <1=>ϼ2 FLT͵ƽЧ
// <o0.5..7> FLT2DEB_ϼ2˲ѡ
//  <0=>˲
//  <1=>0.5us
//  <2=>1us
//  <3=>2us
//  <4=>3us
//  <5=>4us
//  <6=>5us
//  <7=>6us
//  <i>˲ʱ䲻Ǿȷֵ
//  <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define CFG_MCM1_FLTCON_INIT	0x0
//</h>


//<e0.7>PWMʹ
//<i>0x064 b31~8 b7    b6 b5     b4     b3     b2    b1    b0    
//<i>POSCR -     OLSEN -  OLSG21 OLSG11 OLSG01 OLSG2 OLSG1 OLSG0 
// <o0.0> OLSG0_·PWM0Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.1> OLSG1_·PWM1Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.2> OLSG2_·PWM2Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.3> OLSG01_·PWM01Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.4> OLSG11_·PWM11Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
// <o0.5> OLSG21_·PWM21Чƽ趨λ
//    <0=>͵ƽЧ
//    <1=>ߵƽЧ
//</e>
#define     CFG_MCM1_POSCR_INIT 0x0

//<h>PWMDMA
//<i>0x074    b31~8 b7     b6     b5      b4      b3      b2      b1      b0      
//<i>PWMDMAEN -     PWMPDE PWMZDE PTDD2DE PTUD2DE PTDD1DE PTUD1DE PTDD0DE PTUD0DE 
// <q0.0> PTUD0DE_PWMʱʱռձȼĴPWM0DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM0DƥʱDMA
//    <i>1: PWMʱPWM0DƥʱDMA
// <q0.1> PTDD0DE_PWMʱʱռձȼĴPWM0D/PWM01DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM0D/PWM01DƥʱDMA
//    <i>1: PWMʱPWM0D/PWM01DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM0DĶǶԳģʽ£ΪPWM01D
// <q0.2> PTUD1DE_PWMʱʱռձȼĴPWM1DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM1DƥʱDMA
//    <i>1: PWMʱPWM1DƥʱDMA
// <q0.3> PTDD1DE_PWMʱʱռձȼĴPWM1D/PWM11DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM1D/PWM11DƥʱDMA
//    <i>1: PWMʱPWM1D/PWM11DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM1DĶǶԳģʽ£ΪPWM11D
// <q0.4> PTUD2DE_PWMʱʱռձȼĴPWM2DƥʱDMAʹλ
//    <i>0: ֹPWMʱPWM2DƥʱDMA
//    <i>1: PWMʱPWM2DƥʱDMA
// <q0.5> PTDD2DE_PWMʱʱռձȼĴPWM2D/PWM21DƥʱDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱPWM2D/PWM21DƥʱDMA
//    <i>1: PWMʱPWM2D/PWM21DƥʱDMA
//<i>ע:ĶԳģʽ£ΪPWM2DĶǶԳģʽ£ΪPWM21D
// <q0.6> PWMZDE_PWMʱDMAʹλ
//    <i>0: ֹPWMʱDMA
//    <i>1: PWMʱDMA
// <q0.7> PWMPDE_PWMʱƥDMAʹλ
//    <i>ֻĶģʽЧ
//    <i>0: ֹPWMʱƥDMA
//    <i>1: PWMʱƥDMA
#define CFG_MCM1_PWMDMAEN_INIT	0x0
//</h>


//<h>PWMж
//<i>0x074    b31~12 b11    b10 b9     b8     b7     b6     b5      b4      b3      b2      b1      b0      
//<i>PWMINTEN -      OSTDIE OIE FLT2IE FLT1IE PWMPIE PWMZIE PTDD2IE PTUD2IE PTDD1IE PTUD1IE PTDD0IE PTUD0IE
// <q0.0> PTUD0IE_PWMʱPWM0Dƥʱж
//    <i>0: ֹPWMʱPWM0Dƥʱж
//    <i>1: PWMʱPWM0Dƥʱж
// <q0.1> PTDD0IE_PWMʱPWM0Dƥʱж
//<i>ֻĶģʽЧ

// <q0.2> PTUD1IE_PWMʱPWM1Dƥʱж


// <q0.3> PTDD1IE_PWMʱPWM1Dƥʱж
//<i>ֻĶģʽЧ


// <q0.4> PTUD2IE_PWMʱPWM2Dƥʱж


// <q0.5> PTDD2IE_PWMʱPWM2Dƥʱж
//<i>ֻĶģʽЧ


// <q0.6> PWMZIE_PWMʱ㴥ж


// <q0.7> PWMPIE_PWMʱƥ䴥ж
//<i>ֻĶģʽЧ

// <q0.8> FLT1IE_1жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <q0.9> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <q0.10> OIE_·жλ
//    <i>0: ֹж
//    <i>1: ж
// <q0.11> OSTDIE_ֹͣPWMжʹλ
//    <i>0: ֹж
//    <i>1: ж
//</h>
#define     CFG_MCM1_PWMINTEN_INIT 0x0


#if ((CFG_MCM1_PWMINTEN_INIT & 0xFF) != 0)
	#define CFG_MCM1_PWM_INT_EN 1
#else
	#define CFG_MCM1_PWM_INT_EN 0
#endif


#if ((CFG_MCM1_PWMINTEN_INIT & 0x700) != 0)
	#define CFG_MCM1_FLT_INT_EN 1
#else
	#define CFG_MCM1_FLT_INT_EN 0
#endif


//PWMRLDEN
//0x078    b31~8 b7~0     
//PWMRLDEN -     PWMRLDEN 
//PWMRLDEN_Ĵ޸ĺؿƼĴ
//    0x55:ģĴ޸
//    0xAA:ģļĴ
//    PWMģļĴжϱ־ĴPWMINTFֶƼĴPMANUALCON2FLTCONPOSCRPOSTDCR⣬Ĵ޸ֻеPWMRLDEN = 0x55ʱ޸ģ޸Ч
//    PWMģһЩĴлĴռձȼĴPWMxD&PWMx1DڼĴPWMPPWMʱԤƵĴPWMPSQƵĴPOSTPS[2:0]@ PWMCON1DutyѡĴPDCONx@PWMCON1ԳԿƼĴPWMSYM@PWMCON1ĴPWMDTxxȽϼĴADλCMPx@PWMCON2PWM¼ĴPWMCMPx
#define CFG_MCM1_PWMRLDEN_INIT	0x0


//<h>PWM
//<i>PSCON
//<i>0x080 b31~12 b11~10    b9~8      b7 b6~4   b3 b2   b1      b0       
//<i>PSCON -      MINSELECT MAXSELECT -  SECTOR -  OVER STATRUN SHIFTRUN 


// <o0.8..9> MAXSELECT_ʱޱȽֵȽϺPWMռձֵѡλ
//    <0=>κʱPWMxռձȣPWMxDֵ
//    <1=>PWMxDڵPWMDMAXʱPWMxD޸ΪPWMDMAX򲻱
//    <2=>PWMxDڵPWMDMAXʱPWMxD޸ΪֵPWMP򲻱
// <o0.10..11> MINSELECT_ʱޱȽֵȽϺPWMռձֵѡλ
//    <0=>κʱPWMxռձȣPWMxDֵ
//    <1=>PWMxDСڵPWMDMINʱPWMxD޸ΪPWMDMIN򲻱
//    <2=>PWMxDСڵPWMDMINʱPWMxD޸Ϊ0򲻱
#define     CFG_MCM1_PSCON_H_INIT 0x0
#define CFG_MCM1_PSCON_INIT	(CFG_MCM1_PSCON_L_INIT|CFG_MCM1_PSCON_H_INIT)

// PWMDMAX_PWMռձޱȽϼĴ
// <o> PWMDMAX_PWMռձޱȽֵ(0.1%) <0-1000> 
//PWMDMAX
//0x080   b31~16 b15~0   
//PWMDMAX -      PWMDMAX 
#define     CFG_MCM1_PWMDMAX_VALUE 980

// PWMDMIN_PWMռձޱȽϼĴ
// <o> PWMDMIN_PWMռձޱȽֵ(0.1%) <0-1000> 
//PWMDMIN
//0x084   b31~16 b15~0   
//PWMDMIN -      PWMDMIN 
#define     CFG_MCM1_PWMDMIN_VALUE 20

// <o> PWMDCMP1_PWMռձСĴ1 <0-65535>
//<i>PWMDCMP1
//<i>0x088    b31~16 b15~0    
//<i>PWMDCMP1 -      PWMDCMP1 
#define     CFG_MCM1_PWMDCMP1_INIT 330


// <o0> PWMDCMP2_PWMռձСĴ2 <0-65535>
//<i>PWMDCMP2
//<i>0x08C    b31~16 b15~0    
//<i>PWMDCMP2 -      PWMDCMP2 
#define     CFG_MCM1_PWMDCMP2_INIT 330
//</h>


//<h>
//<e0.12>Ƚź0ʹ
//<i>0x094  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC0CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME  
//SCEN_SC0ʹ
// <o0.12..14> SCEN_0ܿλ
//    <0=>0ֹܽ
//    <1=>SC0Ϊ0
//    <2=>CMP0Ϊ0룬ߵƽЧ
//    <3=>CMP0Ϊ0룬͵ƽЧ
//    <4=>CMP1Ϊ0룬ߵƽЧ
//    <5=>CMP1Ϊ0룬͵ƽЧ
//    <6=>CMP2Ϊ0룬ߵƽЧ
//    <7=>CMP2Ϊ0룬͵ƽЧ
// <o0.11> SCS_SC0Чƽѡλ
//    <0=>SC0ߵƽЧ
//    <1=>SC0͵ƽЧ
// <q0.8> SCPWM0EN_SC0PWM0ʹ
// <q0.9> SCPWM1EN_SC0PWM1ʹ
// <q0.10> SCPWM2EN_SC0PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC0˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//   <i>˲ʱ䲻Ǿȷֵ
//   <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM1_SC0CON_INIT 0x0

//<e0.12>Ƚź1ʹ
//<i>0x098  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC1CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME 
// <o0.12..14> SCEN_1ܿλ
//    <0=>1ֹܽ
//    <1=>SC1Ϊ1
//    <2=>CMP0Ϊ1룬ߵƽЧ
//    <3=>CMP0Ϊ1룬͵ƽЧ
//    <4=>CMP1Ϊ1룬ߵƽЧ
//    <5=>CMP1Ϊ1룬͵ƽЧ
//    <6=>CMP2Ϊ1룬ߵƽЧ
//    <7=>CMP2Ϊ1룬͵ƽЧ
// <o0.11> SCS_SC1Чƽѡλ
//    <0=>SC1ߵƽЧ
//    <1=>SC1͵ƽЧ
// <q0.8> SCPWM0EN_SC1PWM0ʹ
// <q0.9> SCPWM1EN_SC1PWM1ʹ
// <q0.10> SCPWM2EN_SC1PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC1˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//   <i>˲ʱ䲻Ǿȷֵ
//   <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM1_SC1CON_INIT 0x0


//<e0.12>Ƚź2ʹ
//<i>0x09C  b31~15 b14~12 b11 b10      b9       b8       b7~4  b3~0   
//<i>SC2CON -      SCEN   SCS SCPWM2EN SCPWM1EN SCPWM0EN SCDEB SCTIME 
// <o0.12..14> SCEN_2ܿλ
//    <0=>2ֹܽ
//    <1=>SC2Ϊ2
//    <2=>CMP0Ϊ2룬ߵƽЧ
//    <3=>CMP0Ϊ2룬͵ƽЧ
//    <4=>CMP1Ϊ2룬ߵƽЧ
//    <5=>CMP1Ϊ2룬͵ƽЧ
//    <6=>CMP2Ϊ2룬ߵƽЧ
//    <7=>CMP2Ϊ2룬͵ƽЧ
// <o0.11> SCS_SC2Чƽѡλ
//    <0=>SC2ߵƽЧ
//    <1=>SC2͵ƽЧ
// <q0.8> SCPWM0EN_SC2PWM0ʹ
// <q0.9> SCPWM1EN_SC2PWM1ʹ
// <q0.10> SCPWM2EN_SC2PWM2ʹ
// <o0.0..3> SCTIME_ʱѡλ
//    <0=>һƥʱָ
//    <1=>1200*HCLK
//    <2=>1600*HCLK
//    <3=>2000*HCLK
//    <4=>2400*HCLK
//    <5=>2800*HCLK
//    <6=>3200*HCLK
//    <7=>4000*HCLK
//    <8=>4800*HCLK
//    <9=>6400*HCLK
//    <10=>8000*HCLK
//    <11=>16000*HCLK
//    <12=>32000*HCLK
//    <13=>48000*HCLK
//    <14=>64000*HCLK
//    <15=>ֶָMCMͨSC0STATC@PWMINTF
// <o0.4..7> SCDEB_SC2˲ѡ
//    <0=>0
//    <1=>40*HCLK
//    <2=>80*HCLK
//    <3=>120*HCLK
//    <4=>160*HCLK
//    <5=>240*HCLK
//    <6=>320*HCLK
//    <7=>480*HCLK
//    <8=>640*HCLK
//    <9=>800*HCLK
//    <10=>960*HCLK
//    <11=>1120*HCLK
//    <12=>1280*HCLK
//    <13=>1600*HCLK
//    <14=>1920*HCLK
//    <15=>2560*HCLK
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲2Ϊ͵ƽ1С˲,˲0ͬʱΪ0
//</e>
#define     CFG_MCM1_SC2CON_INIT 0x0
//</h>
//</e>
#define CFG_MCM1_CLK		(unsigned long)(SYS_CLK/(CFG_MCM1_PWMPSQ_INIT + 1))

#if (CFG_MCM1_PWMCON1_INIT & (1<<10)) 
	#define CFG_MCM1_PWMP_MODE 2
#else
	#define CFG_MCM1_PWMP_MODE 1
#endif


#define CFG_MCM1_PWMP_INIT (uint16_t)(CFG_MCM1_CLK/CFG_MCM1_PWMP_F/CFG_MCM1_PWMP_MODE)

#define CFG_MCM1_PWM0D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM0_DUTY/1000)
#define CFG_MCM1_PWM1D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM1_DUTY/1000)
#define CFG_MCM1_PWM2D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM2_DUTY/1000)
#define CFG_MCM1_PWM01D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM01_DUTY/1000)
#define CFG_MCM1_PWM11D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM11_DUTY/1000)
#define CFG_MCM1_PWM21D_INIT (uint16_t)(CFG_MCM1_PWMP_INIT*CFG_MCM1_PWM21_DUTY/1000)

#define CFG_MCM1_PWMDT21_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT21_DT*CFG_MCM1_CLK/10000000))
#define CFG_MCM1_PWMDT20_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT20_DT*CFG_MCM1_CLK/10000000))
#define CFG_MCM1_PWMDT11_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT11_DT*CFG_MCM1_CLK/10000000))
#define CFG_MCM1_PWMDT10_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT10_DT*CFG_MCM1_CLK/10000000))
#define CFG_MCM1_PWMDT01_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT01_DT*CFG_MCM1_CLK/10000000))
#define CFG_MCM1_PWMDT00_INIT ((uint16_t)((uint64_t)CFG_MCM1_PWMDT00_DT*CFG_MCM1_CLK/10000000))


#define CFG_MCM1_PWMDMIN_INIT	(uint16_t)(CFG_MCM1_PWMDMIN_VALUE*CFG_MCM1_PWMP_INIT/1000)
#define CFG_MCM1_PWMDMAX_INIT	(uint16_t)(CFG_MCM1_PWMDMAX_VALUE*CFG_MCM1_PWMP_INIT/1000)


#define CFG_MCM1_PWMCMP1_INIT	(uint16_t)(CFG_MCM1_PWMCMP1_VALUE*CFG_MCM1_PWMP_INIT/1000)
#define CFG_MCM1_PWMCMP2_INIT	(uint16_t)(CFG_MCM1_PWMCMP2_VALUE*CFG_MCM1_PWMP_INIT/1000)
#define CFG_MCM1_PWMCMP3_INIT	(uint16_t)(CFG_MCM1_PWMCMP3_VALUE*CFG_MCM1_PWMP_INIT/1000)
#define CFG_MCM1_PWMCMP4_INIT	(uint16_t)(CFG_MCM1_PWMCMP4_VALUE*CFG_MCM1_PWMP_INIT/1000)

//</e>



//<e>PCA
#define     CFG_PCA_EN          0
//***********************************************************/
//<e>PCA0
#define     CFG_PCA0_EN         0
//<i>CR_PCAx ʹܼĴ
//<i>0x000 b31~1 b0  
//<i>CR    -     PEN 
// <q0.0> PEN_PCAx /ʱпλ
//  <i>0: ֹPCAx/ʱ
//  <i>1: PCAx/ʱ
#define     CFG_PCA0_CR_INIT    0x0


//<e>ƵռձԶPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA_C0_EN       0


// <o> PCA0Ƶ (0-100000Hz) <0-100000> 
#define     CFG_PCA_PCA0_PR_F   1000
//<o0>PCA0A:  PCA0A/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA0A_DUTY  400
//<o0>PCA0B:  PCA0B/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA0B_DUTY  200
//<o0>PCA0C:  PCA0C/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA0C_DUTY  300
//</e>


//<e>üPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA0_E0_EN      0


//<h>PR_PCAx ֵ
//<i>0x018 b31~16 b15~0 
//<i>PR    PRH    PRL   
// <o0.0..15> PRL_PCAx ֵֽڣLSB<0-0xFFFF> 
// <o0.16..31> PRH_PCAx ֵֽڣMSB<0-0xFFFF> 
#define     CFG_PCA0_PR         0x0
//</h>


//<h>CCR0_PCAx Ƚ/׽ģ0׽ֵ
//<i>0x028 b31~16 b15~0 
//<i>CCR0  CCR0H  CCR0L 
// <o0.0..15> CCR0L_PCAx Ƚ/׽ģ0Ĵֽ <0-0xFFFF>
// <o0.16..31> CCR0H_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF>
//</h>
#define     CFG_PCA0_CCR0       0x0


//<h>CCR1_PCAx Ƚ/׽ģ1׽ֵ
//<i>0x02C b31~16 b15~0 
//<i>CCR1  CCR1H  CCR1L 
// <o0.0..15> CCR1L_PCAx Ƚ/׽ģ1ĴֽڣLSB<0-0xFFFF>
// <o0.16..31> CCR1H_PCAx Ƚ/׽ģ1ĴֽڣMSB<0-0xFFFF>
//</h>
#define     CFG_PCA0_CCR1       0x0


//<h>CCR2_PCAx Ƚ/׽ģ2׽ֵ
//<i>0x030 b31~16 b15~0 
//<i>CCR2  CCR2H  CCR2L 
// <o0.0..15> CCR2L_PCAx Ƚ/׽ģ2ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR2H_PCAx Ƚ/׽ģ2ĴֽڣMSB<0-0xFFFF>
//</h>
#define     CFG_PCA0_CCR2       0x0
//</e>


//<i>PSC_ԤƵ
//<i>0x014 b31~16 b15~0 
//<i>PSC   -      PSC   
// <o0.0..15> PSC_ԤƵ <0-0xFFFF>
#define     CFG_PCA0_PSC_INIT 0x0

// <o0.0..2> CPS_PCAx/ʱʱѡ 
//    <0=>PCLKʱ
//    <1=>PCAxECI½
//    <2=>RC128K
// <o0.6> SDEN_PCAx ģʽѡλ
//    <0=>PCAxϼݲģʽ
//    <1=>PCAxǲģʽ
//    <i>PCAxϼݲģʽ, ʱPCAxȽϲ׽ģģʽֹ
//    <i>PCAxǲģʽʱPCAxȽϲ׽ģľݲģʽֹ

//<h>CNT_PCAxĴ
//<i>0x010 b31~16 b15~0 
//<i>CNT   CNTH   CNTL  
// <o1.0..15> CNTL_PCAxֵ16λ <0-0xFFFF>
// <o1.16..31> CNTH_PCAxֵ16λ <0-0xFFFF>
//</h>

//<h>PCAx ʽ
//<i>0x004 b31~25 b24   b23   b22    b21    b20    b19~18 b17~16 b15 b14   b13   b12   b11 b10~9  b8  b7  b6   b5~3 b2~0 
//<i>CFGR  -      PUDMA CUDMA CC2DMA CC1DMA CC0DMA SYN    ECF    PUE CC2UE CC1UE CC0UE -   ADTSEL PIE CIE SDEN -    CPS  


//<i>PCAx趨ΪģʽеһʱȽϲ׽ģһбģʽʹҲЧġ
// <q0.7> CIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: CIFλʱ PCAx/ʱж
//λPCAx/ʱCIFжϵλ
// <q0.8> PIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: PIFλʱ PCAx/ʱڵж
//<i>λPCAx/ʱڣPIFжϵλ
// <q0.20> CC0DMA_PCA׽/Ƚͨ0¼ʱDMA
// <q0.21> CC1DMA_PCA׽/Ƚͨ1¼ʱDMA
// <q0.22> CC2DMA_PCA׽/Ƚͨ2¼ʱDMA
// <q0.23> CUDMA_PCA/ʱ¼ʱDMA
//    <i>0: /ʱ¼ʱֹDMA
//    <i>1: /ʱ¼ʱDMA
// <q0.24> PUDMAPCA/ʱƥ¼ʱDMA
//    <i>0: /ʱƥ¼ʱֹDMA
//    <i>1: /ʱƥ¼ʱDMA
// <o0.12> CC0UE_PCAxȽģ0ʹλ
//    <0=>д뵽CCR0ֵһЧ
//    <1=>д뵽CCR0ֵЧ
// <o0.13> CC1UE_PCAxȽģ1ʹλ
//    <0=>д뵽CCR1ֵһЧ
//    <1=>д뵽CCR1ֵЧ
// <o0.14> CC2UE_PCAxȽģ2ʹλ
//    <0=>д뵽CCR2ֵһЧ
//    <1=>д뵽CCR2ֵЧ
// <o0.15> PUE_PCAxֵʹλ
//    <0=>д뵽PRĴֵһЧ
//    <1=>д뵽PRĴֵЧ
// <o0.16..17> ECF_PCAx_ECIź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*PCLK
//    <2=>˲ʱΪ16*PCLK
//    <3=>˲ʱΪ32*PCLK
// <o0.18..19> SYN_PCAx_ͬʹλ
//    <0=>ͬ
//    <1=>PENλ10һPCAģͬ
//    <2=>PENλ10MCM0ʹλͬ
//    <3=>PENλ10MCM1ʹλͬ
// <o0.9..10> ADTSEL_PCAxADCѡλ
//    <0=>ADC
//    <1=>ϼԴADC
//    <2=>¼ADC
//    <3=>/¼ԴADC
//<i>ADTSELΪ10b11bֻSDEN=1ʱЧSDENΪ0ʱADTSELΪ1xb൱00b
//<i>ADTR_ȽϴĴ
//<i>0x038 b31~0 
//<i>ADTR  DATA  
// <o2.0..31> DATA_ADCȽϼĴ
//<i>˼ĴֵƥʱᴥADCת <0-0xFFFFFFFF>
//</h>
#define     CFG_PCA0_CFGR_INIT  0x0
#define     CFG_PCA0_CNT_INIT 0x0
#define     CFG_PCA0_ADTR_INIT 0x0


//<h>CCMR0_PCAxA ׽/Ƚģ0ƼĴ
//<i>0x01C b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR0 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7>  PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxA벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxAΪԴ
//    <1=>ѡȽ0C0OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA0_CCMR0_INIT 0x000


//<h>CCMR1_PCAxB ׽/Ƚģ1ƼĴ
//<i>0x020 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR1 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxB벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxBΪԴ
//    <1=>ѡȽ1C1OUTΪ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA0_CCMR1_INIT 0x0


//<h>CCMR2_PCAxC ׽/Ƚģ2ƼĴ
//<i>0x024 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR2 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxC벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxCΪԴ
//    <1=>ѡȽ2C2OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA0_CCMR2_INIT 0x0

//<h>FLTCR_PWM0/1/2ϼⱣĴ
//<i>0x040 b31~30 b29    b28~27 b26~25 b24~23 b22    b21   b20       b19      b18    b17~16 b15~14 b13~12 b11~10 b9~7    b6     b5    b4        b3       b2 b1        b0       
//<i>FLTCR FOUT   FLT1EN CMP2S  CMP1S  CMP0S  FLT1IE FLT1M FLT1STATC FLT1STAT FLT2EN FPIN3S FPIN2S FPIN1S FPIN0S FLT2DEB FLT2IE FLT2M FLT2STATC FLT2STAT -  FLT0STATC FLT0STAT 
// <o0.30..31> FOUT_Ϸ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ

// <o0.29> FLT1EN_ϼ1ܿλ
//    <0=>ϼ1ֹܽ
//    <1=>ϼ1ܴ򿪣ԴѡȽ0/1/2
// <o0.21> FLT1M_⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.22> FLT1IE_жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.23..24> CMP0S_Ƚ0˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.25..26> CMP1S_Ƚ1˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.27..28> CMP2S_Ƚ2˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.18> FLT2EN_ϼ2ܿλ
//    <0=>ϼ2ֹܽ
//    <1=>ϼ2ܴ򿪣ԴΪFLT
// <o0.5> FLT2M_2⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.6> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.7..9> FLT2DEB_ϼ2˲ѡ
//    <0=>˲
//    <1=>0.5us
//    <2=>1us
//    <3=>1.5us
//    <4=>2us
//    <5=>3us
//    <6=>4us
//    <7=>6us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.10..11> FPIN0S_FLTPIN0˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> FPIN1S_FLTPIN1˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> FPIN2S_FLTPIN2˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.16..17> FPIN3S_FLTPIN3˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
//</h>
#define     CFG_PCA0_FLTCR_INIT 0x0


#if (CFG_PCA0_CFGR_INIT & 1) 
	#define CFG_PCA0_CLK		(unsigned long)(SYS_CLK/(CFG_PCA0_PSC_INIT + 1))
#elif (CFG_PCA0_CFGR_INIT & (1<<1)) 
	#define CFG_PCA0_CLK		(unsigned long)(128000/(CFG_PCA0_PSC_INIT + 1))		
#else
	#define CFG_PCA0_CLK		(unsigned long)(SYS_CLK/(CFG_PCA0_PSC_INIT + 1))
#endif


#if (CFG_PCA0_CFGR_INIT & (1<<6)) 
	#define CFG_PCA0_SDEN_MODE 2
#else
	#define CFG_PCA0_SDEN_MODE 1
#endif


#if (CFG_PCA_C0_EN == 1)	
	#define CFG_PCA0_PR_INIT (uint32_t)(CFG_PCA0_CLK/CFG_PCA_PCA0_PR_F/CFG_PCA0_SDEN_MODE)
#else
	#define CFG_PCA0_PR_INIT			CFG_PCA0_PR
#endif


#if (CFG_PCA_C0_EN == 1)	
	#define CFG_PCA0_CCR0_INIT (uint32_t)(CFG_PCA0_PR_INIT*CFG_PCA_PCA0A_DUTY/1000)
	#define CFG_PCA0_CCR1_INIT (uint32_t)(CFG_PCA0_PR_INIT*CFG_PCA_PCA0B_DUTY/1000)
	#define CFG_PCA0_CCR2_INIT (uint32_t)(CFG_PCA0_PR_INIT*CFG_PCA_PCA0C_DUTY/1000)
#else
	#define CFG_PCA0_CCR0_INIT 		CFG_PCA0_CCR0
	#define CFG_PCA0_CCR1_INIT 		CFG_PCA0_CCR1
	#define CFG_PCA0_CCR2_INIT 		CFG_PCA0_CCR2
#endif
#define CFG_PCA0_INT_EN (((CFG_PCA0_CFGR_INIT & 0x80)>>7) | ((CFG_PCA0_CFGR_INIT & 0x100)>>8)| (CFG_PCA0_CCMR0_INIT & 0x01) |(CFG_PCA0_CCMR1_INIT & 0x01)|(CFG_PCA0_CCMR2_INIT & 0x01))
#define CFG_PCA0_FLT_INT_EN ((CFG_PCA0_FLTCR_INIT>>6)&0x01 | (CFG_PCA0_FLTCR_INIT>>22)&0x01)
//</e>


//***********************************************************/
//<e>PCA1
#define     CFG_PCA1_EN         0
//<i>CR_PCAx ʹܼĴ
//<i>0x000 b31~1 b0  
//<i>CR    -     PEN 
// <q0.0> PEN_PCAx /ʱпλ
//  <i>0: ֹPCAx/ʱ
//  <i>1: PCAx/ʱ
#define     CFG_PCA1_CR_INIT    0x0

//<e>ƵռձԶPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA_C1_EN       0


// <o> PCA1Ƶ (0-100000Hz) <0-100000> 
#define     CFG_PCA_PCA1_PR_F   2000
//<o0>PCA1A:  PCA1A/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA1A_DUTY  400
//<o0>PCA1B:  PCA1B/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA1B_DUTY  200
//<o0>PCA1C:  PCA1C/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA1C_DUTY  300
//</e>


//<e>üPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA1_E0_EN      0


//<h>PR_PCAx ֵ
//<i>0x018 b31~16 b15~0 
//<i>PR    PRH    PRL   
// <o0.0..15> PRL_PCAx ֵֽڣLSB<0-0xFFFF> 
// <o0.16..31> PRH_PCAx ֵֽڣMSB<0-0xFFFF> 
#define     CFG_PCA1_PR         0x0
//</h>


//<h>CCR0_PCAx Ƚ/׽ģ0׽ֵ
//<i>0x028 b31~16 b15~0 
//<i>CCR0  CCR0H  CCR0L 
// <o0.0..15> CCR0L_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF> 
// <o0.16..31> CCR0H_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF> 
//</h>
#define     CFG_PCA1_CCR0       0x0


//<h>CCR1_PCAx Ƚ/׽ģ1׽ֵ
//<i>0x02C b31~16 b15~0 
//<i>CCR1  CCR1H  CCR1L 
// <o0.0..15> CCR1L_PCAx Ƚ/׽ģ1ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR1H_PCAx Ƚ/׽ģ1ĴֽڣMSB<0-0xFFFF> 
//</h>
#define     CFG_PCA1_CCR1       0x0

//<h>CCR2_PCAx Ƚ/׽ģ2׽ֵ
//<i>0x030 b31~16 b15~0 
//<i>CCR2  CCR2H  CCR2L 
// <o0.0..15> CCR2L_PCAx Ƚ/׽ģ2ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR2H_PCAx Ƚ/׽ģ2ĴֽڣMSB<0-0xFFFF> 
//</h>
#define     CFG_PCA1_CCR2       0x0
//</e>


//<i>PSC_ԤƵ
//<i>0x014 b31~16 b15~0 
//<i>PSC   -      PSC   
// <o0.0..15> PSC_ԤƵ <0-0xFFFF>
#define     CFG_PCA1_PSC_INIT 0x0

// <o0.0..2> CPS_PCAx/ʱʱѡ 
//    <0=>PCLKʱ
//    <1=>PCAxECI½
//    <2=>RC128K
// <o0.6> SDEN_PCAx ģʽѡλ
//    <0=>PCAxϼݲģʽ
//    <1=>PCAxǲģʽ
//    <i>PCAxϼݲģʽ, ʱPCAxȽϲ׽ģģʽֹ
//    <i>PCAxǲģʽʱPCAxȽϲ׽ģľݲģʽֹ


//<h>CNT_PCAxĴ
//<i>0x010 b31~16 b15~0 
//<i>CNT   CNTH   CNTL  
// <o1.0..15> CNTL_PCAxֵ16λ <0-0xFFFF>
// <o1.16..31> CNTH_PCAxֵ16λ <0-0xFFFF>
//</h>


//<h>PCAx ʽ
//<i>0x004 b31~25 b24   b23   b22    b21    b20    b19~18 b17~16 b15 b14   b13   b12   b11 b10~9  b8  b7  b6   b5~3 b2~0 
//<i>CFGR  -      PUDMA CUDMA CC2DMA CC1DMA CC0DMA SYN    ECF    PUE CC2UE CC1UE CC0UE -   ADTSEL PIE CIE SDEN -    CPS  


//<i>PCAx趨ΪģʽеһʱȽϲ׽ģһбģʽʹҲЧġ
// <q0.7> CIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: CIFλʱ PCAx/ʱж
//<i>λPCAx/ʱCIFжϵλ
// <q0.8> PIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: PIFλʱ PCAx/ʱڵж
//<i>λPCAx/ʱڣPIFжϵλ
// <q0.20> CC0DMA_PCA׽/Ƚͨ0¼ʱDMA
// <q0.21> CC1DMA_PCA׽/Ƚͨ1¼ʱDMA
// <q0.22> CC2DMA_PCA׽/Ƚͨ2¼ʱDMA
// <q0.23> CUDMA_PCA/ʱ¼ʱDMA
//    <i>0: /ʱ¼ʱֹDMA
//    <i>1: /ʱ¼ʱDMA
// <q0.24> PUDMAPCA/ʱƥ¼ʱDMA
//    <i>0: /ʱƥ¼ʱֹDMA
//    <i>1: /ʱƥ¼ʱDMA
// <o0.12> CC0UE_PCAxȽģ0ʹλ
//    <0=>д뵽CCR0ֵһЧ
//    <1=>д뵽CCR0ֵЧ
// <o0.13> CC1UE_PCAxȽģ1ʹλ
//    <0=>д뵽CCR1ֵһЧ
//    <1=>д뵽CCR1ֵЧ
// <o0.14> CC2UE_PCAxȽģ2ʹλ
//    <0=>д뵽CCR2ֵһЧ
//    <1=>д뵽CCR2ֵЧ
// <o0.15> PUE_PCAxֵʹλ
//    <0=>д뵽PRĴֵһЧ
//    <1=>д뵽PRĴֵЧ
// <o0.16..17> ECF_PCAx_ECIź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*PCLK
//    <2=>˲ʱΪ16*PCLK
//    <3=>˲ʱΪ32*PCLK
// <o0.18..19> SYN_PCAx_ͬʹλ
//    <0=>ͬ
//    <1=>PENλ10һPCAģͬ
//    <2=>PENλ10MCM0ʹλͬ
//    <3=>PENλ10MCM1ʹλͬ
// <o0.9..10> ADTSEL_PCAxADCѡλ
//    <0=>ADC
//    <1=>ϼԴADC
//    <2=>¼ADC
//    <3=>/¼ԴADC
//<i>ADTSELΪ10b11bֻSDEN=1ʱЧSDENΪ0ʱADTSELΪ1xb൱00b
//<i>ADTR_ȽϴĴ
//<i>0x038 b31~0 
//<i>ADTR  DATA  
// <o2.0..31> DATA_ADCȽϼĴ
//<i>˼ĴֵƥʱᴥADCת <0-0xFFFFFFFF>
//</h>
#define     CFG_PCA1_CFGR_INIT 0x0
#define     CFG_PCA1_CNT_INIT 0x0
#define     CFG_PCA1_ADTR_INIT 0x0


//<h>CCMR0_PCAxA ׽/Ƚģ0ƼĴ
//<i>0x01C b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR0 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxA벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxAΪԴ
//    <1=>ѡȽ0C0OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA1_CCMR0_INIT 0x0


//<h>CCMR1_PCAxB ׽/Ƚģ1ƼĴ
//<i>0x020 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR1 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxB벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxBΪԴ
//    <1=>ѡȽ1C1OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA1_CCMR1_INIT 0x0


//<h>CCMR2_PCAxC ׽/Ƚģ2ƼĴ
//<i>0x024 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR2 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxC벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxCΪԴ
//    <1=>ѡȽ2C2OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA1_CCMR2_INIT 0x0

//<h>FLTCR_PWM0/1/2ϼⱣĴ
//<i>0x040 b31~30 b29    b28~27 b26~25 b24~23 b22    b21   b20       b19      b18    b17~16 b15~14 b13~12 b11~10 b9~7    b6     b5    b4        b3       b2 b1        b0       
//<i>FLTCR FOUT   FLT1EN CMP2S  CMP1S  CMP0S  FLT1IE FLT1M FLT1STATC FLT1STAT FLT2EN FPIN3S FPIN2S FPIN1S FPIN0S FLT2DEB FLT2IE FLT2M FLT2STATC FLT2STAT -  FLT0STATC FLT0STAT 
// <o0.30..31> FOUT_Ϸ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ
// <o0.29> FLT1EN_ϼ1ܿλ
//    <0=>ϼ1ֹܽ
//    <1=>ϼ1ܴ򿪣ԴѡȽ0/1/2
// <o0.21> FLT1M_⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.22> FLT1IE_жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.23..24> CMP0S_Ƚ0˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.25..26> CMP1S_Ƚ1˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.27..28> CMP2S_Ƚ2˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.18> FLT2EN_ϼ2ܿλ
//    <0=>ϼ2ֹܽ
//    <1=>ϼ2ܴ򿪣ԴΪFLT
// <o0.5> FLT2M_2⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.6> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.7..9> FLT2DEB_ϼ2˲ѡ
//    <0=>˲
//    <1=>0.5us
//    <2=>1us
//    <3=>1.5us
//    <4=>2us
//    <5=>3us
//    <6=>4us
//    <7=>6us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.10..11> FPIN0S_FLTPIN0˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> FPIN1S_FLTPIN1˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> FPIN2S_FLTPIN2˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.16..17> FPIN3S_FLTPIN3˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
//</h>
#define     CFG_PCA1_FLTCR_INIT 0x0


#if (CFG_PCA1_CFGR_INIT & 1) 
	#define CFG_PCA1_CLK		(unsigned long)(SYS_CLK/(CFG_PCA1_PSC_INIT + 1))
#elif (CFG_PCA1_CFGR_INIT & (1<<1)) 
	#define CFG_PCA1_CLK		(unsigned long)(128000/(CFG_PCA1_PSC_INIT + 1))		
#else
	#define CFG_PCA1_CLK		(unsigned long)(SYS_CLK/(CFG_PCA1_PSC_INIT + 1))
#endif


#if (CFG_PCA1_CFGR_INIT & (1<<6)) 
	#define CFG_PCA1_SDEN_MODE 2
#else
	#define CFG_PCA1_SDEN_MODE 1
#endif


#if (CFG_PCA_C1_EN == 1)	
	#define CFG_PCA1_PR_INIT (uint32_t)(CFG_PCA1_CLK/CFG_PCA_PCA1_PR_F/CFG_PCA1_SDEN_MODE)
	#define CFG_PCA1_CCR0_INIT (uint32_t)(CFG_PCA1_PR_INIT*CFG_PCA_PCA1A_DUTY/1000)
	#define CFG_PCA1_CCR1_INIT (uint32_t)(CFG_PCA1_PR_INIT*CFG_PCA_PCA1B_DUTY/1000)
	#define CFG_PCA1_CCR2_INIT (uint32_t)(CFG_PCA1_PR_INIT*CFG_PCA_PCA1C_DUTY/1000)
#else
	#define CFG_PCA1_PR_INIT			CFG_PCA1_PR
	#define CFG_PCA1_CCR0_INIT 		CFG_PCA1_CCR0
	#define CFG_PCA1_CCR1_INIT 		CFG_PCA1_CCR1
	#define CFG_PCA1_CCR2_INIT 		CFG_PCA1_CCR2
#endif


#define CFG_PCA1_INT_EN (((CFG_PCA1_CFGR_INIT & 0x80)>>7) | ((CFG_PCA1_CFGR_INIT & 0x100)>>8)| (CFG_PCA1_CCMR0_INIT & 0x01) |(CFG_PCA1_CCMR1_INIT & 0x01)|(CFG_PCA1_CCMR2_INIT & 0x01))
#define CFG_PCA1_FLT_INT_EN ((CFG_PCA1_FLTCR_INIT>>6)&0x01 | (CFG_PCA1_FLTCR_INIT>>22)&0x01)
//</e>

//<e>PCA2
#define     CFG_PCA2_EN         0
//<i>CR_PCAx ʹܼĴ
//<i>0x000 b31~1 b0  
//<i>CR    -     PEN 
// <q0.0> PEN_PCAx /ʱпλ
//  <i>0: ֹPCAx/ʱ
//  <i>1: PCAx/ʱ
#define     CFG_PCA2_CR_INIT    0x0


//<e>ƵռձԶPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA_C2_EN       0


// <o> PCA2Ƶ (0-100000Hz) <0-100000> 
#define     CFG_PCA_PCA2_PR_F   4000
//<o0>PCA2A:  PCA2A/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA2A_DUTY  400
//<o0>PCA2B:  PCA2B/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA2B_DUTY  200
//<o0>PCA2C:  PCA2C/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA2C_DUTY  300
//</e>

//<e>üPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA2_E0_EN      0

//<h>PR_PCAx ֵ
//<i>0x018 b31~16 b15~0 
//<i>PR    PRH    PRL   
// <o0.0..15> PRL_PCAx ֵֽڣLSB<0-0xFFFF> 
// <o0.16..31> PRH_PCAx ֵֽڣMSB<0-0xFFFF> 
#define     CFG_PCA2_PR         0x0
//</h>

//<h>CCR0_PCAx Ƚ/׽ģ0׽ֵ
//<i>0x028 b31~16 b15~0 
//<i>CCR0  CCR0H  CCR0L 
// <o0.0..15> CCR0L_PCAx Ƚ/׽ģ0Ĵֽ <0-0xFFFF>
// <o0.16..31> CCR0H_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF>
//</h>
#define     CFG_PCA2_CCR0       0x0

//<h>CCR1_PCAx Ƚ/׽ģ1׽ֵ
//<i>0x02C b31~16 b15~0 
//<i>CCR1  CCR1H  CCR1L 
// <o0.0..15> CCR1L_PCAx Ƚ/׽ģ1ĴֽڣLSB<0-0xFFFF>
// <o0.16..31> CCR1H_PCAx Ƚ/׽ģ1ĴֽڣMSB<0-0xFFFF>
//</h>
#define     CFG_PCA2_CCR1       0x0

//<h>CCR2_PCAx Ƚ/׽ģ2׽ֵ
//<i>0x030 b31~16 b15~0 
//<i>CCR2  CCR2H  CCR2L 
// <o0.0..15> CCR2L_PCAx Ƚ/׽ģ2ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR2H_PCAx Ƚ/׽ģ2ĴֽڣMSB<0-0xFFFF>
//</h>
#define     CFG_PCA2_CCR2       0x0
//</e>

//<i>PSC_ԤƵ
//<i>0x014 b31~16 b15~0 
//<i>PSC   -      PSC   
// <o0.0..15> PSC_ԤƵ <0-0xFFFF>
#define     CFG_PCA2_PSC_INIT 0x0

// <o0.0..2> CPS_PCAx/ʱʱѡ 
//    <0=>PCLKʱ
//    <1=>PCAxECI½
//    <2=>RC128K
// <o0.6> SDEN_PCAx ģʽѡλ
//    <0=>PCAxϼݲģʽ
//    <1=>PCAxǲģʽ
//    <i>PCAxϼݲģʽ, ʱPCAxȽϲ׽ģģʽֹ
//    <i>PCAxǲģʽʱPCAxȽϲ׽ģľݲģʽֹ

//<h>CNT_PCAxĴ
//<i>0x010 b31~16 b15~0 
//<i>CNT   CNTH   CNTL  
// <o1.0..15> CNTL_PCAxֵ16λ <0-0xFFFF>
// <o1.16..31> CNTH_PCAxֵ16λ <0-0xFFFF>
//</h>

//<h>PCAx ʽ
//<i>0x004 b31~25 b24   b23   b22    b21    b20    b19~18 b17~16 b15 b14   b13   b12   b11 b10~9  b8  b7  b6   b5~3 b2~0 
//<i>CFGR  -      PUDMA CUDMA CC2DMA CC1DMA CC0DMA SYN    ECF    PUE CC2UE CC1UE CC0UE -   ADTSEL PIE CIE SDEN -    CPS  
//<i>PCAx趨ΪģʽеһʱȽϲ׽ģһбģʽʹҲЧġ
// <q0.7> CIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: CIFλʱ PCAx/ʱж
//λPCAx/ʱCIFжϵλ
// <q0.8> PIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: PIFλʱ PCAx/ʱڵж
//<i>λPCAx/ʱڣPIFжϵλ
// <q0.20> CC0DMA_PCA׽/Ƚͨ0¼ʱDMA
// <q0.21> CC1DMA_PCA׽/Ƚͨ1¼ʱDMA
// <q0.22> CC2DMA_PCA׽/Ƚͨ2¼ʱDMA
// <q0.23> CUDMA_PCA/ʱ¼ʱDMA
//    <i>0: /ʱ¼ʱֹDMA
//    <i>1: /ʱ¼ʱDMA
// <q0.24> PUDMAPCA/ʱƥ¼ʱDMA
//    <i>0: /ʱƥ¼ʱֹDMA
//    <i>1: /ʱƥ¼ʱDMA
// <o0.12> CC0UE_PCAxȽģ0ʹλ
//    <0=>д뵽CCR0ֵһЧ
//    <1=>д뵽CCR0ֵЧ
// <o0.13> CC1UE_PCAxȽģ1ʹλ
//    <0=>д뵽CCR1ֵһЧ
//    <1=>д뵽CCR1ֵЧ
// <o0.14> CC2UE_PCAxȽģ2ʹλ
//    <0=>д뵽CCR2ֵһЧ
//    <1=>д뵽CCR2ֵЧ
// <o0.15> PUE_PCAxֵʹλ
//    <0=>д뵽PRĴֵһЧ
//    <1=>д뵽PRĴֵЧ
// <o0.16..17> ECF_PCAx_ECIź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*PCLK
//    <2=>˲ʱΪ16*PCLK
//    <3=>˲ʱΪ32*PCLK
// <o0.18..19> SYN_PCAx_ͬʹλ
//    <0=>ͬ
//    <1=>PENλ10һPCAģͬ
//    <2=>PENλ10MCM0ʹλͬ
//    <3=>PENλ10MCM1ʹλͬ
// <o0.9..10> ADTSEL_PCAxADCѡλ
//    <0=>ADC
//    <1=>ϼԴADC
//    <2=>¼ADC
//    <3=>/¼ԴADC
//<i>ADTSELΪ10b11bֻSDEN=1ʱЧSDENΪ0ʱADTSELΪ1xb൱00b
//<i>ADTR_ȽϴĴ
//<i>0x038 b31~0 
//<i>ADTR  DATA  
// <o2.0..31> DATA_ADCȽϼĴ
//<i>˼ĴֵƥʱᴥADCת <0-0xFFFFFFFF>
//</h>
#define     CFG_PCA2_CFGR_INIT  0x0
#define     CFG_PCA2_CNT_INIT 0x0
#define     CFG_PCA2_ADTR_INIT 0x0


//<h>CCMR0_PCAxA ׽/Ƚģ0ƼĴ
//<i>0x01C b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR0 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7>  PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxA벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxAΪԴ
//    <1=>ѡȽ0C0OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA2_CCMR0_INIT 0x0


//<h>CCMR1_PCAxB ׽/Ƚģ1ƼĴ
//<i>0x020 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR1 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxB벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxBΪԴ
//    <1=>ѡȽ1C1OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA2_CCMR1_INIT 0x0

//<h>CCMR2_PCAxC ׽/Ƚģ2ƼĴ
//<i>0x024 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR2 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxC벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxCΪԴ
//    <1=>ѡȽ2C2OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA2_CCMR2_INIT 0x0

//<h>FLTCR_PWM0/1/2ϼⱣĴ
//<i>0x040 b31~30 b29    b28~27 b26~25 b24~23 b22    b21   b20       b19      b18    b17~16 b15~14 b13~12 b11~10 b9~7    b6     b5    b4        b3       b2 b1        b0       
//<i>FLTCR FOUT   FLT1EN CMP2S  CMP1S  CMP0S  FLT1IE FLT1M FLT1STATC FLT1STAT FLT2EN FPIN3S FPIN2S FPIN1S FPIN0S FLT2DEB FLT2IE FLT2M FLT2STATC FLT2STAT -  FLT0STATC FLT0STAT 
// <o0.30..31> FOUT_Ϸ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ

// <o0.29> FLT1EN_ϼ1ܿλ
//    <0=>ϼ1ֹܽ
//    <1=>ϼ1ܴ򿪣ԴѡȽ0/1/2
// <o0.21> FLT1M_⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.22> FLT1IE_жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.23..24> CMP0S_Ƚ0˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.25..26> CMP1S_Ƚ1˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.27..28> CMP2S_Ƚ2˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.18> FLT2EN_ϼ2ܿλ
//    <0=>ϼ2ֹܽ
//    <1=>ϼ2ܴ򿪣ԴΪFLT
// <o0.5> FLT2M_2⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.6> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.7..9> FLT2DEB_ϼ2˲ѡ
//    <0=>˲
//    <1=>0.5us
//    <2=>1us
//    <3=>1.5us
//    <4=>2us
//    <5=>3us
//    <6=>4us
//    <7=>6us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.10..11> FPIN0S_FLTPIN0˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> FPIN1S_FLTPIN1˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> FPIN2S_FLTPIN2˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.16..17> FPIN3S_FLTPIN3˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
//</h>
#define     CFG_PCA2_FLTCR_INIT 0x0


#if (CFG_PCA2_CFGR_INIT & 1) 
	#define CFG_PCA2_CLK		(unsigned long)(SYS_CLK/(CFG_PCA2_PSC_INIT + 1))
#elif (CFG_PCA2_CFGR_INIT & (1<<1)) 
	#define CFG_PCA2_CLK		(unsigned long)(128000/(CFG_PCA2_PSC_INIT + 1))		
#else
	#define CFG_PCA2_CLK		(unsigned long)(SYS_CLK/(CFG_PCA2_PSC_INIT + 1))
#endif


#if (CFG_PCA2_CFGR_INIT & (1<<6)) 
	#define CFG_PCA2_SDEN_MODE 2
#else
	#define CFG_PCA2_SDEN_MODE 1
#endif


#if (CFG_PCA_C2_EN == 1)	
	#define CFG_PCA2_PR_INIT (uint32_t)(CFG_PCA2_CLK/CFG_PCA_PCA2_PR_F/CFG_PCA2_SDEN_MODE)
#else
	#define CFG_PCA2_PR_INIT			CFG_PCA2_PR
#endif


#if (CFG_PCA_C2_EN == 1)	
	#define CFG_PCA2_CCR0_INIT (uint32_t)(CFG_PCA2_PR_INIT*CFG_PCA_PCA2A_DUTY/1000)
	#define CFG_PCA2_CCR1_INIT (uint32_t)(CFG_PCA2_PR_INIT*CFG_PCA_PCA2B_DUTY/1000)
	#define CFG_PCA2_CCR2_INIT (uint32_t)(CFG_PCA2_PR_INIT*CFG_PCA_PCA2C_DUTY/1000)
#else
	#define CFG_PCA2_CCR0_INIT 		CFG_PCA2_CCR0
	#define CFG_PCA2_CCR1_INIT 		CFG_PCA2_CCR1
	#define CFG_PCA2_CCR2_INIT 		CFG_PCA2_CCR2
#endif
#define CFG_PCA2_INT_EN (((CFG_PCA2_CFGR_INIT & 0x80)>>7) | ((CFG_PCA2_CFGR_INIT & 0x100)>>8)| (CFG_PCA2_CCMR0_INIT & 0x01) |(CFG_PCA2_CCMR1_INIT & 0x01)|(CFG_PCA2_CCMR2_INIT & 0x01))
#define CFG_PCA2_FLT_INT_EN ((CFG_PCA2_FLTCR_INIT>>6)&0x01 | (CFG_PCA2_FLTCR_INIT>>22)&0x01)
//</e>


//***********************************************************/
//<e>PCA3
#define     CFG_PCA3_EN         0
//<i>CR_PCAx ʹܼĴ
//<i>0x000 b31~1 b0  
//<i>CR    -     PEN 
// <q0.0> PEN_PCAx /ʱпλ
//  <i>0: ֹPCAx/ʱ
//  <i>1: PCAx/ʱ
#define     CFG_PCA3_CR_INIT    0

//<e>ƵռձԶPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA_C3_EN       0

// <o> PCA3Ƶ (0-100000Hz) <0-100000> 
#define     CFG_PCA_PCA3_PR_F   8000
//<o0>PCA3A:  PCA3A/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA3A_DUTY  400
//<o0>PCA3B:  PCA3B/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA3B_DUTY  200
//<o0>PCA3C:  PCA3C/Ƚ/ռձ (0.1%) <0-1000>
#define     CFG_PCA_PCA3C_DUTY  300
//</e>

//<e>üPRCCRx
//<i>÷ʽPCAxģʽ
#define     CFG_PCA3_E0_EN      0

//<h>PR_PCAx ֵ
//<i>0x018 b31~16 b15~0 
//<i>PR    PRH    PRL   
// <o0.0..15> PRL_PCAx ֵֽڣLSB<0-0xFFFF> 
// <o0.16..31> PRH_PCAx ֵֽڣMSB<0-0xFFFF> 
#define     CFG_PCA3_PR         0x0
//</h>

//<h>CCR0_PCAx Ƚ/׽ģ0׽ֵ
//<i>0x028 b31~16 b15~0 
//<i>CCR0  CCR0H  CCR0L 
// <o0.0..15> CCR0L_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF> 
// <o0.16..31> CCR0H_PCAx Ƚ/׽ģ0Ĵֽ<0-0xFFFF> 
//</h>
#define     CFG_PCA3_CCR0       0x0

//<h>CCR1_PCAx Ƚ/׽ģ1׽ֵ
//<i>0x02C b31~16 b15~0 
//<i>CCR1  CCR1H  CCR1L 
// <o0.0..15> CCR1L_PCAx Ƚ/׽ģ1ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR1H_PCAx Ƚ/׽ģ1ĴֽڣMSB<0-0xFFFF> 
//</h>
#define     CFG_PCA3_CCR1       0x0

//<h>CCR2_PCAx Ƚ/׽ģ2׽ֵ
//<i>0x030 b31~16 b15~0 
//<i>CCR2  CCR2H  CCR2L 
// <o0.0..15> CCR2L_PCAx Ƚ/׽ģ2ĴֽڣLSB<0-0xFFFF> 
// <o0.16..31> CCR2H_PCAx Ƚ/׽ģ2ĴֽڣMSB<0-0xFFFF> 
//</h>
#define     CFG_PCA3_CCR2       0x0
//</e>

//<i>PSC_ԤƵ
//<i>0x014 b31~16 b15~0 
//<i>PSC   -      PSC   
// <o0.0..15> PSC_ԤƵ <0-0xFFFF>
#define     CFG_PCA3_PSC_INIT 0x0

// <o0.0..2> CPS_PCAx/ʱʱѡ 
//    <0=>PCLKʱ
//    <1=>PCAxECI½
//    <2=>RC128K
// <o0.6> SDEN_PCAx ģʽѡλ
//    <0=>PCAxϼݲģʽ
//    <1=>PCAxǲģʽ
//    <i>PCAxϼݲģʽ, ʱPCAxȽϲ׽ģģʽֹ
//    <i>PCAxǲģʽʱPCAxȽϲ׽ģľݲģʽֹ
//<h>CNT_PCAxĴ
//<i>0x010 b31~16 b15~0 
//<i>CNT   CNTH   CNTL  
// <o1.0..15> CNTL_PCAxֵ16λ <0-0xFFFF>
// <o1.16..31> CNTH_PCAxֵ16λ <0-0xFFFF>
//</h>

//<h>PCAx ʽ
//<i>0x004 b31~25 b24   b23   b22    b21    b20    b19~18 b17~16 b15 b14   b13   b12   b11 b10~9  b8  b7  b6   b5~3 b2~0 
//<i>CFGR  -      PUDMA CUDMA CC2DMA CC1DMA CC0DMA SYN    ECF    PUE CC2UE CC1UE CC0UE -   ADTSEL PIE CIE SDEN -    CPS  
//<i>PCAx趨ΪģʽеһʱȽϲ׽ģһбģʽʹҲЧġ
// <q0.7> CIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: CIFλʱ PCAx/ʱж
//λPCAx/ʱCIFжϵλ
// <q0.8> PIE_PCAx/ʱжλ
//    <i>0: ֹж
//    <i>1: PIFλʱ PCAx/ʱڵж
//<i>λPCAx/ʱڣPIFжϵλ
// <q0.20> CC0DMA_PCA׽/Ƚͨ0¼ʱDMA
// <q0.21> CC1DMA_PCA׽/Ƚͨ1¼ʱDMA
// <q0.22> CC2DMA_PCA׽/Ƚͨ2¼ʱDMA
// <q0.23> CUDMA_PCA/ʱ¼ʱDMA
//    <i>0: /ʱ¼ʱֹDMA
//    <i>1: /ʱ¼ʱDMA
// <q0.24> PUDMAPCA/ʱƥ¼ʱDMA
//    <i>0: /ʱƥ¼ʱֹDMA
//    <i>1: /ʱƥ¼ʱDMA
// <o0.12> CC0UE_PCAxȽģ0ʹλ
//    <0=>д뵽CCR0ֵһЧ
//    <1=>д뵽CCR0ֵЧ
// <o0.13> CC1UE_PCAxȽģ1ʹλ
//    <0=>д뵽CCR1ֵһЧ
//    <1=>д뵽CCR1ֵЧ
// <o0.14> CC2UE_PCAxȽģ2ʹλ
//    <0=>д뵽CCR2ֵһЧ
//    <1=>д뵽CCR2ֵЧ
// <o0.15> PUE_PCAxֵʹλ
//    <0=>д뵽PRĴֵһЧ
//    <1=>д뵽PRĴֵЧ
// <o0.16..17> ECF_PCAx_ECIź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*PCLK
//    <2=>˲ʱΪ16*PCLK
//    <3=>˲ʱΪ32*PCLK
// <o0.18..19> SYN_PCAx_ͬʹλ
//    <0=>ͬ
//    <1=>PENλ10һPCAģͬ
//    <2=>PENλ10MCM0ʹλͬ
//    <3=>PENλ10MCM1ʹλͬ
// <o0.9..10> ADTSEL_PCAxADCѡλ
//    <0=>ADC
//    <1=>ϼԴADC
//    <2=>¼ADC
//    <3=>/¼ԴADC
//<i>ADTSELΪ10b11bֻSDEN=1ʱЧSDENΪ0ʱADTSELΪ1xb൱00b
//<i>ADTR_ȽϴĴ
//<i>0x038 b31~0 
//<i>ADTR  DATA  
// <o2.0..31> DATA_ADCȽϼĴ
//<i>˼ĴֵƥʱᴥADCת <0-0xFFFFFFFF>
//</h>
#define     CFG_PCA3_CFGR_INIT 0x0
#define     CFG_PCA3_CNT_INIT 0x0
#define     CFG_PCA3_ADTR_INIT 0x0


//<h>CCMR0_PCAxA ׽/Ƚģ0ƼĴ
//<i>0x01C b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR0 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxA벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxAΪԴ
//    <1=>ѡȽ0C0OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA3_CCMR0_INIT 0x0


//<h>CCMR1_PCAxB ׽/Ƚģ1ƼĴ
//<i>0x020 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR1 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxB벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxBΪԴ
//    <1=>ѡȽ1C1OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA3_CCMR1_INIT 0x0


//<h>CCMR2_PCAxC ׽/Ƚģ2ƼĴ
//<i>0x024 b31~15 b14~12   b11~9 b8 b7~6 b5~4 b3  b2  b1 b0   
//<i>CCMR2 -      CSSEL ICF   CC SM   FS   CEN TCP -  CCIE 
// <q0.3> CEN_Ƚ/׽ģ鹦ʹλ
//    <i>0: ֹȽ/׽ģ0
//    <i>1: ʹܱȽ/׽ģ0
// <o0.4..7> PCAxģʽѡλ
//    <0=>׽ʽش
//		<2=>׽ʽڸش
//  	<3=>׽ʽش
//    <4=>Ƚƥ,ƥ
//    <6=>Ƚƥ,ƥ
//    <8=>Ƶģʽ
//  	<12=>PWMģʽ16λPWM1ģʽ
//  	<13=>PWMģʽ32λPWM2ģʽ
//  	<14=>PWMģʽ32λPWM3ģʽ
//  	<15=>PWMģʽ32λPWM4ģʽ
// <o0.2> TCP_λ
//    <0=>ʼƽΪߵƽ
//    <1=>ʼƽΪ͵ƽ
// <q0.0> CCIE_׽/Ƚϱ־жλ
//    <i>0: ֹȽϲ׽/Ƚж
//    <i>1: ֹȽϲ׽/Ƚжϣ CCIFλ1ʱ׽/Ƚϱ־ж
// <o0.8> CC_ʱʹλ
//    <0=>¼ʱֵ
//    <1=>¼ʱֵ
// <o0.9..11> ICF_PCAxC벶׽ź˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ40*PCLK
//    <2=>˲ʱΪ80*PCLK
//    <3=>˲ʱΪ160*PCLK
//    <4=>˲ʱΪ320*PCLK
//    <5=>˲ʱΪ640*PCLK
//    <6=>˲ʱΪ1280*PCLK
//    <7=>˲ʱΪ2560*PCLK
//<i>Mode0ģʽЧ
// <o0.12..14> CSSEL_Դѡλ
//    <0=>ѡPCAxCΪԴ
//    <1=>ѡȽ2C2OUTΪԴ
//    <2=>ѡSDFMFLT1_COMPHBΪԴ
//    <3=>ѡSDFMFLT2_COMPHBΪԴ
//    <4=>ѡSDFMFLT3_COMPHBΪԴ
//    <5=>ѡSDFMFLT4_COMPHBΪԴ
//<i>λֻڲģʽЧSM=00ģʽ²Ϊ1
//</h>
#define     CFG_PCA3_CCMR2_INIT 0x0

//<h>FLTCR_PWM0/1/2ϼⱣĴ
//<i>0x040 b31~30 b29    b28~27 b26~25 b24~23 b22    b21   b20       b19      b18    b17~16 b15~14 b13~12 b11~10 b9~7    b6     b5    b4        b3       b2 b1        b0       
//<i>FLTCR FOUT   FLT1EN CMP2S  CMP1S  CMP0S  FLT1IE FLT1M FLT1STATC FLT1STAT FLT2EN FPIN3S FPIN2S FPIN1S FPIN0S FLT2DEB FLT2IE FLT2M FLT2STATC FLT2STAT -  FLT0STATC FLT0STAT 
// <o0.30..31> FOUT_Ϸ״̬ѡλ
//    <0=>̬
//    <2=>͵ƽ
//    <3=>ߵƽ
// <o0.29> FLT1EN_ϼ1ܿλ
//    <0=>ϼ1ֹܽ
//    <1=>ϼ1ܴ򿪣ԴѡȽ0/1/2
// <o0.21> FLT1M_⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.22> FLT1IE_жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.23..24> CMP0S_Ƚ0˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.25..26> CMP1S_Ƚ1˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.27..28> CMP2S_Ƚ2˲ΪPCAϼ1Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ

// <o0.18> FLT2EN_ϼ2ܿλ
//    <0=>ϼ2ֹܽ
//    <1=>ϼ2ܴ򿪣ԴΪFLT
// <o0.5> FLT2M_2⹦ģʽѡ
//    <0=>ģʽ
//    <1=>ģʽ
//    <i>ģʽ⵽ЧFLT1STAT Ӳ1PWMֹͣΪЧʱ״ֲ̬䣬ֻеFLTSTATPWMβŻPWMʱ0ʱ̻ڼĴPWMPֵƥʱָһֱЧʱFLT1STAT޷
//    <i>ģʽPWMֱɹϼƣЧFLT1STATӲ1رPWMΪЧFLT1STATԶ0PWM0/1/2νPWMʱ0ʱԶָ
// <q0.6> FLT2IE_2жλ
//    <i>0: ֹϼж
//    <i>1: ϼж
// <o0.7..9> FLT2DEB_ϼ2˲ѡ
//    <0=>˲
//    <1=>0.5us
//    <2=>1us
//    <3=>1.5us
//    <4=>2us
//    <5=>3us
//    <6=>4us
//    <7=>6us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.10..11> FPIN0S_FLTPIN0˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.12..13> FPIN1S_FLTPIN1˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.14..15> FPIN2S_FLTPIN2˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
// <o0.16..17> FPIN3S_FLTPIN3˲ΪPCAϼ2Դѡλ
//    <0=>Ч
//    <2=>͵ƽЧ
//    <3=>ߵƽЧ
//</h>
#define     CFG_PCA3_FLTCR_INIT 0x0


#if (CFG_PCA3_CFGR_INIT & 1) 
	#define CFG_PCA3_CLK		(unsigned long)(SYS_CLK/(CFG_PCA3_PSC_INIT + 1))
#elif (CFG_PCA3_CFGR_INIT & (1<<1)) 
	#define CFG_PCA3_CLK		(unsigned long)(128000/(CFG_PCA3_PSC_INIT + 1))		
#else
	#define CFG_PCA3_CLK		(unsigned long)(SYS_CLK/(CFG_PCA3_PSC_INIT + 1))
#endif

#if (CFG_PCA3_CFGR_INIT & (1<<6)) 
	#define CFG_PCA3_SDEN_MODE 2
#else
	#define CFG_PCA3_SDEN_MODE 1
#endif


#if (CFG_PCA_C3_EN == 1)	
	#define CFG_PCA3_PR_INIT (uint32_t)(CFG_PCA3_CLK/CFG_PCA_PCA3_PR_F/CFG_PCA3_SDEN_MODE)
	#define CFG_PCA3_CCR0_INIT (uint32_t)(CFG_PCA3_PR_INIT*CFG_PCA_PCA3A_DUTY/1000)
	#define CFG_PCA3_CCR1_INIT (uint32_t)(CFG_PCA3_PR_INIT*CFG_PCA_PCA3B_DUTY/1000)
	#define CFG_PCA3_CCR2_INIT (uint32_t)(CFG_PCA3_PR_INIT*CFG_PCA_PCA3C_DUTY/1000)
#else
	#define CFG_PCA3_PR_INIT			CFG_PCA3_PR
	#define CFG_PCA3_CCR0_INIT 		CFG_PCA3_CCR0
	#define CFG_PCA3_CCR1_INIT 		CFG_PCA3_CCR1
	#define CFG_PCA3_CCR2_INIT 		CFG_PCA3_CCR2
#endif

#define CFG_PCA3_INT_EN (((CFG_PCA3_CFGR_INIT & 0x80)>>7) | ((CFG_PCA3_CFGR_INIT & 0x100)>>8)| (CFG_PCA3_CCMR0_INIT & 0x01) |(CFG_PCA3_CCMR1_INIT & 0x01)|(CFG_PCA3_CCMR2_INIT & 0x01))
#define CFG_PCA3_FLT_INT_EN ((CFG_PCA3_FLTCR_INIT>>6)&0x01 | (CFG_PCA3_FLTCR_INIT>>22)&0x01)

//***********************************************************/
//</e>
//***********************************************************/
//</e>



//<e>TIMER
#define     CFG_TIM_EN          0
//***********************************************************/
//<e>TIM0
#define     CFG_TIM0_EN         0
//<i>CR_TIMƼĴ
//<i>0x000 b31~16 b15    b14 b13~12 b11 b10 b9   b8     b7 b6 b5~4 b3  b2~1 b0  
//<i>CR    -      CASCEN -   ECF    -   TC  ETEN TRIGEN IE -  CLKS OPM -    STR 
// <o0.4..5> CLKS_TIMxʱԴICLKѡλ
//    <0=>ڲAPB0ʱ
//    <1=>Tn(n=0,1)룬Զ
//    <2=>ڲƵRC4Ƶ128KHz CLK / 4 = 32KHz
// <o1>TIM0ʼֵ(us) <0-100000000>

// <o2>TIM0ֵ(us) <0-100000000>

//<i>PSQ_TIMԤƵĴ
//<i>0x00C b31~16 b15~0 
//<i>PSQ   PSQH   PSQL  
// <o3> ʱƵֵ<0-65535>
//<i>ȫTIMЧ
//<i>ʱʱΪ(ICLK/(PSQ[15:0]+1))  

// <o0.3> ģʽ
//    <0=>ʱxʱʱֹͣ
//    <1=>ʱxʱʱֹͣ
// <q0.7> TIMxжʹλ
//    <i>0: ֹʱTIMxж
//    <i>1: ʱTIMxж
// <q0.8> TIMxⲿģʹλ
//    <i>0: ʱxֹģ
//    <i>1: ʱxģ
// <q0.9> TIMx¼ʹλ
//    <i>0: ֹʱں
//    <i>1: ʱں
// <q0.10> Ƚλ
//    <i>0: ֹʱxȽ
//    <i>1: ʱxȽ
//<i>ʱȽʱ(TC=1)ʱԴѡTn(n=0,1)ڡ
// <o0.12..13> TxΪʱԴ˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*APB0
//    <2=>˲ʱΪ16*APB0
//    <3=>˲ʱΪ32*APB0
//<q4>ʱ0ϵʼʱ
#define     CFG_TIM0_CR_INIT    0x0
#define     CFG_TIM0_PERIOD_VALUE 0
#define     CFG_TIM0_RL_VALUE   0
#define     CFG_TIM0_PSQ_INIT 0x0
#define     CFG_TIM0_CR_START   0


#if ((CFG_TIM0_CR_INIT & (1<<7)) != 0)
	#define CFG_TIM0_INT_EN 1
#else
	#define CFG_TIM0_INT_EN 0
#endif


#if (CFG_TIM0_CR_INIT & 0x30) == 0x0
#define CFG_TIM0_CLK		(unsigned long)(SYS_CLK/APB_F/(CFG_TIM0_PSQ_INIT + 1)) 
#elif (CFG_TIM0_CR_INIT & 0x30) == 0x10
#define CFG_TIM0_CLK		100000 	
#elif (CFG_TIM0_CR_INIT & 0x30) == 0x20
#define CFG_TIM0_CLK		128000 	
#endif


#if (CFG_TIM0_CR_INIT & (1<<15))
#define CFG_TIM0_TCNT_INIT ((unsigned long)(CFG_TIM0_PERIOD_VALUE*(CFG_TIM0_CLK/1000000.0)))
#define CFG_TIM0_TPR_INIT ((unsigned long)(CFG_TIM0_RL_VALUE*(CFG_TIM0_CLK/1000000.0)))
#else
#define CFG_TIM0_TCNT_INIT (unsigned long)(CFG_TIM0_PERIOD_VALUE*(CFG_TIM0_CLK/1000000.0))
#define CFG_TIM0_TPR_INIT (unsigned long)(CFG_TIM0_RL_VALUE*(CFG_TIM0_CLK/1000000.0))
#endif
//</e>


//***********************************************************/
//<e>TIM1
#define     CFG_TIM1_EN         0
//<i>CR_TIMƼĴ
//<i>0x000 b31~16 b15    b14 b13~12 b11 b10 b9   b8     b7 b6 b5~4 b3  b2~1 b0  
//<i>CR    -      CASCEN -   ECF    -   TC  ETEN TRIGEN IE -  CLKS OPM -    STR 
// <o0.4..5> TIMxʱԴICLKѡλ
//    <0=>ڲAPB0ʱ
//    <1=>Tn(n=0,1)룬Զ
//    <2=>ڲƵRC4Ƶ128KHz CLK / 4 = 32KHz
// <o1>TIM1ʼֵ(us) <0-100000000>

// <o2>TIM1ֵ(us) <0-100000000>

//<i>TIMԤƵĴ
//<i>0x00C b31~16 b15~0 
//<i>PSQ   PSQH   PSQL  
// <o3> ʱƵֵ<0-65535>
//<i>ȫTIMЧ
//<i>ʱʱΪ(ICLK/(PSQ[15:0]+1))  
// <o0.3> OPM_ģʽ
//    <0=>ʱxʱʱֹͣ
//    <1=>ʱxʱʱֹͣ
// <q0.7> TIMxжʹλ
//    <i>0: ֹʱTIMxж
//    <i>1: ʱTIMxж
// <q0.8> TIMxⲿģʹλ
//    <i>0: ʱxֹģ
//    <i>1: ʱxģ
// <q0.9> TIMx¼ʹλ
//    <i>0: ֹʱں
//    <i>1: ʱں
// <q0.10> Ƚλ
//    <i>0: ֹʱxȽ
//    <i>1: ʱxȽ
//<i>ʱȽʱ(TC=1)ʱԴѡTn(n=0,1)ڡ
// <o0.12..13> TxΪʱԴ˲ʱ
//    <0=>˲
//    <1=>˲ʱΪ8*APB0
//    <2=>˲ʱΪ16*APB0
//    <3=>˲ʱΪ32*APB0
//<q4>TIM1ϵʼʱ

#define     CFG_TIM1_CR_INIT    0x0
#define     CFG_TIM1_PERIOD_VALUE 0
#define     CFG_TIM1_RL_VALUE   0
#define     CFG_TIM1_PSQ_INIT 0x0
#define     CFG_TIM1_CR_START   0


#if ((CFG_TIM1_CR_INIT & (1<<7)) != 0)
	#define CFG_TIM1_INT_EN 1
#else
	#define CFG_TIM1_INT_EN 0
#endif

#if (CFG_TIM1_CR_INIT & 0x30) == 0x0
#define CFG_TIM1_CLK		(unsigned long)(SYS_CLK/APB_F/(CFG_TIM1_PSQ_INIT + 1)) 
#elif (CFG_TIM1_CR_INIT & 0x30) == 0x10
#define CFG_TIM1_CLK		100000 	
#elif (CFG_TIM1_CR_INIT & 0x30) == 0x20
#define CFG_TIM1_CLK		128000 	
#endif

#if (CFG_TIM1_CR_INIT & (1<<15))
#define CFG_TIM1_TCNT_INIT ((unsigned long)(CFG_TIM1_PERIOD_VALUE*(CFG_TIM1_CLK/1000000.0)))
#define CFG_TIM1_TPR_INIT ((unsigned long)(CFG_TIM1_RL_VALUE*(CFG_TIM1_CLK/1000000.0)))
#else
#define CFG_TIM1_TCNT_INIT (unsigned long)(CFG_TIM1_PERIOD_VALUE*(CFG_TIM1_CLK/1000000.0))
#define CFG_TIM1_TPR_INIT (unsigned long)(CFG_TIM1_RL_VALUE*(CFG_TIM1_CLK/1000000.0))
#endif
//</e>
//</e>

//<e>MACP
#define     CFG_MACP_EN         0
//<h>CORDCSR0_CORDIC״̬Ĵ0
//<i>0x000    b31~15 b14~13 b12~8 b7   b6~5   b4 b3    b2   b1  b0  
//<i>CORDCSR0 -      ARITH  ITERA MODE FORMAT -  XYMRS KADJ OVF RUN 
// <o0.13..14> ARITH_CORDIC㷨ѡ
//   <0=>Բ
//   <1=>Ժ
//   <2=>˫
// <o0.8..12> ITERA_ѡ<0-31>
//<i>Ϊ31ĬϰFORMAT
//    <i>IQ24/IQ26ʽ£Ĭ27ε
//    <i>IQ15ʽ£16ΣҪûֶ޸
//<i>һҪݸʽƥ䣬ûҲΪھٶ֮Ȩ⣬ϸľҪο۷ûȷϣ˴ֻжϡ
// <o0.7> MODE_CORDICģʽѡλ
//    <0=>תģʽ
//    <1=>ģʽ
// <o0.5..6> FORMAT_CORDICݸʽ
//   <0=>IQ26ʽ
//   <1=>IQ24ʽ
//   <2=>IQ15ʽ
//<i>IQ26/IQ24ʹ27λCORDIC㣬IQ15ʹ16λCORDIC㡣
//<i>ݸʽ֣ûʹʽ룬Ҫе
// <o0.3> XYMRS_CORDICXYȵѡλ
//    <0=>һεXYΪս
//    <1=>һεXY2Ϊս
//    <i>ȷXYMRSλɱ
// <o0.2> KADJ_CORDICKӵѡλ
//    <0=>1/K
//    <1=>1/KK
//    <i>K=1.64676
//</h>
#define     CFG_MACP_CORDCSR0_INIT 0x0


//<h>CORDCSR1_CORDIC״̬Ĵ1
//<i>0x010    b31~15 b14~13 b12~8 b7   b6~5   b4 b3    b2   b1  b0  
//<i>CORDCSR1 -      ARITH  ITERA MODE FORMAT -  XYMRS KADJ OVF RUN 
// <o0.13..14> ARITH_CORDIC㷨ѡ
//   <0=>Բ
//   <1=>Ժ
//   <2=>˫
// <o0.8..12> ITERA_ѡ<0-31>
//<i>Ϊ31ĬϰFORMAT
//    <i>IQ24/IQ26ʽ£Ĭ27ε
//    <i>IQ15ʽ£16ΣҪûֶ޸
//<i>һҪݸʽƥ䣬ûҲΪھٶ֮Ȩ⣬ϸľҪο۷ûȷϣ˴ֻжϡ
// <o0.7> MODE_CORDICģʽѡλ
//    <0=>תģʽ
//    <1=>ģʽ
// <o0.5..6> FORMAT_CORDICݸʽ
//   <0=>IQ26ʽ
//   <1=>IQ24ʽ
//   <2=>IQ15ʽ
//<i>IQ26/IQ24ʹ27λCORDIC㣬IQ15ʹ16λCORDIC㡣
//<i>ݸʽ֣ûʹʽ룬Ҫе
// <o0.3> XYMRS_CORDICXYȵѡλ
//    <0=>һεXYΪս
//    <1=>һεXY2Ϊս
//    <i>ȷXYMRSλɱ
// <o0.2> KADJ_CORDICKӵѡλ
//    <0=>1/K
//    <1=>1/KK
//    <i>K=1.64676
//</h>
#define     CFG_MACP_CORDCSR1_INIT 0x0

//<h>SVUALPHA_SVPWMUalphaֵĴ
//<i>0x204    b31~0    
//<i>SVUALPHA SVUALPHA 
// <o0.0..31> SVUALPHA_SVPWMUalphaֵ <0-2147483648>
//</h>
#define     CFG_MACP_SVUALPHA_INIT 0x0

//<h>SVUBETA_SVPWMUbetaֵĴ
//<i>0x208   b31~0   
//<i>SVUBETA SVUBETA 
// <o0.0..31> SVUBETA_SVPWMUbetaֵ <0-2147483648>
//</h>
#define     CFG_MACP_SVUBETA_INIT 0x0

//<h>SVIQN_SVPWMݸʽüĴ
//<i>0x20C b31~16 b15    b14    b13    b12    b11    b10    b9    b8    b7~0  
//<i>SVIQN -      OVERCH OVERBH OVERAH OVERCL OVERBL OVERAL SATEN SVTYP SVIQN 
// <o0.0..7> SVIQN_SVPWMݸʽ
// <o0.8> SVTYP_ʽ߶ʽѡλ
//  <0=>߶ʽ
//  <1=>ʽ
// <q0.9> SATEN_ʹλ
//  <i>0: ֹ
//  <i>1: ʹ
//</h>
#define     CFG_MACP_SVIQN_INIT 0x0
//***********************************************************/
//</e>


//<e>ADC
#define CFG_ADC_EN	0
//***********************************************************/
//<e>ADC0
#define CFG_ADC0_EN	0
//<h>ADCON1_ADCƼĴ1
//<i>0x000  b30~24  b23   b22   b21~15  b14   b13   b12  b11~5  b4   b3   b2   b1~0     
//<i>ADCON1 ADSTRS2 ADIE2 ADDE2 ADSTRS1 ADIE1 ADDE1 ADON ADSTRS ADIE ADDE REFC ADCTU
// <q0.12> ADON_ADCλ
//    <i>0: ֹADCģ
//    <i>1: ADCģ
// <o0.0..1> ADCTU_ADCͨеתʽλ
//    <0=>תʽ
//    <2=>תʽ
//    <i>ע:޸ĸÿλһADתʱŲЧ
// <o0.2> REFC_׼ѹѡλ
//    <0=>ѡAVDDΪ׼ѹ
//    <1=>ѡⲿVREF˿Ϊ׼ѹ

// <q0.3> ADDE_ͨAתDMAʹλ
//    <i>0: ֹADCAתDMA
//    <i>1: ʹADCAתDMA
// <q0.4> ADIE_ͨAתжʹλ
//    <i>0: ֹADCAתж
//    <i>1: ʹADCAתж

// <q0.13> ADDE1_ͨBתDMAʹλ
//    <i>0: ֹADCBתDMA
//    <i>1: ʹADCBתDMA
// <q0.14> ADIE1_ͨBתжʹλ
//    <i>0: ֹADCBתж
//    <i>1: ʹADCBתж

// <q0.22> ADDE2_ͨCתDMAʹλ
//    <i>0: ֹADCCתDMA
//    <i>1: ʹADCCתDMA
// <q0.23> ADIE2_ͨCתжʹλ
//    <i>0: ֹADCCתж
//    <i>1: ʹADCCתж
#define CFG_ADC0_ADCON1_VALUE   0x0
// <e> ADSTRS_ADCAӲԴѡ(5ѡ1)
#define CFG_ADC0_ADCON1_ADSTRSA 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC0_ADTRGA_EX_EN 0x0
#define CFG_ADC0_ADTRGA_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC0_ADTRGA_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGA_MCM0_EN 0x0
#define CFG_ADC0_ADTRGA_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGA_MCM1_EN 0x0
#define CFG_ADC0_ADTRGA_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0	PCA0TRG	PCA0Ƚƥ䴥
//    <q1.1>PCA1	PCA1TRG	PCA1Ƚƥ䴥
//    <q1.2>PCA2	PCA2TRG	PCA2Ƚƥ䴥
//    <q1.3>PCA3	PCA3TRG	PCA3Ƚƥ䴥
//</e>
#define CFG_ADC0_ADTRGA_PCA_EN 0x0
#define CFG_ADC0_ADTRGA_PCA_VALUE 0x0

//</e>
#define IS_ADC0_ADCON1_ADSTRSA() ((CFG_ADC0_ADTRGA_EX_EN && (CFG_ADC0_ADTRGA_T1_EN || CFG_ADC0_ADTRGA_MCM1_EN || CFG_ADC0_ADTRGA_MCM0_EN || CFG_ADC0_ADTRGA_PCA_EN ))\
									||(CFG_ADC0_ADTRGA_T1_EN && (CFG_ADC0_ADTRGA_EX_EN || CFG_ADC0_ADTRGA_MCM1_EN || CFG_ADC0_ADTRGA_MCM0_EN || CFG_ADC0_ADTRGA_PCA_EN ))\
									||(CFG_ADC0_ADTRGA_MCM1_EN && (CFG_ADC0_ADTRGA_EX_EN || CFG_ADC0_ADTRGA_T1_EN || CFG_ADC0_ADTRGA_MCM0_EN || CFG_ADC0_ADTRGA_PCA_EN ))\
									||(CFG_ADC0_ADTRGA_MCM0_EN && (CFG_ADC0_ADTRGA_EX_EN || CFG_ADC0_ADTRGA_T1_EN || CFG_ADC0_ADTRGA_MCM1_EN || CFG_ADC0_ADTRGA_PCA_EN ))\
									||(CFG_ADC0_ADTRGA_PCA_EN && (CFG_ADC0_ADTRGA_EX_EN || CFG_ADC0_ADTRGA_T1_EN || CFG_ADC0_ADTRGA_MCM1_EN || CFG_ADC0_ADTRGA_MCM0_EN ))\
									)
                                    
#if ((CFG_ADC0_ADCON1_ADSTRSA) && (CFG_ADC0_EN) && IS_ADC0_ADCON1_ADSTRSA())
#error "ADC0AӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_EX_EN && (CFG_ADC0_ADTRGA_EX_VALUE==0))
#error "ADC0AΪӲԴתδⲿţ"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_MCM1_EN && (CFG_ADC0_ADTRGA_MCM1_VALUE==0))
#error "ADC0AΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_MCM0_EN && (CFG_ADC0_ADTRGA_MCM0_VALUE==0))
#error "ADC0AΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_PCA_EN && (CFG_ADC0_ADTRGA_PCA_VALUE==0))
#error "ADC0AΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_EX_EN)
#define CFG_ADC0_ADCON1_ADSTRSA_INIT (CFG_ADC0_ADTRGA_EX_VALUE<<5)
#elif(CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_T1_EN)
#define CFG_ADC0_ADCON1_ADSTRSA_INIT ((1<<9))
#elif(CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_MCM0_EN)
#define CFG_ADC0_ADCON1_ADSTRSA_INIT ((2<<9)|(CFG_ADC0_ADTRGA_MCM0_VALUE<<5))
#elif(CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_MCM1_EN)
#define CFG_ADC0_ADCON1_ADSTRSA_INIT ((3<<9)|(CFG_ADC0_ADTRGA_MCM1_VALUE<<5))
#elif(CFG_ADC0_ADCON1_ADSTRSA && CFG_ADC0_EN && CFG_ADC0_ADTRGA_PCA_EN)
#define CFG_ADC0_ADCON1_ADSTRSA_INIT ((4<<9)|(CFG_ADC0_ADTRGA_PCA_VALUE<<5))
#else
#define CFG_ADC0_ADCON1_ADSTRSA_INIT 0x0
#endif

// <e> ADSTRS1_ADCBӲԴѡ(5ѡ1)
#define CFG_ADC0_ADCON1_ADSTRSB 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC0_ADTRGB_EX_EN 0x0
#define CFG_ADC0_ADTRGB_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC0_ADTRGB_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGB_MCM0_EN 0x0
#define CFG_ADC0_ADTRGB_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGB_MCM1_EN 0x0
#define CFG_ADC0_ADTRGB_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC0_ADTRGB_PCA_EN 0x0
#define CFG_ADC0_ADTRGB_PCA_VALUE 0x0

//</e>
#define IS_ADC0_ADCON1_ADSTRSB() ((CFG_ADC0_ADTRGB_EX_EN && (CFG_ADC0_ADTRGB_T1_EN || CFG_ADC0_ADTRGB_MCM1_EN || CFG_ADC0_ADTRGB_MCM0_EN || CFG_ADC0_ADTRGB_PCA_EN ))\
									||(CFG_ADC0_ADTRGB_T1_EN && (CFG_ADC0_ADTRGB_EX_EN || CFG_ADC0_ADTRGB_MCM1_EN || CFG_ADC0_ADTRGB_MCM0_EN || CFG_ADC0_ADTRGB_PCA_EN ))\
									||(CFG_ADC0_ADTRGB_MCM1_EN && (CFG_ADC0_ADTRGB_EX_EN || CFG_ADC0_ADTRGB_T1_EN || CFG_ADC0_ADTRGB_MCM0_EN || CFG_ADC0_ADTRGB_PCA_EN ))\
									||(CFG_ADC0_ADTRGB_MCM0_EN && (CFG_ADC0_ADTRGB_EX_EN || CFG_ADC0_ADTRGB_T1_EN || CFG_ADC0_ADTRGB_MCM1_EN || CFG_ADC0_ADTRGB_PCA_EN ))\
									||(CFG_ADC0_ADTRGB_PCA_EN && (CFG_ADC0_ADTRGB_EX_EN || CFG_ADC0_ADTRGB_T1_EN || CFG_ADC0_ADTRGB_MCM1_EN || CFG_ADC0_ADTRGB_MCM0_EN ))\
									)
                                    
#if ((CFG_ADC0_ADCON1_ADSTRSB) && (CFG_ADC0_EN) && IS_ADC0_ADCON1_ADSTRSB())
#error "ADC0BӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_EX_EN && (CFG_ADC0_ADTRGB_EX_VALUE==0))
#error "ADC0BΪӲԴתδⲿţ"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_MCM1_EN && (CFG_ADC0_ADTRGB_MCM1_VALUE==0))
#error "ADC0BΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_MCM0_EN && (CFG_ADC0_ADTRGB_MCM0_VALUE==0))
#error "ADC0BΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_PCA_EN && (CFG_ADC0_ADTRGB_PCA_VALUE==0))
#error "ADC0BΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_EX_EN)
#define CFG_ADC0_ADCON1_ADSTRSB_INIT ((uint32_t)(CFG_ADC0_ADTRGB_EX_VALUE<<15))
#elif(CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_T1_EN)
#define CFG_ADC0_ADCON1_ADSTRSB_INIT ((uint32_t)(1<<19))
#elif(CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_MCM0_EN)
#define CFG_ADC0_ADCON1_ADSTRSB_INIT ((uint32_t)(2<<19)|(CFG_ADC0_ADTRGB_MCM0_VALUE<<15))
#elif(CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_MCM1_EN)
#define CFG_ADC0_ADCON1_ADSTRSB_INIT ((uint32_t)(3<<19)|(CFG_ADC0_ADTRGB_MCM1_VALUE<<15))
#elif(CFG_ADC0_ADCON1_ADSTRSB && CFG_ADC0_EN && CFG_ADC0_ADTRGB_PCA_EN)
#define CFG_ADC0_ADCON1_ADSTRSB_INIT ((uint32_t)(4<<19)|(CFG_ADC0_ADTRGB_PCA_VALUE<<15))
#else
#define CFG_ADC0_ADCON1_ADSTRSB_INIT 0x0
#endif

// <e> ADSTRS2_ADCCӲԴѡ(5ѡ1)
#define CFG_ADC0_ADCON1_ADSTRSC 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC0_ADTRGC_EX_EN 0x0
#define CFG_ADC0_ADTRGC_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC0_ADTRGC_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGC_MCM0_EN 0x0
#define CFG_ADC0_ADTRGC_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC0_ADTRGC_MCM1_EN 0x0
#define CFG_ADC0_ADTRGC_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC0_ADTRGC_PCA_EN 0x0
#define CFG_ADC0_ADTRGC_PCA_VALUE 0x0

//</e>
#define IS_ADC0_ADCON1_ADSTRSC() ((CFG_ADC0_ADTRGC_EX_EN && (CFG_ADC0_ADTRGC_T1_EN || CFG_ADC0_ADTRGC_MCM0_EN || CFG_ADC0_ADTRGC_MCM1_EN || CFG_ADC0_ADTRGC_PCA_EN ))\
									||(CFG_ADC0_ADTRGC_T1_EN && (CFG_ADC0_ADTRGC_EX_EN || CFG_ADC0_ADTRGC_MCM1_EN || CFG_ADC0_ADTRGC_MCM0_EN || CFG_ADC0_ADTRGC_PCA_EN ))\
									||(CFG_ADC0_ADTRGC_MCM0_EN && (CFG_ADC0_ADTRGC_EX_EN || CFG_ADC0_ADTRGC_T1_EN || CFG_ADC0_ADTRGC_MCM0_EN || CFG_ADC0_ADTRGC_PCA_EN ))\
									||(CFG_ADC0_ADTRGC_MCM1_EN && (CFG_ADC0_ADTRGC_EX_EN || CFG_ADC0_ADTRGC_T1_EN || CFG_ADC0_ADTRGC_MCM1_EN || CFG_ADC0_ADTRGC_PCA_EN ))\
									||(CFG_ADC0_ADTRGC_PCA_EN && (CFG_ADC0_ADTRGC_EX_EN || CFG_ADC0_ADTRGC_T1_EN || CFG_ADC0_ADTRGC_MCM1_EN || CFG_ADC0_ADTRGC_MCM0_EN ))\
									)
                                    
#if ((CFG_ADC0_ADCON1_ADSTRSC) && (CFG_ADC0_EN) && IS_ADC0_ADCON1_ADSTRSC())
#error "ADC0CӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_EX_EN && (CFG_ADC0_ADTRGC_EX_VALUE==0))
#error "ADC0CΪӲԴתδⲿţ"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_MCM0_EN && (CFG_ADC0_ADTRGC_MCM0_VALUE==0))
#error "ADC0CΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_MCM1_EN && (CFG_ADC0_ADTRGC_MCM1_VALUE==0))
#error "ADC0CΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_PCA_EN && (CFG_ADC0_ADTRGC_PCA_VALUE==0))
#error "ADC0CΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_EX_EN)
#define CFG_ADC0_ADCON1_ADSTRSC_INIT ((uint32_t)(CFG_ADC0_ADTRGC_EX_VALUE<<24))
#elif(CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_T1_EN)
#define CFG_ADC0_ADCON1_ADSTRSC_INIT ((uint32_t)(1<<28))
#elif(CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_MCM0_EN)
#define CFG_ADC0_ADCON1_ADSTRSC_INIT ((uint32_t)(2<<28)|(CFG_ADC0_ADTRGC_MCM0_VALUE<<24))
#elif(CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_MCM1_EN)
#define CFG_ADC0_ADCON1_ADSTRSC_INIT ((uint32_t)(3<<28)|(CFG_ADC0_ADTRGC_MCM1_VALUE<<24))
#elif(CFG_ADC0_ADCON1_ADSTRSC && CFG_ADC0_EN && CFG_ADC0_ADTRGC_PCA_EN)
#define CFG_ADC0_ADCON1_ADSTRSC_INIT ((uint32_t)(4<<28)|(CFG_ADC0_ADTRGC_PCA_VALUE<<24))
#else
#define CFG_ADC0_ADCON1_ADSTRSC_INIT 0x0
#endif


#define CFG_ADC0_ADCON1_INIT ((uint32_t)(CFG_ADC0_ADCON1_VALUE)|CFG_ADC0_ADCON1_ADSTRSA_INIT|CFG_ADC0_ADCON1_ADSTRSB_INIT|CFG_ADC0_ADCON1_ADSTRSC_INIT)
//</h>

//<h>ADCON2_ADCƼĴ2
//<i>0x004  b31~26 b25   b24~22   b21~19   b18~16 b15~12 b11~8 b7 b6~4    b3~0 
//<i>ADCON2 -      ADOUT ADMAXCH2 ADMAXCH1 TGAP   -      TS    -  ADMAXCH TADC 
// <o0.0..3> TADC_ADCʱѡ
//    <0=>ADCʱTAD = 1 ʱ
//    <1=>ADCʱTAD = 2 ʱ
//    <2=>ADCʱTAD = 3 ʱ
//    <3=>ADCʱTAD = 4 ʱ
//    <4=>ADCʱTAD = 5 ʱ
//    <5=>ADCʱTAD = 6 ʱ
//    <6=>ADCʱTAD = 8 ʱ
//    <7=>ADCʱTAD = 12 ʱ
//    <8=>ADCʱTAD = 16 ʱ
//    <9=>ADCʱTAD = 24 ʱ
//    <10=>ADCʱTAD = 32 ʱ
//    <11=>ADCʱTAD = 48 ʱ
//    <12=>ADCʱTAD = 64 ʱ
//    <13=>ADCʱTAD = 128 ʱ
//    <14=>ADCʱTAD = 256 ʱ
//    <15=>ADCʱTAD = 320 ʱ
// <o0.8..11> TS _ʱ
//<0=>1ADCʱ
//<1=>2ADCʱ
//<2=>3ADCʱ
//<3=>4ADCʱ
//<4=>5ADCʱ
//<5=>6ADCʱ
//<6=>7ADCʱ
//<7=>8ADCʱ
//<8=>9ADCʱ
//<9=>10ADCʱ
//<10=>11ADCʱ
//<11=>12ADCʱ
//<12=>13ADCʱ
//<13=>14ADCʱ
//<14=>15ADCʱ
//<15=>16ADCʱ
//<i>1 tAD   (TS [<3=>0]+1) * tAD  16 tAD
//<i>ע12bitģʽһܹͨתʱ = 15tAD + ʱ
// <o0.16..18> TGAP _ʱ֮ͨʱλ
//<i>ע:һתУһͨתϵһͨʼ֮ʱ
//<0=>޵ȴʱ
//<1=>2ADCʱ
//<2=>4ADCʱ
//<3=>8ADCʱ
//<4=>16ADCʱ
//<5=>32ADCʱ
//<6=>64ADCʱ
//<7=>128ADCʱ
// <o0.4..6> ADMAXCH_ADCͨAܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.19..21> ADMAXCH1_ADCͨBܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.22..24> ADMAXCH2_ADCͨCܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <q0.25> ADOUT_ͨADTRGADSOC״̬λ


//</h>
#define CFG_ADC0_ADCON2_INIT	0x0
//<h>ADCMPCON_ADCȽϿƼĴ
//<i>0x02C    b31~16 b15   b14   b13   b12   b11~10 b9~8    b7    b6    b5    b4    b3    b2    b1    b0    
//<i>ADCMPCON -      ADGIE ADGDE ADLIE ADLDE -      ADGLCNT CSEN7 CSEN6 CSEN5 CSEN4 CSEN3 CSEN2 CSEN1 CSEN0 
// <q0.0> CSEN0_ڱȽϵĽĴʹ
//    <i>0: ֹADDR0еֵADDGTADDLTȽ
//    <i>1: ʹADDR0еֵADDGTADDLTȽ
// <q0.1> CSEN1_ڱȽϵĽĴʹ
//    <i>0: ֹADDR1еֵADDGTADDLTȽ
//    <i>1: ʹADDR1еֵADDGTADDLTȽ
// <q0.2> CSEN2_ڱȽϵĽĴʹ
//    <i>0: ֹADDR2еֵADDGTADDLTȽ
//    <i>1: ʹADDR2еֵADDGTADDLTȽ
// <q0.3> CSEN3_ڱȽϵĽĴʹ
//    <i>0: ֹADDR3еֵADDGTADDLTȽ
//    <i>1: ʹADDR3еֵADDGTADDLTȽ
// <q0.4> CSEN4_ڱȽϵĽĴʹ
//    <i>0: ֹADDR4еֵADDGTADDLTȽ
//    <i>1: ʹADDR4еֵADDGTADDLTȽ
// <q0.5> CSEN5_ڱȽϵĽĴʹ
//    <i>0: ֹADDR5еֵADDGTADDLTȽ
//    <i>1: ʹADDR5еֵADDGTADDLTȽ
// <q0.6> CSEN6_ڱȽϵĽĴʹ
//    <i>0: ֹADDR6еֵADDGTADDLTȽ
//    <i>1: ʹADDR6еֵADDGTADDLTȽ
// <q0.7> CSEN7_ڱȽϵĽĴʹ
//    <i>0: ֹADDR7еֵADDGTADDLTȽ
//    <i>1: ʹADDR7еֵADDGTADDLTȽ
// <o0.8..9> ADGLCNT_ȽϽ˲
//    <0=>1˲
//    <1=>2˲
//    <2=>3˲
//    <3=>4˲
// <q0.12> ADLDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽDMA
//    <i>1: ʹޱȽDMA
// <q0.13> ADLIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
// <q0.14> ADGDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽϴDMA
//    <i>1: ʹޱȽϴDMA
// <q0.15> ADGIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
//</h>
#define CFG_ADC0_ADCMPCON_INIT	0x0

//<h>ADDGT_ADCޱȽϼĴ
//<i>0x030 b31~16 b15~0 
//<i>ADDGT -      GT    
// <o0.0..15> GT_ADCޱȽϼĴ<0-4095>
//<i>ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDGTеֵȽ 
//</h>
#define CFG_ADC0_ADDGT_INIT	0x000

//<h>ADDLT_ADCޱȽϼĴ
//<i>0x034 b31~16 b15~0 
//<i>ADDLT -      LT    
// <o0.0..15> LT_ADCޱȽϼĴ <0-4095>
//<i>ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDLTеֵȽ
//</h>
#define CFG_ADC0_ADDLT_INIT	0x000

//<h>SEQCHSEL0_ADCͨѡĴ0
//<i>0x038     b31~28 b27~24 b23~20 b19~16 b15~12 b11~8  b7~4   b3~0   
//<i>SEQCHSEL0 SEQCH7 SEQCH6 SEQCH5 SEQCH4 SEQCH3 SEQCH2 SEQCH1 SEQCH0 
// <o0.0..3> SEQCH0_ģͨ0ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.4..7> SEQCH1_ģͨ1ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.8..11> SEQCH2_ģͨ2ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.12..15> SEQCH3_ģͨ3ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.16..19> SEQCH4_ģͨ4ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.20..23> SEQCH5_ģͨ5ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.24..27> SEQCH6_ģͨ6ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.28..31> SEQCH7_ģͨ7ѡĴ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
//</h>
#define CFG_ADC0_SEQCHSEL0_INIT	0x00000000

//<h>ADGAPON_ADCͨGAPλ
//<i>0x040   b31~8 b7     b6     b5     b4     b3     b2     b1     b0     
//<i>ADGAPON -     GAPEN7 GAPEN6 GAPEN5 GAPEN4 GAPEN3 GAPEN2 GAPEN1 GAPEN0 
// <q0.0> GAPEN0_ADCͨ0GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.1> GAPEN1_ADCͨ1GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.2> GAPEN2_ADCͨ2GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.3> GAPEN3_ADCͨ3GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.4> GAPEN4_ADCͨ4GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.5> GAPEN5_ADCͨ5GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> GAPEN6_ADCͨ6GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.7> GAPEN7_ADCͨ7GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
//</h>
#define CFG_ADC0_ADGAPON_INIT	0x0

#define CFG_ADC0_INT_EN (((CFG_ADC0_ADCON1_VALUE & 0x10)>>4) | ((CFG_ADC0_ADCON1_VALUE & (1<<14))>>14)| ((CFG_ADC0_ADCON1_VALUE & (1<<23))>>23))

//</e>

//***********************************************************/
//<e>ADC1
#define CFG_ADC1_EN	0
//<h>ADCON1_ADCƼĴ1
//<i>0x000  b30~24  b23   b22   b21~15  b14   b13   b12  b11~5  b4   b3   b2   b1~0     
//<i>ADCON1 ADSTRS2 ADIE2 ADDE2 ADSTRS1 ADIE1 ADDE1 ADON ADSTRS ADIE ADDE REFC ADCTU
// <q0.12> ADON_ADCλ
//    <i>0: ֹADCģ
//    <i>1: ADCģ
// <o0.0..1> ADCTU_ADCͨеתʽλ
//    <0=>תʽ
//    <2=>תʽ
//    <i>ע:޸ĸÿλһADתʱŲЧ
// <o0.2> REFC_׼ѹѡλ
//    <0=>ѡAVDDΪ׼ѹ
//    <1=>ѡⲿVREF˿Ϊ׼ѹ
// <q0.3> ADDE_ͨAתDMAʹλ
//    <i>0: ֹADCAתDMA
//    <i>1: ʹADCAתDMA
// <q0.4> ADIE_ͨAתжʹλ
//    <i>0: ֹADCAתж
//    <i>1: ʹADCAתж
// <q0.13> ADDE1_ͨBתDMAʹλ
//    <i>0: ֹADCBתDMA
//    <i>1: ʹADCBתDMA
// <q0.14> ADIE1_ͨBתжʹλ
//    <i>0: ֹADCBתж
//    <i>1: ʹADCBתж
// <q0.22> ADDE2_ͨCתDMAʹλ
//    <i>0: ֹADCCתDMA
//    <i>1: ʹADCCתDMA
// <q0.23> ADIE2_ͨCתжʹλ
//    <i>0: ֹADCCתж
//    <i>1: ʹADCCתж
#define CFG_ADC1_ADCON1_VALUE   0x00
// <e> ADSTRS_ADCAӲԴѡ(5ѡ1)
#define CFG_ADC1_ADCON1_ADSTRSA 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥
//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC1_ADTRGA_EX_EN 0x0
#define CFG_ADC1_ADTRGA_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC1_ADTRGA_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGA_MCM0_EN 0x0
#define CFG_ADC1_ADTRGA_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGA_MCM1_EN 0x0
#define CFG_ADC1_ADTRGA_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0	PCA0TRG	PCA0Ƚƥ䴥
//    <q1.1>PCA1	PCA1TRG	PCA1Ƚƥ䴥
//    <q1.2>PCA2	PCA2TRG	PCA2Ƚƥ䴥
//    <q1.3>PCA3	PCA3TRG	PCA3Ƚƥ䴥
//</e>
#define CFG_ADC1_ADTRGA_PCA_EN 0x0
#define CFG_ADC1_ADTRGA_PCA_VALUE 0x0

//</e>
#define IS_ADC1_ADCON1_ADSTRSA() ((CFG_ADC1_ADTRGA_EX_EN && (CFG_ADC1_ADTRGA_T1_EN || CFG_ADC1_ADTRGA_MCM1_EN || CFG_ADC1_ADTRGA_MCM0_EN || CFG_ADC1_ADTRGA_PCA_EN ))\
									||(CFG_ADC1_ADTRGA_T1_EN && (CFG_ADC1_ADTRGA_EX_EN || CFG_ADC1_ADTRGA_MCM1_EN || CFG_ADC1_ADTRGA_MCM0_EN || CFG_ADC1_ADTRGA_PCA_EN ))\
									||(CFG_ADC1_ADTRGA_MCM1_EN && (CFG_ADC1_ADTRGA_EX_EN || CFG_ADC1_ADTRGA_T1_EN || CFG_ADC1_ADTRGA_MCM0_EN || CFG_ADC1_ADTRGA_PCA_EN ))\
									||(CFG_ADC1_ADTRGA_MCM0_EN && (CFG_ADC1_ADTRGA_EX_EN || CFG_ADC1_ADTRGA_T1_EN || CFG_ADC1_ADTRGA_MCM1_EN || CFG_ADC1_ADTRGA_PCA_EN ))\
									||(CFG_ADC1_ADTRGA_PCA_EN && (CFG_ADC1_ADTRGA_EX_EN || CFG_ADC1_ADTRGA_T1_EN || CFG_ADC1_ADTRGA_MCM1_EN || CFG_ADC1_ADTRGA_MCM0_EN ))\
									)
#if ((CFG_ADC1_ADCON1_ADSTRSA) && (CFG_ADC1_EN) && IS_ADC1_ADCON1_ADSTRSA())
#error "ADC1AӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_EX_EN && (CFG_ADC1_ADTRGA_EX_VALUE==0))
#error "ADC1AΪӲԴתδⲿţ"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_MCM1_EN && (CFG_ADC1_ADTRGA_MCM1_VALUE==0))
#error "ADC1AΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_MCM0_EN && (CFG_ADC1_ADTRGA_MCM0_VALUE==0))
#error "ADC1AΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_PCA_EN && (CFG_ADC1_ADTRGA_PCA_VALUE==0))
#error "ADC1AΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_EX_EN)
#define CFG_ADC1_ADCON1_ADSTRSA_INIT (CFG_ADC1_ADTRGA_EX_VALUE<<5)
#elif(CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_T1_EN)
#define CFG_ADC1_ADCON1_ADSTRSA_INIT ((1<<9))
#elif(CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_MCM0_EN)
#define CFG_ADC1_ADCON1_ADSTRSA_INIT ((2<<9)|(CFG_ADC1_ADTRGA_MCM0_VALUE<<5))
#elif(CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_MCM1_EN)
#define CFG_ADC1_ADCON1_ADSTRSA_INIT ((3<<9)|(CFG_ADC1_ADTRGA_MCM1_VALUE<<5))
#elif(CFG_ADC1_ADCON1_ADSTRSA && CFG_ADC1_EN && CFG_ADC1_ADTRGA_PCA_EN)
#define CFG_ADC1_ADCON1_ADSTRSA_INIT ((4<<9)|(CFG_ADC1_ADTRGA_PCA_VALUE<<5))
#else
#define CFG_ADC1_ADCON1_ADSTRSA_INIT 0x0
#endif

// <e> ADSTRS1_ADCBӲԴѡ(5ѡ1)
#define CFG_ADC1_ADCON1_ADSTRSB 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>

#define CFG_ADC1_ADTRGB_EX_EN 0x0
#define CFG_ADC1_ADTRGB_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC1_ADTRGB_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGB_MCM0_EN 0x0
#define CFG_ADC1_ADTRGB_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGB_MCM1_EN 0x0
#define CFG_ADC1_ADTRGB_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC1_ADTRGB_PCA_EN 0x0
#define CFG_ADC1_ADTRGB_PCA_VALUE 0x0

//</e>
#define IS_ADC1_ADCON1_ADSTRSB()    ((CFG_ADC1_ADTRGB_EX_EN && (CFG_ADC1_ADTRGB_T1_EN || CFG_ADC1_ADTRGB_MCM1_EN || CFG_ADC1_ADTRGB_MCM0_EN || CFG_ADC1_ADTRGB_PCA_EN ))\
									||(CFG_ADC1_ADTRGB_T1_EN && (CFG_ADC1_ADTRGB_EX_EN || CFG_ADC1_ADTRGB_MCM1_EN || CFG_ADC1_ADTRGB_MCM0_EN || CFG_ADC1_ADTRGB_PCA_EN ))\
									||(CFG_ADC1_ADTRGB_MCM1_EN && (CFG_ADC1_ADTRGB_EX_EN || CFG_ADC1_ADTRGB_T1_EN || CFG_ADC1_ADTRGB_MCM0_EN || CFG_ADC1_ADTRGB_PCA_EN ))\
									||(CFG_ADC1_ADTRGB_MCM0_EN && (CFG_ADC1_ADTRGB_EX_EN || CFG_ADC1_ADTRGB_T1_EN || CFG_ADC1_ADTRGB_MCM1_EN || CFG_ADC1_ADTRGB_PCA_EN ))\
									||(CFG_ADC1_ADTRGB_PCA_EN && (CFG_ADC1_ADTRGB_EX_EN || CFG_ADC1_ADTRGB_T1_EN || CFG_ADC1_ADTRGB_MCM1_EN || CFG_ADC1_ADTRGB_MCM0_EN ))\
									)
                                    
#if ((CFG_ADC1_ADCON1_ADSTRSB) && (CFG_ADC1_EN) && IS_ADC1_ADCON1_ADSTRSB())
#error "ADC1BӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_EX_EN && (CFG_ADC1_ADTRGB_EX_VALUE==0))
#error "ADC1BΪӲԴתδⲿţ"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_MCM1_EN && (CFG_ADC1_ADTRGB_MCM1_VALUE==0))
#error "ADC1BΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_MCM0_EN && (CFG_ADC1_ADTRGB_MCM0_VALUE==0))
#error "ADC1BΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_PCA_EN && (CFG_ADC1_ADTRGB_PCA_VALUE==0))
#error "ADC1BΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_EX_EN)
#define CFG_ADC1_ADCON1_ADSTRSB_INIT (CFG_ADC1_ADTRGB_EX_VALUE<<15)
#elif(CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_T1_EN)
#define CFG_ADC1_ADCON1_ADSTRSB_INIT (1<<19)
#elif(CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_MCM0_EN)
#define CFG_ADC1_ADCON1_ADSTRSB_INIT ((2<<19)|(CFG_ADC1_ADTRGB_MCM0_VALUE<<15))
#elif(CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_MCM1_EN)
#define CFG_ADC1_ADCON1_ADSTRSB_INIT ((3<<19)|(CFG_ADC1_ADTRGB_MCM1_VALUE<<15))
#elif(CFG_ADC1_ADCON1_ADSTRSB && CFG_ADC1_EN && CFG_ADC1_ADTRGB_PCA_EN)
#define CFG_ADC1_ADCON1_ADSTRSB_INIT ((4<<19)|(CFG_ADC1_ADTRGB_PCA_VALUE<<15))
#else
#define CFG_ADC1_ADCON1_ADSTRSB_INIT 0x0
#endif

// <e> ADSTRS2_ADCCӲԴѡ(5ѡ1)
#define CFG_ADC1_ADCON1_ADSTRSC 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC1_ADTRGC_EX_EN 0x0
#define CFG_ADC1_ADTRGC_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC1_ADTRGC_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGC_MCM0_EN 0x0
#define CFG_ADC1_ADTRGC_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC1_ADTRGC_MCM1_EN 0x0
#define CFG_ADC1_ADTRGC_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC1_ADTRGC_PCA_EN 0x0
#define CFG_ADC1_ADTRGC_PCA_VALUE 0x0

//</e>
#define IS_ADC1_ADCON1_ADSTRSC() ((CFG_ADC1_ADTRGC_EX_EN && (CFG_ADC1_ADTRGC_T1_EN || CFG_ADC1_ADTRGC_MCM1_EN || CFG_ADC1_ADTRGC_MCM0_EN || CFG_ADC1_ADTRGC_PCA_EN ))\
									||(CFG_ADC1_ADTRGC_T1_EN && (CFG_ADC1_ADTRGC_EX_EN || CFG_ADC1_ADTRGC_MCM1_EN || CFG_ADC1_ADTRGC_MCM0_EN || CFG_ADC1_ADTRGC_PCA_EN ))\
									||(CFG_ADC1_ADTRGC_MCM1_EN && (CFG_ADC1_ADTRGC_EX_EN || CFG_ADC1_ADTRGC_T1_EN || CFG_ADC1_ADTRGC_MCM0_EN || CFG_ADC1_ADTRGC_PCA_EN ))\
									||(CFG_ADC1_ADTRGC_MCM0_EN && (CFG_ADC1_ADTRGC_EX_EN || CFG_ADC1_ADTRGC_T1_EN || CFG_ADC1_ADTRGC_MCM1_EN || CFG_ADC1_ADTRGC_PCA_EN ))\
									||(CFG_ADC1_ADTRGC_PCA_EN && (CFG_ADC1_ADTRGC_EX_EN || CFG_ADC1_ADTRGC_T1_EN || CFG_ADC1_ADTRGC_MCM1_EN || CFG_ADC1_ADTRGC_MCM0_EN ))\
									)
#if ((CFG_ADC1_ADCON1_ADSTRSC) && (CFG_ADC1_EN) && IS_ADC1_ADCON1_ADSTRSC())
#error "ADC1CӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_EX_EN && (CFG_ADC1_ADTRGC_EX_VALUE==0))
#error "ADC1CΪӲԴתδⲿţ"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_MCM1_EN && (CFG_ADC1_ADTRGC_MCM1_VALUE==0))
#error "ADC1CΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_MCM0_EN && (CFG_ADC1_ADTRGC_MCM0_VALUE==0))
#error "ADC1CΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_PCA_EN && (CFG_ADC1_ADTRGC_PCA_VALUE==0))
#error "ADC1CΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_EX_EN)
#define CFG_ADC1_ADCON1_ADSTRSC_INIT ((uint32_t)(CFG_ADC1_ADTRGC_EX_VALUE<<24))
#elif(CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_T1_EN)
#define CFG_ADC1_ADCON1_ADSTRSC_INIT ((uint32_t)(1<<28))
#elif(CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_MCM0_EN)
#define CFG_ADC1_ADCON1_ADSTRSC_INIT ((uint32_t)(2<<28)|(CFG_ADC1_ADTRGC_MCM0_VALUE<<24))
#elif(CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_MCM1_EN)
#define CFG_ADC1_ADCON1_ADSTRSC_INIT ((uint32_t)(3<<28)|(CFG_ADC1_ADTRGC_MCM1_VALUE<<24))
#elif(CFG_ADC1_ADCON1_ADSTRSC && CFG_ADC1_EN && CFG_ADC1_ADTRGC_PCA_EN)
#define CFG_ADC1_ADCON1_ADSTRSC_INIT ((uint32_t)(4<<28)|(CFG_ADC1_ADTRGC_PCA_VALUE<<24))
#else
#define CFG_ADC1_ADCON1_ADSTRSC_INIT 0x0
#endif

#define CFG_ADC1_ADCON1_INIT ((uint32_t)(CFG_ADC1_ADCON1_VALUE)|CFG_ADC1_ADCON1_ADSTRSA_INIT|CFG_ADC1_ADCON1_ADSTRSB_INIT|CFG_ADC1_ADCON1_ADSTRSC_INIT)
//</h>

//<h>ADCON2_ADCƼĴ2
//<i>0x004  b31~26 b25   b24~22   b21~19   b18~16 b15~12 b11~8 b7 b6~4    b3~0 
//<i>ADCON2 -      ADOUT ADMAXCH2 ADMAXCH1 TGAP   -      TS    -  ADMAXCH TADC 
// <o0.0..3> TADC_ADCʱѡ
//    <0=>ADCʱTAD = 1ʱ
//    <1=>ADCʱTAD = 2ʱ
//    <2=>ADCʱTAD = 3ʱ
//    <3=>ADCʱTAD = 4 ʱ
//    <4=>ADCʱTAD = 5 ʱ
//    <5=>ADCʱTAD = 6 ʱ
//    <6=>ADCʱTAD = 8 ʱ
//    <7=>ADCʱTAD = 12 ʱ
//    <8=>ADCʱTAD = 16 ʱ
//    <9=>ADCʱTAD = 24ʱ
//    <10=>ADCʱTAD = 32ʱ
//    <11=>ADCʱTAD = 48 ʱ
//    <12=>ADCʱTAD = 64ʱ
//    <13=>ADCʱTAD = 128ʱ
//    <14=>ADCʱTAD = 256ʱ
//    <15=>ADCʱTAD = 320 ʱ
// <o0.8..11> TS _ʱ
//<0=>1ADCʱ
//<1=>2ADCʱ
//<2=>3ADCʱ
//<3=>4ADCʱ
//<4=>5ADCʱ
//<5=>6ADCʱ
//<6=>7ADCʱ
//<7=>8ADCʱ
//<8=>9ADCʱ
//<9=>10ADCʱ
//<10=>11ADCʱ
//<11=>12ADCʱ
//<12=>13ADCʱ
//<13=>14ADCʱ
//<14=>15ADCʱ
//<15=>16ADCʱ
//<i>1 tAD   (TS [<3=>0]+1) * tAD  16 tAD
//<i>ע12bitģʽһܹͨתʱ = 15tAD + ʱ
// <o0.16..18> TGAP _ʱ֮ͨʱλ
//<i>ע:һתУһͨתϵһͨʼ֮ʱ
//<0=>޵ȴʱ
//<1=>2ADCʱ
//<2=>4ADCʱ
//<3=>8ADCʱ
//<4=>16ADCʱ
//<5=>32ADCʱ
//<6=>64ADCʱ
//<7=>128ADCʱ
// <o0.4..6> ADMAXCH_ADCͨAܳλֵ0~7
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.19..21> ADMAXCH1_ADCͨBܳλֵ0~7
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.22..24> ADMAXCH2_ADCͨCܳλֵ0~7
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <q0.25> ADOUT_λ
//<i>0: ֹͨADTRGADSOC״̬
//<i>1: ʹͨADTRGADSOC״̬
//</h>
#define CFG_ADC1_ADCON2_INIT	0x0

//<h>ADCMPCON_ADCȽϿƼĴ
//<i>0x02C    b31~16 b15   b14   b13   b12   b11~10 b9~8    b7    b6    b5    b4    b3    b2    b1    b0    
//<i>ADCMPCON -      ADGIE ADGDE ADLIE ADLDE -      ADGLCNT CSEN7 CSEN6 CSEN5 CSEN4 CSEN3 CSEN2 CSEN1 CSEN0 
// <q0.0> CSEN0_ڱȽϵĽĴʹ
//    <i>0: ֹADDR0еֵADDGTADDLTȽ
//    <i>1: ʹADDR0еֵADDGTADDLTȽ
// <q0.1> CSEN1_ڱȽϵĽĴʹ
//    <i>0: ֹADDR1еֵADDGTADDLTȽ
//    <i>1: ʹADDR1еֵADDGTADDLTȽ
// <q0.2> CSEN2_ڱȽϵĽĴʹ
//    <i>0: ֹADDR2еֵADDGTADDLTȽ
//    <i>1: ʹADDR2еֵADDGTADDLTȽ
// <q0.3> CSEN3_ڱȽϵĽĴʹ
//    <i>0: ֹADDR3еֵADDGTADDLTȽ
//    <i>1: ʹADDR3еֵADDGTADDLTȽ
// <q0.4> CSEN4_ڱȽϵĽĴʹ
//    <i>0: ֹADDR4еֵADDGTADDLTȽ
//    <i>1: ʹADDR4еֵADDGTADDLTȽ
// <q0.5> CSEN5_ڱȽϵĽĴʹ
//    <i>0: ֹADDR5еֵADDGTADDLTȽ
//    <i>1: ʹADDR5еֵADDGTADDLTȽ
// <q0.6> CSEN6_ڱȽϵĽĴʹ
//    <i>0: ֹADDR6еֵADDGTADDLTȽ
//    <i>1: ʹADDR6еֵADDGTADDLTȽ
// <q0.7> CSEN7_ڱȽϵĽĴʹ
//    <i>0: ֹADDR7еֵADDGTADDLTȽ
//    <i>1: ʹADDR7еֵADDGTADDLTȽ
// <o0.8..9> ADGLCNT_ȽϽ˲
//    <0=>1˲
//    <1=>2˲
//    <2=>3˲
//    <3=>4˲
// <q0.12> ADLDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽDMA
//    <i>1: ʹޱȽDMA
// <q0.13> ADLIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
// <q0.14> ADGDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽϴDMA
//    <i>1: ʹޱȽϴDMA
// <q0.15> ADGIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
//</h>
#define CFG_ADC1_ADCMPCON_INIT	0x0

//<h>ADDGT_ADCޱȽϼĴ
//<i>0x030 b31~16 b15~0 
//<i>ADDGT -      GT    
// <o0.0..15> GT_ADCޱȽϼĴ
//ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDGTеֵȽ <0-32768>
//</h>
#define CFG_ADC1_ADDGT_INIT	0x0

//<h>ADDLT_ADCޱȽϼĴ
//<i>0x034 b31~16 b15~0 
//<i>ADDLT -      LT    
// <o0.0..15> LT_ADCޱȽϼĴ
//ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDLTеֵȽ <0-32768>
//</h>
#define CFG_ADC1_ADDLT_INIT	0x0

//<h>SEQCHSEL0_ADCͨѡĴ0
//<i>0x038     b31~28 b27~24 b23~20 b19~16 b15~12 b11~8  b7~4   b3~0   
//<i>SEQCHSEL0 SEQCH7 SEQCH6 SEQCH5 SEQCH4 SEQCH3 SEQCH2 SEQCH1 SEQCH0 
// <o0.0..3> SEQCH0_ģͨ0ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.4..7> SEQCH1_ģͨ1ѡĴ飬ֱָӦģͨ:
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.8..11> SEQCH2_ģͨ2ѡĴ飬ֱָӦģͨ:
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.12..15> SEQCH3_ģͨ3ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.16..19> SEQCH4_ģͨ4ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.20..23> SEQCH5_ģͨ5ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.24..27> SEQCH6_ģͨ6ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.28..31> SEQCH7_ģͨ7ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg 
//</h>
#define CFG_ADC1_SEQCHSEL0_INIT	0x00

//<h>ADGAPON_ADCͨGAPλ
//<i>0x040   b31~8 b7     b6     b5     b4     b3     b2     b1     b0     
//<i>ADGAPON -     GAPEN7 GAPEN6 GAPEN5 GAPEN4 GAPEN3 GAPEN2 GAPEN1 GAPEN0 
// <q0.0> GAPEN0_ADCͨ0GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.1> GAPEN1_ADCͨ1GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.2> GAPEN2_ADCͨ2GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.3> GAPEN3_ADCͨ3GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.4> GAPEN4_ADCͨ4GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.5> GAPEN5_ADCͨ5GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> GAPEN6_ADCͨ6GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.7> GAPEN7_ADCͨ7GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
//</h>
#define CFG_ADC1_ADGAPON_INIT	0x0
#define CFG_ADC1_INT_EN (((CFG_ADC1_ADCON1_VALUE & 0x10)>>4) | ((CFG_ADC1_ADCON1_VALUE & (1<<14))>>14)| ((CFG_ADC1_ADCON1_VALUE & (1<<23))>>23))
//</e>

//***********************************************************/
//<e>ADC2
#define CFG_ADC2_EN	0
//<h>ADCON1_ADCƼĴ1
//<i>0x000  b30~24  b23   b22   b21~15  b14   b13   b12  b11~5  b4   b3   b2   b1~0     
//<i>ADCON1 ADSTRS2 ADIE2 ADDE2 ADSTRS1 ADIE1 ADDE1 ADON ADSTRS ADIE ADDE REFC ADCTU
// <q0.12> ADON_ADCλ
//    <i>0: ֹADCģ
//    <i>1: ADCģ
// <o0.0..1> ADCTU_ADCͨеתʽλ
//    <0=>תʽ
//    <2=>תʽ
//    <i>ע:޸ĸÿλһADתʱŲЧ
// <o0.2> REFC_׼ѹѡλ
//    <0=>ѡAVDDΪ׼ѹ
//    <1=>ѡⲿVREF˿Ϊ׼ѹ
// <q0.3> ADDE_ͨAתDMAʹλ
//    <i>0: ֹADCAתDMA
//    <i>1: ʹADCAתDMA
// <q0.4> ADIE_ͨAתжʹλ
//    <i>0: ֹADCAתж
//    <i>1: ʹADCAתж
// <q0.13> ADDE1_ͨBתDMAʹλ
//    <i>0: ֹADCBתDMA
//    <i>1: ʹADCBתDMA
// <q0.14> ADIE1_ͨBתжʹλ
//    <i>0: ֹADCBתж
//    <i>1: ʹADCBתж
// <q0.22> ADDE2_ͨCתDMAʹλ
//    <i>0: ֹADCCתDMA
//    <i>1: ʹADCCתDMA
// <q0.23> ADIE2_ͨCתжʹλ
//    <i>0: ֹADCCתж
//    <i>1: ʹADCCתж
#define CFG_ADC2_ADCON1_VALUE   0x00
// <e> ADSTRS_ADCAӲԴѡ(5ѡ1)
#define CFG_ADC2_ADCON1_ADSTRSA 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥
//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC2_ADTRGA_EX_EN 0x0
#define CFG_ADC2_ADTRGA_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC2_ADTRGA_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGA_MCM0_EN 0x0
#define CFG_ADC2_ADTRGA_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGA_MCM1_EN 0x0
#define CFG_ADC2_ADTRGA_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC2_ADTRGA_PCA_EN 0x0
#define CFG_ADC2_ADTRGA_PCA_VALUE 0x0

//</e>
#define IS_ADC2_ADCON1_ADSTRSA() ((CFG_ADC2_ADTRGA_EX_EN && (CFG_ADC2_ADTRGA_T1_EN || CFG_ADC2_ADTRGA_MCM1_EN || CFG_ADC2_ADTRGA_MCM0_EN || CFG_ADC2_ADTRGA_PCA_EN ))\
									||(CFG_ADC2_ADTRGA_T1_EN && (CFG_ADC2_ADTRGA_EX_EN || CFG_ADC2_ADTRGA_MCM1_EN || CFG_ADC2_ADTRGA_MCM0_EN || CFG_ADC2_ADTRGA_PCA_EN ))\
									||(CFG_ADC2_ADTRGA_MCM1_EN && (CFG_ADC2_ADTRGA_EX_EN || CFG_ADC2_ADTRGA_T1_EN || CFG_ADC2_ADTRGA_MCM0_EN || CFG_ADC2_ADTRGA_PCA_EN ))\
									||(CFG_ADC2_ADTRGA_MCM0_EN && (CFG_ADC2_ADTRGA_EX_EN || CFG_ADC2_ADTRGA_T1_EN || CFG_ADC2_ADTRGA_MCM1_EN || CFG_ADC2_ADTRGA_PCA_EN ))\
									||(CFG_ADC2_ADTRGA_PCA_EN && (CFG_ADC2_ADTRGA_EX_EN || CFG_ADC2_ADTRGA_T1_EN || CFG_ADC2_ADTRGA_MCM1_EN || CFG_ADC2_ADTRGA_MCM0_EN ))\
									)
#if ((CFG_ADC2_ADCON1_ADSTRSA) && (CFG_ADC2_EN) && IS_ADC2_ADCON1_ADSTRSA())
#error "ADC2AӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_EX_EN && (CFG_ADC2_ADTRGA_EX_VALUE==0))
#error "ADC2AΪӲԴתδⲿţ"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_MCM1_EN && (CFG_ADC2_ADTRGA_MCM1_VALUE==0))
#error "ADC2AΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_MCM0_EN && (CFG_ADC2_ADTRGA_MCM0_VALUE==0))
#error "ADC2AΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_PCA_EN && (CFG_ADC2_ADTRGA_PCA_VALUE==0))
#error "ADC2AΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_EX_EN)
#define CFG_ADC2_ADCON1_ADSTRSA_INIT (CFG_ADC2_ADTRGA_EX_VALUE<<5)
#elif(CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_T1_EN)
#define CFG_ADC2_ADCON1_ADSTRSA_INIT ((1<<9))
#elif(CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_MCM0_EN)
#define CFG_ADC2_ADCON1_ADSTRSA_INIT ((2<<9)|(CFG_ADC2_ADTRGA_MCM0_VALUE<<5))
#elif(CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_MCM1_EN)
#define CFG_ADC2_ADCON1_ADSTRSA_INIT ((3<<9)|(CFG_ADC2_ADTRGA_MCM1_VALUE<<5))
#elif(CFG_ADC2_ADCON1_ADSTRSA && CFG_ADC2_EN && CFG_ADC2_ADTRGA_PCA_EN)
#define CFG_ADC2_ADCON1_ADSTRSA_INIT ((4<<9)|(CFG_ADC2_ADTRGA_PCA_VALUE<<5))
#else
#define CFG_ADC2_ADCON1_ADSTRSA_INIT 0x0
#endif

// <e> ADSTRS1_ADCBӲԴѡ(5ѡ1)
#define CFG_ADC2_ADCON1_ADSTRSB 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC2_ADTRGB_EX_EN 0x0
#define CFG_ADC2_ADTRGB_EX_VALUE 0x0

//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC2_ADTRGB_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGB_MCM0_EN 0x0
#define CFG_ADC2_ADTRGB_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGB_MCM1_EN 0x0
#define CFG_ADC2_ADTRGB_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC2_ADTRGB_PCA_EN 0x0
#define CFG_ADC2_ADTRGB_PCA_VALUE 0x0

//</e>
#define IS_ADC2_ADCON1_ADSTRSB() ((CFG_ADC2_ADTRGB_EX_EN && (CFG_ADC2_ADTRGB_T1_EN || CFG_ADC2_ADTRGB_MCM1_EN || CFG_ADC2_ADTRGB_MCM0_EN || CFG_ADC2_ADTRGB_PCA_EN ))\
									||(CFG_ADC2_ADTRGB_T1_EN && (CFG_ADC2_ADTRGB_EX_EN || CFG_ADC2_ADTRGB_MCM1_EN || CFG_ADC2_ADTRGB_MCM0_EN || CFG_ADC2_ADTRGB_PCA_EN ))\
									||(CFG_ADC2_ADTRGB_MCM1_EN && (CFG_ADC2_ADTRGB_EX_EN || CFG_ADC2_ADTRGB_T1_EN || CFG_ADC2_ADTRGB_MCM0_EN || CFG_ADC2_ADTRGB_PCA_EN ))\
									||(CFG_ADC2_ADTRGB_MCM0_EN && (CFG_ADC2_ADTRGB_EX_EN || CFG_ADC2_ADTRGB_T1_EN || CFG_ADC2_ADTRGB_MCM1_EN || CFG_ADC2_ADTRGB_PCA_EN ))\
									||(CFG_ADC2_ADTRGB_PCA_EN && (CFG_ADC2_ADTRGB_EX_EN || CFG_ADC2_ADTRGB_T1_EN || CFG_ADC2_ADTRGB_MCM1_EN || CFG_ADC2_ADTRGB_MCM0_EN ))\
									)
#if ((CFG_ADC2_ADCON1_ADSTRSB) && (CFG_ADC2_EN) && IS_ADC2_ADCON1_ADSTRSB())
#error "ADC2BӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_EX_EN && (CFG_ADC2_ADTRGB_EX_VALUE==0))
#error "ADC2BΪӲԴתδⲿţ"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_MCM1_EN && (CFG_ADC2_ADTRGB_MCM1_VALUE==0))
#error "ADC2BΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_MCM0_EN && (CFG_ADC2_ADTRGB_MCM0_VALUE==0))
#error "ADC2BΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_PCA_EN && (CFG_ADC2_ADTRGB_PCA_VALUE==0))
#error "ADC2BΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_EX_EN)
#define CFG_ADC2_ADCON1_ADSTRSB_INIT (CFG_ADC2_ADTRGB_EX_VALUE<<15)
#elif(CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_T1_EN)
#define CFG_ADC2_ADCON1_ADSTRSB_INIT (1<<19)
#elif(CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_MCM0_EN)
#define CFG_ADC2_ADCON1_ADSTRSB_INIT ((2<<19)|(CFG_ADC2_ADTRGB_MCM0_VALUE<<15))
#elif(CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_MCM1_EN)
#define CFG_ADC2_ADCON1_ADSTRSB_INIT ((3<<19)|(CFG_ADC2_ADTRGB_MCM1_VALUE<<15))
#elif(CFG_ADC2_ADCON1_ADSTRSB && CFG_ADC2_EN && CFG_ADC2_ADTRGB_PCA_EN)
#define CFG_ADC2_ADCON1_ADSTRSB_INIT ((4<<19)|(CFG_ADC2_ADTRGB_PCA_VALUE<<15))
#else
#define CFG_ADC2_ADCON1_ADSTRSB_INIT 0x0
#endif

// <e> ADSTRS2_ADCCӲԴѡ(5ѡ1)
#define CFG_ADC2_ADCON1_ADSTRSC 0x0
//<i>ADSTRS[6:0]
//<i>000 0000	ADSOC
//<i>------------------------------------------------	
//<i>000 0001	ⲿADTRG0
//<i>000 0010	ⲿADTRG1
//<i>000 0100	ⲿADTRG2
//<i>-------------------------------------------------	
//<i>001 0000	ʱ1 TIMTRG_ʱ1
//<i>-------------------------------------------------	
//<i>010 0001	  MCM0TRG1	MCM0. PWMCMP1Ƚƥ
//<i>010 0010		MCM0TRG2	MCM0. PWMCMP2Ƚƥ
//<i>010 0100		MCM0TRG3	MCM0. PWMCMP3Ƚƥ
//<i>010 1000		MCM0TRG4	MCM0. PWMCMP4Ƚƥ
//<i>-------------------------------------------------	
//<i>011 0001	  MCM1TRG1	MCM1. PWMCMP1Ƚƥ
//<i>011 0010		MCM1TRG2	MCM1. PWMCMP2Ƚƥ
//<i>011 0100		MCM1TRG3	MCM1. PWMCMP3Ƚƥ
//<i>011 1000		MCM1TRG4	MCM1. PWMCMP4Ƚƥ
//<i>-------------------------------------------------
//<i>100 0001	PCA0TRG_PCA0Ƚƥ䴥
//<i>100 0010	PCA1TRG_PCA1Ƚƥ䴥
//<i>100 0100	PCA2TRG_PCA2Ƚƥ䴥
//<i>100 1000	PCA3TRG_PCA3Ƚƥ䴥

//<e0>ⲿ
//    <q1.0>ADTRG0 0
//    <q1.1>ADTRG1 1
//    <q1.2>ADTRG2 2
//</e>
#define CFG_ADC2_ADTRGC_EX_EN 0x0
#define CFG_ADC2_ADTRGC_EX_VALUE 0x0


//<q>ʱ1 TIMTRG_ʱ1
#define CFG_ADC2_ADTRGC_T1_EN 0x0

//<e0>MCM0
//    <q1.0>MCM0TRG1_MCM0.PWMCMP1Ƚƥ
//    <q1.1>MCM0TRG2_MCM0.PWMCMP2Ƚƥ
//    <q1.2>MCM0TRG3_MCM0.PWMCMP3Ƚƥ
//    <q1.3>MCM0TRG4_MCM0.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGC_MCM0_EN 0x0
#define CFG_ADC2_ADTRGC_MCM0_VALUE 0x0

//<e0>MCM1
//    <q1.0>MCM1TRG1_MCM1.PWMCMP1Ƚƥ
//    <q1.1>MCM1TRG2_MCM1.PWMCMP2Ƚƥ
//    <q1.2>MCM1TRG3_MCM1.PWMCMP3Ƚƥ
//    <q1.3>MCM1TRG4_MCM1.PWMCMP4Ƚƥ
//</e>
#define CFG_ADC2_ADTRGC_MCM1_EN 0x0
#define CFG_ADC2_ADTRGC_MCM1_VALUE 0x0

//<e0>PCA
//    <q1.0>PCA0TRG_PCA0Ƚƥ䴥
//    <q1.1>PCA1TRG_PCA1Ƚƥ䴥
//    <q1.2>PCA2TRG_PCA2Ƚƥ䴥
//    <q1.3>PCA3TRG_PCA3Ƚƥ䴥
//</e>
#define CFG_ADC2_ADTRGC_PCA_EN 0x0
#define CFG_ADC2_ADTRGC_PCA_VALUE 0x0

//</e>
#define IS_ADC2_ADCON1_ADSTRSC() ((CFG_ADC2_ADTRGC_EX_EN && (CFG_ADC2_ADTRGC_T1_EN || CFG_ADC2_ADTRGC_MCM1_EN || CFG_ADC2_ADTRGC_MCM0_EN || CFG_ADC2_ADTRGC_PCA_EN ))\
									||(CFG_ADC2_ADTRGC_T1_EN && (CFG_ADC2_ADTRGC_EX_EN || CFG_ADC2_ADTRGC_MCM1_EN || CFG_ADC2_ADTRGC_MCM0_EN || CFG_ADC2_ADTRGC_PCA_EN ))\
									||(CFG_ADC2_ADTRGC_MCM1_EN && (CFG_ADC2_ADTRGC_EX_EN || CFG_ADC2_ADTRGC_T1_EN || CFG_ADC2_ADTRGC_MCM0_EN || CFG_ADC2_ADTRGC_PCA_EN ))\
									||(CFG_ADC2_ADTRGC_MCM0_EN && (CFG_ADC2_ADTRGC_EX_EN || CFG_ADC2_ADTRGC_T1_EN || CFG_ADC2_ADTRGC_MCM1_EN || CFG_ADC2_ADTRGC_PCA_EN ))\
									||(CFG_ADC2_ADTRGC_PCA_EN && (CFG_ADC2_ADTRGC_EX_EN || CFG_ADC2_ADTRGC_T1_EN || CFG_ADC2_ADTRGC_MCM1_EN || CFG_ADC2_ADTRGC_MCM0_EN ))\
									)
#if ((CFG_ADC2_ADCON1_ADSTRSC) && (CFG_ADC2_EN) && IS_ADC2_ADCON1_ADSTRSC())
#error "ADC2CӲԴѡֻѡ5ӲԴеһ࣡"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_EX_EN && (CFG_ADC2_ADTRGC_EX_VALUE==0))
#error "ADC2CΪӲԴתδⲿţ"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_MCM1_EN && (CFG_ADC2_ADTRGC_MCM1_VALUE==0))
#error "ADC2CΪMCM1ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_MCM0_EN && (CFG_ADC2_ADTRGC_MCM0_VALUE==0))
#error "ADC2CΪMCM0ӲԴתδþ崥"
#endif

#if (CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_PCA_EN && (CFG_ADC2_ADTRGC_PCA_VALUE==0))
#error "ADC2CΪPCAӲԴתδþ崥"
#endif

#if(CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_EX_EN)
#define CFG_ADC2_ADCON1_ADSTRSC_INIT ((uint32_t)(CFG_ADC2_ADTRGC_EX_VALUE<<24))
#elif(CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_T1_EN)
#define CFG_ADC2_ADCON1_ADSTRSC_INIT ((uint32_t)(1<<28))
#elif(CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_MCM0_EN)
#define CFG_ADC2_ADCON1_ADSTRSC_INIT ((uint32_t)(2<<28)|(CFG_ADC2_ADTRGC_MCM0_VALUE<<24))
#elif(CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_MCM1_EN)
#define CFG_ADC2_ADCON1_ADSTRSC_INIT ((uint32_t)(3<<28)|(CFG_ADC2_ADTRGC_MCM1_VALUE<<24))
#elif(CFG_ADC2_ADCON1_ADSTRSC && CFG_ADC2_EN && CFG_ADC2_ADTRGC_PCA_EN)
#define CFG_ADC2_ADCON1_ADSTRSC_INIT ((uint32_t)(4<<28)|(CFG_ADC2_ADTRGC_PCA_VALUE<<24))
#else
#define CFG_ADC2_ADCON1_ADSTRSC_INIT 0x0
#endif

#define CFG_ADC2_ADCON1_INIT ((uint32_t)(CFG_ADC2_ADCON1_VALUE)|CFG_ADC2_ADCON1_ADSTRSA_INIT|CFG_ADC2_ADCON1_ADSTRSB_INIT|CFG_ADC2_ADCON1_ADSTRSC_INIT)

//</h>
//<h>ADCON2_ADCƼĴ2
//<i>0x004  b31~26 b25   b24~22   b21~19   b18~16 b15~12 b11~8 b7 b6~4    b3~0 
//<i>ADCON2 -      ADOUT ADMAXCH2 ADMAXCH1 TGAP   -      TS    -  ADMAXCH TADC 
// <o0.0..3> TADC_ADCʱѡ
//    <0=>ADCʱTAD = 1ʱ
//    <1=>ADCʱTAD = 2ʱ
//    <2=>ADCʱTAD = 3ʱ
//    <3=>ADCʱTAD = 4ʱ
//    <4=>ADCʱTAD = 5ʱ
//    <5=>ADCʱTAD = 6ʱ
//    <6=>ADCʱTAD = 8ʱ
//    <7=>ADCʱTAD = 12ʱ
//    <8=>ADCʱTAD = 16ʱ
//    <9=>ADCʱTAD = 24ʱ
//    <10=>ADCʱTAD = 32ʱ
//    <11=>ADCʱTAD = 48ʱ
//    <12=>ADCʱTAD = 64ʱ
//    <13=>ADCʱTAD = 128ʱ
//    <14=>ADCʱTAD = 256ʱ
//    <15=>ADCʱTAD = 320ʱ
// <o0.8..11> TS _ʱ
//<0=>1ADCʱ
//<1=>2ADCʱ
//<2=>3ADCʱ
//<3=>4ADCʱ
//<4=>5ADCʱ
//<5=>6ADCʱ
//<6=>7ADCʱ
//<7=>8ADCʱ
//<8=>9ADCʱ
//<9=>10ADCʱ
//<10=>11ADCʱ
//<11=>12ADCʱ
//<12=>13ADCʱ
//<13=>14ADCʱ
//<14=>15ADCʱ
//<15=>16ADCʱ
//<i>1 tAD   (TS [<3=>0]+1) * tAD  16 tAD
//<i>ע12bitģʽһܹͨתʱ = 15tAD + ʱ
// <o0.16..18> TGAP _ʱ֮ͨʱλ
//<i>ע:һתУһͨתϵһͨʼ֮ʱ
//<0=>޵ȴʱ
//<1=>2ADCʱ
//<2=>4ADCʱ
//<3=>8ADCʱ
//<4=>16ADCʱ
//<5=>32ADCʱ
//<6=>64ADCʱ
//<7=>128ADCʱ
// <o0.4..6> ADMAXCH_ADCͨAܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.19..21> ADMAXCH1_ADCͨBܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <o0.22..24> ADMAXCH2_ADCͨCܳλ
//    <0=>תгΪ1
//    <1=>תгΪ2
//    <2=>תгΪ3
//    <3=>תгΪ4
//    <4=>תгΪ5
//    <5=>תгΪ6
//    <6=>תгΪ7
//    <7=>תгΪ8
//    <i>ע:ADMAxCH2+ADMAxCH1+ADMAxCH<=8
// <q0.25> ADOUT_λ
//<i>0: ֹͨADTRGADSOC״̬
//<i>1: ʹͨADTRGADSOC״̬
//</h>
#define CFG_ADC2_ADCON2_INIT	0x0

//<h>ADCMPCON_ADCȽϿƼĴ
//<i>0x02C    b31~16 b15   b14   b13   b12   b11~10 b9~8    b7    b6    b5    b4    b3    b2    b1    b0    
//<i>ADCMPCON -      ADGIE ADGDE ADLIE ADLDE -      ADGLCNT CSEN7 CSEN6 CSEN5 CSEN4 CSEN3 CSEN2 CSEN1 CSEN0 
// <q0.0> CSEN0_ڱȽϵĽĴʹ
//    <i>0: ֹADDR0еֵADDGTADDLTȽ
//    <i>1: ʹADDR0еֵADDGTADDLTȽ
// <q0.1> CSEN1_ڱȽϵĽĴʹ
//    <i>0: ֹADDR1еֵADDGTADDLTȽ
//    <i>1: ʹADDR1еֵADDGTADDLTȽ
// <q0.2> CSEN2_ڱȽϵĽĴʹ
//    <i>0: ֹADDR2еֵADDGTADDLTȽ
//    <i>1: ʹADDR2еֵADDGTADDLTȽ
// <q0.3> CSEN3_ڱȽϵĽĴʹ
//    <i>0: ֹADDR3еֵADDGTADDLTȽ
//    <i>1: ʹADDR3еֵADDGTADDLTȽ
// <q0.4> CSEN4_ڱȽϵĽĴʹ
//    <i>0: ֹADDR4еֵADDGTADDLTȽ
//    <i>1: ʹADDR4еֵADDGTADDLTȽ
// <q0.5> CSEN5_ڱȽϵĽĴʹ
//    <i>0: ֹADDR5еֵADDGTADDLTȽ
//    <i>1: ʹADDR5еֵADDGTADDLTȽ
// <q0.6> CSEN6_ڱȽϵĽĴʹ
//    <i>0: ֹADDR6еֵADDGTADDLTȽ
//    <i>1: ʹADDR6еֵADDGTADDLTȽ
// <q0.7> CSEN7_ڱȽϵĽĴʹ
//    <i>0: ֹADDR7еֵADDGTADDLTȽ
//    <i>1: ʹADDR7еֵADDGTADDLTȽ
// <o0.8..9> ADGLCNT_ȽϽ˲
//    <0=>1˲
//    <1=>2˲
//    <2=>3˲
//    <3=>4˲
// <q0.12> ADLDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽDMA
//    <i>1: ʹޱȽDMA
// <q0.13> ADLIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
// <q0.14> ADGDE_ޱȽDMAʹλ
//    <i>0: ֹޱȽϴDMA
//    <i>1: ʹޱȽϴDMA
// <q0.15> ADGIE_ޱȽжʹλ
//    <i>0: ֹޱȽж
//    <i>1: ʹޱȽж
//</h>
#define CFG_ADC2_ADCMPCON_INIT	0x0

//<h>ADDGT_ADCޱȽϼĴ
//<i>0x030 b31~16 b15~0 
//<i>ADDGT -      GT    
// <o0.0..15> GT_ADCޱȽϼĴ
//ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDGTеֵȽ <0-32768>
//</h>
#define CFG_ADC2_ADDGT_INIT	0x0

//<h>ADDLT_ADCޱȽϼĴ
//<i>0x034 b31~16 b15~0 
//<i>ADDLT -      LT    
// <o0.0..15> LT_ADCޱȽϼĴ
//ÿͨתʱADCMPCONĴCSENxλʹܵĽĴֵADDLTеֵȽ <0-32768>
//</h>
#define CFG_ADC2_ADDLT_INIT	0x0

//<h>SEQCHSEL0_ADCͨѡĴ0
//<i>0x038     b31~28 b27~24 b23~20 b19~16 b15~12 b11~8  b7~4   b3~0   
//<i>SEQCHSEL0 SEQCH7 SEQCH6 SEQCH5 SEQCH4 SEQCH3 SEQCH2 SEQCH1 SEQCH0 
// <o0.0..3> SEQCH0_ģͨ0ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.4..7> SEQCH1_ģͨ1ѡĴ飬ֱָӦģͨ:
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.8..11> SEQCH2_ģͨ2ѡĴ飬ֱָӦģͨ:
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.12..15> SEQCH3_ģͨ3ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.16..19> SEQCH4_ģͨ4ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.20..23> SEQCH5_ģͨ5ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.24..27> SEQCH6_ģͨ6ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
// <o0.28..31> SEQCH7_ģͨ7ѡĴ飬ֱָӦģͨ
//    <0=>AN0
//    <1=>AN1
//    <2=>AN2
//    <3=>AN3
//    <4=>AN4
//    <5=>AN5
//    <6=>OP0OUT
//    <7=>OP1OUT
//    <8=>OP2OUT
//    <9=>Vbg
//</h>
#define CFG_ADC2_SEQCHSEL0_INIT	0x00

//<h>ADGAPON_ADCͨGAPλ
//<i>0x040   b31~8 b7     b6     b5     b4     b3     b2     b1     b0     
//<i>ADGAPON -     GAPEN7 GAPEN6 GAPEN5 GAPEN4 GAPEN3 GAPEN2 GAPEN1 GAPEN0 
// <q0.0> GAPEN0_ADCͨ0GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.1> GAPEN1_ADCͨ1GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.2> GAPEN2_ADCͨ2GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.3> GAPEN3_ADCͨ3GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.4> GAPEN4_ADCͨ4GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.5> GAPEN5_ADCͨ5GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> GAPEN6_ADCͨ6GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
// <q0.7> GAPEN7_ADCͨ7GAPʱʹλ
//<i>0: ֹ
//<i>1: ʹ
//</h>
#define CFG_ADC2_ADGAPON_INIT	0x0
#define CFG_ADC2_INT_EN (((CFG_ADC2_ADCON1_VALUE & 0x10)>>4) | ((CFG_ADC2_ADCON1_VALUE & (1<<14))>>14)| ((CFG_ADC2_ADCON1_VALUE & (1<<23))>>23))
//</e>
//***********************************************************/
//</e>


//<e>QEI
#define     CFG_QEI_EN          0

//<e>QEI0
#define     CFG_QEI0_EN          0
//<h>QEICON_QEIƼĴ
//<i>0x000  b31~12 b11~8 b7    b6    b5    b4     b3    b2   b1~0  
//<i>QEICON -      QTEPS QPDIR QIDXS QSWAP QIDXEN QEIEN QTSR QPMOD 
// <o0.3> QEIEN_QEIʹܿλ
//    <0=>QEIܹر
//    <1=>QEIܴ򿪣幤ģʽQPMOD
// <o0.0..1> QPMOD_QEIλüѡλ
//    <0=>x2ģʽʹINDExλλü
//    <1=>x2ģʽλü0~QPOSMAxԶѭ
//    <2=>x4ģʽʹINDExλλü
//    <3=>x4ģʽλü0~QPOSMAxԶѭ
// <q0.2> QTSR_QEIʱλ
//    <i>0: Qtimer
//    <i>1: Qtimer
//<i>λΪQEIENΪ0ʱQtimerǷĿλQEIENΪ1ʱλЧ
// <o0.4> QIDXEN_λüλʹλ
//    <0=>INDExܸλλü
//    <1=>INDExԸλλü
//<i>QPMOD=00/10ʱЧ
// <o0.5> QSWAP_QEAQEB뽻ѡλ
//    <0=>QEAQEBδֱṩ
//    <1=>QEAQEBṩ
// <o0.8..11> QTEPS_QEI(QTimer)׽¼Ƶλ
//    <0=>QCLK/1
//    <1=>QCLK/2
//    <2=>QCLK/4
//    <3=>QCLK/8
//    <4=>QCLK/16
//    <5=>QCLK/32
//    <6=>QCLK/64
//    <7=>QCLK/128
//    <8=>QCLK/256
//    <9=>QCLK/512
//    <10=>QCLK/1024
//    <11=>QCLK/2048
//<i>QEIܴʱЧ
//</h>
#define     CFG_QEI0_QEICON_INIT 0x0


//<h>QFLTCON_QEI˲ƼĴ
//<i>0x004   b31~8 b7      b6~4    b3     b2~0   
//<i>QFLTCON -     QIDXFEN QIDXCPS QABFEN QABCPS 
// <q0.3> QABFEN_QEAQEB˲ʹλ
//    <i>0: ֹQEAQEBŵ˲
//    <i>1: ʹQEAQEBŵ˲
// <o0.0..2> QABCPS_QEAQEB˲ʱӷƵϵ
//    <0=>1/1 PCLK
//    <1=>1/2 PCLK
//    <2=>1/4 PCLK
//    <3=>1/16 PCLK
//    <4=>1/32 PCLK
//    <5=>1/64 PCLK
//    <6=>1/128 PCLK
//    <7=>1/256 PCLK
// <q0.7> QIDXFEN_INDEx˲ʹλ
//    <i>0: ֹINDExŵ˲
//    <i>1: ʹINDExŵ˲
// <o0.4..6> QIDXCPS_INDEx˲ʱӷƵϵ
//    <0=>1/1 PCLK
//    <1=>1/2 PCLK
//    <2=>1/4 PCLK
//    <3=>1/16 PCLK
//    <4=>1/32 PCLK
//    <5=>1/64 PCLK
//    <6=>1/128 PCLK
//    <7=>1/256 PCLK
//</h>
#define     CFG_QEI0_QFLTCON_INIT 0x0


//<h>QPOSMAX_QEIֵĴ
//<i>0x00C   b31~16 b15~0   
//<i>QPOSMAX -      QPOSMAX 
// <o0.0..15> QPOSMAX_λüֵĴ <0-65535>
//</h>
#define     CFG_QEI0_QPOSMAX_INIT 0x0


//<h>QCNTMIN_QEIڲ׽ʱСֵ
//<i>0x018   b31~16 b15~0   
//<i>QCNTMIN -      QCNTMIN 
// <o0.0..15> QCNTMIN_׽¼СQCNTֵ<0-65535>
//<i>׽¼ʱQCNTֵСQCNTMINĴֵ˴β׽¼
//<i>ͬʱڲĲ׽Ƶ1ֱ׽QCNTֵQCNTMINѷƵʹʱQCNTԶ浽QTMLATС 
//</h>
#define     CFG_QEI0_QCNTMIN_INIT 0x0


//<h>QTPR_QEIڼĴ
//<i>0x01C b31~0 
//<i>QTPR  QTPR  
// <o0.0..31> QTPR_32bitڼĴ
//</h>
#define     CFG_QEI0_QTPR_INIT 0x0


//<h>QTPSQ_QEIԤƵֵ
//<i>0x020 b31~16 b15~0 
//<i>QTPSQ -      QTPSQ 
// <o0.0..15> QTPSQ_ԤƵֵ <0-65535>
//</h>
#define     CFG_QEI0_QTPSQ_INIT 0x0


//<h>QEIINT_QEIжϺ״̬Ĵ
//<i>0x024  b31~5 b4   b3      b2    b1    b0    
//<i>QEIINT -     QTIE QTCAPIE QEIIE QCEIE QPEIE 
// <q0.0> QPEIE_λжʹλ
//    <i>0: ֹλж
//    <i>1: ʹλж
//<i>QEA/QEBźͬʱΪλ
// <q0.1> QCEIE_жʹλ,INDExźŸλλüģʽ
//    <i>0: ֹж
//    <i>1: ʹܴж
//<i>INDExźŸλλüģʽ£жʹλ
// <q0.2> QEIIE_QEIжʹλ
//    <i>0: ֹQEIж
//    <i>1: ʹQEIж
// <q0.3> QTCAPIE_QTimer׽¼жʹλ
//    <i>0: ֹ׽¼ʱж
//    <i>1: ׽¼ʱж
//<i>QEIܴʱЧ
// <q0.4> QTIE_Qtimerжʹλ
//    <i>0: ֹQtimer¼ʱж
//    <i>1: Qtimer¼ʱж
//</h>
#define     CFG_QEI0_QEIINT_INIT 0x0


#if ((CFG_QEI0_QEIINT_INIT & 0x1F) != 0)
	#define CFG_QEI0_INT_EN 1
#else
	#define CFG_QEI0_INT_EN 0
#endif
//</e>

//<e>QEI1
#define     CFG_QEI1_EN          0
//<h>QEICON_QEIƼĴ
//<i>0x000  b31~12 b11~8 b7    b6    b5    b4     b3    b2   b1~0  
//<i>QEICON -      QTEPS QPDIR QIDXS QSWAP QIDXEN QEIEN QTSR QPMOD 
// <o0.3> QEIEN_QEIʹܿλ
//    <0=>QEIܹر
//    <1=>QEIܴ򿪣幤ģʽQPMOD
// <o0.0..1> QPMOD_QEIλüѡλ
//    <0=>x2ģʽʹINDExλλü
//    <1=>x2ģʽλü0~QPOSMAxԶѭ
//    <2=>x4ģʽʹINDExλλü
//    <3=>x4ģʽλü0~QPOSMAxԶѭ
// <q0.2> QTSR_QEIʱλ
//    <i>0: Qtimer
//    <i>1: Qtimer
//<i>λΪQEIENΪ0ʱQtimerǷĿλQEIENΪ1ʱλЧ
// <o0.4> QIDXEN_λüλʹλ
//    <0=>INDExܸλλü
//    <1=>INDExԸλλü
//<i>QPMOD=00/10ʱЧ
// <o0.5> QSWAP_QEAQEB뽻ѡλ
//    <0=>QEAQEBδֱṩ
//    <1=>QEAQEBṩ
// <o0.8..11> QTEPS_QEI(QTimer)׽¼Ƶλ
//    <0=>QCLK/1
//    <1=>QCLK/2
//    <2=>QCLK/4
//    <3=>QCLK/8
//    <4=>QCLK/16
//    <5=>QCLK/32
//    <6=>QCLK/64
//    <7=>QCLK/128
//    <8=>QCLK/256
//    <9=>QCLK/512
//    <10=>QCLK/1024
//    <11=>QCLK/2048
//<i>QEIܴʱЧ
//</h>
#define     CFG_QEI1_QEICON_INIT 0x0


//<h>QFLTCON_QEI˲ƼĴ
//<i>0x004   b31~8 b7      b6~4    b3     b2~0   
//<i>QFLTCON -     QIDXFEN QIDXCPS QABFEN QABCPS 
// <q0.3> QABFEN_QEAQEB˲ʹλ
//    <i>0: ֹQEAQEBŵ˲
//    <i>1: ʹQEAQEBŵ˲
// <o0.0..2> QABCPS_QEAQEB˲ʱӷƵϵ
//    <0=>1/1 PCLK
//    <1=>1/2 PCLK
//    <2=>1/4 PCLK
//    <3=>1/16 PCLK
//    <4=>1/32 PCLK
//    <5=>1/64 PCLK
//    <6=>1/128 PCLK
//    <7=>1/256 PCLK
// <q0.7> QIDXFEN_INDEx˲ʹλ
//    <i>0: ֹINDExŵ˲
//    <i>1: ʹINDExŵ˲
// <o0.4..6> QIDXCPS_INDEx˲ʱӷƵϵ
//    <0=>1/1 PCLK
//    <1=>1/2 PCLK
//    <2=>1/4 PCLK
//    <3=>1/16 PCLK
//    <4=>1/32 PCLK
//    <5=>1/64 PCLK
//    <6=>1/128 PCLK
//    <7=>1/256 PCLK
//</h>
#define     CFG_QEI1_QFLTCON_INIT 0x0


//<h>QPOSMAX_QEIֵĴ
//<i>0x00C   b31~16 b15~0   
//<i>QPOSMAX -      QPOSMAX 
// <o0.0..15> QPOSMAX_λüֵĴ <0-65535>
//</h>
#define     CFG_QEI1_QPOSMAX_INIT 0x0


//<h>QCNTMIN_QEIڲ׽ʱСֵ
//<i>0x018   b31~16 b15~0   
//<i>QCNTMIN -      QCNTMIN 
// <o0.0..15> QCNTMIN_׽¼СQCNTֵ<0-65535>
//<i>׽¼ʱQCNTֵСQCNTMINĴֵ˴β׽¼
//<i>ͬʱڲĲ׽Ƶ1ֱ׽QCNTֵQCNTMINѷƵʹʱQCNTԶ浽QTMLATС 
//</h>
#define     CFG_QEI1_QCNTMIN_INIT 0x0


//<h>QTPR_QEIڼĴ
//<i>0x01C b31~0 
//<i>QTPR  QTPR  
// <o0.0..31> QTPR_32bitڼĴ
//</h>
#define     CFG_QEI1_QTPR_INIT 0x0


//<h>QTPSQ_QEIԤƵֵ
//<i>0x020 b31~16 b15~0 
//<i>QTPSQ -      QTPSQ 
// <o0.0..15> QTPSQ_ԤƵֵ <0-65535>
//</h>
#define     CFG_QEI1_QTPSQ_INIT 0x0


//<h>QEIINT_QEIжϺ״̬Ĵ
//<i>0x024  b31~5 b4   b3      b2    b1    b0    
//<i>QEIINT -     QTIE QTCAPIE QEIIE QCEIE QPEIE 
// <q0.0> QPEIE_λжʹλ
//    <i>0: ֹλж
//    <i>1: ʹλж
//<i>QEA/QEBźͬʱΪλ
// <q0.1> QCEIE_жʹλ,INDExźŸλλüģʽ
//    <i>0: ֹж
//    <i>1: ʹܴж
//<i>INDExźŸλλüģʽ£жʹλ
// <q0.2> QEIIE_QEIжʹλ
//    <i>0: ֹQEIж
//    <i>1: ʹQEIж
// <q0.3> QTCAPIE_QTimer׽¼жʹλ
//    <i>0: ֹ׽¼ʱж
//    <i>1: ׽¼ʱж
//<i>QEIܴʱЧ
// <q0.4> QTIE_Qtimerжʹλ
//    <i>0: ֹQtimer¼ʱж
//    <i>1: Qtimer¼ʱж
//</h>
#define     CFG_QEI1_QEIINT_INIT 0x0


#if ((CFG_QEI1_QEIINT_INIT & 0x1F) != 0)
	#define CFG_QEI1_INT_EN 1
#else
	#define CFG_QEI1_INT_EN 0
#endif

//</e>
//***********************************************************/
//</e>

//<e>AMOC
#define CFG_AMOC_EN	0
//<h>CMP0CON_Ƚ0ƼĴ
//<i>0x000   b31~23 b22~16  b15    b14      b13    b12     b11~10 b9~8  b7     b6~4   b3    b2~0  
//<i>CMP0CON -      CMP0VRS CMP0EN CMP0VCMP C0DE   C0OUTEN C0IES  C0SMT C0NCHS C0PCHS C0OUT C0DEB 
// <q0.15> CMP0EN_Ƚ0ʹܿλ
//    0: Ƚ0ر
//    1: Ƚ0
// <o0.0..2> C0DEB_Ƚ0ź˲ʱ
//    <0=>˲
//    <1=>˲ʱ䳣Ϊ0.5us
//    <2=>˲ʱ䳣Ϊ1us
//    <3=>˲ʱ䳣Ϊ2us
//    <4=>˲ʱ䳣Ϊ4us
//    <5=>˲ʱ䳣Ϊ8us
//    <6=>˲ʱ䳣Ϊ12us
//    <7=>˲ʱ䳣Ϊ16us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.4..6> C0PCHS_Ƚ0ͬѡλ
//    <0=>ѡCP0P0ΪȽ0ͬ
//    <1=>ѡCP0P1ΪȽ0ͬ
//    <2=>ѡCP0P2ΪȽ0ͬ
//    <4=>ѡOP0OUTΪȽ0ͬ
//    <5=>ѡOP1OUTΪȽ0ͬ
//    <6=>ѡOP2OUTΪȽ0ͬ
// <o0.7> C0NCHS_Ƚ0ѡλ
//    <0=>ѡC0NΪȽ0
//    <1=>ѡڲοCMP0VRSΪȽ0
// <o0.8..9> C0SMT_Ƚ0ʩصѹѡλ
//    <0=>no
//    <1=>10mV
//    <2=>20mV
//    <3=>50mV
// <o0.10..11> C0IES_Ƚ0жģʽѡλ
//    <0=>жϱ־
//    <1=>½شȽ0ɸ߱ʱжϱ־
//    <2=>شȽ0ɵͱʱжϱ־
//    <3=>˫شȽ0ɸ߱ʱɵͱʱжϱ־
// <o0.12> C0OUTEN_ȽI/Oʹ
//    <0=>ͨȽ0CP0OUTCP0OUTΪͨIO
//    <1=>ͨȽ0CP0OUT
//    <i>I/OǴ˲ܵģ˲ΪC0DEB[2:0]趨ֵ
// <q0.13> C0DE_Ƚ0DMAʹλ
//<i>    0жϱ־λʱDMA
//<i>    1жϱ־λʱDMA
//<i>    עжϱ־λʽC0IES[1:0]
// <o0.14> CMP0VCMP_Ƚ0˲οѹԴVcmpѡλ
//    <0=>AVDD
//    <1=>Vref

// <o0.16..22> CMP0VRS_Ƚ0˲οѹѡλ<0-127>
//    <i>(1+CMP0VRS)/ 128 *Vcmp
//    <i>˵ѹΧΪ1/128~128/128
//</h>
#define CFG_AMOC_CMP0CON_INIT	0x00


//<h>CMP1CON_Ƚ1ƼĴ
//<i>0x004   b31~23 b22~16  b15    b14      b13    b12     b11~10 b9~8     b7     b6~4   b3    b2~0  
//<i>CMP1CON -      CMP1VRS CMP1EN CMP1VCMP C0DE   C1OUTEN C1IES  C1SMT    C1NCHS C1PCHS C1OUT C1DEB 
// <q0.15> CMP1EN_Ƚ1ʹܿλ
//    <i>0: Ƚ1ر
//    <i>1: Ƚ1
// <o0.0..2> C1DEB_Ƚ1ź˲ʱ
//    <0=>˲
//    <1=>˲ʱ䳣Ϊ0.5us
//    <2=>˲ʱ䳣Ϊ1us
//    <3=>˲ʱ䳣Ϊ2us
//    <4=>˲ʱ䳣Ϊ4us
//    <5=>˲ʱ䳣Ϊ8us
//    <6=>˲ʱ䳣Ϊ12us
//    <7=>˲ʱ䳣Ϊ16us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.4..6> C1PCHS_Ƚ1ͬѡλ
//    <0=>ѡCP1P0ΪȽ1ͬ
//    <1=>ѡCP1P1ΪȽ1ͬ
//    <2=>ѡCP1P2ΪȽ1ͬ
//    <4=>ѡOP0OUTΪȽ1ͬ
//    <5=>ѡOP1OUTΪȽ1ͬ
//    <6=>ѡOP2OUTΪȽ1ͬ
// <o0.7> C1NCHS_Ƚ1ѡλ
//    <0=>ѡC1NΪȽ1
//    <1=>ѡڲοCMP1VRSΪȽ1
// <o0.8..9> C1SMT_Ƚ1ʩصѹѡλ
//    <0=>no
//    <1=>10mV
//    <2=>20mV
//    <3=>50mV
// <o0.10..11> C1IES_Ƚ1жģʽѡλ
//    <0=>жϱ־
//    <1=>½شȽ1ɸ߱ʱжϱ־
//    <2=>شȽ1ɵͱʱжϱ־
//    <3=>˫شȽ1ɸ߱ʱɵͱʱжϱ־
// <o0.12> C1OUTEN_ȽI/Oʹ
//    <0=>ͨȽ1CP1OUTCP1OUTΪͨIO
//    <1=>ͨȽ1CP1OUT
//    <i>I/OǴ˲ܵģ˲ΪC1DEB[2:0]趨ֵ
// <q0.13> C1DE_Ƚ1DMAʹλ
//<i>    0жϱ־λʱDMA
//<i>    1жϱ־λʱDMA
//<i>    עжϱ־λʽC1IES[1:0]
// <o0.14> CMP1VCMP_Ƚ1˲οѹԴVcmpѡλ:
//    <0=>AVDD
//    <1=>Vref
// <o0.16..22> CMP1VRS_Ƚ1˲οѹѡλ<0-127>
//    <i>(1+CMP1VRS)/ 128 *Vcmp
//    <i>˵ѹΧΪ1/128~128/128
//</h>
#define CFG_AMOC_CMP1CON_INIT	0x0

//<h>CMP2CON_Ƚ2ƼĴ
//<i>0x008   b31~23 b22~16  b15    b14      b13    b12     b11~10 b9~8     b7     b6~4   b3    b2~0  
//<i>CMP2CON -      CMP2VRS CMP2EN CMP2VCMP C2DE   C2OUTEN C2IES  C2SMT    C2NCHS C2PCHS C2OUT C2DEB 
// <q0.15> CMP2EN_Ƚ2ʹܿλ
//    <i>0: Ƚ2ر
//    <i>1: Ƚ2
// <o0.0..2> C2DEB_Ƚ2ź˲ʱ
//    <0=>˲
//    <1=>˲ʱ䳣Ϊ0.5us
//    <2=>˲ʱ䳣Ϊ1us
//    <3=>˲ʱ䳣Ϊ2us
//    <4=>˲ʱ䳣Ϊ4us
//    <5=>˲ʱ䳣Ϊ8us
//    <6=>˲ʱ䳣Ϊ12us
//    <7=>˲ʱ䳣Ϊ16us
//    <i>˲ʱ䲻Ǿȷֵ
//    <i>˲㷨˵:HSIʱӲ˲롣ֻ֣Ϊߵƽ1趨ĳ˲1ͬʱΪ˲Ϊ͵ƽ1С˲,˲0ͬʱΪ0
// <o0.4..6> C2PCHS_Ƚ2ͬѡλ
//    <0=>ѡCP2P0ΪȽ2ͬ
//    <1=>ѡCP2P1ΪȽ2ͬ
//    <2=>ѡCP2P2ΪȽ2ͬ
//    <4=>ѡOP0OUTΪȽ2ͬ
//    <5=>ѡOP1OUTΪȽ2ͬ
//    <6=>ѡOP2OUTΪȽ2ͬ
// <o0.7> C2NCHS_Ƚ2ѡλ
//    <0=>ѡC2NΪȽ2
//    <1=>ѡڲοCMP2VRSΪȽ1
// <o0.8..9> C2SMT_Ƚ2ʩصѹѡλ
//    <0=>no
//    <1=>10mV
//    <2=>20mV
//    <3=>50mV
// <o0.10..11> C2IES_Ƚ2жģʽѡλ
//    <0=>жϱ־
//    <1=>½شȽ2ɸ߱ʱжϱ־
//    <2=>شȽ2ɵͱʱжϱ־
//    <3=>˫شȽ1ɸ߱ʱɵͱʱжϱ־
// <o0.12> C2OUTEN_ȽI/Oʹ
//    <0=>ͨȽ2CP2OUTCP2OUTΪͨIO
//    <1=>ͨȽ2CP2OUT
//    <i>I/OǴ˲ܵģ˲ΪC2DEB[2:0]趨ֵ
// <q0.13> C2DE_Ƚ2DMAʹλ
//<i>    0жϱ־λʱDMA
//<i>    1жϱ־λʱDMA
//<i>    עжϱ־λʽC2IES[1:0]
// <o0.14> CMP2VCMP_Ƚ2˲οѹԴVcmpѡλ:
//    <0=>AVDD
//    <1=>Vref
// <o0.16..22> CMP2VRS_Ƚ1˲οѹѡλ<0-127>
//    <i>(1+CMP2VRS)/ 128 *Vcmp
//    <i>˵ѹΧΪ1/128~128/128
//</h>
#define CFG_AMOC_CMP2CON_INIT	0x0

//<h>OPCON_ŴƼĴ1
//<i>0x010 b31~3 b2    b1    b0    
//<i>OPCON -     OP2EN OP1EN OP0EN 
// <q0.0> OP0EN_Ŵ0ʹܿλ
//    <i>0: Ŵ0ر
//    <i>1: Ŵ0
//   <i>IOóģ룬ӦI/O AFĴΪģAF
// <q0.1> OP1EN_Ŵ1ʹܿλ
//    <i>0: Ŵ1ر
//    <i>1: Ŵ1
//<i>IOóģ룬ӦI/O AFĴΪģAF
// <q0.2> OP2EN_Ŵ2ʹܿλ
//    <i>0: Ŵ2ر
//    <i>1: Ŵ2
//<i>IOóģ룬ӦI/O AFĴΪģAF
//</h>
#define CFG_AMOC_OPCON_INIT	0x0

#define CFG_CMP0_INT_EN ((CFG_AMOC_CMP0CON_INIT & (3<<11) >>11) != 0)
#define CFG_CMP1_INT_EN ((CFG_AMOC_CMP1CON_INIT & (3<<11) >>11) != 0)
#define CFG_CMP2_INT_EN ((CFG_AMOC_CMP2CON_INIT & (3<<11) >>11) != 0)

//***********************************************************/
//</e>

//<e>UART
#define     CFG_UART_EN         0
//***********************************************************/
//<e>UART0
#define     CFG_UART0_EN        0
//       <o>Baudrate
//             <115200=> 115200
//             <57600=>  57600
//             <38400=>  38400
//             <19200=>  19200
//             <9600=>   9600
//             <4800=>   4800
//             <2400=>   2400
//             <1200=>   1200
//             <600=>     600
#define     CFG_UART0_BAUDRATE  9600

//<h>ADDR_UARTַüĴ
//<i>0x10 b31~16 b15~8 b7~0  
//<i>ADDR -      SMAR  SADDR 
// <o0.0..7> SADDR_UARTӲַ<0-0xFF>
// <o0.8..15> SMAR_λμĴ<0-0xFF>
//<i>SADDRЩλᱻ
//</h>
#define     CFG_UART0_ADDR_INIT 0x0

//<h>CR_UARTƼĴ
//<i>0x18 b31~24 b23 b22  b21  b20    b19    b18 b17 b16   b15 b14~13 b12~11 b10 b9 b8    b7   b6    b5   b4  b3  b2    b1     b0   
//<i>CR   -      FER TNEG RNEG DMAT   DMAR   TEN REN LINEN SBK SM     MULTIE PS  -  SMOD0 LBDL LBDIE TCIE TIE RIE SMOD2 SBRTEN STOP 
// <o0.13..14> SM_UARTзʽλ
//    <0=>ʽ0ͬʽ̶
//    <1=>ʽ18λ첽ʽɱ䲨
//    <2=>ʽ29λ첽ʽ̶
//    <3=>ʽ39λ첽ʽɱ䲨
// <o0.0> STOP_ֹͣλλѡλ
//     <0=>ѡʹ1λֹͣλ
//     <1=>ѡʹ2λֹͣλ
// <o0.1> SBRTEN_UARTʷʹܿλ
//    <0=>òʷ
//    <1=>ʹܲʷ
// <o0.2> SMOD2_UARTʽ2ʿλ
//    <0=>ڷʽ2УΪPCLK1ʱӵ1/256
//    <1=>ڷʽ2УΪPCLK1ʱӵ1/128
// <q0.3> RIE_ֽڽжʹλ
//    <i>0: ֹжϱ־λRIж
//    <i>1: жϱ־λRIж
// <q0.4> TIE_ֽڴжʹλ
//    <i>0: ֹжϱ־λTIж
//    <i>1: жϱ־λTIж 
// <q0.5> TCIE_ȫݷжʹλ
//    <i>0: ֹݷɱ־λTCжϹ
//    <i>1: ݷɱ־λTCжϹ
// <o0.8> SMOD0_UARTʽ0ʿλ
//    <0=>ڷʽ0УΪPCLK1ʱӵ1/48
//    <1=>ڷʽ0УΪPCLK1ʱӵ1/12
// <o0.10> PS_żУѡλ
//  <0=>У
//  <1=>żУ
// <o0.11..12> MULTIE_ھλѡλ
//  <0=>Զ
//  <1=>ͨѶ־λ
//  <2=>żУλ
// <q0.17> REN_UARTչʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.18> TEN_UART͹ʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <q0.19> DMAR_UARTDMAͨʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.20> DMAT_UARTDMAͨʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <o0.21> RNEG_UARTռԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <o0.22> TNEG_UARTͼԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <q0.23> FER_֡λ
//    <i>0: ֹֹͣλȷϼ飬κֹͣλλRI
//    <i>1: ֹͣλȷϼ飬ֻЧֹͣλ(1)λRI
// <e0.16> LINģʽ
//    <i>0: ֹLINģʽ
//    <i>1: ʹLINģʽ
// <o0.15> SBK_LINͬʹλ
//    <0=>ͬ
//    <1=>ͬɺԶ
// <q0.6> LBDIE_LINϷָжʹλ
//    <i>0: ֹLINϷָжϹ
//    <i>1: LINϷָжϹ
// <o0.7> LBDL_ֵͬ
//  <0=>͵ƽȴ10BIT͵ƽ
//  <1=>͵ƽȴ11BIT͵ƽ
//</e>
//</h>
#define     CFG_UART0_CR_INIT   0x62002


#if ((CFG_UART0_CR_INIT & (0xF<<3)) != 0)
	#define CFG_UART0_INT_EN 1
#else
	#define CFG_UART0_INT_EN 0
#endif

#define CFG_UART0_CLK		(SYS_CLK/APB_F)
#if ((CFG_UART0_CR_INIT & 0x2000) == 0x2000)
#define CFG_UART0_SBRT_VALUE 	(CFG_UART0_CLK/16/CFG_UART0_BAUDRATE-1)
#define CFG_UART0_BFINE_VALUE 	(CFG_UART0_CLK/CFG_UART0_BAUDRATE - (16*CFG_UART0_SBRT_VALUE+1))
#define CFG_UART0_BRT_INIT	(CFG_UART0_SBRT_VALUE | (CFG_UART0_BFINE_VALUE<<16))
#else
#define CFG_UART0_BRT_INIT	0
#endif
//</e>




//***********************************************************/
//<e>UART1
#define     CFG_UART1_EN        0
//       <o>Baudrate
//             <115200=> 115200
//             <57600=>  57600
//             <38400=>  38400
//             <19200=>  19200
//             <9600=>   9600
//             <4800=>   4800
//             <2400=>   2400
//             <1200=>   1200
//             <600=>     600     
#define     CFG_UART1_BAUDRATE  9600

//<h>ADDR_UARTַüĴ
//<i>0x10 b31~16 b15~8 b7~0  
//<i>ADDR -      SMAR  SADDR 
// <o0.0..7> SADDR_UARTӲַ<0-0xFF>
// <o0.8..15> SMAR_λμĴ<0-0xFF>
//<i>SADDRЩλᱻ
//</h>
#define     CFG_UART1_ADDR_INIT 0x0

//<h>CR_UARTƼĴ
//<i>0x18 b31~24 b23 b22  b21  b20    b19    b18 b17 b16   b15 b14~13 b12~11 b10 b9 b8    b7   b6    b5   b4  b3  b2    b1     b0   
//<i>CR   -      FER TNEG RNEG DMAT   DMAR   TEN REN LINEN SBK SM     MULTIE PS  -  SMOD0 LBDL LBDIE TCIE TIE RIE SMOD2 SBRTEN STOP 
// <o0.13..14> SM_UARTзʽλ
//    <0=>ʽ0ͬʽ̶
//    <1=>ʽ18λ첽ʽɱ䲨
//    <2=>ʽ29λ첽ʽ̶
//    <3=>ʽ39λ첽ʽɱ䲨
// <o0.0> STOP_ֹͣλλѡλ
//     <0=>ѡʹ1λֹͣλ
//     <1=>ѡʹ2λֹͣλ
// <o0.1> SBRTEN_UARTʷʹܿλ
//    <0=>òʷ
//    <1=>ʹܲʷ
// <o0.2> SMOD2_UARTʽ2ʿλ
//    <0=>ڷʽ2УΪPCLK1ʱӵ1/256
//    <1=>ڷʽ2УΪPCLK1ʱӵ1/128
// <q0.3> RIE_ֽڽжʹλ
//    <i>0: ֹжϱ־λRIж
//    <i>1: жϱ־λRIж
// <q0.4> TIE_ֽڴжʹλ
//    <i>0: ֹжϱ־λTIж
//    <i>1: жϱ־λTIж 
// <q0.5> TCIE_ȫݷжʹλ
//    <i>0: ֹݷɱ־λTCжϹ
//    <i>1: ݷɱ־λTCжϹ
// <o0.8> SMOD0_UARTʽ0ʿλ
//    <0=>ڷʽ0УΪPCLK1ʱӵ1/48
//    <1=>ڷʽ0УΪPCLK1ʱӵ1/12
// <o0.10> PS_żУѡλ
//  <0=>У
//  <1=>żУ
// <o0.11..12> MULTIE_ھλѡλ
//  <0=>Զ
//  <1=>ͨѶ־λ
//  <2=>żУλ
// <q0.17> REN_UARTչʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.18> TEN_UART͹ʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <q0.19> DMAR_UARTDMAͨʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.20> DMAT_UARTDMAͨʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <o0.21> RNEG_UARTռԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <o0.22> TNEG_UARTͼԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <q0.23> FER_֡λ
//    <i>0: ֹֹͣλȷϼ飬κֹͣλλRI
//    <i>1: ֹͣλȷϼ飬ֻЧֹͣλ(1)λRI
// <e0.16> LINģʽ
//    <i>0: ֹLINģʽ
//    <i>1: ʹLINģʽ
// <o0.15> SBK_LINͬʹλ
//    <0=>ͬ
//    <1=>ͬɺԶ
// <q0.6> LBDIE_LINϷָжʹλ
//    <i>0: ֹLINϷָжϹ
//    <i>1: LINϷָжϹ
// <o0.7> LBDL_ֵͬ
//  <0=>͵ƽȴ10BIT͵ƽ
//  <1=>͵ƽȴ11BIT͵ƽ
//</e>
//</h>
#define     CFG_UART1_CR_INIT   0x62002


#if ((CFG_UART1_CR_INIT & (0xF<<3)) != 0)
	#define CFG_UART1_INT_EN 1
#else
	#define CFG_UART1_INT_EN 0
#endif


#define CFG_UART1_CLK		(SYS_CLK/APB_F)
#if ((CFG_UART1_CR_INIT & 0x2000) == 0x2000)
#define CFG_UART1_SBRT_VALUE 	(CFG_UART1_CLK/16/CFG_UART1_BAUDRATE-1)
#define CFG_UART1_BFINE_VALUE 	(CFG_UART1_CLK/CFG_UART1_BAUDRATE - (16*CFG_UART1_SBRT_VALUE+1))
#define CFG_UART1_BRT_INIT	(CFG_UART1_SBRT_VALUE | (CFG_UART1_BFINE_VALUE<<16))
#else
#define CFG_UART1_BRT_INIT	0
#endif
//</e>

//***********************************************************/
//<e>UART2
#define     CFG_UART2_EN        0
//       <o>Baudrate
//             <115200=> 115200
//             <57600=>  57600
//             <38400=>  38400
//             <19200=>  19200
//             <9600=>   9600
//             <4800=>   4800
//             <2400=>   2400
//             <1200=>   1200
//             <600=>     600     
#define     CFG_UART2_BAUDRATE  115200

//<h>ADDR_UARTַüĴ
//<i>0x10 b31~16 b15~8 b7~0  
//<i>ADDR -      SMAR  SADDR 
// <o0.0..7> SADDR_UARTӲַ<0-0xFF>
// <o0.8..15> SMAR_λμĴ<0-0xFF>
//<i>SADDRЩλᱻ
//</h>
#define     CFG_UART2_ADDR_INIT 0x0
//<h>CR_UARTƼĴ
//<i>0x18 b31~24 b23 b22  b21  b20    b19    b18 b17 b16   b15 b14~13 b12~11 b10 b9 b8    b7   b6    b5   b4  b3  b2    b1     b0   
//<i>CR   -      FER TNEG RNEG DMAT   DMAR   TEN REN LINEN SBK SM     MULTIE PS  -  SMOD0 LBDL LBDIE TCIE TIE RIE SMOD2 SBRTEN STOP 
// <o0.13..14> SM_UARTзʽλ
//    <0=>ʽ0ͬʽ̶
//    <1=>ʽ18λ첽ʽɱ䲨
//    <2=>ʽ29λ첽ʽ̶
//    <3=>ʽ39λ첽ʽɱ䲨
// <o0.0> STOP_ֹͣλλѡλ
//     <0=>ѡʹ1λֹͣλ
//     <1=>ѡʹ2λֹͣλ
// <o0.1> SBRTEN_UARTʷʹܿλ
//    <0=>òʷ
//    <1=>ʹܲʷ
// <o0.2> SMOD2_UARTʽ2ʿλ
//    <0=>ڷʽ2УΪPCLK1ʱӵ1/256
//    <1=>ڷʽ2УΪPCLK1ʱӵ1/128
// <q0.3> RIE_ֽڽжʹλ
//    <i>0: ֹжϱ־λRIж
//    <i>1: жϱ־λRIж
// <q0.4> TIE_ֽڴжʹλ
//    <i>0: ֹжϱ־λTIж
//    <i>1: жϱ־λTIж 
// <q0.5> TCIE_ȫݷжʹλ
//    <i>0: ֹݷɱ־λTCжϹ
//    <i>1: ݷɱ־λTCжϹ
// <o0.8> SMOD0_UARTʽ0ʿλ
//    <0=>ڷʽ0УΪPCLK1ʱӵ1/48
//    <1=>ڷʽ0УΪPCLK1ʱӵ1/12
// <o0.10> PS_żУѡλ
//  <0=>У
//  <1=>żУ
// <o0.11..12> MULTIE_ھλѡλ
//  <0=>Զ
//  <1=>ͨѶ־λ
//  <2=>żУλ
// <q0.17> REN_UARTչʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.18> TEN_UART͹ʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <q0.19> DMAR_UARTDMAͨʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.20> DMAT_UARTDMAͨʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <o0.21> RNEG_UARTռԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <o0.22> TNEG_UARTͼԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <q0.23> FER_֡λ
//    <i>0: ֹֹͣλȷϼ飬κֹͣλλRI
//    <i>1: ֹͣλȷϼ飬ֻЧֹͣλ(1)λRI
// <e0.16> LINģʽ
//    <i>0: ֹLINģʽ
//    <i>1: ʹLINģʽ
// <o0.15> SBK_LINͬʹλ
//    <0=>ͬ
//    <1=>ͬɺԶ
// <q0.6> LBDIE_LINϷָжʹλ
//    <i>0: ֹLINϷָжϹ
//    <i>1: LINϷָжϹ
// <o0.7> LBDL_ֵͬ
//  <0=>͵ƽȴ10BIT͵ƽ
//  <1=>͵ƽȴ11BIT͵ƽ
//</e>
//</h>
#define     CFG_UART2_CR_INIT   0x6062002

#if ((CFG_UART2_CR_INIT & (0xF<<3)) != 0)
	#define CFG_UART2_INT_EN 1
#else
	#define CFG_UART2_INT_EN 0
#endif


#define CFG_UART2_CLK		(SYS_CLK/APB_F)
#if ((CFG_UART2_CR_INIT & 0x2000) == 0x2000)
#define CFG_UART2_SBRT_VALUE 	(CFG_UART2_CLK/16/CFG_UART2_BAUDRATE-1)
#define CFG_UART2_BFINE_VALUE 	(CFG_UART2_CLK/CFG_UART2_BAUDRATE - (16*CFG_UART2_SBRT_VALUE+1))
#define CFG_UART2_BRT_INIT	(CFG_UART2_SBRT_VALUE | (CFG_UART2_BFINE_VALUE<<16))
#else
#define CFG_UART2_BRT_INIT	0
#endif
//</e>

//***********************************************************/
//<e>UART3
#define     CFG_UART3_EN        0
//       <o>Baudrate
//             <115200=> 115200
//             <57600=>  57600
//             <38400=>  38400
//             <19200=>  19200
//             <9600=>   9600
//             <4800=>   4800
//             <2400=>   2400
//             <1200=>   1200
//             <600=>     600     
#define     CFG_UART3_BAUDRATE  115200


//<h>ADDR_UARTַüĴ
//<i>0x10 b31~16 b15~8 b7~0  
//<i>ADDR -      SMAR  SADDR 
// <o0.0..7> SADDR_UARTӲַ<0-0xFF>
// <o0.8..15> SMAR_λμĴ<0-0xFF>
//<i>SADDRЩλᱻ
//</h>
#define     CFG_UART3_ADDR_INIT 0x0

//<h>CR_UARTƼĴ
//<i>0x18 b31~24 b23 b22  b21  b20    b19    b18 b17 b16   b15 b14~13 b12~11 b10 b9 b8    b7   b6    b5   b4  b3  b2    b1     b0   
//<i>CR   -      FER TNEG RNEG DMAT   DMAR   TEN REN LINEN SBK SM     MULTIE PS  -  SMOD0 LBDL LBDIE TCIE TIE RIE SMOD2 SBRTEN STOP 
// <o0.13..14> SM_UARTзʽλ
//    <0=>ʽ0ͬʽ̶
//    <1=>ʽ18λ첽ʽɱ䲨
//    <2=>ʽ29λ첽ʽ̶
//    <3=>ʽ39λ첽ʽɱ䲨
// <o0.0> STOP_ֹͣλλѡλ
//     <0=>ѡʹ1λֹͣλ
//     <1=>ѡʹ2λֹͣλ
// <o0.1> SBRTEN_UARTʷʹܿλ
//    <0=>òʷ
//    <1=>ʹܲʷ
// <o0.2> SMOD2_UARTʽ2ʿλ
//    <0=>ڷʽ2УΪPCLK1ʱӵ1/256
//    <1=>ڷʽ2УΪPCLK1ʱӵ1/128
// <q0.3> RIE_ֽڽжʹλ
//    <i>0: ֹжϱ־λRIж
//    <i>1: жϱ־λRIж
// <q0.4> TIE_ֽڴжʹλ
//    <i>0: ֹжϱ־λTIж
//    <i>1: жϱ־λTIж 
// <q0.5> TCIE_ȫݷжʹλ
//    <i>0: ֹݷɱ־λTCжϹ
//    <i>1: ݷɱ־λTCжϹ
// <o0.8> SMOD0_UARTʽ0ʿλ
//    <0=>ڷʽ0УΪPCLK1ʱӵ1/48
//    <1=>ڷʽ0УΪPCLK1ʱӵ1/12
// <o0.10> PS_żУѡλ
//  <0=>У
//  <1=>żУ
// <o0.11..12> MULTIE_ھλѡλ
//  <0=>Զ
//  <1=>ͨѶ־λ
//  <2=>żУλ
// <q0.17> REN_UARTչʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.18> TEN_UART͹ʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <q0.19> DMAR_UARTDMAͨʹλ
//    <i>0:ýչ
//    <i>1:ʹܽչ
// <q0.20> DMAT_UARTDMAͨʹλ
//    <i>0:÷͹
//    <i>1:ʹܷ͹
// <o0.21> RNEG_UARTռԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <o0.22> TNEG_UARTͼԿλ
//    <0=>ݲȡ
//    <1=>ȡ
// <q0.23> FER_֡λ
//    <i>0: ֹֹͣλȷϼ飬κֹͣλλRI
//    <i>1: ֹͣλȷϼ飬ֻЧֹͣλ(1)λRI
// <e0.16> LINģʽ
//    <i>0: ֹLINģʽ
//    <i>1: ʹLINģʽ
// <o0.15> SBK_LINͬʹλ
//    <0=>ͬ
//    <1=>ͬɺԶ
// <q0.6> LBDIE_LINϷָжʹλ
//    <i>0: ֹLINϷָжϹ
//    <i>1: LINϷָжϹ
// <o0.7> LBDL_ֵͬ
//  <0=>͵ƽȴ10BIT͵ƽ
//  <1=>͵ƽȴ11BIT͵ƽ
//</e>
//</h>
#define     CFG_UART3_CR_INIT   0x6062002

#if ((CFG_UART3_CR_INIT & (0xF<<3)) != 0)
	#define CFG_UART3_INT_EN 1
#else
	#define CFG_UART3_INT_EN 0
#endif

#define CFG_UART3_CLK		(SYS_CLK/APB_F)
#if ((CFG_UART3_CR_INIT & 0x2000) == 0x2000)
#define CFG_UART3_SBRT_VALUE 	(CFG_UART3_CLK/16/CFG_UART3_BAUDRATE-1)
#define CFG_UART3_BFINE_VALUE 	(CFG_UART3_CLK/CFG_UART3_BAUDRATE - (16*CFG_UART3_SBRT_VALUE+1))
#define CFG_UART3_BRT_INIT	(CFG_UART3_SBRT_VALUE | (CFG_UART3_BFINE_VALUE<<16))
#else
#define CFG_UART3_BRT_INIT	0
#endif
//</e>

//***********************************************************/
//</e>


//<e>SPI
#define     CFG_SPI_EN          0

//<e>SPI0
#define     CFG_SPI0_EN        0
//<i>CR_SPIƼĴ
//<i>0x00C b31~19 b18    b17~16 b15   b14   b13 b12 b11   b10   b9 b8  b7   b6   b5   b4    b3~0 
//<i>CR    -      FIFOEN SPDATL SPSFF SPIEN -   -   SPTIE SPRIE -  DIR MSTR CPHA CPOL SSDIS SPR 
// <q0.14> SPIEN_SPIʹλ
// <o0.7> MSTR_SPI豸λ
//     <0=>SPIΪ豸
//     <1=>SPIΪ豸
// <o0.0..3> SPR_ʱӷƵλ
//    <0=>PCLK1 /2
//    <1=>PCLK1 /4
//    <2=>PCLK1 /8
//    <3=>PCLK1 /16
//    <4=>PCLK1 /32
//    <5=>PCLK1 /64
//    <6=>PCLK1 /128
//    <7=>PCLK1 /256
//    <8=>PCLK1 /512
//    <9=>PCLK1 /1024
//    :
// <o0.4> SSDIS_NSSſλ
//    <0=>/ģʽ£NSS
//    <1=>/ģʽ£رNSSţMODFж
//    <i>ڴģʽ£CPHA=0λ
// <o0.5> CPOL_ʱӼԿλ
//    <0=>IDLE״̬CKڵ͵ƽ
//    <1=>IDLE״̬CKڸߵƽ
// <o0.6> CPHA_ʱλλ
//     <0=>CKڵĵһزɼ
//     <1=>CKڵĵڶزɼ
// <o0.8> DIR_ݴͷѡλ
//     <0=>MSBȷ
//     <1=>LSBȷ
// <q0.10> SPRIE_SPIĽжϱ־ʹλ
//    <i>0: ֹжϱ־λSPRIж
//    <i>1: жϱ־λSPRIж
// <q0.11> SPTIE_SPIĴжϱ־ʹλ
//    <i>0: ֹжϱ־λSPTIж
//    <i>1: жϱ־λSPTIж
// <o0.15> SPSFF_ӻٷģʽλ
//  <0=>ͨģʽ
//  <1=>ģʽ
//<i>CPHA=0ʱЧ
//<i>ͨģʽ£ӻSSÿδһݣ8λ16λ뱻ߣһǰSSݵλĴ
//<i>ģʽ£ӻ͵ĵһҪSS½ڶϸһCLKصλĴעͻû򲻻أ
// <o0.16..17> SPDATL_ֳѡλ
//     <0=>ÿη8bit
//     <1=>ÿη16bit 
//     <2=>ÿη32bit
#define     CFG_SPI0_CR_INIT     0


#if ((CFG_SPI0_CR_INIT & (3<<10)) != 0)
	#define CFG_SPI0_INT_EN 1
#else
	#define CFG_SPI0_INT_EN 0
#endif

//</e>

//<e>SPI1
#define     CFG_SPI1_EN        0
//<i>CR_SPIƼĴ
//<i>0x00C b31~19 b18    b17~16 b15   b14   b13 b12 b11   b10   b9 b8  b7   b6   b5   b4    b3~0 
//<i>CR    -      FIFOEN SPDATL SPSFF SPIEN -   -   SPTIE SPRIE -  DIR MSTR CPHA CPOL SSDIS SPR 
// <q0.14> SPIEN_SPIʹλ
// <o0.7> MSTR_SPI豸λ
//     <0=>SPIΪ豸
//     <1=>SPIΪ豸
// <o0.0..3> SPR_ʱӷƵλ
//    <0=>PCLK1 /2
//    <1=>PCLK1 /4
//    <2=>PCLK1 /8
//    <3=>PCLK1 /16
//    <4=>PCLK1 /32
//    <5=>PCLK1 /64
//    <6=>PCLK1 /128
//    <7=>PCLK1 /256
//    <8=>PCLK1 /512
//    <9=>PCLK1 /1024
// <o0.4> SSDIS_NSSſλ
//    <0=>/ģʽ£NSS
//    <1=>/ģʽ£رNSSţMODFж
//    <i>ڴģʽ£CPHA=0λ
// <o0.5> CPOL_ʱӼԿλ
//    <0=>IDLE״̬CKڵ͵ƽ
//    <1=>IDLE״̬CKڸߵƽ
// <o0.6> CPHA_ʱλλ
//     <0=>CKڵĵһزɼ
//     <1=>CKڵĵڶزɼ
// <o0.8> DIR_ݴͷѡλ
//     <0=>MSBȷ
//     <1=>LSBȷ
// <q0.10> SPRIE_SPIĽжϱ־ʹλ
//    <i>0: ֹжϱ־λSPRIж
//    <i>1: жϱ־λSPRIж
// <q0.11> SPTIE_SPIĴжϱ־ʹλ
//    <i>0: ֹжϱ־λSPTIж
//    <i>1: жϱ־λSPTIж
// <o0.15> SPSFF_ӻٷģʽλ
//  <0=>ͨģʽ
//  <1=>ģʽ
//<i>CPHA=0ʱЧ
//<i>ͨģʽ£ӻSSÿδһݣ8λ16λ뱻ߣһǰSSݵλĴ
//<i>ģʽ£ӻ͵ĵһҪSS½ڶϸһCLKصλĴעͻû򲻻أ
// <o0.16..17> SPDATL_ֳѡλ
//     <0=>ÿη8bit
//     <1=>ÿη16bit 
//     <2=>ÿη32bit

#define     CFG_SPI1_CR_INIT     0x0


#if ((CFG_SPI1_CR_INIT & (3<<10)) != 0)
	#define CFG_SPI1_INT_EN 1
#else
	#define CFG_SPI1_INT_EN 0
#endif
//</e>
//***********************************************************/
//</e>


//<e>TWI
#define     CFG_TWI_EN          0
//<h>CR_TWIƼĴ
//<i>0x018 b31~16 b15   b14~11 b10    b9   b8    b7~6 b5~4 b3 b2  b1  b0 
//<i>CR    -      TWIEN -      TWINTE ETOT EFREE CNT  CR   -  STA STO AA 
// <o0.15> TWIEN_TWIʹλ
//    <0=>رTWI
//    <1=>ʹTWI
// <q0.10> TWINTE_TWIķͽжϱ־ʹλ
//    <i>0: ֹTWIжϱ־λTWINTж
//    <i>1: TWIжϱ־λTWINTж
// <q0.9> ETOT_߳ʱ־ʹλ
//    <i>0: ֹTWI߳ʱTOUT־
//    <i>1: TWI߳ʱTOUT־
// <q0.8> EFREE_SCLߵƽʱ־ʹλ
//    <i>0: ֹSCLߵƽʱTFREE־
//    <i>1: SCLߵƽʱTFREE־
// <o0.6..7> CNT_߳ʱλ
//    <0=>N=25000
//    <1=>N=50000
//    <2=>N=100000
//    <3=>N=200000
//    <i>Nֵ߳ʱʹ
// <o0.4..5> CR_TWIƵϵ
//    <0=>64
//    <1=>16
//    <2=>4
//    <3=>1
// <o0.2> STA_ʼλ
//    <0=>ʼ
//    <1=>߿ʱʼ
//<i>STA=1ʱʼģʽѾģʽظʼ
//<i>STASTOλʱģʽ߷һֹͣȻһʼӿڴڴģʽһڲֹͣ͵
// <o0.0> AA_Ӧ־λ
//    <0=>ظӦźţSDAߵƽ
//    <1=>ظӦźţSDA͵ƽ 
//</h>
#define     CFG_TWI_CR_INIT     0x0


//<h>ADDR_TWIַüĴ
//<i>0x014 b31~24 b23~17 b16~8 b7~1 b0 
//<i>ADDR  -      TWIAMR -     ADDR GC 
// <q0.0> GC_ͨõַʹλ
//    <i>0: ֹӦͨõַ
//    <i>1: Ӧͨõַ
// <o0.1..7> ADDR_TWIַλ<0-0x7F>
// <o0.17..23> TWIAMR_TWIַλ<0-0x7F>
//<i>TWIAMRÿһλTWIADRַĴӦĵַλλ1ַƥ߼ĵַλTWIADRеӦַλıȽϽλ0ӦλıȽϽ
//</h>
#define     CFG_TWI_ADDR_INIT 0x0


//<h>BRT_TWIüĴ
//<i>0x00C b31~8 b7~0 
//<i>BRT   -     BRT  
// <o0.0..7> BRT_TWIıλ<0-255>
//</h>
#define     CFG_TWI_BRT_INIT    0x0


//<h>HOC_TWIߵƽʱüĴ
//<i>0x008 b31~8 b7~0 
//<i>HOC   -     HOC  
// <o0.0..7> HOC_TWISCL߸ߵƽʱλ<0-255>
//</h>
#define     CFG_TWI_HOC_INIT    255


#if ((CFG_TWI_CR_INIT & (1<<10)) != 0)
	#define CFG_TWI_INT_EN 1
#else
	#define CFG_TWI_INT_EN 0
#endif
//</e>



//<e>CAN
#define     CFG_CAN_EN          0
//<h>CGCR_CANȫüĴ
//<i>0x200 b31~12 b11 b10    b9       b8        b7     b6 b5   b4     b3       b2  b1  b0    
//<i>CGCR  -      EIT DIAGEN INTERNAL LOOP-BACK IGNACK LO DDIR TSTPEN BUFFLOCK CTX CRX CANEN 
// <o0.0> CANEN_CANģʹλ
//    <0=>CANģʧ
//    <1=>CANģʹ
// <o0.1> CRX_յƽ
//    <0=>ԵƽΪ0ԵƽΪ1
//    <1=>ԵƽΪ1ԵƽΪ0
// <o0.2> CTX_͵ƽ
//    <0=>ԵƽΪ0ԵƽΪ1
//    <1=>ԵƽΪ1ԵƽΪ0
// <o0.3> BUFFLOCK_λ
//    <0=>ֹ
//    <1=>
// <o0.4> TSTPEN_ʱͬʹ
//    <0=>ʱͬʧ
//    <1=>ʱͬʹ
// <o0.5> DDIR_ݴ洢λ
//    <0=>ȽյĴ洢ڸλַյĴ洢ڵλַ
//    <1=>ȽյĴ洢ڵλַյĴ洢ڸλַ
// <o0.6> LO_ģʽ
//    <0=>/
//    <1=>ֻݣϢӦ
// <q0.7> IGNACK_ACKλ
//    <i>0:ģʽ
//    <i>1:ݺACKλ
// <q0.8> LOOPBACK_ػģʽ
//    <i>0:ģʽ
//    <i>1:ػģʽʹ
// <q0.9> INTERNAL_ڲģʽ
//    <i>0:ģʽ
//    <i>1:ڲģʽʹ
// <q0.10> DIAGEN_Ϲʹλ
//    <i>0:ģʽ
//    <i>1:Ϲʹ
// <o0.11> EIT_ж
//    <0=>ÿߴEIPNDλ
//    <1=>ֻе״̬ıʱEIPND
//</h>
#define     CFG_CAN0_CGCR_INIT 0x0
//<h>CTIM_CANʱĴ
//<i>0x204 b31~16 b15~9 b8~7 b6~3  b2~0  
//<i>CTIM  -      PSC   SJW  TSEG1 TSEG2 
// <o0.0..2> TSEG2_λ2
//    <0=>1tq
//    <1=>2tq
//    <2=>3tq
//    <3=>4tq
//    <4=>5tq
//    <5=>6tq
//    <6=>7tq
//    <7=>8tq
// <o0.3..6> TSEG1_λ1
//    <0=>Ƽ
//    <1=>2tq
//    <2=>3tq
//    <3=>4tq
//    <4=>5tq
//    <5=>6tq
//    <6=>7tq
//    <7=>8tq
//    <8=>9tq
//    <9=>10tq
//    <10=>11tq
//    <11=>12tq
//    <12=>13tq
//    <13=>14tq
//    <14=>15tq
//    <15=>16tq
// <o0.7..8> SJW_ͬת
//    <0=>1tq
//    <1=>2tq
//    <2=>3tq
//    <3=>4tq
// <o0.9..15> PSC_ԤƵ<0-127>
//<i>ʵԤƵֵ = ֵ + 2СԤƵΪ2ƵԤƵΪ128Ƶ
//</h>
#define     CFG_CAN0_CTIM_INIT  0x0

//<h>GMSKX_CANȫμĴ
//<i>0x208 b31~16 b15~1 b0   
//<i>GMSKX -      GM    XRTR 
// <q0.0> XRTR_չ֡Զλ
//  <i>0: ֹ
//  <i>1: ʹ
// <o0.1..15> XGM_IDȫλXGM bit14~0 <0-0x7FFF>
//</h>
#define     CFG_CAN0_GMSKX_INIT 0x0

//<h>GMSKB_CANȫμĴ
//<i>0x20C b31~16 b15~5 b4  b3  b2~0 
//<i>GMSKB -      GME   RTR IDE GM   
// <o0.0..2> XGM_չ֡IDȫλXGM bit17~15<0-0x7>
// <o0.3> IDE_չָ֡ʾλ
// <o0.4> RTRSRR_Զָ֡ʾλ
// <o0.5..15> GMXGM_չ֡IDλ bit28-18 <0-0x7FF>
//</h>
#define     CFG_CAN0_GMSKB_INIT 0x0

//<h>BMSKX_CANμĴ
//<i>0x210 b31~16 b15~1 b0   
//<i>BMSKX -      BM    XRTR 
// <q0.0> XRTR_չ֡Զλ
//  <i>0: ֹ
//  <i>1: ʹ
// <o0.1..15> XBM_չ֡IDλ bit14~0 <0-0x7FFF>
//</h>
#define     CFG_CAN0_BMSKX_INIT 0x0

//<h>BMSKB_CANμĴ
//<i>0x214 b31~16 b15~5 b4  b3  b2~0 
//<i>BMSKB -      BME   RTR IDE BM   
// <o0.0..2> XBM_IDȫλBM bit17~15<0-0x7>
// <o0.3> IDE_չָ֡ʾλ
// <o0.4> RTRSRR_Զָ֡ʾλ
// <o0.5..15> BMXBM_չ֡IDλbit28-18 <0-0x7FF>
//</h>
#define     CFG_CAN0_BMSKB_INIT 0x0

//<h>CIEN_CANжʹܼĴ
//<i>0x218 b31~16 b15  b14   b13   b12   b11   b10   b9   b8   b7   b6   b5   b4   b3   b2   b1   b0   
//<i>CIEN  -      EIEN IEN14 IEN13 IEN12 IEN11 IEN10 IEN9 IEN8 IEN7 IEN6 IEN5 IEN4 IEN3 IEN2 IEN1 IEN0 
// <o0.15> EIEN_жʹλ
//    <0=>ֹж
//    <1=>жϣҵTECRECб仯ʱһж
// <q0.0> IEN0_0жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.1> IEN1_1жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.2> IEN2_2жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.3> IEN3_3жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.4> IEN4_4жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.5> IEN5_5жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.6> IEN6_6жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.7> IEN7_7жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.8> IEN8_8жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.9> IEN9_9жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.10> IEN10_10жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.11> IEN11_11жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.12> IEN12_12жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.13> IEN13_13жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
// <q0.14> IEN14_14жʹλ
//    <i>0:xжʧ
//    <i>1:xжʹ
//</h>
#define     CFG_CAN0_CIEN_INIT 0x0


#if (CFG_CAN0_CIEN_INIT != 0)
	#define CFG_CAN_INT_EN 1
#else
	#define CFG_CAN_INT_EN 0
#endif


//<h>CICEN_CANжʹܼĴ
//<i>0x224 b31~16 b15   b14    b13    b12    b11    b10    b9    b8    b7    b6    b5    b4    b3    b2    b1    b0    
//<i>CICEN -      EICEN ICEN14 ICEN13 ICEN12 ICEN11 ICEN10 ICEN9 ICEN8 ICEN7 ICEN6 ICEN5 ICEN4 ICEN3 ICEN2 ICEN1 ICEN0 
// <q0.15> EICEN_жʹλ
//    <i>0=жϹʾжĴCSTPND
//    <i>1=жϹʾжĴCSTPND
// <q0.0> ICEN0_0жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.1> ICEN1_1жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.2> ICEN2_2жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.3> ICEN3_3жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.4> ICEN4_4жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.5> ICEN5_5жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.6> ICEN6_6жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.7> ICEN7_7жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.8> ICEN8_8жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.9> ICEN9_9жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.10> ICEN10_10жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.11> ICEN11_11жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.12> ICEN12_12жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.13> ICEN13_13жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
// <q0.14> ICEN14_14жʹλ
//    <0=>xжϹλûиı
//    <1=>λxλIPND
//</h>
#define     CFG_CAN0_CICEN_INIT 0x0

//<h>CTMR_CANʱĴ
//<i>0x234 b31~16 b15~0 
//<i>CTMR  -      CTMR  
// <o0.0..15> CTMR_涨ʱֵ <0-65535>
//</h>
#define     CFG_CAN0_CTMR_INIT 0x0

//***********************************************************/
//</e>



//<e>SDFM
#define CFG_SDFM_EN	0

//<i>0x00C    b31~12 b11 b10~0 
//<i>SDMFILEN -      MFE -     
// <q0.11> MFE_ݹʹλ
//<i>0: ĸݹֹFIFO
//<i>1: FENΪ1ʱݹʹ
#define CFG_SDFM0_SDMFILEN_INIT	0x0


//<i>0x008 b31~14 b13 b12~4 b3  b2  b1  b0  
//<i>SDCTL -      MIE -     HZ4 HZ3 HZ2 HZ1 
// <q0.13> MIE_Sdy_ERRжʹλ (CEVT1/2,MF,FFOVжԴ)
//<i>0: жϽֹ
//<i>1: жʹ
#define CFG_SDFM0_SDCTL_INIT	0x0000

#if ((CFG_SDFM0_SDCTL_INIT & (1<<13)) != 0)
	#define CFG_SDFM_ERR_INT_EN 1
#else
	#define CFG_SDFM_ERR_INT_EN 0
#endif

//<e>SDFM_CH1
#define CFG_SDFM_CH1_EN 0
//<h>MCSTCLKCFG1_ͨ1 ˹ʱüĴ
//<i>0x020       b31~16 b15~8  b7~0      
//<i>MCSTCLKCFG1 -      SMPPOS SCLKCYCLE 
// <o0.0..7> SCLKCYCLE_˹رSYSCLKSD_D1ʱӵĽʱӱȣõֵС10ܿܵ´Ľ롣ӦʵSD_D1 :(int)(fSCLK/fSDC1)
// <o0.8..15> SMPPOS_˹ؽλãͨΪ(int)(0.75*fSCLK/fSDC)ûԸʵźʧ΢
//</h>
#define CFG_SDFM0_MCSTCLKCFG1_INIT	0x0

//<h>SD1_CTLPARM_ͨ1ƲĴ
//<i>0x040       b31~7 b6         b5 b4        b3       b2~0 
//<i>SD1_CTLPARM -     SDDATASYNC -  SDCLKSYNC SDCLKSEL MOD  
// <o0.0..2> MOD_
//<0=>ʱز
//<1=>ʱ½ز
//<2=>ʱغ½ز
//<3=>˹ؽ룬Ϊ1
//<4=>˹ؽ룬Ϊ0
//<5=>2ʱز
// <o0.3> SDCLKSEL_SD ʱԴѡ
//<0=>SDFM ˲ʱͨʱ
//<1=>SDFM ˲ʱSD˲ʱ
// <o0.4> SDCLKSYNC_<0=>SD Clock ͨͬ
//<1=>SD Clock ͨͬ
// <o0.6> SDDATASYNC_<0=>SD Data ͨͬ
//<1=>SD Data ͨͬ
//</h>
#define CFG_SDFM0_SD1_CTLPARM_INIT	0x0

//<h>SD1_DFPARM_ͨ1 ݹ˲Ĵ
//<i>0x044      b31~13 b12      b11~10 b9 b8  b7~0 
//<i>SD1_DFPARM -      SDSYNCEN SST    AE FEN DOSR 
// <o0.0..7> DOSR_ݹ<0-255>
//ʵʹΪDOSR+1, 0xFFʾ256
// <q0.8> FEN_ݹʹ
//<i>0: ݹֹûݲ
//<i>1: ݹʹܣɹ
//<i>ע:ֹʱDOSRָλݱȻҲḴλFIFOָFIFO
// <o0.9> AE_ݹȷ
//<0=>ֹȷϱ־
//<1=>ȷϱ־
// <o0.10..11> SST_ݹṹ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.12> SDSYNCEN_ݹPWMͬ
//<i>0: ֹͬ
//<i>1: ͬʹ
//</h>
#define CFG_SDFM0_SD1_DFPARM_INIT	0x0

//<h>SD1_DPARM_ͨ1ݲĴ
//<i>0x048     b31~16 b15~11 b10 b9~0 
//<i>SD1_DPARM -      SH     DR  -    
// <o0.10> DR_ݹݱʾ
//<0=>16λ 2Ĳ
//<1=>32λ 2Ĳ
// <o0.11..15> SH_λ
//Щλָʾѡ16λݱʾʱ16λ˶λ
//</h>
#define CFG_SDFM0_SD1_DPARM_INIT	0x0

//<h>SDCMP1_ͨ1ߵ͵ƽֵĴ
// <o0.0..14> HLT_Ƚ˲޷Ÿߵƽֵ <0-16384>
//<i>0x020   b31~15 b14~0 
//<i>SDCMPH1 -      HLT   
#define CFG_SDFM0_SDCMPH1_INIT	0x7fff
 
// <o0.0..14> LLT_Ƚ˲޷ŵ͵ƽֵ <0-16384>
//<i>0x024   b31~15 b14~0 
//<i>SDCMPL1 -      LLT  
//</h>
#define CFG_SDFM0_SDCMPL1_INIT	0x0

//<h>SD1_CPARM_ͨ1 Ƚ˲Ĵ
//<i>0x054     b31~16 b15~14   b13 b12~11   b10  b9   b8~7   b6      b5      b4~0 
//<i>SD1_CPARM -      CEVT2SEL CEN CEVT1SEL HZEN MFIE CS1CS0 CEVT2EN CEVT1EN COSR 
// <o0.0..4> COSR_Ƚ
//ʵʹΪCOSR+1, 0x1Fʾ32
// <q0.5> CEVT1EN_CEVT1жʹ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> CEVT2EN_CEVT2жʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.7..8> CS1CS0_Ƚ˲ṹѡ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.9> MFIE_ж
//<i>0: ֹ
//<i>1: ʹ
// <q0.10> HZEN_"0"ƽʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.11..12> CEVT1SEL_Ƚ¼1ѡ
//<0=>ڵCOMPH1
//<1=>СڵCOMPL1 ڵCOMPH1
//<2=>ڵCOMPH2
//<3=>СڵCOMPL2ڵCOMPH2
// <q0.13> CEN_Ƚ˲ʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.14..15> CEVT2SEL_Ƚ¼2ѡ
//<0=>СڵCOMPL1
//<1=>СڵCOMPL1ڵCOMPH1
//<2=>СڵCOMPL2
//<3=>СڵCOMPL2ڵCOMPH2
//</h>
#define CFG_SDFM0_SD1_CPARM_INIT	0x0

//<h>SD1_CDATA_ͨ1ȽĴ
//<i>0x060     b31~16 b15~0  
//<i>SD1_CDATA -      DATA16 
// <o0.0..15> DATA16_Ƚֻ16λ <0-32768>
//</h>
#define CFG_SDFM0_SD1_CDATA_INIT	0x0

//<h>SD1_CMPH2_ͨ1 ֵĴ2
//<i>0x064     b31~15 b14~0 
//<i>SD1_CMPH2 -      HLT2  
// <o0.0..14> HLT2_Ƚ˲ĵڶ޷Ÿֵ <0-16384>
//</h>
#define CFG_SDFM0_SD1_CMPH2_INIT	0x0

//<h>SD1_CMPHZ_ͨ1 "0"ֵĴ
//<i>0x068     b31~15 b14~0 
//<i>SD1_CMPHZ -      HLTZ  
// <o0.0..14> HLTZ_Ƚ˲޷ŸֵҪڼ㡱¼
//ȽSDCMPHxͬжϡ <0-16384>
//</h>
#define CFG_SDFM0_SD1_CMPHZ_INIT	0x0

//<h>SD1_FIFOCTL_ͨ1FIFOƼĴ
//<i>0x06C       b31~16 b15    b14      b13  b12   b11 b10~6  b5 b4~0   
//<i>SD1_FIFOCTL -      OVFIEN DRINTSEL FFEN FFIEN -   SDFFST -  SDFFIL 
// <o0.0..4> SDFFIL_SDFIFOж<0-16>
//<i>SDFFST>= FIFO (SDFFIL)ʱж
// <o0.12> FFIEN_SDFIFOݾжʹ
// <q0.13> FFEN_SDFIFOʹ
//<i>0: ֹFIFO
//<i>1: ʹFIFO
//<i>ע:FIFOֹʱFIFOݻᱻ
// <o0.14> DRINTSEL_ݾжԴѡ
//<0=>AF1(ѡFIFO ݾж)
//<1=>SDFFINT1(ѡFIFOݾж)
// <o0.15> OVFIEN_SDFIFOжʹ
//<0=>ж
//<1=>ж(Sdy_ERR)
//</h>
#define CFG_SDFM0_SD1_FIFOCTL_INIT	0x0

//<h>SD1_SYNC_ͨ1ͬ
//<i>0x070    b31~11 b10      b9          b8       b7       b6       b5~0    
//<i>SD1_SYNC -      WTSCLREN FFSYNCCLREN WTSYNCLR WTSYNFLG WTSYNCEN SYNCSEL 
// <o0.0..5> SYNCSEL_ͨSDSYNCԴѡ
//<0=>MCM0Ƚ1ADCź
//<1=>MCM1Ƚ1ADCź
//<4=>MCM0Ƚ2ADCź
//<5=>MCM1Ƚ2ADCź
//<8=>MCM0Ƚ3ADCź
//<9=>MCM1Ƚ3ADCź
//<12=>MCM0Ƚ4ADCź
//<13=>MCM1Ƚ4ADCź
//<16=>PCA0ADCź
//<17=>TIM1Cź
//<20=>PCA1ADCź
//<21=>EXTI1ⲿжź
//<24=>PCA2ADCź
//<28=>PCA3ADCź
// <o0.6> WTSYNCEN_ȴͬʹ
//<0=>ÿݾ¼ʱдSDFIFO
//<1=>SDSYNC¼ݾ¼дSDFIFO.
//<i>ע:FIFOжź,ͬ־дFIFO
// <o0.9> FFSYNCCLREN_SDSYNCFIFOʹ
//<0=>յSDSYCԶFIFO
//<1=>յSDSYNCԶFIFO
// <o0.10> WTSCLREN_WTSYNFLG FIFOжʹ
//<0=>ֶֻWTSYNCLRλ
//<1=>SDFFINTжʱԶ
//</h>
#define CFG_SDFM0_SD1_SYNC_INIT	0x0

//<h>SD1_CMPL2_ֵĴ2
//<i>0x074     b31~15 b14~0 
//<i>SD1_CMPL2 -      LLT2  
// <o0.0..14> LLT2_Ƚ˲޷ŵֵ2 <0-16384>
//</h>
#define CFG_SDFM0_SD1_CMPL2_INIT	0x0

//<h>SD1_COMPCTL_SDȽ¼1ƼĴ
//<i>0x078       b31~12 b11~10   b9~4 b3~2     b1~0 
//<i>SD1_COMPCTL -      CEVT2DFS -    CEVT1DFS -    
// <o0.2..3> CEVT1DFS_Ƚ¼1 Դѡ
//<0=>CEVT1ֱ
//<2=>CEVT1˲
// <o0.10..11> CEVT2DFS_Ƚ¼2 Դѡ
//<0=>CEVT2ֱ
//<2=>CEVT2˲
//</h>
#define CFG_SDFM0_SD1_COMPCTL_INIT	0x0

//<h>SD1_COMPEVT2CTL_SD COMPL/CEVT2 ˲ƼĴ
//<i>0x07C           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD1_COMPEVT2CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD1_COMPEVT2CTL_INIT	0x0

//<h>SD1_COMPEVT2CLKCTL_SD COMPL/CEVT2 ˲ʱӿƼĴ
//<i>0x080              b31~10 b9~0        
//<i>SD1_COMPEVT2CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD1_COMPEVT2CLKCTL_INIT	0x0

//<h>SD1_COMPEVT1CTL_SD COMPL/CEVT1 ˲ƼĴ
//<i>0x084           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD1_COMPEVT1CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD1_COMPEVT1CTL_INIT	0x0

//<h>SD1_COMPEVT1CLKCTL_SD COMPL/CEVT1 ˲1ʱӿƼĴ
//<i>0x088              b31~10 b9~0        
//<i>SD1_COMPEVT1CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD1_COMPEVT1CLKCTL_INIT	0x0


//</e>
//<e>SDFM_CH2
#define CFG_SDFM_CH2_EN 0

//<h>MCSTCLKCFG2_ͨ2 ˹ʱüĴ
//<i>0x024       b31~16 b15~8  b7~0      
//<i>MCSTCLKCFG2 -      SMPPOS SCLKCYCLE 
// <o0.0..7> SCLKCYCLE_˹رSYSCLKSD_D1ʱӵĽʱӱ,õֵС10ܿܵ´Ľ롣ӦʵSD_D1 :(int)(fSCLK/fSDC1)
// <o0.8..15> SMPPOS_˹ؽλãͨΪ(int)(0.75*fSCLK/fSDC).ûԸʵźʧ΢
//</h>
#define CFG_SDFM0_MCSTCLKCFG2_INIT	0x0

//<h>SD2_CTLPARM_ͨ2ƲĴ
//<i>0x090       b31~7 b6         b5 b4        b3       b2~0 
//<i>SD2_CTLPARM -     SDDATASYNC -  SDCLKSYNC SDCLKSEL MOD  
// <o0.0..2> MOD_<0=>ʱز
//<1=>ʱ½ز
//<2=>ʱغ½ز
//<3=>˹ؽ룬Ϊ1
//<4=> ˹ؽ룬Ϊ0
//default:ʱز
// <o0.3> SDCLKSEL_SD ʱԴѡ
//<0=>SDFM ˲ʱͨʱ
//<1=>SDFM ˲ʱSD˲ʱ
// <o0.4> SDCLKSYNC_<0=>SD Clock ͨͬ
//<1=>SD Clock ͨͬ
// <o0.6> SDDATASYNC_<0=>SD Data ͨͬ
//<1=>SD Data ͨͬ
//</h>
#define CFG_SDFM0_SD2_CTLPARM_INIT	0x0

//<h>SD2_DFPARM_ͨ2 ݹ˲Ĵ
//<i>0x094      b31~13 b12      b11~10 b9 b8  b7~0 
//<i>SD2_DFPARM -      SDSYNCEN SST    AE FEN DOSR 
// <o0.0..7> DOSR_ݹ
//ʵʹΪDOSR+1, 0xFFʾ256
// <q0.8> FEN_ݹʹ
//<i>0: ݹֹûݲ
//<i>1: ݹʹܣɹ
//<i>ע:ֹʱDOSRָλݱȻҲḴλFIFOָFIFO
// <o0.9> AE_ݹȷ
//<0=>ֹȷϱ־
//<1=>ȷϱ־
// <o0.10..11> SST_ݹṹ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.12> SDSYNCEN_ݹPWMͬ
//<i>0: ֹͬ
//<i>1: ͬʹ
//</h>
#define CFG_SDFM0_SD2_DFPARM_INIT	0x0

//<h>SD2_DPARM_ͨ2ݲĴ
//<i>0x098     b31~16 b15~11 b10 b9~0 
//<i>SD2_DPARM -      SH     DR  -    
// <o0.10> DR_ݹݱʾ
//<0=>16λ 2Ĳ
//<1=>32λ 2Ĳ
// <o0.11..15> SH_λ
//Щλָʾѡ16λݱʾʱ16λ˶λ
//</h>
#define CFG_SDFM0_SD2_DPARM_INIT	0x0

//<h>SDCMPH2_ͨ2ߵ͵ƽֵĴ
// <o0.0..14> HLT_Ƚ˲޷Ÿߵƽֵ <0-16384>
//<i>0x050   b31~15 b14~0 
//<i>SDCMPH2 -      HLT   
#define CFG_SDFM0_SDCMPH2_INIT	0x7fff

// <o0.0..14> LLT_Ƚ˲޷ŵ͵ƽֵ <0-16384>
//<i>0x054   b31~15 b14~0 
//<i>SDCMPL2 -      LLT  
//</h>
#define CFG_SDFM0_SDCMPL2_INIT	0x0

//<h>SD2_CPARM_ͨ2 Ƚ˲Ĵ
//<i>0x0A4     b31~16 b15~14   b13 b12~11   b10  b9   b8~7   b6      b5      b4~0 
//<i>SD2_CPARM -      CEVT2SEL CEN CEVT1SEL HZEN MFIE CS1CS0 CEVT2EN CEVT1EN COSR 
// <o0.0..4> COSR_Ƚ
//ʵʹΪCOSR+1, 0x1Fʾ32
// <q0.5> CEVT1EN_CEVT1жʹ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> CEVT2EN_CEVT2жʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.7..8> CS1CS0_Ƚ˲ṹѡ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.9> MFIE_ж
//<i>0: ֹ
//<i>1: ʹ
// <q0.10> HZEN_"0"ƽʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.11..12> CEVT1SEL_Ƚ¼1ѡ
//<0=>ڵCOMPH1
//<1=>СڵCOMPL1ڵCOMPH1
//<2=>ڵCOMPH2
//<3=>СڵCOMPL2ڵCOMPH2
// <q0.13> CEN_Ƚ˲ʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.14..15> CEVT2SEL_Ƚ¼2ѡ
//<0=>СڵCOMPL1
//<1=>СڵCOMPL1ڵCOMPH1
//<2=>СڵCOMPL2
//<3=>СڵCOMPL2ڵCOMPH2
//</h>
#define CFG_SDFM0_SD2_CPARM_INIT	0x0

//<h>SD2_CDATA_ͨ2ȽĴ
//<i>0x0B0     b31~16 b15~0  
//<i>SD2_CDATA -      DATA16 
// <o0.0..15> DATA16_Ƚֻ16λ <0-32768>
//</h>
#define CFG_SDFM0_SD2_CDATA_INIT	0x0

//<h>SD2_CMPH2_ͨ2 ֵĴ
//<i>0x0B4     b31~15 b14~0 
//<i>SD2_CMPH2 -      HLT2  
// <o0.0..14> HLT2_Ƚ˲ĵڶ޷Ÿֵ <0-16384>
//</h>
#define CFG_SDFM0_SD2_CMPH2_INIT	0x0

//<h>SD2_CMPHZ_ͨ2 ֵĴ
//<i>0x0B8     b31~15 b14~0 
//<i>SD2_CMPHZ -      HLTZ  
// <o0.0..14> HLTZ_Ƚ˲޷ŸֵҪڼ㡱¼
//ȽSDCMPHxͬжϵ <0-16384>
//</h>
#define CFG_SDFM0_SD2_CMPHZ_INIT	0x0

//<h>SD2_FIFOCTL_ͨ2FIFOƼĴ
//<i>0x0BC       b31~16 b15    b14      b13  b12   b11 b10~6  b5 b4~0   
//<i>SD2_FIFOCTL -      OVFIEN DRINTSEL FFEN FFIEN -   SDFFST -  SDFFIL 
// <o0.0..4> SDFFIL_SDFIFOж<0-16>
//<i>SDFFST>= FIFO (SDFFIL)ʱж
// <o0.12> FFIEN_SDFIFOݾжʹ
// <q0.13> FFEN_SDFIFOʹ
//<i>0: ֹFIFO
//<i>1: ʹFIFO
//<i>ע:FIFOֹʱFIFOݻᱻ
// <o0.14> DRINTSEL_ݾжԴѡ
//<0=>AF1(ѡFIFO ݾж)
//<1=>SDFFINT1(ѡFIFOݾж)
// <o0.15> OVFIEN_SDFIFOжʹ
//<0=>ж
//<1=>ж(Sdy_ERR)
//</h>
#define CFG_SDFM0_SD2_FIFOCTL_INIT	0x0

//<h>SD2_SYNC_ͨ2ͬ
//<i>0x0C0    b31~11 b10      b9          b8       b7       b6       b5~0    
//<i>SD2_SYNC -      WTSCLREN FFSYNCCLREN WTSYNCLR WTSYNFLG WTSYNCEN SYNCSEL 
// <o0.0..5> SYNCSEL_ͨSDSYNCԴѡ
//<0=>MCM0Ƚ1ADCź
//<1=>MCM1Ƚ1ADCź
//<4=>MCM0Ƚ2ADCź
//<5=>MCM1Ƚ2ADCź
//<8=>MCM0Ƚ3ADCź
//<9=>MCM1Ƚ3ADCź
//<12=>MCM0Ƚ4ADCź
//<13=>MCM1Ƚ4ADCź
//<16=>PCA0ADCź
//<17=>TIM1Cź
//<20=>PCA1ADCź
//<21=>EXTI1ⲿжź
//<24=>PCA2ADCź
//<28=>PCA3ADCź
// <o0.6> WTSYNCEN_ȴͬʹ
//<0=>ÿݾ¼ʱдSDFIFO
//<1=>SDSYNC¼ݾ¼дSDFIFO.(<i>ע:FIFOжź,ͬ־дFIFO)
// <o0.9> FFSYNCCLREN_SDSYNCFIFOʹ
//<0=>յSDSYCԶFIFO
//<1=>յSDSYNCԶFIFO
// <o0.10> WTSCLREN_WTSYNFLG FIFOжʹ
//<0=>ֶֻWTSYNCLRλ
//<1=>SDFFINTжʱԶ
//</h>
#define CFG_SDFM0_SD2_SYNC_INIT	0x0

//<h>SD2_CMPL2_ֵĴ2
//<i>0x0C4     b31~15 b14~0 
//<i>SD2_CMPL2 -      LLT2  
// <o0.0..14> LLT2_Ƚ˲޷ŵֵ2 <0-16384>
//</h>
#define CFG_SDFM0_SD2_CMPL2_INIT	0x0

//<h>SD2_COMPCTL_SDȽ¼1ƼĴ
//<i>0x0C8       b31~12 b11~10   b9~4 b3~2     b1~0 
//<i>SD2_COMPCTL -      CEVT2DFS -    CEVT1DFS -    
// <o0.2..3> CEVT1DFS_Ƚ¼1 Դѡ
//<0=>CEVT1ֱ
//<1=>
//<2=>CEVT1˲
//<3=>
// <o0.10..11> CEVT2DFS_Ƚ¼2 Դѡ
//<0=>CEVT2ֱ
//<1=>
//<2=>CEVT2˲
//<3=>
//</h>
#define CFG_SDFM0_SD2_COMPCTL_INIT	0x0

//<h>SD2_COMPEVT2CTL_SDCOMPL/CEVT2˲2ƼĴ
//<i>0x0CC           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD2_COMPEVT2CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD2_COMPEVT2CTL_INIT	0x0

//<h>SD2_COMPEVT2CLKCTL_SDCOMPL/CEVT2˲2ʱӿƼĴ
//<i>0x0D0              b31~10 b9~0        
//<i>SD2_COMPEVT2CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD2_COMPEVT2CLKCTL_INIT	0x0

//<h>SD2_COMPEVT1CTL_SDCOMPL/CEVT1˲2ƼĴ
//<i>0x0D4           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD2_COMPEVT1CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD2_COMPEVT1CTL_INIT	0x0

//<h>SD2_COMPEVT1CLKCTL_SDCOMPL/CEVT1˲2ʱӿƼĴ
//<i>0x0D8              b31~10 b9~0        
//<i>SD2_COMPEVT1CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD2_COMPEVT1CLKCTL_INIT	0x0


//</e>
//<e>SDFM_CH3
#define CFG_SDFM_CH3_EN 0

//<h>MCSTCLKCFG3_ͨ3 ˹ʱüĴ
//<i>0x028       b31~16 b15~8  b7~0      
//<i>MCSTCLKCFG3 -      SMPPOS SCLKCYCLE 
// <o0.0..7> SCLKCYCLE_˹رSYSCLKSD_D1ʱӵĽʱӱ,õֵС10ܿܵ´Ľ롣ӦʵSD_D1 :(int)(fSCLK/fSDC1)
// <o0.8..15> SMPPOS_˹ؽλãͨΪ(int)(0.75*fSCLK/fSDC).ûԸʵźʧ΢
//</h>
#define CFG_SDFM0_MCSTCLKCFG3_INIT	0x0

//<h>SD3_CTLPARM_ͨ3ƲĴ
//<i>0x0E0       b31~7 b6         b5 b4        b3       b2~0 
//<i>SD3_CTLPARM -     SDDATASYNC -  SDCLKSYNC SDCLKSEL MOD  
// <o0.0..2> MOD_<0=>ʱز
//<1=>ʱ½ز
//<2=>ʱغ½ز
//<3=>˹ؽ룬Ϊ1
//<4=> ˹ؽ룬Ϊ0
//default:ʱز
// <o0.3> SDCLKSEL_SD ʱԴѡ
//<0=>SDFM ˲ʱͨʱ
//<1=>SDFM ˲ʱSD˲ʱ
// <o0.4> SDCLKSYNC_<0=>SD Clock ͨͬ
//<1=>SD Clock ͨͬ
// <o0.6> SDDATASYNC_<0=>SD Data ͨͬ
//<1=>SD Data ͨͬ
//</h>
#define CFG_SDFM0_SD3_CTLPARM_INIT	0x0

//<h>SD3_DFPARM_ͨ3 ݹ˲Ĵ
//<i>0x0E4      b31~13 b12      b11~10 b9 b8  b7~0 
//<i>SD3_DFPARM -      SDSYNCEN SST    AE FEN DOSR 
// <o0.0..7> DOSR_ݹ
//ʵʹΪDOSR+1, 0xFFʾ256
// <q0.8> FEN_ݹʹ
//<i>0: ݹֹûݲ
//<i>1: ݹʹܣɹ
//<i>ע:ֹʱDOSRָλݱȻҲḴλFIFOָFIFO
// <o0.9> AE_ݹȷ
//<0=>ֹȷϱ־
//<1=>ȷϱ־
// <o0.10..11> SST_ݹṹ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.12> SDSYNCEN_ݹPWMͬ
//<i>0: ֹͬ
//<i>1: ͬʹ
//</h>
#define CFG_SDFM0_SD3_DFPARM_INIT	0x0

//<h>SD3_DPARM_ͨ3ݲĴ
//<i>0x0E8     b31~16 b15~11 b10 b9~0 
//<i>SD3_DPARM -      SH     DR  -    
// <o0.10> DR_ݹݱʾ
//<0=>16λ 2Ĳ
//<1=>32λ 2Ĳ
// <o0.11..15> SH_λ
//Щλָʾѡ16λݱʾʱ16λ˶λ
//</h>
#define CFG_SDFM0_SD3_DPARM_INIT	0x0

//<h>SDCMPH3_ͨ3ߵ͵ƽֵĴ
// <o0.0..14> HLT_Ƚ˲޷Ÿߵƽֵ <0-16384>
//<i>0x080   b31~15 b14~0 
//<i>SDCMPH3 -      HLT  
#define CFG_SDFM0_SDCMPH3_INIT	0x7fff

// <o0.0..14> LLT_Ƚ˲޷ŵ͵ƽֵ <0-16384>
//<i>0x084   b31~15 b14~0 
//<i>SDCMPL3 -      LLT  
//</h>
#define CFG_SDFM0_SDCMPL3_INIT	0x0

//<h>SD3_CPARM_ͨ3 Ƚ˲Ĵ
//<i>0x0F4     b31~16 b15~14   b13 b12~11   b10  b9   b8~7   b6      b5      b4~0 
//<i>SD3_CPARM -      CEVT2SEL CEN CEVT1SEL HZEN MFIE CS1CS0 CEVT2EN CEVT1EN COSR 
// <o0.0..4> COSR_Ƚ
//ʵʹΪCOSR+1, 0x1Fʾ32
// <q0.5> CEVT1EN_CEVT1жʹ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> CEVT2EN_CEVT2жʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.7..8> CS1CS0_Ƚ˲ṹѡ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.9> MFIE_ж
//<i>0: ֹ
//<i>1: ʹ
// <q0.10> HZEN_"0"ƽʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.11..12> CEVT1SEL_Ƚ¼1ѡ
//<0=>ڵCOMPH1
//<1=>СڵCOMPL1 ڵCOMPH1
//<2=>ڵCOMPH2
//<3=>СڵCOMPL2ڵCOMPH2
// <q0.13> CEN_Ƚ˲ʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.14..15> CEVT2SEL_Ƚ¼2ѡ
//<0=>СڵCOMPL1
//<1=>СڵCOMPL1ڵCOMPH1
//<2=>СڵCOMPL2
//<3=>СڵCOMPL2ڵCOMPH2
//</h>
#define CFG_SDFM0_SD3_CPARM_INIT	0x0

//<h>SD3_CDATA_ͨ3ȽĴ
//<i>0x100     b31~16 b15~0  
//<i>SD3_CDATA -      DATA16 
// <o0.0..15> DATA16_Ƚֻ16λ <0-32768>
//</h>
#define CFG_SDFM0_SD3_CDATA_INIT	0x0

//<h>SD3_CMPH2_ͨ3 ֵĴ
//<i>0x104     b31~15 b14~0 
//<i>SD3_CMPH2 -      HLT2  
// <o0.0..14> HLT2_Ƚ˲ĵڶ޷Ÿֵ <0-16384>
//</h>
#define CFG_SDFM0_SD3_CMPH2_INIT	0x0

//<h>SD3_CMPHZ_ͨ3 ֵĴ
//<i>0x108     b31~15 b14~0 
//<i>SD3_CMPHZ -      HLTZ  
// <o0.0..14> HLTZ_Ƚ˲޷ŸֵҪڼ㡱¼
//ȽSDCMPHxͬжϵ <0-16384>
//</h>
#define CFG_SDFM0_SD3_CMPHZ_INIT	0x0

//<h>SD3_FIFOCTL_ͨ3FIFOƼĴ
//<i>0x10C       b31~16 b15    b14      b13  b12   b11 b10~6  b5 b4~0   
//<i>SD3_FIFOCTL -      OVFIEN DRINTSEL FFEN FFIEN -   SDFFST -  SDFFIL 
// <o0.0..4> SDFFIL_SDFIFOж<0-16>
//<i>SDFFST>= FIFO (SDFFIL)ʱж
// <o0.12> FFIEN_SDFIFOݾжʹ
// <q0.13> FFEN_SDFIFOʹ
//<i>0: ֹFIFO
//<i>1: ʹFIFO
//<i>ע:FIFOֹʱFIFOݻᱻ
// <o0.14> DRINTSEL_ݾжԴѡ
//<0=>AF1(ѡFIFO ݾж)
//<1=>SDFFINT1(ѡFIFOݾж)
// <o0.15> OVFIEN_SDFIFOжʹ
//<0=>ж
//<1=>ж(Sdy_ERR)
//</h>
#define CFG_SDFM0_SD3_FIFOCTL_INIT	0x0

//<h>SD3_SYNC_ͨ3ͬ
//<i>0x110    b31~11 b10      b9          b8       b7       b6       b5~0    
//<i>SD3_SYNC -      WTSCLREN FFSYNCCLREN WTSYNCLR WTSYNFLG WTSYNCEN SYNCSEL 
// <o0.0..5> SYNCSEL_ͨSDSYNCԴѡ
//<0=>MCM0Ƚ1ADCź
//<1=>MCM1Ƚ1ADCź
//<4=>MCM0Ƚ2ADCź
//<5=>MCM1Ƚ2ADCź
//<8=>MCM0Ƚ3ADCź
//<9=>MCM1Ƚ3ADCź
//<12=>MCM0Ƚ4ADCź
//<13=>MCM1Ƚ4ADCź
//<16=>PCA0ADCź
//<17=>TIM1Cź
//<20=>PCA1ADCź
//<21=>EXTI1ⲿжź
//<24=>PCA2ADCź
//<28=>PCA3ADCź
// <o0.6> WTSYNCEN_ȴͬʹ
//<0=>ÿݾ¼ʱдSDFIFO
//<1=>SDSYNC¼ݾ¼дSDFIFO.(<i>ע:FIFOжź,ͬ־дFIFO)
// <o0.9> FFSYNCCLREN_SDSYNCFIFOʹ
//<0=>յSDSYCԶFIFO
//<1=>յSDSYNCԶFIFO
// <o0.10> WTSCLREN_WTSYNFLG FIFOжʹ
//<0=>ֶֻWTSYNCLRλ
//<1=>SDFFINTжʱԶ
//</h>
#define CFG_SDFM0_SD3_SYNC_INIT	0x0

//<h>SD3_CMPL2_ֵĴ2
//<i>0x114     b31~15 b14~0 
//<i>SD3_CMPL2 -      LLT2  
// <o0.0..14> LLT2_Ƚ˲޷ŵֵ2 <0-16384>
//</h>
#define CFG_SDFM0_SD3_CMPL2_INIT	0x0

//<h>SD3_COMPCTL_SDȽ¼1ƼĴ
//<i>0x118       b31~12 b11~10   b9~4 b3~2     b1~0 
//<i>SD3_COMPCTL -      CEVT2DFS -    CEVT1DFS -    
// <o0.2..3> CEVT1DFS_Ƚ¼1 Դѡ
//<0=>CEVT1ֱ
//<1=>
//<2=>CEVT1˲
//<3=>
// <o0.10..11> CEVT2DFS_Ƚ¼2 Դѡ
//<0=>CEVT2ֱ
//<1=>
//<2=>CEVT2˲
//<3=>
//</h>
#define CFG_SDFM0_SD3_COMPCTL_INIT	0x0

//<h>SD3_COMPEVT2CTL_SD COMPL/CEVT2˲3ƼĴ
//<i>0x11C           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD3_COMPEVT2CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD3_COMPEVT2CTL_INIT	0x0

//<h>SD3_COMPEVT2CLKCTL_SD COMPL/CEVT2˲3ʱӿƼĴ
//<i>0x120              b31~10 b9~0        
//<i>SD3_COMPEVT2CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD3_COMPEVT2CLKCTL_INIT	0x0

//<h>SD3_COMPEVT1CTL_SD COMPL/CEVT1˲3ƼĴ
//<i>0x124           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD3_COMPEVT1CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD3_COMPEVT1CTL_INIT	0x0

//<h>SD3_COMPEVT1CLKCTL_SD COMPL/CEVT1˲3ʱӿƼĴ
//<i>0x128              b31~10 b9~0        
//<i>SD3_COMPEVT1CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD3_COMPEVT1CLKCTL_INIT	0x0
//</e>

//<e>SDFM_CH4
#define CFG_SDFM_CH4_EN 0

//<h>MCSTCLKCFG4_ͨ4 ˹ʱüĴ
//<i>0x02C       b31~16 b15~8  b7~0      
//<i>MCSTCLKCFG4 -      SMPPOS SCLKCYCLE 
// <o0.0..7> SCLKCYCLE_˹رSYSCLKSD_D1ʱӵĽʱӱ,õֵС10ܿܵ´Ľ롣ӦʵSD_D1 :(int)(fSCLK/fSDC1)
// <o0.8..15> SMPPOS_˹ؽλãͨΪ(int)(0.75*fSCLK/fSDC).ûԸʵźʧ΢
//</h>
#define CFG_SDFM0_MCSTCLKCFG4_INIT	0x0

//<h>SD4_CTLPARM_ͨ4ƲĴ
//<i>0x130       b31~7 b6         b5 b4        b3       b2~0 
//<i>SD4_CTLPARM -     SDDATASYNC -  SDCLKSYNC SDCLKSEL MOD  
// <o0.0..2> MOD_<0=>ʱز
//<1=>ʱ½ز
//<2=>ʱغ½ز
//<3=>˹ؽ룬Ϊ1
//<4=> ˹ؽ룬Ϊ0
//default:ʱز
// <o0.3> SDCLKSEL_SD ʱԴѡ
//<0=>SDFM ˲ʱͨʱ
//<1=>SDFM ˲ʱSD˲ʱ
// <o0.4> SDCLKSYNC_<0=>SD Clock ͨͬ
//<1=>SD Clock ͨͬ
// <o0.6> SDDATASYNC_<0=>SD Data ͨͬ
//<1=>SD Data ͨͬ
//</h>
#define CFG_SDFM0_SD4_CTLPARM_INIT	0x0

//<h>SD4_DFPARM_ͨ4 ݹ˲Ĵ
//<i>0x134      b31~13 b12      b11~10 b9 b8  b7~0 
//<i>SD4_DFPARM -      SDSYNCEN SST    AE FEN DOSR 
// <o0.0..7> DOSR_ݹ
//ʵʹΪDOSR+1, 0xFFʾ256
// <q0.8> FEN_ݹʹ
//<i>0: ݹֹûݲ
//<i>1: ݹʹܣɹ
//<i>ע:ֹʱDOSRָλݱȻҲḴλFIFOָFIFO
// <o0.9> AE_ݹȷ
//<0=>ֹȷϱ־
//<1=>ȷϱ־
// <o0.10..11> SST_ݹṹ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.12> SDSYNCEN_ݹPWMͬ
//<i>0: ֹͬ
//<i>1: ͬʹ
//</h>
#define CFG_SDFM0_SD4_DFPARM_INIT	0x0

//<h>SD4_DPARM_ͨ4ݲĴ
//<i>0x138     b31~16 b15~11 b10 b9~0 
//<i>SD4_DPARM -      SH     DR  -    
// <o0.10> DR_ݹݱʾ
//<0=>16λ 2Ĳ
//<1=>32λ 2Ĳ
// <o0.11..15> SH_λ
//Щλָʾѡ16λݱʾʱ16λ˶λ
//</h>
#define CFG_SDFM0_SD4_DPARM_INIT	0x0

//<h>SDCMPH4_ͨ4ߵ͵ƽֵĴ
// <o0.0..14> HLT_Ƚ˲޷Ÿߵƽֵ <0-16384>
//<i>0x0B0   b31~15 b14~0 
//<i>SDCMPH4 -      HLT   
#define CFG_SDFM0_SDCMPH4_INIT	0x7fff

// <o0.0..14> LLT_Ƚ˲޷ŵ͵ƽֵ <0-16384>
//<i>SDCMPL4_ͨ4͵ƽֵĴ
//<i>0x0B4   b31~15 b14~0 
//<i>SDCMPL4 -      LLT   
//</h>
#define CFG_SDFM0_SDCMPL4_INIT	0x0

//<h>SD4_CPARM_ͨ4 Ƚ˲Ĵ
//<i>0x144     b31~16 b15~14   b13 b12~11   b10  b9   b8~7   b6      b5      b4~0 
//<i>SD4_CPARM -      CEVT2SEL CEN CEVT1SEL HZEN MFIE CS1CS0 CEVT2EN CEVT1EN COSR 
// <o0.0..4> COSR_Ƚ
//ʵʹΪCOSR+1, 0x1Fʾ32
// <q0.5> CEVT1EN_CEVT1жʹ
//<i>0: ֹ
//<i>1: ʹ
// <q0.6> CEVT2EN_CEVT2жʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.7..8> CS1CS0_Ƚ˲ṹѡ
//<0=>ʹSincfastṹ
//<1=>ʹSinc1ṹ
//<2=>ʹSinc2ṹ
//<3=>ʹSinc3ṹ
// <q0.9> MFIE_ж
//<i>0: ֹ
//<i>1: ʹ
// <q0.10> HZEN_"0"ƽʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.11..12> CEVT1SEL_Ƚ¼1ѡ
//<0=>ڵCOMPH1
//<1=>СڵCOMPL1 ڵCOMPH1
//<2=>ڵCOMPH2
//<3=>СڵCOMPL2ڵCOMPH2
// <q0.13> CEN_Ƚ˲ʹ
//<i>0: ֹ
//<i>1: ʹ
// <o0.14..15> CEVT2SEL_Ƚ¼2ѡ
//<0=>СڵCOMPL1
//<1=>СڵCOMPL1ڵCOMPH1
//<2=>СڵCOMPL2
//<3=>СڵCOMPL2ڵCOMPH2
//</h>
#define CFG_SDFM0_SD4_CPARM_INIT	0x0

//<h>SD4_CDATA_ͨ4ȽĴ
//<i>0x150     b31~16 b15~0  
//<i>SD4_CDATA -      DATA16 
// <o0.0..15> DATA16_Ƚֻ16λ <0-32768>
//</h>
#define CFG_SDFM0_SD4_CDATA_INIT	0x0

//<h>SD4_CMPH2_ͨ4 ֵĴ
//<i>0x154     b31~15 b14~0 
//<i>SD4_CMPH2 -      HLT2  
// <o0.0..14> HLT2_Ƚ˲ĵڶ޷Ÿֵ <0-16384>
//</h>
#define CFG_SDFM0_SD4_CMPH2_INIT	0x0

//<h>SD4_CMPHZ_ͨ4 ֵĴ
//<i>0x158     b31~15 b14~0 
//<i>SD4_CMPHZ -      HLTZ  
// <o0.0..14> HLTZ_Ƚ˲޷ŸֵҪڼ㡱¼
//ȽSDCMPHxͬжϵ <0-16384>
//</h>
#define CFG_SDFM0_SD4_CMPHZ_INIT	0x0

//<h>SD4_FIFOCTL_ͨ4 FIFOƼĴ
//<i>0x15C       b31~16 b15    b14      b13  b12   b11 b10~6  b5 b4~0   
//<i>SD4_FIFOCTL -      OVFIEN DRINTSEL FFEN FFIEN -   SDFFST -  SDFFIL 
// <o0.0..4> SDFFIL_SDFIFOж<0-16>
//<i>SDFFST>= FIFO (SDFFIL)ʱж
// <o0.12> FFIEN_SDFIFOݾжʹ

// <q0.13> FFEN_SDFIFOʹ
//<i>0: ֹFIFO
//<i>1: ʹFIFO
//<i>ע:FIFOֹʱFIFOݻᱻ
// <o0.14> DRINTSEL_ݾжԴѡ
//<0=>AF1(ѡFIFO ݾж)
//<1=>SDFFINT1(ѡFIFOݾж)
// <o0.15> OVFIEN_SDFIFOжʹ
//<0=>ж
//<1=>ж(Sdy_ERR)
//</h>
#define CFG_SDFM0_SD4_FIFOCTL_INIT	0x0

//<h>SD4_SYNC_ͨ4ͬ
//<i>0x160    b31~11 b10      b9          b8       b7       b6       b5~0    
//<i>SD4_SYNC -      WTSCLREN FFSYNCCLREN WTSYNCLR WTSYNFLG WTSYNCEN SYNCSEL 
// <o0.0..5> SYNCSEL_ͨSDSYNCԴѡ
//<0=>MCM0Ƚ1ADCź
//<1=>MCM1Ƚ1ADCź
//<4=>MCM0Ƚ2ADCź
//<5=>MCM1Ƚ2ADCź
//<8=>MCM0Ƚ3ADCź
//<9=>MCM1Ƚ3ADCź
//<12=>MCM0Ƚ4ADCź
//<13=>MCM1Ƚ4ADCź
//<16=>PCA0ADCź
//<17=>TIM1Cź
//<20=>PCA1ADCź
//<21=>EXTI1ⲿжź
//<24=>PCA2ADCź
//<28=>PCA3ADCź

// <o0.6> WTSYNCEN_ȴͬʹ
//<0=>ÿݾ¼ʱдSDFIFO
//<1=>SDSYNC¼ݾ¼дSDFIFO.(<i>ע:FIFOжź,ͬ־дFIFO)

// <o0.9> FFSYNCCLREN_SDSYNCFIFOʹ
//<0=>յSDSYCԶFIFO
//<1=>յSDSYNCԶFIFO
// <o0.10> WTSCLREN_WTSYNFLG FIFOжʹ
//<0=>ֶֻWTSYNCLRλ
//<1=>SDFFINTжʱԶ
//</h>
#define CFG_SDFM0_SD4_SYNC_INIT	0x0

//<h>SD4_CMPL2_ֵĴ2
//<i>0x164     b31~15 b14~0 
//<i>SD4_CMPL2 -      LLT2  
// <o0.0..14> LLT2_Ƚ˲޷ŵֵ2 <0-16384>
//</h>
#define CFG_SDFM0_SD4_CMPL2_INIT	0x0

//<h>SD4_COMPCTL_SDȽ¼1ƼĴ
//<i>0x168       b31~12 b11~10   b9~4 b3~2     b1~0 
//<i>SD4_COMPCTL -      CEVT2DFS -    CEVT1DFS -    
// <o0.2..3> CEVT1DFS_Ƚ¼1 Դѡ
//<0=>CEVT1ֱ
//<1=>
//<2=>CEVT1˲
//<3=>
// <o0.10..11> CEVT2DFS_Ƚ¼2 Դѡ
//<0=>CEVT2ֱ
//<1=>
//<2=>CEVT2˲
//<3=>
//</h>
#define CFG_SDFM0_SD4_COMPCTL_INIT	0x0

//<h>SD4_COMPEVT2CTL_SD COMPL/CEVT2˲4ƼĴ
//<i>0x16C           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD4_COMPEVT2CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD4_COMPEVT2CTL_INIT	0x0

//<h>SD4_COMPEVT2CLKCTL_SD COMPL/CEVT2˲4ʱӿƼĴ
//<i>0x170              b31~10 b9~0        
//<i>SD4_COMPEVT2CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD4_COMPEVT2CLKCTL_INIT	0x0

//<h>SD4_COMPEVT1CTL_SD COMPL/CEVT1˲4ƼĴ
//<i>0x174           b31~16 b15     b14 b13~9  b8~4    b3~0 
//<i>SD4_COMPEVT1CTL -      FILINIT -   THRESH SAMPWIN -    
// <o0.4..8> SAMPWIN_ֵ˲ڡ ҪΪSAMPWIN+1
// <o0.9..13> THRESH_ֵ˲ֵΪʹı״̬smpleбٳTHRESH෴״̬
// <o0.15> FILINIT_ֵ˲ʼ
//<0=>
//<1=>ʼΪֵ
//</h>
#define CFG_SDFM0_SD4_COMPEVT1CTL_INIT	0

//<h>SD4_COMPEVT1CLKCTL_SD COMPL/CEVT1˲4ƼĴ
//<i>0x178              b31~10 b9~0        
//<i>SD4_COMPEVT1CLKCTL -      CLKPRESCALE 
// <o0.0..9> CLKPRESCALE_ֵ˲ʱԤš֮ϵͳʱ
//</h>
#define CFG_SDFM0_SD4_COMPEVT1CLKCTL_INIT	0x0

#if ((CFG_SDFM0_SD1_FIFOCTL_INIT & (1<<12)) != 0)
	#define CFG_SDFM_DR1_INT_EN 1
#else
	#define CFG_SDFM_DR1_INT_EN 0
#endif

#if ((CFG_SDFM0_SD2_FIFOCTL_INIT & (1<<12)) != 0)
	#define CFG_SDFM_DR2_INT_EN 1
#else
	#define CFG_SDFM_DR2_INT_EN 0
#endif

#if ((CFG_SDFM0_SD3_FIFOCTL_INIT & (1<<12)) != 0)
	#define CFG_SDFM_DR3_INT_EN 1
#else
	#define CFG_SDFM_DR3_INT_EN 0
#endif

#if ((CFG_SDFM0_SD4_FIFOCTL_INIT & (1<<12)) != 0)
	#define CFG_SDFM_DR4_INT_EN 1
#else
	#define CFG_SDFM_DR4_INT_EN 0
#endif
//***********************************************************/
//</e>
//***********************************************************/
//</e>

#define CFG_BOD_INT_EN 0
#define CFG_RCC_INT_EN 0
#define CFG_NVIC_ISER0_INIT ((uint32_t)CFG_WWDT_INT_EN\
													|(CFG_BOD_INT_EN << 1)\
													|(CFG_RCC_INT_EN << 2)\
													|(CFG_EXTI0_INT_EN << 3)\
													|(CFG_EXTI1_INT_EN << 4)\
													|(CFG_EXTI2_INT_EN << 5)\
													|(CFG_EXTI3_INT_EN << 6)\
  												|(CFG_EXTI7_4_INT_EN << 7)\
													|(CFG_DMA_CH0_INT_EN << 8)\
													|(CFG_DMA_CH1_INT_EN << 9)\
													|(CFG_DMA_CH2_INT_EN << 10)\
													|(CFG_DMA_CH3_INT_EN << 11)\
  												|(CFG_DMA_CH4_INT_EN << 12)\
  												|(CFG_DMA_CH5_7_INT_EN << 13)\
													|(CFG_MCM0_FLT_INT_EN << 14)\
													|(CFG_MCM1_FLT_INT_EN << 15)\
													|(CFG_MCM0_PWM_INT_EN << 16)\
													|(CFG_MCM1_PWM_INT_EN << 17)\
													|(CFG_ADC0_INT_EN << 18)\
													|(CFG_ADC1_INT_EN << 19)\
													|(CFG_ADC2_INT_EN << 20)\
													|(CFG_PCA0_FLT_INT_EN << 21)\
													|(CFG_PCA1_FLT_INT_EN << 22)\
													|(CFG_PCA2_FLT_INT_EN << 23)\
													|(CFG_PCA3_FLT_INT_EN << 24)\
													|(CFG_PCA0_INT_EN << 25)\
													|(CFG_PCA1_INT_EN << 26)\
													|(CFG_PCA2_INT_EN << 27)\
													|(CFG_PCA3_INT_EN << 28)\
													|(CFG_CMP0_INT_EN << 29)\
													|(CFG_CMP1_INT_EN << 30)\
													|(CFG_CMP2_INT_EN << 31))
													
#define CFG_NVIC_ISER1_INIT ((uint32_t)CFG_QEI0_INT_EN\
													|(CFG_QEI1_INT_EN << 1)\
													|(CFG_CAN_INT_EN << 2)\
													|(CFG_TWI_INT_EN << 3)\
													|(CFG_SPI0_INT_EN << 4)\
													|(CFG_SPI1_INT_EN << 5)\
													|(CFG_UART0_INT_EN << 6)\
													|(CFG_UART1_INT_EN << 7)\
													|(CFG_UART2_INT_EN << 8)\
													|(CFG_UART3_INT_EN << 9)\
													|(CFG_TIM0_INT_EN << 10)\
													|(CFG_TIM1_INT_EN << 11)\
													|(CFG_EXTI15_8_INT_EN << 12)\
													|(CFG_SDFM_ERR_INT_EN << 13)\
													|(CFG_SDFM_DR1_INT_EN << 14)\
													|(CFG_SDFM_DR2_INT_EN << 15)\
													|(CFG_SDFM_DR3_INT_EN << 16)\
													|(CFG_SDFM_DR4_INT_EN << 17)\
													)
										
void IPCA_InitCfg(void);        
void WWDT_InitCfg(void);        
void GPIO_InitCfg(void);        
void EXTI_InitCfg(void);
void DMA_InitCfg(void);          
void MCM0_InitCfg(void);        
void MCM1_InitCfg(void);
void PCA_InitCfg(void);        
void MACP_InitCfg(void);           
void TIM_InitCfg(void);        
void ADC_InitCfg(void);        
void QEI_InitCfg(void);        
void AMOC_InitCfg(void);       
void UART_InitCfg(void);       
void SPI_InitCfg(void);        
void TWI_InitCfg(void);   
void CAN_InitCfg(void); 
void SysTick_InitCfg(void);    
void NVIC_InitCfg(void);       
void Peripherals_InitCfg(void);
void IWDT_InitCfg(void);
void SYSCFG_InitCfg(void);
void SDFM_InitCfg(void);

#endif //__CFG33F2053_H__
